{***********************************************************************}
{ TTIWDBPlanner component                                               }
{ for Delphi & C++Builder                                               }
{                                                                       }
{ written by TMS Software                                               }
{            copyright  2002 - 2017                                    }
{            Email: info@tmssoftware.com                                }
{            Web: http://www.tmssoftware.com                            }
{                                                                       }
{ The source code is given as is. The author is not responsible         }
{ for any possible damage done due to the use of this code.             }
{ The component can be freely used in any application. The complete     }
{ source code remains property of the author and may not be distributed,}
{ published, given or sold in any form as such. No parts of the source  }
{ code can be included in any other component or application without    }
{ written authorization of the author.                                  }
{***********************************************************************}


unit IWDBWebPlanner;

interface

{$I TMSDEFS.INC}

uses
  IWWebPlanner, Classes, DB, Windows, SysUtils, Graphics, Dialogs, ComObj, ActiveX;


const
  CDaysInMonth:array[1..12] of word = (31,28,31,30,31,30,31,31,30,31,30,31);

  MAJ_VER = 3; // Major version nr.
  MIN_VER = 2; // Minor version nr.
  REL_VER = 5; // Release nr.
  BLD_VER = 0; // Build nr.
  DATE_VER = 'April, 2017'; // Month version

type
  TTIWDBItemSource = class;

  TResourceMapItem = class(TCollectionItem)
  private
    FResourceIndex: Integer;
    FPositionIndex: Integer;
    FDisplayName: string;
  public
    constructor Create(Collection: TCollection); override;
  published
    property ResourceIndex: Integer read FResourceIndex write FResourceIndex;
    property PositionIndex: Integer read FPositionIndex write FPositionIndex;
    property DisplayName: string read FDisplayName write FDisplayName;
  end;

  TResourceMap = class(TCollection)
  private
    FOwner: TComponent;
    function GetItem(Index: Integer): TResourceMapItem;
    procedure SetItem(Index: Integer; const Value: TResourceMapItem);
  protected
  public
    function Add: TResourceMapItem;
    function Insert(index: Integer): TResourceMapItem;
    property Items[Index: Integer]: TResourceMapItem read GetItem write SetItem; default;
    constructor Create(AOwner: TComponent);
    function GetOwner: TPersistent; override;
    function ResToPosition(Value: Integer): Integer;
    function PositionToRes(Value: Integer): Integer;
    function PositionName(Value: Integer): string;
  end;

  TCreateKeyEvent = procedure(Sender:TObject; APlannerItem: TPlannerItem; var Key:string) of object;

  TAcceptEvent = procedure(Sender:TObject; Fields:TFields; var Accept: Boolean) of object;

  TDBItemSQLEvent = procedure(Sender:TObject; APlannerItem:TPlannerItem) of object;

  TTIWItemSource = class(TComponent)
  private
    FPlanner: TTIWWebPlanner;
    FUpdateCount: Integer;
    FOnCreateKey: TCreateKeyEvent;
    FActive: Boolean;
    FOnSetFilter: TNotifyEvent;
    FOnResetFilter: TNotifyEvent;
    FOnAccept: TAcceptEvent;
    FLocateIdx: Integer;
    FLocateSpecifier: string;
    FLocateParams: TFindTextParams;
    procedure SetPlanner(Value: TTIWWebPlanner);
    procedure SetActive(const Value: Boolean);
  protected
  public
    procedure ClearDBItems; virtual;
    procedure ReadDBItems; virtual;
    procedure WriteDBItem; virtual;
    procedure ReadDBItem; virtual;
    procedure SynchDBItems; virtual;
    procedure AddDBItem; virtual;
    procedure GotoDBItem; virtual;
    procedure DeleteDBItem(APlanner:TTIWWebPlanner); virtual;
    procedure ItemChanged(DBKey:string); virtual;
    procedure Refresh; virtual;
    procedure Reload; virtual;
    procedure Next; virtual;
    procedure Prev; virtual;
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;
    procedure PlannerChanged; virtual;
    property Planner:TTIWWebPlanner read FPlanner write SetPlanner;
    procedure BeginUpdate;
    procedure EndUpdate;
    function IsUpdating: Boolean;
    function CreateKey:string; virtual;
    function LocateFirstItem(Specifier: string; Params: TFindTextParams;
      var StartTime,EndTime: TDateTime;var Key:string): Boolean; virtual;
    function LocateNextItem(var StartTime,EndTime: TDateTime;
      var Key:string):Boolean; virtual;
    function PosToDay(Pos: Integer): TDateTime; virtual;
    function PosToRes(Pos: Integer): Integer; virtual;
    procedure MapItemTimeOnPlanner(APlannerItem: TPlannerItem); virtual;
  published
    property Active: Boolean read FActive write SetActive default true;
    property OnCreateKey: TCreateKeyEvent read FOnCreateKey write FOnCreateKey;
    property OnResetFilter: TNotifyEvent read FOnResetFilter write FOnResetFilter;
    property OnSetFilter: TNotifyEvent read FOnSetFilter write FOnSetFilter;
    property OnAccept: TAcceptEvent read FOnAccept write FOnAccept;
  end;

  TTIWDBPlannerDataLink = class(TDataLink)
  private
    FDBItemSource: TTIWDBItemSource;
  protected
    procedure ActiveChanged; override;
    procedure DataSetChanged; override;
    procedure DataSetScrolled(Distance: Integer); override;
    procedure RecordChanged(Field: TField); override;
  public
    constructor Create(ADBItemSource: TTIWDBItemSource);
    destructor Destroy; override;
  end;

  TFieldsItemEvent = procedure(Sender: TObject; Fields:TFields; Item: TPlannerItem) of object;

  TResourceToPosEvent = procedure(Sender: TObject; Field: TField;
    var Position: Integer; var Accept: Boolean) of object;

  TPosToResourceEvent = procedure(Sender: TObject; Field: TField;
    Position: Integer) of object;

  TFieldsToTimeEvent = procedure(Sender: TObject; Fields:TFields;
    var dtS: TDateTime; var dtE: TDateTime) of object;

  TTimeToFieldsEvent = procedure(Sender: TObject; Fields:TFields;
    dtS: TDateTime; dtE: TDateTime) of object;


  TTIWDBItemSource = class(TTIWItemSource)
  private
    FEndTimeField: string;
    FStartTimeField: string;
    FKeyField: string;
    FResourceField: string;
    FSubjectField: string;
    FNotesField: string;
    FAutoIncKey: Boolean;
    FDataLink: TTIWDBPlannerDataLink;
    FOnFieldsToItem: TFieldsItemEvent;
    FOnItemToFields: TFieldsItemEvent;
    FOnChangeQuery: TNotifyEvent;
    FOnPositionToResource: TPosToResourceEvent;
    FOnResourceToPosition: TResourceToPosEvent;
    FReadOnly: Boolean;
    FOnInsertItem: TDBItemSQLEvent;
    FOnDeleteItem: TDBItemSQLEvent;
    FOnUpdateItem: TDBItemSQLEvent;
    FOnItemsRead: TNotifyEvent;
    FOnFieldsToTime: TFieldsToTimeEvent;
    FOnTimeToFields: TTimeToFieldsEvent;
    FUpdateByQuery: Boolean;
    FResourceMap: TResourceMap;
    function GetDataSource: TDataSource;
    procedure SetDataSource(const Value: TDataSource);
    procedure SetEndTimeField(const Value: string);
    procedure SetStartTimeField(const Value: string);
    procedure SetKeyField(const Value: string);
    procedure SetNotesField(const Value: string);
    procedure SetSubjectField(const Value: string);
    procedure SetResourceField(const Value: string);
    procedure SetReadOnly(const Value: Boolean);
    procedure SetResourceMap(const Value: TResourceMap);
  protected
    procedure Notification(AComponent: TComponent; AOperation: TOperation); override;
    function CheckDataSet: Boolean;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Next; override;
    procedure Prev; override;
    function LocateFirstItem(Specifier: string; Params: TFindTextParams;
      var StartTime,EndTime: TDateTime;var Key:string): Boolean; override;
    function LocateNextItem(var StartTime,EndTime: TDateTime;
      var Key:string):Boolean; override;
    procedure FieldsToTime(Fields:TFields; var dtS: TDateTime; var dtE: TDateTime); virtual;
    procedure TimeToFields(Fields:TFields; dtS: TDateTime; dtE: TDateTime); virtual;
    procedure FieldsToRes(Fields: TFields; var Position: Integer; var Accept: Boolean); virtual;
    procedure ResToFields(Fields: TFields; Position: Integer); virtual;
  published
    property AutoIncKey: Boolean read FAutoIncKey write FAutoIncKey;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property ResourceMap: TResourceMap read FResourceMap write SetResourceMap;
    property StartTimeField: string read FStartTimeField write SetStartTimeField;
    property EndTimeField: string read FEndTimeField write SetEndTimeField;
    property KeyField: string  read FKeyField write SetKeyField;
    property ReadOnly: Boolean read FReadOnly write SetReadOnly;
    property ResourceField: string read FResourceField write SetResourceField;
    property SubjectField: string read FSubjectField write SetSubjectField;
    property NotesField: string read FNotesField write SetNotesField;
    property UpdateByQuery: Boolean read FUpdateByQuery write FUpdateByQuery;
    property OnFieldsToItem: TFieldsItemEvent read FOnFieldsToItem write FOnFieldsToItem;
    property OnItemToFields: TFieldsItemEvent read FOnItemToFields write FOnItemToFields;
    property OnChangeQuery: TNotifyEvent read FOnChangeQuery write FOnChangeQuery;
    property OnResourceToPosition: TResourceToPosEvent read FOnResourceToPosition
      write FOnResourceToPosition;
    property OnPositionToResource: TPosToResourceEvent read FOnPositionToResource
      write FOnPositionToResource;
    property OnInsertItem: TDBItemSQLEvent read FOnInsertItem write FOnInsertItem;
    property OnDeleteItem: TDBItemSQLEvent read FOnDeleteItem write FOnDeleteItem;
    property OnUpdateItem: TDBItemSQLEvent read FOnUpdateItem write FOnUpdateItem;
    property OnItemsRead: TNotifyEvent read FOnItemsRead write FOnItemsRead;
    property OnFieldsToTime: TFieldsToTimeEvent read FOnFieldsToTime write FOnFieldsToTime;
    property OnTimeToFields: TTimeToFieldsEvent read FOnTimeToFields write FOnTimeToFields;
  end;

  TDBDayMode = (dmMultiDay, dmMultiResource, dmMultiDayRes, dmMultiResDay);

  TResourceNameEvent = procedure(Sender: TObject; ResourceIndex: Integer; var ResourceName: string) of object;

  TTIWDBDaySource = class(TTIWDBItemSource)
  private
    FMode: TDBDayMode;
    FDayIncrement: Integer;
    FDay: TDateTime;
    FAutoHeaderUpdate: Boolean;
    FDateFormat: string;
    FNumberOfDays: Integer;
    FNumberOfResources: Integer;
    FOnGetResourceName: TResourceNameEvent;
    procedure SetDay(const Value: TDateTime);
    procedure SetMode(const Value: TDBDayMode);
    procedure SetNumberOfDays(const Value: Integer);
    procedure SetNumberOfResources(const Value: Integer);
    procedure ConfigurePlanner(var Span: Integer);
    function CalcItemPos(DatePos,ResourcePos: Integer): Integer;
  protected
  public
    constructor Create(AOwner: TComponent); override;
    procedure SynchDBItems; override;
    procedure ReadDBItems; override;
    procedure WriteDBItem; override;
    procedure ReadDBItem; override;
    procedure AddDBItem; override;
    procedure GotoDBItem; override;
    procedure DeleteDBItem(APlanner: TTIWWebPlanner); override;
    procedure ItemChanged(DBKey:string); override;
    procedure Next; override;
    procedure Prev; override;
    procedure PlannerChanged; override;
    function PosToDay(Pos: Integer): TDateTime; override;
    function PosToRes(Pos: Integer): Integer; override;
    procedure MapItemTimeOnPlanner(APlannerItem: TPlannerItem); override;
  published
    property AutoHeaderUpdate: Boolean read FAutoHeaderUpdate write FAutoHeaderUpdate default False;
    property DateFormat: string read FDateFormat write FDateFormat;
    property Day: TDateTime read FDay write SetDay;
    property DayIncrement: Integer read FDayIncrement write FDayIncrement default 1;
    property Mode: TDBDayMode read FMode write SetMode;
    property NumberOfDays: Integer read FNumberOfDays write SetNumberOfDays default 7;
    property NumberOfResources: Integer read FNumberOfResources write SetNumberOfResources default 1;
    property OnGetResourceName: TResourceNameEvent read FOnGetResourceName write FOnGetResourceName;
  end;

  TTIWDBCustomPeriodSource = class(TTIWDBItemSource)
  private
    FNumberOfResources: Integer;
    FOnGetResourceName: TResourceNameEvent;
    FAutoHeaderUpdate: Boolean;
    procedure ConfigurePlanner;
    procedure SetNumberOfResources(const Value: Integer);
  public
    constructor Create(AOwner: TComponent); override;
    procedure SynchDBItems; override;
    procedure ReadDBItems; override;
    procedure WriteDBItem; override;
    procedure ReadDBItem; override;
    procedure AddDBItem; override;
    procedure GotoDBItem; override;
    procedure DeleteDBItem(APlanner: TTIWWebPlanner); override;
    procedure ItemChanged(DBKey:string); override;
    procedure Next; override;
    procedure Prev; override;
  public
  published
    property AutoHeaderUpdate: Boolean read FAutoHeaderUpdate write FAutoHeaderUpdate default False;
    property NumberOfResources: Integer read FNumberOfResources write SetNumberOfResources default 1;
    property OnGetResourceName: TResourceNameEvent read FOnGetResourceName write FOnGetResourceName;
  end;

  TTIWDBPeriodSource = class(TTIWDBCustomPeriodSource)
  private
    FStartDate: TDateTime;
    FEndDate: TDateTime;
    procedure SetEndDate(const Value: TDateTime);
    procedure SetStartDate(const Value: TDateTime);
  public
    constructor Create(AOwner: TComponent); override;
    procedure Loaded; override;
    procedure PlannerChanged; override;
    procedure UpdatePlanner; virtual;
  published
    property StartDate: TDateTime read FStartDate write SetStartDate;
    property EndDate: TDateTime read FEndDate write SetEndDate;
  end;

  TTIWDBHalfDayPeriodSource = class(TTIWDBPeriodSource)
  private
  public
    procedure PlannerChanged; override;
    procedure UpdatePlanner; override;
  published
  end;

  TTIWDBMonthSource = class(TTIWDBCustomPeriodSource)
  private
    FMonth: integer;
    FYear: integer;
    procedure SetMonth(const Value: integer);
    procedure SetYear(const Value: integer);
  public
    constructor Create(AOwner: TComponent); override;
    procedure Loaded; override;
    procedure PlannerChanged; override;
    procedure UpdatePlanner;
  published
    property Month: integer read FMonth write SetMonth;
    property Year: integer read FYear write SetYear;
  end;

  TTIWDBWeekSource = class(TTIWDBCustomPeriodSource)
  private
    FMonth: Integer;
    FYear: Integer;
    FWeekStart: Integer;
    FWeeks: Integer;
    procedure SetMonth(const Value: integer);
    procedure SetYear(const Value: integer);
    procedure SetWeekStart(const Value: Integer);
    procedure SetWeeks(const Value: Integer);
  public
    constructor Create(AOwner: TComponent); override;
    procedure Loaded; override;
    procedure PlannerChanged; override;
    procedure UpdatePlanner;
  published
    property Month: Integer read FMonth write SetMonth;
    property Year: Integer read FYear write SetYear;
    property WeekStart: Integer read FWeekStart write SetWeekStart;
    property Weeks: Integer read FWeeks write SetWeeks;
  end;

  TTIWDBMultiMonthSource = class(TTIWDBItemSource)
  private
    FNumberOfMonths: Integer;
    FStartMonth: Integer;
    FYear: Integer;
    FStartDate, FEndDate: TDateTime;
    FAutoHeaderUpdate: Boolean;
    procedure ConfigurePlanner;
    procedure SetNumberOfMonths(const Value: Integer);
    procedure SetStartMonth(const Value: Integer);
    procedure SetYear(const Value: Integer);
  public
    procedure Loaded; override;
    procedure PlannerChanged; override;
    procedure UpdatePlanner;
    constructor Create(AOwner: TComponent); override;
    procedure SynchDBItems; override;
    procedure ReadDBItems; override;
    procedure WriteDBItem; override;
    procedure ReadDBItem; override;
    procedure AddDBItem; override;
    procedure GotoDBItem; override;
    procedure DeleteDBItem(APlanner: TTIWWebPlanner); override;
    procedure ItemChanged(DBKey:string); override;
    procedure Next; override;
    procedure Prev; override;
  published
    property AutoHeaderUpdate: Boolean read FAutoHeaderUpdate write FAutoHeaderUpdate default False;
    property StartMonth: Integer read FStartMonth write SetStartMonth;
    property Year: Integer read FYear write SetYear;
    property NumberOfMonths: Integer read FNumberOfMonths write SetNumberOfMonths;
  end;

  TTIWDBTimeLineSource = class(TTIWDBItemSource)
  private
    FStartDate: TDateTime;
    FEndDate: TDateTime;
    FAutoHeaderUpdate: Boolean;
    FNumberOfResources: Integer;
    FOnGetResourceName: TResourceNameEvent;
    procedure SetEndDate(const Value: TDateTime);
    procedure SetStartDate(const Value: TDateTime);
    procedure SetNumberOfResources(const Value: Integer);
    procedure ConfigurePlanner;
  protected
  public
    constructor Create(AOwner: TComponent); override;
    procedure SynchDBItems; override;
    procedure ReadDBItems; override;
    procedure WriteDBItem; override;
    procedure ReadDBItem; override;
    procedure AddDBItem; override;
    procedure GotoDBItem; override;
    procedure DeleteDBItem(APlanner: TTIWWebPlanner); override;
    procedure ItemChanged(DBKey:string); override;
    procedure Next; override;
    procedure Prev; override;
    procedure PlannerChanged; override;
    procedure UpdatePlanner;
    function PosToRes(Pos: Integer): Integer; override;
  published
    property AutoHeaderUpdate: Boolean read FAutoHeaderUpdate write FAutoHeaderUpdate default False;
    property NumberOfResources: Integer read FNumberOfResources write SetNumberOfResources default 3;
    property StartDate: TDateTime read FStartDate write SetStartDate;
    property EndDate: TDateTime read FEndDate write SetEndDate;
    property OnGetResourceName: TResourceNameEvent read FOnGetResourceName write FOnGetResourceName;
  end;

  TDateItem = class(TCollectionItem)
  private
    FDate: TDateTime;
  published
    property Date: TDateTime read FDate write FDate;
  end;

  TDateCollection = class(TCollection)
  private
    function GetItem(Index: Integer): TDateItem;
    procedure SetItem(Index: Integer; const Value: TDateItem);  
  public
    function Add: TDateItem;
    function Insert(index: Integer): TDateItem;
    property Items[Index: Integer]: TDateItem read GetItem write SetItem; default;
    constructor Create;
    function HasDate(ADate: TDateTime): Boolean;
    function DatePos(ADate: TDateTime): Integer;
    function PosDate(Pos: Integer): TDateTime;
  end;

  TTIWDBDisjunctDaySource = class(TTIWDBItemSource)
  private
    FDates: TDateCollection;
    FAutoHeaderUpdate: Boolean;
    FDateFormat: string;
    procedure ConfigurePlanner;
    procedure SetDates(const Value: TDateCollection);
  protected
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SynchDBItems; override;
    procedure ReadDBItems; override;
    procedure WriteDBItem; override;
    procedure ReadDBItem; override;
    procedure AddDBItem; override;
    procedure GotoDBItem; override;
    procedure DeleteDBItem(APlanner: TTIWWebPlanner); override;
    procedure ItemChanged(DBKey:string); override;
    procedure PlannerChanged; override;
    procedure MapItemTimeOnPlanner(APlannerItem: TPlannerItem); override;
    function PosToDay(Pos: Integer): TDateTime; override;    
  published
    property AutoHeaderUpdate: Boolean read FAutoHeaderUpdate write FAutoHeaderUpdate default False;
    property DateFormat: string read FDateFormat write FDateFormat;
    property Dates: TDateCollection read FDates write SetDates;
  end;

  TTIWDBActiveDaySource = class(TTIWDBItemSource)
  private
    FMode: TDBDayMode;
    FDay: TDateTime;
    FAutoHeaderUpdate: Boolean;
    FDateFormat: string;
    FNumberOfDays: Integer;
    FNumberOfResources: Integer;
    FOnGetResourceName: TResourceNameEvent;
    procedure SetDay(const Value: TDateTime);
    procedure SetMode(const Value: TDBDayMode);
    procedure SetNumberOfDays(const Value: Integer);
    procedure SetNumberOfResources(const Value: Integer);
    procedure ConfigurePlanner(var Span: Integer);
    function CalcItemPos(DatePos,ResourcePos: Integer): Integer;
    function DaysToSpan(Days: Integer): Integer;
    function DayInSpan(ADay: Integer): TDateTime;
    function IDayToPos(ADate: TDateTime): Integer;
    function IPosToDay(ADay: Integer): TDateTime;
  protected

  public
    constructor Create(AOwner: TComponent); override;
    procedure SynchDBItems; override;
    procedure ReadDBItems; override;
    procedure WriteDBItem; override;
    procedure ReadDBItem; override;
    procedure AddDBItem; override;
    procedure GotoDBItem; override;
    procedure DeleteDBItem(APlanner: TTIWWebPlanner); override;
    procedure ItemChanged(DBKey:string); override;
    procedure Next; override;
    procedure Prev; override;
    procedure PlannerChanged; override;
    function PosToDay(Pos: Integer): TDateTime; override;
    function PosToRes(Pos: Integer): Integer; override;
    procedure MapItemTimeOnPlanner(APlannerItem: TPlannerItem); override;
  published
    property AutoHeaderUpdate: Boolean read FAutoHeaderUpdate write FAutoHeaderUpdate default False;
    property DateFormat: string read FDateFormat write FDateFormat;
    property Day: TDateTime read FDay write SetDay;
    property Mode: TDBDayMode read FMode write SetMode;
    property NumberOfDays: Integer read FNumberOfDays write SetNumberOfDays default 7;
    property NumberOfResources: Integer read FNumberOfResources write SetNumberOfResources default 1;
    property OnGetResourceName: TResourceNameEvent read FOnGetResourceName write FOnGetResourceName;
  end;

  TTIWDBWebPlanner = class(TTIWWebPlanner)
  private
    FItemSource: TTIWItemSource;
    procedure SetItemSource(const Value: TTIWItemSource);
  protected
    procedure Notification(AComponent: TComponent; AOperation: TOperation); override;
    procedure ItemMoved(FromBegin, FromEnd, FromPos, ToBegin, ToEnd, ToPos: Integer); override;
    procedure ItemSized(FromBegin, FromEnd, ToBegin, ToEnd: Integer); override;
    procedure ItemEdited(Item: TPlannerItem); override;
    procedure ItemChanged(Item : TPlannerItem); override;
    procedure ItemSelected(Item: TPlannerItem); override;
    procedure LoadItems;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Loaded; override;
    procedure Refresh;
    procedure FreeItem(APlannerItem:TPlannerItem); override;
    procedure UpdateItem(APlannerItem:TPlannerItem); override;
    procedure RefreshItem(APlannerItem:TPlannerItem); virtual;
    function CreateItem: TPlannerItem; override;
    function CreateItemAtSelection:TPlannerItem; override;
    function CreateNonDBItem: TPlannerItem;
    function CreateNonDBItemAtSelection: TPlannerItem;
    function PosToDay(Pos: Integer): TDateTime; override;
    function GetVersionNr: Integer; override;
    function GetVersionString:string; override;
  published
    property ItemSource: TTIWItemSource read FItemSource write SetItemSource;
  end;




implementation

function EndOfMonth(dt: TDateTime): TDateTime;
var
  da,mo,ye: word;
begin
  DecodeDate(dt,ye,mo,da);
  da := DaysInMonth(mo,ye);
  Result := EncodeDate(ye,mo,da);
end;

function NextMonth(mo: word): word;
begin
  Result := mo + 1;
  if mo = 13 then
    Result := 1;
end;

{ TTIWDBPlanner }

constructor TTIWDBWebPlanner.Create(AOwner: TComponent);
begin
  inherited;
  FItemSource := nil;
end;

function TTIWDBWebPlanner.CreateItem: TPlannerItem;
begin
  Result := inherited CreateItem;
  Items.DBItem := Result;
  if Assigned(FItemSource) then
    FItemSource.AddDBItem;
end;

function TTIWDBWebPlanner.CreateNonDBItem: TPlannerItem;
begin
  Result := inherited CreateItem;
  Result.NonDBItem := True;
end;

function TTIWDBWebPlanner.CreateNonDBItemAtSelection: TPlannerItem;
begin
  Result := inherited CreateItemAtSelection;
  Result.NonDBItem := True;
end;

destructor TTIWDBWebPlanner.Destroy;
begin
  if Assigned(FItemSource) then
    FItemSource.Planner := nil;
  inherited;
end;

procedure TTIWDBWebPlanner.FreeItem(APlannerItem: TPlannerItem);
begin
  Items.DBItem := APlannerItem.ParentItem;
  if not Items.DBItem.NonDBItem then
  begin
    if Assigned(FItemSource) then
      FItemSource.DeleteDBItem(Self);
  end;

  inherited FreeItem(APlannerItem);
end;


procedure TTIWDBWebPlanner.ItemEdited(Item: TPlannerItem);
begin
  inherited;
  if Item.NonDBItem then
    Exit;

  Items.DBItem := Item.ParentItem;
  if Assigned(FItemSource) then
    FItemSource.WriteDBItem;
end;

procedure TTIWDBWebPlanner.ItemChanged(Item: TPlannerItem);
begin
  inherited;

  if Item.NonDBItem then
    Exit;

  Items.DBItem := Item.ParentItem;
  Items.DBItem.RealTime := false;
  if Assigned(FItemSource) then
    FItemSource.WriteDBItem;
end;


procedure TTIWDBWebPlanner.ItemMoved(FromBegin, FromEnd, FromPos, ToBegin, ToEnd,
  ToPos: Integer);
begin
  inherited;

  if Items.Selected.NonDBItem then
    Exit;

  Items.DBItem := Items.Selected.ParentItem;
  if Assigned(FItemSource) then
    FItemSource.WriteDBItem;
end;

procedure TTIWDBWebPlanner.ItemSized(FromBegin, FromEnd, ToBegin,
  ToEnd: Integer);
begin
  inherited;

  if Items.Selected.NonDBItem then
    Exit;

  Items.DBItem := Items.Selected.ParentItem;
  if Assigned(FItemSource) then
    FItemSource.WriteDBItem;
end;

procedure TTIWDBWebPlanner.Loaded;
begin
  inherited;
end;

procedure TTIWDBWebPlanner.Notification(AComponent: TComponent;
  AOperation: TOperation);
begin
  inherited;
  if (AOperation = opRemove) and
     (AComponent = FItemSource) then
    begin
      FItemSource := Nil;
      Items.Clear;
    end;
end;

procedure TTIWDBWebPlanner.SetItemSource(const Value: TTIWItemSource);
begin
  if Assigned(Value) and Assigned(Value.Planner) then
  begin
    if FItemSource = Value then
      Exit;

    if Value.Planner = Self then
    begin
      ShowMessage('Cannot assign itemsource to multiple TTIWDBWebPlanner components');
      Exit;
    end;
  end;

  if Assigned(FItemSource) and (Value <> FItemSource) then
    FItemSource.Planner := nil;

  FItemSource := Value;
  if Assigned(FItemSource) then
  begin
    FItemSource.Planner := Self;
    if not (csLoading in ComponentState) then LoadItems;
  end
  else
  begin
    Items.BeginUpdate;
    Items.Clear;
    Items.EndUpdate;
  end;
end;

procedure TTIWDBWebPlanner.RefreshItem(APlannerItem: TPlannerItem);
begin
  inherited;

  if APlannerItem.NonDBItem then
    Exit;

  Items.DBItem := APlannerItem.ParentItem;
  if Assigned(FItemSource) then
    FItemSource.ReadDBItem;
end;

procedure TTIWDBWebPlanner.UpdateItem(APlannerItem: TPlannerItem);
begin
  inherited;

  if APlannerItem.NonDBItem then
    Exit;

  Items.DBItem := APlannerItem.ParentItem;
  if Assigned(FItemSource) then
    FItemSource.WriteDBItem;
end;

procedure TTIWDBWebPlanner.ItemSelected(Item: TPlannerItem);
begin
  inherited;
  if Item.NonDBItem then
    Exit;

  Items.DBItem := Item.ParentItem;
  Items.Selected := Items.DBItem;
  if Assigned(FItemSource) then
    FItemSource.GotoDBItem;
end;

procedure TTIWDBWebPlanner.LoadItems;
begin
  Items.BeginUpdate;
  Items.ClearDB;
  FItemSource.ReadDBItems;
  Items.EndUpdate;
end;

function TTIWDBWebPlanner.CreateItemAtSelection: TPlannerItem;
begin
  Result := Inherited CreateItemAtSelection;
  Items.DBItem := Result;
  if Assigned(FItemSource) then
    FItemSource.AddDBItem;
end;


procedure TTIWDBWebPlanner.Refresh;
begin
  inherited;
  if Assigned(ItemSource) then
    ItemSource.Reload;
end;

function TTIWDBWebPlanner.PosToDay(Pos: Integer): TDateTime;
begin
  if Assigned(ItemSource) then
    Result := ItemSource.PosToDay(Pos)
  else
    Result := inherited PosToDay(Pos);
end;

function TTIWDBWebPlanner.GetVersionNr: Integer;
begin
  Result := MakeLong(MakeWord(BLD_VER,REL_VER),MakeWord(MIN_VER,MAJ_VER));
end;

function TTIWDBWebPlanner.GetVersionString:string;
var
  vn: Integer;
begin
  vn := GetVersionNr;
  Result := IntToStr(Hi(Hiword(vn)))+'.'+IntToStr(Lo(Hiword(vn)))+'.'+IntToStr(Hi(Loword(vn)))+'.'+IntToStr(Lo(Loword(vn)))+' '+DATE_VER;
end;


{ TTIWItemSource }

procedure TTIWItemSource.AddDBItem;
begin

end;

constructor TTIWItemSource.Create(AOwner: TComponent);
begin
  inherited;
  FPlanner := Nil;
  FUpdateCount := 0;
  FActive := True;
end;


procedure TTIWItemSource.DeleteDBItem(APlanner: TTIWWebPlanner);
begin

end;

destructor TTIWItemSource.Destroy;
begin
  inherited;
end;


procedure TTIWItemSource.Next;
begin
  ClearDBItems;
  ReadDBItems;
end;

procedure TTIWItemSource.Prev;
begin
  ClearDBItems;
  ReadDBItems;
end;

procedure TTIWItemSource.ReadDBItems;
begin
end;

procedure TTIWItemSource.ItemChanged(DBKey: string);
begin
end;

procedure TTIWItemSource.WriteDBItem;
begin
end;

procedure TTIWItemSource.ReadDBItem;
begin
end;

procedure TTIWItemSource.BeginUpdate;
begin
  Inc(FUpdateCount);
end;

procedure TTIWItemSource.EndUpdate;
begin
  if FUpdateCount > 0 then Dec(FUpdateCount);
end;

function TTIWItemSource.IsUpdating: Boolean;
begin
  Result := FUpdateCount > 0;
end;

procedure TTIWItemSource.SynchDBItems;
begin
end;

procedure TTIWItemSource.ClearDBItems;
begin
  // 1.5.0.3 optimization
  if IsUpdating then
    Exit;

  if Assigned(FPlanner) then
  begin
    FPlanner.Items.BeginUpdate;
    FPlanner.Items.ClearDB;
    FPlanner.Items.EndUpdate;
    FPlanner.Items.Selected := nil;
  end;
end;

procedure TTIWItemSource.Refresh;
begin
  if Assigned(FPlanner) then
    FPlanner.Invalidate;
end;

procedure TTIWItemSource.GotoDBItem;
begin

end;


function TTIWItemSource.CreateKey: string;
var
  GUID: TGUID;
  Key: string;
begin
  Key := '';

  if Assigned(FOnCreateKey) then
    FOnCreateKey(Self,FPlanner.Items.DBItem, Key)
  else
  begin
    CoCreateGUID(GUID);
    Key := GUIDToString(GUID);
  end;

  Result := Key;
end;

procedure TTIWItemSource.SetPlanner(Value :TTIWWebPlanner);
begin
  FPlanner := Value;
  PlannerChanged;
end;

procedure TTIWItemSource.PlannerChanged;
begin

end;

procedure TTIWItemSource.SetActive(const Value: Boolean);
begin
  if FActive <> Value then
  begin
    FActive := Value;
    if not FActive then
    begin
      ClearDBItems;
      Inc(FUpdateCount)
    end
    else
    begin
      if FUpdateCount > 0 then
      begin
        Dec(FUpdateCount);
        if FUpdateCount = 0 then Reload;
      end;
    end;
  end;
end;

procedure TTIWItemSource.Reload;
begin
  ClearDBItems;
  ReadDBItems;
end;

function TTIWItemSource.LocateFirstItem(Specifier: string; Params: TFindTextParams;
  var StartTime, EndTime: TDateTime;var Key:string): Boolean;
begin
  Result := False;
end;

function TTIWItemSource.LocateNextItem(var StartTime,
  EndTime: TDateTime;var Key:string): Boolean;
begin
  Result := False;
end;

function TTIWItemSource.PosToDay(Pos: Integer): TDateTime;
begin
  Result := 0;
end;

function TTIWItemSource.PosToRes(Pos: Integer): Integer;
begin
  Result := 0;
end;

procedure TTIWItemSource.MapItemTimeOnPlanner(APlannerItem: TPlannerItem);
begin

end;

{ TDBPlannerDataLink }

procedure TTIWDBPlannerDataLink.ActiveChanged;
begin
  inherited;
  {$IFDEF TMSCODESITE}
  SendMsg('ActiveChanged');
  {$ENDIF}
  FDBItemSource.ClearDBItems;
  FDBItemSource.ReadDBItems;
  FDBItemSource.Refresh;
end;


constructor TTIWDBPlannerDataLink.Create(ADBItemSource: TTIWDBItemSource);
begin
  inherited Create;
  FDBItemSource := ADBItemSource;
end;

procedure TTIWDBPlannerDataLink.DataSetChanged;
begin
  inherited;
  {$IFDEF TMSCODESITE}
  SendMsg('DatasetChanged');
  {$ENDIF}
end;

procedure TTIWDBPlannerDataLink.DataSetScrolled(Distance: Integer);
begin
  inherited;
  {$IFDEF TMSCODESITE}
  SendMsg('DatasetScrolled');
  {$ENDIF}
end;

destructor TTIWDBPlannerDataLink.Destroy;
begin
  inherited;
end;

procedure TTIWDBPlannerDataLink.RecordChanged(Field: TField);
begin
  inherited;

  if FDBItemSource.IsUpdating then Exit;

  {what is the connected planner ??}

  if Assigned(DataSet) then
    if DataSet.Active then
    begin
      {$IFDEF TMSCODESITE}
      if Assigned(Field) then
       SendMsg('RecordChanged : '+Field.FieldName +' of '+DataSet.FieldByName(FDBItemSource.KeyField).AsString)
      else
       SendMsg('RecordChanged : '+DataSet.FieldByName(FDBItemSource.KeyField).AsString);
      {$ENDIF}
      if DataSet.State = dsBrowse then
      FDBItemSource.ItemChanged(DataSet.FieldByName(FDBItemSource.KeyField).AsString);
    end;
end;


{ TTIWDBItemSource }

constructor TTIWDBItemSource.Create(AOwner: TComponent);
begin
  inherited;
  FDataLink := TTIWDBPlannerDataLink.Create(Self);
  FResourceMap := TResourceMap.Create(Self);
end;

destructor TTIWDBItemSource.Destroy;
begin
  FDataLink.Free;
  FResourceMap.Free;
  inherited;
end;

function TTIWDBItemSource.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TTIWDBItemSource.SetDataSource(const Value: TDataSource);
begin
  if FDataLink.DataSource <> Value then
  begin
    FDataLink.DataSource := Value;
  end;
end;

procedure TTIWDBItemSource.SetStartTimeField(const Value: string);
begin
  FStartTimeField := Value;
end;

procedure TTIWDBItemSource.SetKeyField(const Value: string);
begin
  FKeyField := Value;
end;

procedure TTIWDBItemSource.SetResourceField(const Value: string);
begin
  FResourceField := Value;
end;

procedure TTIWDBItemSource.SetEndTimeField(const Value: string);
begin
  FEndTimeField := Value;
end;

procedure TTIWDBItemSource.SetNotesField(const Value: string);
begin
  FNotesField := Value;
end;

procedure TTIWDBItemSource.SetSubjectField(const Value: string);
begin
  FSubjectField := Value;
end;


procedure TTIWDBItemSource.Next;
begin
  inherited;
  if Assigned(FOnChangeQuery) then
    FOnChangeQuery(Self);
end;

procedure TTIWDBItemSource.Prev;
begin
  inherited;
  if Assigned(FOnChangeQuery) then
    FOnChangeQuery(Self);
end;

procedure TTIWDBItemSource.Notification(AComponent: TComponent;
  AOperation: TOperation);
begin
  inherited;
  if (AOperation = opRemove) and (AComponent = FDataLink.DataSource) then
     FDataLink.DataSource := nil;
end;

procedure TTIWDBItemSource.SetReadOnly(const Value: Boolean);
begin
  FReadOnly := Value;
end;

function MatchItem(Specifier,Subject,Notes: string; Param: TFindTextParams): Boolean;
var
  SrchText: string;

begin
  Result := False;

  if fnCaptionText in Param then
  begin
    SrchText := Subject;

    if fnIgnoreHTMLTags in Param then
      SrchText := HTMLStrip(SrchText);

    if not (fnMatchCase in Param) then
    begin
      SrchText := AnsiUpperCase(SrchText);
      Specifier := AnsiUpperCase(Specifier);
    end;

    if fnMatchFull in Param then
      Result := (Specifier = SrchText)
    else
      if fnMatchStart in Param then
        Result := Pos(Specifier,SrchText) = 1
      else
        if fnMatchRegular in Param then
          Result := MatchStr(Specifier,SrchText,fnMatchCase in Param);
  end;

  if Result then Exit;

  if fnText in Param then
  begin
    SrchText := Notes;

    if fnIgnoreHTMLTags in Param then
      SrchText := HTMLStrip(SrchText);

    if not (fnMatchCase in Param) then
    begin
      SrchText := AnsiUpperCase(SrchText);
      Specifier := AnsiUpperCase(Specifier);
    end;

    if fnMatchFull in Param then
      Result := (Specifier = SrchText)
    else
      if fnMatchStart in Param then
        Result := Pos(Specifier,SrchText) = 1
      else
        if fnMatchRegular in Param then
          Result := MatchStr(Specifier,SrchText,fnMatchCase in Param);
  end;
end;


function TTIWDBItemSource.LocateFirstItem(Specifier: string;
  Params: TFindTextParams; var StartTime, EndTime: TDateTime;var Key:string): Boolean;
var
  D: TDataSet;
  B: TBookMark;
  Subject,Notes: string;

begin
  Result := False;

  if IsUpdating then Exit;
  if not CheckDataSet then Exit;

  FLocateIdx := 0;
  FLocateSpecifier := Specifier;
  FLocateParams := Params;

  BeginUpdate;

  D := DataSource.DataSet;

  D.DisableControls;

  B := D.GetBookMark;

  D.First;

  while not D.Eof do
  begin
    Inc(FLocateIdx);

    Subject := '';
    Notes := '';

    if SubjectField <> '' then
      Subject := D.FieldByName(SubjectField).AsString;
    if NotesField <> '' then
      Notes := D.FieldByName(NotesField).AsString;

    if MatchItem(FLocateSpecifier,Subject,Notes,FLocateParams) then
    begin
      FieldsToTime(D.Fields,StartTime,EndTime);
      Key := D.FieldByName(KeyField).AsString;
      Result := True;
      Break;
    end;
    D.Next;

  end;

  D.GotoBookMark(B);

  D.EnableControls;

  D.FreeBookMark(B);

  EndUpdate;
end;

function TTIWDBItemSource.LocateNextItem(var StartTime,
  EndTime: TDateTime;var Key:string): Boolean;
var
  D: TDataSet;
  B: TBookMark;
  Subject,Notes: string;

begin
  Result := False;

  if IsUpdating then Exit;
  if not CheckDataSet then Exit;

  BeginUpdate;

  D := DataSource.DataSet;

  D.DisableControls;

  B := D.GetBookMark;

  D.First;
  D.MoveBy(FLocateIdx);


  while not D.Eof do
  begin
    Inc(FLocateIdx);
    Subject := '';
    Notes := '';

    if SubjectField <> '' then
      Subject := D.FieldByName(SubjectField).AsString;
    if NotesField <> '' then
      Notes := D.FieldByName(NotesField).AsString;

    if MatchItem(FLocateSpecifier,Subject,Notes,FLocateParams) then
    begin
      FieldsToTime(D.Fields,StartTime,EndTime);	
      Key := D.FieldByName(KeyField).AsString;
      Result := True;
      Break;
    end;

    D.Next;


  end;

  D.GotoBookMark(B);

  D.EnableControls;

  D.FreeBookMark(B);

  EndUpdate;
end;

function TTIWDBItemSource.CheckDataSet: Boolean;
begin
  Result := False;

  if KeyField = '' then Exit;
  if not Assigned(FPlanner) then Exit;

  if Assigned(DataSource) then
  begin
    if Assigned(DataSource.DataSet) then
    begin
      if DataSource.DataSet.Active then
      begin        
        Result := (((StartTimeField <> '') AND (EndTimeField <> '')) OR
          (Assigned(FOnFieldsToTime) AND Assigned(FOnTimeToFields))) AND (KeyField <> '');
      end;
    end
  end;
end;

procedure TTIWDBItemSource.FieldsToTime(Fields:TFields; var dtS: TDateTime; var dtE: TDateTime);
begin
  if Assigned(FOnFieldsToTime) then 
  begin
    FOnFieldsToTime(Self,Fields,dtS,dtE);
  end
  else
  begin
    dtS := Fields.FieldByName(StartTimeField).AsDateTime;
    dtE := Fields.FieldByName(EndTimeField).AsDateTime;
  end;
end;


procedure TTIWDBItemSource.TimeToFields(Fields:TFields; dtS: TDateTime; dtE: TDateTime);
begin
  if Assigned(FOnTimeToFields) then 
  begin
    FOnTimeToFields(Self,Fields,dtS,dtE);
  end
  else
  begin
    Fields.FieldByName(StartTimeField).AsDateTime := dtS;
    Fields.FieldByName(EndTimeField).AsDateTime := dtE;
  end;
end;



procedure TTIWDBItemSource.SetResourceMap(const Value: TResourceMap);
begin
  FResourceMap.Assign(Value);
end;

procedure TTIWDBItemSource.FieldsToRes(Fields: TFields; var Position: Integer;
  var Accept: Boolean);
begin
  Accept := True;

  if ResourceMap.Count > 0 then
  begin
    Position := ResourceMap.ResToPosition(Fields.FieldByName(ResourceField).AsInteger);
    Accept := Position <> -1;
  end
  else
  begin
    if Assigned(FOnResourceToPosition) then
      FOnResourceToPosition(Self,Fields.FieldByName(ResourceField),Position, Accept)
    else
      Position := Fields.FieldByName(ResourceField).AsInteger;
  end;
end;

procedure TTIWDBItemSource.ResToFields(Fields: TFields; Position: Integer);
begin
  if ResourceMap.Count > 0 then
  begin
    Fields.FieldByName(ResourceField).AsInteger := ResourceMap.PositionToRes(Position);
  end
  else
  begin
    if Assigned(FOnPositionToResource) then
      FOnPositionToResource(Self,Fields.FieldByName(ResourceField),Position)
    else
      Fields.FieldByName(ResourceField).AsInteger := Position;
  end;
end;

{ TTIWDBDaySource }

procedure TTIWDBDaySource.ReadDBItems;
var
  D: TDataSet;
  B: TBookMark;
  dts,dte: TDateTime;
  plIt: TPlannerItem;
  Span, j: Integer;
  Accept: Boolean;
  ResourcePos,DatePos: Integer;
  Zone: TDateTime;
  dis, die: TDateTime;
  crossflg: Boolean;

begin
  if IsUpdating then Exit;
  if not CheckDataSet then Exit;

  {$IFDEF TMSCODESITE}
  SendMsg(FPlanner.Name+' READ DB ITEMS : '+FPlanner.Name);
  {$ENDIF}

  BeginUpdate;

  if Assigned(FOnSetFilter) then
    FOnSetFilter(Self);


  D := DataSource.DataSet;

  D.DisableControls;

  B := D.GetBookMark;

  D.First;

  ConfigurePlanner(Span);

  j := FPlanner.Display.DisplayUnit * FPlanner.Display.DisplayStart;
  j := j mod 1440;

  Zone := EncodeTime(j div 60,j mod 60,0,0);

  j := FPlanner.Display.DisplayUnit * FPlanner.Display.DisplayStart;
  j := j mod 1440;
  dis := EncodeTime(j div 60, j mod 60,0,0);

  j := FPlanner.Display.DisplayUnit * FPlanner.Display.DisplayEnd;
  crossflg := j > 1440;

  j := j mod 1440;
  die := EncodeTime(j div 60, j mod 60,0,0);


  FPlanner.Items.BeginUpdate;

  while not D.Eof do
  begin
    FieldsToTime(D.Fields,dts,dte);

    {$IFDEF TMSCODESITE}
    SendMsg(FPlanner.Name+' READ ITEM '+D.FieldByName(KeyField).AsString);
    SendMsg(FormatDateTime('dd/mm/yyyy hh:nn',FDay)+'->'+FormatDateTime('dd/mm/yyyy hh:nn',FDay+Span));
    {$ENDIF}


    Accept := True;

    // resource based item acception
    if (ResourceField <> '') and
       (Mode in [dmMultiResource,dmMultiDayRes,dmMultiResDay]) and
       (dte >= FDay + Zone) and (dts < FDay + Span + Zone) then
    begin
      FieldsToRes(D.Fields, ResourcePos, Accept);
    end;

    if (Int(dte) = Int(dts)) and not crossflg and ((Frac(dte) < Frac(dis)) or (Frac(dts) > Frac(die))) then
      Accept := False;

    // date based item acception
    if (dte >= FDay + Zone) and (dts < FDay + Span + Zone) and Accept and Assigned(FOnAccept) then
    begin
      FOnAccept(Self,D.Fields,Accept);
    end;

    if (dte >= FDay + Zone) and (dts < FDay + Span + Zone) and Accept then
    begin
      plIt := FPlanner.Items.FindKey(D.FieldByName(KeyField).AsString);

      if not Assigned(plIt) then
      begin
        {$IFDEF TMSCODESITE}
        SendMsg(FPlanner.Name+' ADD ITEM '+D.FieldByName(KeyField).AsString +
          ' ' +DateToStr(dts) + ' ' +DateToStr(dte));
        {$ENDIF}
        plIt := FPlanner.Items.Add;
      end;

      with plIt do
      begin
        FPlanner.RemoveClones(plIt);

        // set time here correct

        ItemRealStartTime := dts;
        ItemRealEndTime := dte;

        ItemStartTime := dts;
        ItemEndTime := dte;

        InHeader := dte - dts > 1;
        RealTime := true;

        // map to start of day
        if (Int(dts) < Int(FDay)) and not InHeader then
          ChangeCrossing;

        if dts > FDay then
          DatePos := Round(Int(dts - FDay))
        else
          DatePos := 0;

        ItemPos := CalcItemPos(DatePos,ResourcePos);

        if KeyField <> '' then
          DBKey := D.FieldByName(KeyField).AsString;
        if SubjectField <> '' then
          CaptionText := D.FieldByName(SubjectField).AsString;
        if NotesField <> '' then
          Text.Text := D.FieldByName(NotesField).AsString;

        if FReadOnly then
        begin
          FixedPos := True;
          FixedSize := True;
          ReadOnly := True;
        end;
      end;

      if Assigned(FOnFieldsToItem) then
        FOnFieldsToItem(Self, D.Fields, plIt);

      if plIt.InHeader then
      begin
        if dts < Day then
          dts := Day;

        for j := 1  to Round(Int(dte) - Int(dts)) do
        begin
          if (j + plIt.ItemPos < FPlanner.Positions) and (dts + j < Day + Span) then
            with FPlanner.CloneItem(plIt) do
            begin
              DatePos := DatePos + 1;
              ItemPos := CalcItemPos(DatePos,ResourcePos);
            end;
        end;

      end;
    end;

    D.Next;
  end;

  D.GotoBookMark(B);

  D.EnableControls;

  D.FreeBookMark(B);

  if Assigned(FOnResetFilter) then
    FOnResetFilter(Self);

  EndUpdate;

  FPlanner.Items.EndUpdate;

  if Assigned(OnItemsRead) then
    OnItemsRead(Self);

  {$IFDEF TMSCODESITE}
  SendMsg('DONE READ DB ITEMS');
  {$ENDIF}
end;

procedure TTIWDBDaySource.SetDay(const Value: TDateTime);
begin
  if FDay <> Value then
  begin
    FDay := Int(Value);
    ClearDBItems;
    ReadDBItems;
  end;
end;

procedure TTIWDBDaySource.Next;
begin
  FDay := FDay + FDayIncrement;
  inherited;
end;

procedure TTIWDBDaySource.Prev;
begin
  FDay := FDay - FDayIncrement;
  inherited;
end;

procedure TTIWDBDaySource.WriteDBItem;
var
  D: TDataSet;
  B: TBookMark;
  DTE, DTS: TDateTime;
  ResPos: Integer;

begin
  if IsUpdating then Exit;

  if not CheckDataSet then Exit;

  {$IFDEF TMSCODESITE}
  SendMsg('write selected here?');
  {$ENDIF}

  if Assigned(FOnUpdateItem) then
  begin
    BeginUpdate;
    FOnUpdateItem(Self,FPlanner.Items.DBItem);
    EndUpdate;
    if not FUpdateByQuery then
      Exit;
  end;

  BeginUpdate;

  D := DataSource.DataSet;

  B := D.GetBookMark;

  {$IFNDEF TMSDOTNET}
  D.Locate(KeyField,FPlanner.Items.DBItem.DBKey,[]);
  {$ENDIF}

  {$IFDEF TMSDOTNET}
  D.Locate(KeyField,Variant(FPlanner.Items.DBItem.DBKey),[]);
  {$ENDIF}

  D.Edit;

  if FPlanner.Items.DBItem.RealTime then
  begin
    DTS := FPlanner.Items.DBItem.ItemRealStartTime;
    DTE := FPlanner.Items.DBItem.ItemRealEndTime;
  end
  else
  begin
    DTS := FPlanner.Items.DBItem.ItemStartTime;
    DTE := FPlanner.Items.DBItem.ItemEndTime;
  end;

  {$IFDEF TMSCODESITE}
  SendMsg('writedb:'+formatdatetime('dd/mm/yyyy hh:nn',dts)+'-'+formatdatetime('dd/mm/yyyy hh:nn',dte));
  {$ENDIF}

  FPlanner.Items.DBItem.RealTime := False;

  //  if DTE < DTS then
  //    DTE := DTE + 1;

  TimeToFields(D.Fields,DTS,DTE);

  if (ResourceField <> '') and (Mode in [dmMultiResource,dmMultiDayRes,dmMultiResDay]) then
  begin
    ResPos := PosToRes(FPlanner.Items.DBItem.ItemPos);

    ResToFields(D.Fields, ResPos);
  end;

  if (SubjectField <> '') and
     (D.FieldByName(SubjectField).CanModify) then
    D.FieldByName(SubjectField).AsString := FPlanner.Items.DBItem.CaptionText;

  if (NotesField <> '') and
     (D.FieldByName(NotesField).CanModify) then
    D.FieldByName(NotesField).AsString := FPlanner.Items.DBItem.ItemText;

  if Assigned(FOnItemToFields) then
    FOnItemToFields(Self, D.Fields, FPlanner.Items.DBItem);

  try
    D.Post;
  finally
    D.GotoBookMark(B);
    D.FreeBookMark(B);
    EndUpdate;
  end;

end;


procedure TTIWDBDaySource.SetMode(const Value: TDBDayMode);
begin
  if FMode <> Value then
  begin
    FMode := Value;
    case FMode of
    dmMultiDay:
      if Assigned(FPlanner) then
      begin
        FPlanner.PositionGroup := 0;
        FPlanner.Positions := NumberOfDays;
      end;
    dmMultiResource:
      if Assigned(FPlanner) then
      begin
        FPlanner.PositionGroup := 0;
        FPlanner.Positions := NumberOfResources;
      end;
    end;
    ClearDBItems;
    ReadDBItems;
  end;
end;

procedure TTIWDBDaySource.AddDBItem;
var
  D: TDataSet;
  B: TBookMark;
  DTS, DTE: TDateTime;
  ResPos: Integer;

begin
  inherited;

  if not CheckDataSet then Exit;


  {$IFDEF TMSCODESITE}
  SendMsg('add selected here?');
  {$ENDIF}

  if Assigned(FOnInsertItem) then
  begin
    BeginUpdate;
    FOnInsertItem(Self,FPlanner.Items.DBItem);
    EndUpdate;
    Exit;
  end;

  BeginUpdate;

  D := DataSource.DataSet;

  B := D.GetBookMark;

  D.Append;

  if not AutoIncKey then
  begin
    D.FieldByName(KeyField).AsString := CreateKey;
    FPlanner.Items.DBItem.DBKey := D.FieldByName(KeyField).AsString;
  end;

  DTS := FPlanner.Items.DBItem.ItemStartTime;
  DTE := FPlanner.Items.DBItem.ItemEndTime;

  FPlanner.Items.DBItem.ItemRealStartTime := DTS;
  FPlanner.Items.DBItem.ItemRealEndTime := DTE;

  TimeToFields(D.Fields,DTS,DTE);

  if (ResourceField <> '') and (Mode in [dmMultiResource,dmMultiDayRes,dmMultiResDay]) then
  begin
    ResPos := PosToRes(FPlanner.Items.DBItem.ItemPos);

    ResToFields(D.Fields, ResPos);
  end;

  if SubjectField <> '' then
    D.FieldByName(SubjectField).AsString := FPlanner.Items.DBItem.CaptionText;

  if NotesField <> '' then
    D.FieldByName(NotesField).AsString := FPlanner.Items.DBItem.ItemText;

  if Assigned(FOnItemToFields) then
    FOnItemToFields(Self, D.Fields, FPlanner.Items.DBItem);

  try
    D.Post;
    if AutoIncKey then
      FPlanner.Items.DBItem.DBKey := D.FieldByName(KeyField).AsString;
  finally
    D.GotoBookMark(B);
    D.FreeBookMark(B);
    EndUpdate;
  end;
end;

procedure TTIWDBDaySource.DeleteDBItem(APlanner: TTIWWebPlanner);
var
  D: TDataSet;
begin
  inherited;
  if not CheckDataSet then Exit;

  {$IFDEF TMSCODESITE}
  SendMsg('delete selected here?');
  {$ENDIF}

  if Assigned(FOnDeleteItem) then
  begin
    BeginUpdate;
    FOnDeleteItem(Self,APlanner.Items.DBItem);
    EndUpdate;
    Exit;
  end;

  BeginUpdate;
  D := DataSource.DataSet;

  {$IFNDEF TMSDOTNET}
  D.Locate(KeyField,FPlanner.Items.DBItem.DBKey,[]);
  {$ENDIF}

  {$IFDEF TMSDOTNET}
  D.Locate(KeyField,Variant(FPlanner.Items.DBItem.DBKey),[]);
  {$ENDIF}

  D.Delete;
  EndUpdate;
end;


procedure TTIWDBDaySource.ItemChanged(DBKey: string);
var
  plIt: TPlannerItem;
  dt, dtE: TDateTime;
  Span: Integer;
  D: TDataSet;

begin
  inherited;
  if IsUpdating then Exit;
  if (DBKEY='') then Exit;
  if not Assigned(FPlanner) then Exit;

  BeginUpdate;

  {$IFDEF TMSCODESITE}
  SendMsg(FPlanner.Name+' Itemchanged: '+DBKEY);
  {$ENDIF}

  plIt := FPlanner.Items.FindKey(DBKey);

  if Assigned(plIt) then
  begin
    {$IFDEF TMSCODESITE}
    SendMsg('Found it : '+ plIt.Name);
    {$ENDIF}

    FPlanner.Items.DBItem := plIt;
    ReadDBItem;

    FPlanner.Items.Select(plIt);
  end
  else
  begin
    {$IFDEF TMSCODESITE}
    SendMsg('Add it to '+inttostr(FPlanner.Items.Count)+' items');
    {$ENDIF}

    if not CheckDataSet then Exit;

    D := DataSource.DataSet;
    FieldsToTime(D.Fields,dt,dtE);

    if Mode = dmMultiResource then
      Span := 1
    else
      Span := FPlanner.Positions;


    if (dt > FDay) and (dt < FDay + Span) then
    begin
      plIt := FPlanner.Items.Add;
      plIt.DBKey := DBKey;
      FPlanner.Items.DBItem := plIt;
      ReadDBItem;
    end;

  end;

  EndUpdate;
end;

procedure TTIWDBDaySource.ReadDBItem;
var
  D: TDataSet;
  dts,dte: TDateTime;
  ResourcePos,DatePos,j: Integer;
  Accept: Boolean;
  Zone: TDateTime;
  Span: Integer;

begin
  if not CheckDataSet then Exit;

  {$IFDEF TMSCODESITE}
  SendMsg('read selected');
  {$ENDIF}

  D := DataSource.DataSet;

  FPlanner.RemoveClones(FPlanner.Items.DBItem);

  j := FPlanner.Display.DisplayUnit * FPlanner.Display.DisplayStart;
  j := j mod 1440;

  Zone := EncodeTime(j div 60,j mod 60,0,0);

  case Mode of
  dmMultiDay,dmMultiDayRes,dmMultiResDay: Span := NumberOfDays;
  dmMultiResource: Span := 1;
  end;

  ConfigurePlanner(Span);

  with FPlanner.Items.DBItem do
  begin
    FieldsToTime(D.Fields,dts,dte);

    {$IFDEF TMSCODESITE}
    SendMsg('read:'+formatdatetime('dd/mm/yyyy hh:nn',dts)+'-'+formatdatetime('dd/mm/yyyy hh:nn',dte));
    {$ENDIF}
    
    ItemStartTime := dts;
    ItemEndTime := dte;
    ItemRealStartTime := dts;
    ItemRealEndTime := dte;

    InHeader := dte - dts > 1;

    // resource based item acception
    if (ResourceField <> '') and
       (Mode in [dmMultiResource,dmMultiDayRes,dmMultiResDay]) and
       (dte >= FDay + Zone) and (dts < FDay + Span + Zone) then
    begin
      FieldsToRes(D.Fields,ResourcePos, Accept);
    end;

    // date based item acception
    if (dte >= FDay + Zone) and (dts < FDay + Span + Zone) then
    begin
      FPlanner.RemoveClones(FPlanner.Items.DBItem);

      DatePos := Round(Int(dts - FDay));

      ItemPos := CalcItemPos(DatePos,ResourcePos);

      if SubjectField <> '' then
        CaptionText := D.FieldByName(SubjectField).AsString;

      if NotesField <> '' then
        Text.Text := D.FieldByName(NotesField).AsString;

      if FReadOnly then
      begin
        FixedPos := True;
        FixedSize := True;
        ReadOnly := True;
      end;
      
      if InHeader then
      begin
        if dts < Day then
          dts := Day;

        for j := 1  to Round(Int(dte) - Int(dts)) do
        begin
          if (j + ItemPos < FPlanner.Positions) and (dts + j < Day + Span)  then
            with FPlanner.CloneItem(FPlanner.Items.DBItem) do
            begin
              DatePos := DatePos + 1;
              ItemPos := CalcItemPos(DatePos,ResourcePos);
            end;
        end;
      end;


      if Assigned(FOnFieldsToItem) then
        FOnFieldsToItem(Self, D.Fields, FPlanner.Items.DBItem);
    end
    else
    begin
      FPlanner.Items.DBItem.Free;
    end;
  end;

end;

procedure TTIWDBDaySource.SynchDBItems;
var
  D: TDataSet;
  B: TBookMark;
  dt, dtE: TDateTime;
  plIt: TPlannerItem;
  Span,i: Integer;

begin
  if IsUpdating then Exit;
  if not CheckDataSet then Exit;
  if not Assigned(FPlanner) then Exit;

  BeginUpdate;

  if Assigned(FOnSetFilter) then
    FOnSetFilter(Self);

  D := DataSource.DataSet;
  B := D.GetBookMark;
  D.First;

  if Mode = dmMultiResource then
    Span := 1
  else
  begin
    Span := FPlanner.Positions;
  end;

  for i := 1 to FPlanner.Items.Count do
    FPlanner.Items[i - 1].Synched := False;

  while not D.Eof do
  begin
    FieldsToTime(D.Fields,dt,dtE);

    if (dt > FDay) and (dt < FDay + Span) then
    begin
      plIt := FPlanner.Items.FindKey(D.FieldByName(KeyField).AsString);
      if Assigned(plIt) then
        plIt.Synched := True;
    end;
    
    D.Next;
  end;

  for i := FPlanner.Items.Count downto 1 do
  begin
    if not FPlanner.Items[i - 1].Synched then
      FPlanner.Items[i - 1].Free;
  end;

  D.GotoBookMark(B);
  D.FreeBookMark(B);

  if Assigned(FOnResetFilter) then
    FOnResetFilter(Self);

  EndUpdate;
end;

procedure TTIWDBDaySource.GotoDBItem;
var
  D: TDataSet;
begin
  inherited;
  if IsUpdating then Exit;
  if not CheckDataSet then Exit;
  if not Assigned(FPlanner) then Exit;

  BeginUpdate;
  D := DataSource.DataSet;

  {$IFNDEF TMSDOTNET}
  D.Locate(KeyField,FPlanner.Items.DBItem.DBKey,[]);
  {$ENDIF}

  {$IFDEF TMSDOTNET}
  D.Locate(KeyField,Variant(FPlanner.Items.DBItem.DBKey),[]);
  {$ENDIF}

  EndUpdate;
end;

constructor TTIWDBDaySource.Create(AOwner: TComponent);
begin
  inherited;
  FDayIncrement := 1;
  // Initialize on US date format
  FDateFormat := 'mm/dd/yyyy';
  FNumberOfDays := 7;
  FNumberOfResources := 1;
end;

procedure TTIWDBDaySource.PlannerChanged;
begin
  inherited;
  if Assigned(FPlanner) then
    FPlanner.Mode.PlannerType := plDay;
end;

function TTIWDBDaySource.PosToDay(Pos: Integer): TDateTime;
begin
  case Mode of
  dmMultiDay:
    Result := FDay + Pos;
  dmMultiResource:
    Result := Day;
  dmMultiDayRes:
    Result := FDay + (Pos mod NumberOfDays);
  dmMultiResDay:
    Result := FDay + (Pos div NumberOfResources);
  else
    Result := FDay;
  end;
end;

function TTIWDBDaySource.PosToRes(Pos: Integer): Integer;
begin
  case Mode of
  dmMultiDay:
    Result := 0;
  dmMultiResource:
    Result := Pos;
  dmMultiDayRes:
    Result := Pos div NumberOfDays;
  dmMultiResDay:
    Result := Pos mod NumberOfResources;
  else
    Result := 0;
  end;
end;


procedure TTIWDBDaySource.SetNumberOfDays(const Value: Integer);
begin
  if Value <= 0 then
    Exit;

  if FNumberOfDays <> Value then
  begin
    FNumberOfDays := Value;
    ClearDBItems;
    ReadDBItems;
  end;
end;

procedure TTIWDBDaySource.SetNumberOfResources(const Value: Integer);
begin
  if Value <= 0 then
    Exit;
  if FNumberOfResources <> Value then
  begin
    FNumberOfResources := Value;
    ClearDBItems;
    ReadDBItems;
  end;
end;

procedure TTIWDBDaySource.ConfigurePlanner(var Span: Integer);
var
  i,j,k,dx: Integer;
  ResName: string;
begin
  if not Assigned(FPlanner) then
    Exit;

  case Mode of
  dmMultiResource:
    begin
      FPlanner.Positions := NumberOfResources;
      Span := 1;
      if FAutoHeaderUpdate then
      begin
        FPlanner.Header.Captions.Clear;
        FPlanner.Header.Captions.Add('');

        for i := 1 to NumberOfResources do
        begin
          ResName := ResourceMap.PositionName(i - 1);
          if Assigned(FOnGetResourceName) then
            FOnGetResourceName(Self,i,ResName);
          if ResName <> '' then
          begin
            if FPlanner.Header.Captions.Count <= i then
              FPlanner.Header.Captions.Add(ResName)
            else
              FPlanner.Header.Captions.Strings[i] := ResName;
          end;
        end;
      end;
      FPlanner.Display.CurrentPosFrom := -1;
      FPlanner.Display.CurrentPosTo := -1;
    end;
  dmMultiDay:
    begin
      FPlanner.Positions := NumberOfDays;
      Span := NumberOfDays;

      if FAutoHeaderUpdate then
      begin
        FPlanner.Header.Captions.Clear;
        FPlanner.Header.Captions.Add('');

        for i := 1 to NumberOfDays do
        begin
          if FPlanner.Header.Captions.Count <= i then
            FPlanner.Header.Captions.Add(FormatDateTime(FDateFormat,Day + i - 1))
          else
            FPlanner.Header.Captions.Strings[i] := FormatDateTime(FDateFormat,Day + i - 1);
        end;
      end;
      dx := Round(Int(Now) - Int(Day));
      FPlanner.Display.CurrentPosFrom := dx;
      FPlanner.Display.CurrentPosTo := dx;
    end;
  dmMultiDayRes:
    begin
      FPlanner.Positions := NumberOfResources * NumberOfDays;
      FPlanner.PositionGroup := NumberOfDays;
      Span := NumberOfDays;

      if FAutoHeaderUpdate then
      begin
        FPlanner.Header.Captions.Clear;
        FPlanner.Header.Captions.Add('');
        FPlanner.Header.GroupCaptions.Clear;
        FPlanner.Header.GroupCaptions.Add('');

        for i := 1 to NumberOfResources do
        begin
          for j := 1 to NumberOfDays do
          begin
             k := j + (i - 1) * NumberOfDays;

             if FPlanner.Header.Captions.Count <= k then
               FPlanner.Header.Captions.Add(FormatDateTime(FDateFormat,Day + j - 1))
             else
               FPlanner.Header.Captions.Strings[k] := FormatDateTime(FDateFormat,Day + j - 1);
          end;

          ResName := ResourceMap.PositionName(i - 1);
          if Assigned(FOnGetResourceName) then
            FOnGetResourceName(Self,i,ResName);

          if FPlanner.Header.GroupCaptions.Count < i then
            FPlanner.Header.GroupCaptions.Add(ResName)
          else
            FPlanner.Header.GroupCaptions[i - 1] := ResName;
        end;
      end;
      FPlanner.Display.CurrentPosFrom := -1;
      FPlanner.Display.CurrentPosTo := -1;
    end;
  dmMultiResDay:
    begin
      FPlanner.PositionGroup := NumberOfResources;
      FPlanner.Positions := NumberOfResources * NumberOfDays;
      Span := NumberOfDays;

      if FAutoHeaderUpdate then
      begin
        FPlanner.Header.Captions.Clear;
        FPlanner.Header.Captions.Add('');
        FPlanner.Header.GroupCaptions.Clear;
        FPlanner.Header.GroupCaptions.Add('');

        for i := 1 to NumberOfDays do
        begin
          if FPlanner.Header.GroupCaptions.Count < i then
            FPlanner.Header.GroupCaptions.Add(FormatDateTime(FDateFormat,Day + i - 1))
          else
            FPlanner.Header.GroupCaptions[i - 1] := FormatDateTime(FDateFormat,Day + i - 1);
        end;

        for i := 1 to NumberOfDays do
          for j := 1 to NumberOfResources do
          begin
            ResName := ResourceMap.PositionName(j - 1);
            if Assigned(FOnGetResourceName) then
              FOnGetResourceName(Self,j,ResName);
            k := j + (i - 1) * NumberOfResources;
            if FPlanner.Header.Captions.Count <= k then
              FPlanner.Header.Captions.Add(ResName)
            else
              FPlanner.Header.Captions.Strings[k] := ResName;
          end;
      end;
      dx := Round(Int(Now) - Int(Day));
      FPlanner.Display.CurrentPosFrom := dx * NumberOfResources;
      FPlanner.Display.CurrentPosTo := (dx + 1) * NumberOfResources - 1;
    end;
  end;

end;

function TTIWDBDaySource.CalcItemPos(DatePos, ResourcePos: Integer): Integer;
begin
  case Mode of
  dmMultiDay:
    Result := DatePos;
  dmMultiResource:
    Result := ResourcePos;
  dmMultiResDay:
    Result := ResourcePos + NumberOfResources * DatePos;
  dmMultiDayRes:
    Result := ResourcePos * NumberOfDays + DatePos;
  else
    Result := DatePos;
  end;
end;


procedure TTIWDBDaySource.MapItemTimeOnPlanner(APlannerItem: TPlannerItem);
var
  dts,dte: TDateTime;
  j, DatePos: Integer;
begin
  with APlannerItem do
  begin
    RealTime := True;
    dts := ItemRealStartTime;
    dte := ItemRealEndTime;

    // map to start of day
    if Int(dts) < Int(FDay) then
      ChangeCrossing;

    InHeader := dte - dts > 1;

    DatePos := Round(Int(dts - FDay));

    ItemPos := CalcItemPos(DatePos,ItemPos);

    if APlannerItem.InHeader then
    begin
      for j := 1  to round(Int(dte) - Int(dts)) do
        begin
          if j + APlannerItem.ItemPos < FPlanner.Positions then
            with FPlanner.CloneItem(APlannerItem) do
              ItemPos := ItemPos + j;
        end;
    end;
  end;
end;

{ TTIWDBCustomPeriodSource }

procedure TTIWDBCustomPeriodSource.AddDBItem;
var
  D: TDataSet;
  B: TBookMark;

begin
  if not CheckDataSet then Exit;

  {$IFDEF TMSCODESITE}
  SendMsg('add selected here?');
  {$ENDIF}

  if Assigned(FOnInsertItem) then
  begin
    BeginUpdate;
    FOnInsertItem(Self,FPlanner.Items.DBItem);
    EndUpdate;
    Exit;
  end;

  BeginUpdate;

  D := DataSource.DataSet;

  B := D.GetBookMark;

  D.Append;

  if not AutoIncKey then
  begin
    D.FieldByName(KeyField).AsString := CreateKey;
    FPlanner.Items.DBItem.DBKey := D.FieldByName(KeyField).AsString;
  end;

  TimeToFields(D.Fields,FPlanner.Items.DBItem.ItemRealStartTime,
    FPlanner.Items.DBItem.ItemRealEndTime);


  if (ResourceField <> '') then
  begin
    ResToFields(D.Fields, FPlanner.Items.DBItem.ItemPos);
  end;

  if SubjectField <> '' then
    D.FieldByName(SubjectField).AsString := FPlanner.Items.DBItem.CaptionText;

  if NotesField <> '' then
    D.FieldByName(NotesField).AsString := FPlanner.Items.DBItem.ItemText;

  if Assigned(FOnItemToFields) then
    FOnItemToFields(Self, D.Fields, FPlanner.Items.DBItem);

  try
    D.Post;
    if AutoIncKey then
      FPlanner.Items.DBItem.DBKey := D.FieldByName(KeyField).AsString;
  finally
    D.GotoBookMark(B);
    D.FreeBookMark(B);
    EndUpdate;
  end;

end;


procedure TTIWDBCustomPeriodSource.ConfigurePlanner;
var
  i: Integer;
  ResName: string;
begin
  FPlanner.Positions := NumberOfResources;

  if FAutoHeaderUpdate then
    FPlanner.Header.Captions.Clear; //added by Pro Med
  FPlanner.Header.Captions.Add(''); //added by Pro Med

  for i := 1 to NumberOfResources do
  begin
    ResName := ResourceMap.PositionName(i - 1);
    if Assigned(FOnGetResourceName) then
      FOnGetResourceName(Self,i,ResName);
    if ResName <> '' then
    begin
      if FPlanner.Header.Captions.Count <= i then    //added by Pro Med
        FPlanner.Header.Captions.Add(ResName) //added by Pro Med
      Else  //added by Pro Med
        FPlanner.Header.Captions.Strings[i] := ResName; //added by Pro Med
    end; //added by Pro Med
  end;
end;


constructor TTIWDBCustomPeriodSource.Create(AOwner: TComponent);
begin
  inherited;
  FNumberOfResources := 1;
end;

procedure TTIWDBCustomPeriodSource.DeleteDBItem(APlanner: TTIWWebPlanner);
var
  D: TDataSet;
begin
  inherited;
  {$IFDEF TMSCODESITE}
  SendMsg('delete selected here?');
  {$ENDIF}

  BeginUpdate;
  D := DataSource.DataSet;

  {$IFNDEF TMSDOTNET}
  D.Locate(KeyField,FPlanner.Items.DBItem.DBKey,[]);
  {$ENDIF}

  {$IFDEF TMSDOTNET}
  D.Locate(KeyField,Variant(FPlanner.Items.DBItem.DBKey),[]);
  {$ENDIF}

  D.Delete;
  EndUpdate;
end;

procedure TTIWDBCustomPeriodSource.GotoDBItem;
var
  D: TDataSet;
begin
  inherited;
  if IsUpdating then Exit;
  if not CheckDataSet then Exit;
  if not Assigned(FPlanner) then Exit;

  BeginUpdate;
  D := DataSource.DataSet;

  {$IFNDEF TMSDOTNET}
  D.Locate(KeyField,FPlanner.Items.DBItem.DBKey,[]);
  {$ENDIF}

  {$IFDEF TMSDOTNET}
  D.Locate(KeyField,Variant(FPlanner.Items.DBItem.DBKey),[]);
  {$ENDIF}

  EndUpdate;
end;

procedure TTIWDBCustomPeriodSource.ItemChanged(DBKey: string);
var
  plIt: TPlannerItem;
  dts,dte: TDateTime;
  D: TDataSet;

begin
  inherited;
  if IsUpdating then Exit;
  if (DBKEY='') then Exit;
  if not Assigned(FPlanner) then Exit;

  BeginUpdate;

  {$IFDEF TMSCODESITE}
  SendMsg(FPlanner.Name+' Itemchanged: '+DBKEY);
  {$ENDIF}

  plIt := FPlanner.Items.FindKey(DBKey);

  if Assigned(plIt) then
  begin
    {$IFDEF TMSCODESITE}
    SendMsg('Found it : '+ plIt.Name);
    {$ENDIF}
    FPlanner.Items.DBItem := plIt;
    ReadDBItem;

    FPlanner.Items.Select(plIt);
  end
  else
  begin
    {$IFDEF TMSCODESITE}
    SendMsg('Add it to '+inttostr(FPlanner.Items.Count)+' items');
    {$ENDIF}

    if not CheckDataSet then Exit;

    D := DataSource.DataSet;

    FieldsToTime(D.Fields,dts,dte);

    if (dts < FPlanner.Mode.PeriodEndDate) and
       (dte > FPlanner.Mode.PeriodStartDate) then
    begin
      plIt := FPlanner.Items.Add;
      plIt.DBKey := DBKey;
      FPlanner.Items.DBItem := plIt;
      ReadDBItem;
    end;
  end;

  EndUpdate;
end;

procedure TTIWDBCustomPeriodSource.Next;
begin
  inherited;

end;

procedure TTIWDBCustomPeriodSource.Prev;
begin
  inherited;

end;

procedure TTIWDBCustomPeriodSource.ReadDBItem;
var
  D: TDataSet;
  dtS, dtE: TDateTime;
  ResourcePos: Integer;
  Accept: Boolean;
  B: TBookmark;

begin
  if not CheckDataSet then Exit;

  {$IFDEF TMSCODESITE}
  SendMsg('read selected');
  {$ENDIF}

  D := DataSource.DataSet;

  B := D.GetBookMark;

  {$IFNDEF TMSDOTNET}
  D.Locate(KeyField,FPlanner.Items.DBItem.DBKey,[]);
  {$ENDIF}

  {$IFDEF TMSDOTNET}
  D.Locate(KeyField,Variant(FPlanner.Items.DBItem.DBKey),[]);
  {$ENDIF}


  with FPlanner.Items.DBItem do
  begin
    FieldsToTime(D.Fields,dtS,dtE);
    ItemStartTime := dtS;
    ItemEndTime := dtE;

    if (ResourceField <> '') then
    begin
      FieldsToRes(D.Fields, ResourcePos, Accept);
      ItemPos := ResourcePos;
    end;

    if SubjectField <> '' then
      CaptionText := D.FieldByName(SubjectField).AsString;

    if NotesField <> '' then
      Text.Text := D.FieldByName(NotesField).AsString;

    if FReadOnly then
    begin
      FixedPos := True;
      FixedSize := True;
      ReadOnly := True;
    end;
  end;

  if Assigned(FOnFieldsToItem) then
    FOnFieldsToItem(Self, D.Fields, FPlanner.Items.DBItem);

  D.GotoBookMark(B);
  D.FreeBookMark(B);
end;

procedure TTIWDBCustomPeriodSource.ReadDBItems;
var
  D: TDataSet;
  B: TBookMark;
  dts,dte: TDateTime;
  plIt: TPlannerItem;
  Accept: Boolean;
  ResourcePos: Integer;

begin
  if IsUpdating then Exit;
  if not CheckDataSet then Exit;

  {$IFDEF TMSCODESITE}
  SendMsg('READ DB ITEMS : '+FPlanner.Name);
  {$ENDIF}

  BeginUpdate;

  if Assigned(FOnSetFilter) then
    FOnSetFilter(Self);

  D := DataSource.DataSet;

  D.DisableControls;

  B := D.GetBookMark;

  D.First;

  ConfigurePlanner;

  FPlanner.Items.BeginUpdate;

  while not D.Eof do
  begin
    Accept := True;

    FieldsToTime(D.Fields,dts,dte);

    if (ResourceField <> '') and
       (dts < FPlanner.Mode.PeriodEndDate + 1) and
       (dte >= FPlanner.Mode.PeriodStartDate) then
    begin
      FieldsToRes(D.Fields, ResourcePos, Accept);
    end;

    if (dts < FPlanner.Mode.PeriodEndDate + 1) and
       (dte >= FPlanner.Mode.PeriodStartDate) and
       Accept and Assigned(FOnAccept) then
    begin
      FOnAccept(Self,D.Fields,Accept);
    end;

    if (dts < FPlanner.Mode.PeriodEndDate + 1) and
       (dte >= FPlanner.Mode.PeriodStartDate) and Accept then
    begin
      plIt := FPlanner.Items.FindKey(D.FieldByName(KeyField).AsString);

      if not Assigned(plIt) then
      begin
        {$IFDEF TMSCODESITE}
        SendMsg(FPlanner.Name+' Add:'+D.FieldByName(KeyField).AsString);
        {$ENDIF}
        plIt := FPlanner.Items.Add;
      end;

      with plIt do
      begin
        Assign(FPlanner.DefaultItem);

        ItemStartTime := dts;
        ItemEndTime := dte;

        if (ResourceField <> '') then
        begin
          FieldsToRes(D.Fields, ResourcePos, Accept);
          ItemPos := ResourcePos;
        end;

        if KeyField <> '' then
          DBKey := D.FieldByName(KeyField).AsString;
        if SubjectField <> '' then
          CaptionText := D.FieldByName(SubjectField).AsString;
        if NotesField <> '' then
          Text.Text := D.FieldByName(NotesField).AsString;

        if FReadOnly then
        begin
          FixedPos := True;
          FixedSize := True;
          ReadOnly := True;
        end;

      end;

      if Assigned(FOnFieldsToItem) then
        FOnFieldsToItem(Self, D.Fields, plIt);

    end;
    D.Next;
  end;

  D.GotoBookMark(B);
  D.EnableControls;
  D.FreeBookMark(B);

  if Assigned(FOnResetFilter) then
    FOnResetFilter(Self);

  EndUpdate;

  FPlanner.Items.EndUpdate;

  if Assigned(OnItemsRead) then
    OnItemsRead(Self);  
  
  {$IFDEF TMSCODESITE}
  SendMsg('DONE READ DB ITEMS');
  {$ENDIF}
end;


procedure TTIWDBCustomPeriodSource.SetNumberOfResources(const Value: Integer);
begin
  if Value <= 0 then
    Exit;
  if FNumberOfResources <> Value then
  begin
    FNumberOfResources := Value;
    ClearDBItems;
    ReadDBItems;
  end;
end;

procedure TTIWDBCustomPeriodSource.SynchDBItems;
var
  D: TDataSet;
  B: TBookMark;
  dts,dte: TDateTime;
  plIt: TPlannerItem;
  i: Integer;

begin
  if IsUpdating then Exit;
  if not CheckDataSet then Exit;
  if not Assigned(FPlanner) then Exit;

  BeginUpdate;

  if Assigned(FOnSetFilter) then
    FOnSetFilter(Self);

  D := DataSource.DataSet;
  B := D.GetBookMark;
  D.First;


  for i := 1 to FPlanner.Items.Count do
    FPlanner.Items[i - 1].Synched := False;

  while not D.Eof do
  begin
    FieldsToTime(D.Fields,dts,dte);

    if (dts < FPlanner.Mode.PeriodEndDate) and
       (dte > FPlanner.Mode.PeriodStartDate) then
    begin
      plIt := FPlanner.Items.FindKey(D.FieldByName(KeyField).AsString);
      if Assigned(plIt) then
        plIt.Synched := True;
    end;
    D.Next;
  end;

  for i := FPlanner.Items.Count downto 1 do
  begin
    if not FPlanner.Items[i - 1].Synched then
      FPlanner.Items[i - 1].Free;
  end;

  D.GotoBookMark(B);
  D.FreeBookMark(B);

  if Assigned(FOnResetFilter) then
    FOnResetFilter(Self);

  EndUpdate;
end;

procedure TTIWDBCustomPeriodSource.WriteDBItem;
var
  D: TDataSet;
  B: TBookMark;
begin
  if IsUpdating then Exit;

  if not CheckDataSet then Exit;

  {$IFDEF TMSCODESITE}
  SendMsg('write selected here?');
  {$ENDIF}

  if Assigned(FOnUpdateItem) then
  begin
    BeginUpdate;
    FOnUpdateItem(Self,FPlanner.Items.DBItem);
    EndUpdate;
    if not FUpdateByQuery then
      Exit;
  end;

  BeginUpdate;

  D := DataSource.DataSet;

  B := D.GetBookMark;

  {$IFNDEF TMSDOTNET}
  D.Locate(KeyField,FPlanner.Items.DBItem.DBKey,[]);
  {$ENDIF}

  {$IFDEF TMSDOTNET}
  D.Locate(KeyField,Variant(FPlanner.Items.DBItem.DBKey),[]);
  {$ENDIF}

  D.Edit;

  if FPlanner.Items.DBItem.RealTime then
  begin
    TimeToFields(D.Fields,FPlanner.Items.DBItem.ItemRealStartTime,
      FPlanner.Items.DBItem.ItemRealEndTime);
  end
  else
  begin
    TimeToFields(D.Fields,FPlanner.Items.DBItem.ItemStartTime,
      FPlanner.Items.DBItem.ItemEndTime);
  end;

  if ResourceField <> '' then
  begin
    ResToFields(D.Fields, FPlanner.Items.DBItem.ItemPos);
  end;

  if (SubjectField <> '') and
     (D.FieldByName(SubjectField).CanModify) then
    D.FieldByName(SubjectField).AsString := FPlanner.Items.DBItem.CaptionText;

  if (NotesField <> '') and
     (D.FieldByName(NotesField).CanModify) then
    D.FieldByName(NotesField).AsString := FPlanner.Items.DBItem.ItemText;


  if Assigned(FOnItemToFields) then
    FOnItemToFields(Self, D.Fields, FPlanner.Items.DBItem);

  try
    D.Post;
  finally
    D.GotoBookMark(B);
    D.FreeBookMark(B);
    EndUpdate;
  end;
end;

{ TTIWDBPeriodSource }

constructor TTIWDBPeriodSource.Create(AOwner: TComponent);
begin
  inherited;
  // initialize it to a default 14 day period
  FStartDate := Trunc(Now);
  FEndDate := Trunc(Now) + 14;
end;

procedure TTIWDBPeriodSource.Loaded;
begin
  inherited;
  if Assigned(FPlanner) then
    UpdatePlanner;
end;

procedure TTIWDBPeriodSource.PlannerChanged;
begin
  inherited;
  if Assigned(FPlanner) then
    UpdatePlanner;
end;

procedure TTIWDBPeriodSource.SetEndDate(const Value: TDateTime);
begin
  if (FEndDate <> Value) and (Value > FStartDate) then
  begin
    FEndDate := Value;
    if not Assigned(FPlanner) then
      Exit;
    if csLoading in ComponentState then
      Exit;

    FPlanner.Mode.PeriodEndDate := Value;
  end;
end;

procedure TTIWDBPeriodSource.SetStartDate(const Value: TDateTime);
begin
  if (FStartDate <> Value) and (Value < FEndDate) then
  begin
    FStartDate := Value;
    if not Assigned(FPlanner) then
      Exit;
    if csLoading in ComponentState then
      Exit;

    FPlanner.Mode.PeriodStartDate := Value;
  end;
end;

procedure TTIWDBPeriodSource.UpdatePlanner;
begin
  if Assigned(FPlanner) and (FUpdateCount = 0) then
  begin
    FPlanner.Mode.PlannerType := plDayPeriod;
    FPlanner.Mode.BeginUpdate;
    FPlanner.Mode.PeriodEndDate := FEndDate;
    FPlanner.Mode.PeriodStartDate := FStartDate;
    FPlanner.Mode.EndUpdate;
    FPlanner.Items.BeginUpdate;
    ClearDBItems;
    ReadDBItems;
    FPlanner.Items.EndUpdate;
  end;
end;

{ TTIWDBMonthSource }

function DaysInMonth(mo,ye:word):word;
begin
  if (mo < 1) or (mo > 12) then
    mo := 1;

  if (mo <> 2) then
    Result := CDaysInMonth[mo]
  else
  begin
    if ye mod 4 = 0 then
      Result := 29
    else
      Result := 28;

    if ye mod 100 = 0 then

      Result := 28;

    if ye mod 400 = 0 then
      Result := 29;
  end;
end;

constructor TTIWDBMonthSource.Create(AOwner: TComponent);
var
  da,mo,ye: word;
begin
  inherited;
  DecodeDate(Now,ye,mo,da);
  FMonth := mo;
  FYear := ye;
end;

procedure TTIWDBMonthSource.Loaded;
begin
  inherited;
  if Assigned(FPlanner) then
  begin
    FPlanner.Mode.PlannerType := plMonth;
    FPlanner.Mode.Year := FYear;
    FPlanner.Mode.Month := FMonth;
    FPlanner.Display.DisplayStart := 0;
    FPlanner.Display.DisplayEnd := DaysInMonth(FMonth,FYear) - 1;
  end;
end;

procedure TTIWDBMonthSource.PlannerChanged;
begin
  inherited;
  if Assigned(FPlanner) then
  begin
    FPlanner.Mode.PlannerType := plMonth;
    FPlanner.Mode.Year := FYear;
    FPlanner.Mode.Month := FMonth;
  end;
end;


procedure TTIWDBMonthSource.SetMonth(const Value: integer);
begin
  if (Value > 0) and (Value < 13) and (Value <> FMonth) then
  begin
    FMonth := Value;

    if not Assigned(FPlanner) then Exit;
    if csLoading in ComponentState then Exit;
    FPlanner.Mode.Month := Value;

    { reload items to reflect the new month }
    if CheckDataSet then
      UpdatePlanner;
  end;
end;


procedure TTIWDBMonthSource.SetYear(const Value: integer);
begin
  if (Value > 0) and (Value <> FYear) then
  begin
    FYear := Value;
    if not Assigned(FPlanner) then Exit;
    if csLoading in ComponentState then Exit;

    FPlanner.Mode.Year := Value;

    { reload items to reflect the new month }
    if CheckDataSet then
      UpdatePlanner;
  end;
end;

procedure TTIWDBMonthSource.UpdatePlanner;
begin
  if Assigned(FPlanner) then
  begin
    FPlanner.Display.DisplayStart := 0;
    FPlanner.Display.DisplayEnd := DaysInMonth(FMonth,FYear) - 1;

    FPlanner.Items.BeginUpdate;
    ClearDBItems;
    ReadDBItems;
    FPlanner.Items.EndUpdate;
  end;
end;

{ TTIWDBWeekSource }

constructor TTIWDBWeekSource.Create(AOwner: TComponent);
var
  da,mo,ye: word;
begin
  inherited;
  DecodeDate(Now,ye,mo,da);
  FMonth := mo;
  FYear := ye;
  FWeekStart := 0;
  FWeeks := 4;
end;


procedure TTIWDBWeekSource.Loaded;
begin
  inherited;
  if Assigned(FPlanner) then
  begin
    FPlanner.Mode.PlannerType := plWeek;
    FPlanner.Mode.Year := FYear;
    FPlanner.Mode.Month := FMonth;
    FPlanner.Mode.WeekStart := FWeekStart;
    FPlanner.Display.DisplayStart := 0;
    FPlanner.Display.DisplayEnd := FWeeks * 7;
  end;
end;

procedure TTIWDBWeekSource.PlannerChanged;
begin
  inherited;
  if Assigned(FPlanner) then
  begin
    FPlanner.Mode.PlannerType := plWeek;
    FPlanner.Mode.Year := FYear;
    FPlanner.Mode.Month := FMonth;
    FPlanner.Mode.WeekStart := FWeekStart;
  end;
end;

procedure TTIWDBWeekSource.SetMonth(const Value: integer);
begin
  if (Value > 0) and (Value < 13) and (Value <> FMonth) then
  begin
    FMonth := Value;

    if not Assigned(FPlanner) then Exit;
    if csLoading in ComponentState then Exit;
    FPlanner.Mode.Month := Value;

    { reload items to reflect the new month }
    if CheckDataSet then
      UpdatePlanner;
  end;
end;

procedure TTIWDBWeekSource.SetWeeks(const Value: Integer);
begin
  if FWeeks <> Value then
  begin
    FWeeks := Value;
    if CheckDataSet then
      UpdatePlanner;
  end;
end;

procedure TTIWDBWeekSource.SetWeekStart(const Value: Integer);
begin
  FWeekStart := Value;
  if Assigned(FPlanner) then
  begin
    FPlanner.Mode.WeekStart := FWeekStart;
  end;
end;

procedure TTIWDBWeekSource.SetYear(const Value: integer);
begin
  if (Value > 0) and (Value <> FYear) then
  begin
    FYear := Value;
    if not Assigned(FPlanner) then Exit;
    if csLoading in ComponentState then Exit;

    FPlanner.Mode.Year := Value;

    { reload items to reflect the new month }
    if CheckDataSet then
      UpdatePlanner;
  end;
end;

procedure TTIWDBWeekSource.UpdatePlanner;
begin
  FPlanner.Display.DisplayStart := 0;
  FPlanner.Display.DisplayEnd := FWeeks * 7;

  FPlanner.Items.BeginUpdate;
  ClearDBItems;
  ReadDBItems;
  FPlanner.Items.EndUpdate;
end;

{ TTIWDBMultiMonthSource }

procedure TTIWDBMultiMonthSource.AddDBItem;
var
  D: TDataSet;
  B: TBookMark;
begin
  inherited;
  if not CheckDataSet then Exit;

  {$IFDEF TMSCODESITE}
  SendMsg('add selected here?');
  {$ENDIF}

  if Assigned(FOnInsertItem) then
  begin
    BeginUpdate;
    FOnInsertItem(Self,FPlanner.Items.DBItem);
    EndUpdate;
    Exit;
  end;

  BeginUpdate;

  D := DataSource.DataSet;

  B := D.GetBookMark;

  D.Append;

  if not AutoIncKey then
  begin
    D.FieldByName(KeyField).AsString := CreateKey;
    FPlanner.Items.DBItem.DBKey := D.FieldByName(KeyField).AsString;
  end;  

  TimeToFields(D.Fields,FPlanner.Items.DBItem.ItemRealStartTime,
    FPlanner.Items.DBItem.ItemRealEndTime - 1);

  if SubjectField <> '' then
    D.FieldByName(SubjectField).AsString := FPlanner.Items.DBItem.CaptionText;

  if NotesField <> '' then
    D.FieldByName(NotesField).AsString := FPlanner.Items.DBItem.ItemText;

  if Assigned(FOnItemToFields) then
    FOnItemToFields(Self, D.Fields, FPlanner.Items.DBItem);

  try
    D.Post;
    if AutoIncKey then
      FPlanner.Items.DBItem.DBKey := D.FieldByName(KeyField).AsString;
  finally
    D.GotoBookMark(B);
    D.FreeBookMark(B);
    EndUpdate;
  end;

end;

procedure TTIWDBMultiMonthSource.ConfigurePlanner;
var
  i,j: Integer;
begin
  FPlanner.Positions := NumberOfMonths;
  FPlanner.Display.DisplayStart := 0;
  FPlanner.Display.DisplayEnd := 30;

  if FAutoHeaderUpdate then
  begin
    j := FStartMonth;
    for i := 1 to NumberOfMonths do
    begin
     {$IFDEF DELPHIXE_LVL}
      FPlanner.Header.Captions.Strings[i] := FormatSettings.LongMonthNames[j];
      {$ELSE}
      FPlanner.Header.Captions.Strings[i] := LongMonthNames[j];
      {$ENDIF}
      inc(j);
      if j > 12 then
        j := 1;
    end;
  end;  
end;

constructor TTIWDBMultiMonthSource.Create(AOwner: TComponent);
var
  da,mo,ye: word;
begin
  inherited;
  FStartMonth := 1;
  FNumberOfMonths := 3;
  DecodeDate(Now, ye, mo, da);
  FYear := ye;
end;

procedure TTIWDBMultiMonthSource.DeleteDBItem(APlanner: TTIWWebPlanner);
var
  D: TDataSet;
begin
  inherited;
  {$IFDEF TMSCODESITE}
  SendMsg('delete selected here?');
  {$ENDIF}

  BeginUpdate;
  D := DataSource.DataSet;

  {$IFNDEF TMSDOTNET}
  D.Locate(KeyField,FPlanner.Items.DBItem.DBKey,[]);
  {$ENDIF}

  {$IFDEF TMSDOTNET}
  D.Locate(KeyField,Variant(FPlanner.Items.DBItem.DBKey),[]);
  {$ENDIF}

  D.Delete;
  EndUpdate;
end;

procedure TTIWDBMultiMonthSource.GotoDBItem;
var
  D: TDataSet;
begin
  inherited;
  if IsUpdating then Exit;
  if not CheckDataSet then Exit;
  if not Assigned(FPlanner) then Exit;

  BeginUpdate;
  D := DataSource.DataSet;

  {$IFNDEF TMSDOTNET}
  D.Locate(KeyField,FPlanner.Items.DBItem.DBKey,[]);
  {$ENDIF}

  {$IFDEF TMSDOTNET}
  D.Locate(KeyField,Variant(FPlanner.Items.DBItem.DBKey),[]);
  {$ENDIF}

  EndUpdate;
end;

procedure TTIWDBMultiMonthSource.ItemChanged(DBKey: string);
var
  plIt: TPlannerItem;
  dts,dte: TDateTime;
  D: TDataSet;

begin
  inherited;
  if IsUpdating then Exit;
  if (DBKEY='') then Exit;
  if not Assigned(FPlanner) then Exit;

  BeginUpdate;

  {$IFDEF TMSCODESITE}
  SendMsg(FPlanner.Name+' Itemchanged: '+DBKEY);
  {$ENDIF}

  plIt := FPlanner.Items.FindKey(DBKey);

  if Assigned(plIt) then
  begin
    {$IFDEF TMSCODESITE}
    SendMsg('Found it : '+ plIt.Name);
    {$ENDIF}
    FPlanner.Items.DBItem := plIt;
    ReadDBItem;
    FPlanner.Items.Select(plIt);
  end
  else
  begin
    {$IFDEF TMSCODESITE}
    SendMsg('Add it to '+inttostr(FPlanner.Items.Count)+' items');
    {$ENDIF}

    if not CheckDataSet then Exit;

    D := DataSource.DataSet;

    FieldsToTime(D.Fields,dts,dte);

    if (dts <= FEndDate) and
       (dte >= FStartDate) then
    begin
      plIt := FPlanner.Items.Add;
      plIt.DBKey := DBKey;
      FPlanner.Items.DBItem := plIt;
      ReadDBItem;
    end;
  end;

  EndUpdate;
end;

procedure TTIWDBMultiMonthSource.Loaded;
begin
  inherited;
  if Assigned(FPlanner) then
    UpdatePlanner;
end;

procedure TTIWDBMultiMonthSource.Next;
var
  NSM,NSY: Integer;
begin
  NSM := StartMonth + 1;
  NSY := Year;
  if NSM > 12 then
  begin
    NSM := 1;
    NSY := Year + 1;
  end;
  FStartMonth := NSM;
  FYear := NSY;
  if CheckDataSet then
    UpdatePlanner;
end;

procedure TTIWDBMultiMonthSource.PlannerChanged;
begin
  inherited;
  if Assigned(FPlanner) then
    UpdatePlanner;
end;

procedure TTIWDBMultiMonthSource.Prev;
var
  NSM,NSY: Integer;
begin
  NSM := StartMonth - 1;
  NSY := Year;
  if NSM < 1 then
  begin
    NSM := 12;
    NSY := Year - 1;
  end;
  FStartMonth := NSM;
  FYear := NSY;
  if CheckDataSet then
    UpdatePlanner;
end;

procedure TTIWDBMultiMonthSource.ReadDBItem;
var
  D: TDataSet;
  dt, dts, dte: TDateTime;
  smo,sye,sda: word;
  emo,eye,eda: word;

begin
  if not CheckDataSet then Exit;

  {$IFDEF TMSCODESITE}
  SendMsg('read selected');
  {$ENDIF}

  D := DataSource.DataSet;

//  B := D.GetBookMark;
//  D.Locate(KeyField,APlanner.Items.Selected.DBKey,[]);

  FPlanner.RemoveClones(FPlanner.Items.DBItem);

  FieldsToTime(D.Fields,dts,dte);

  dt := dts;

  if dts >= FStartDate then
    DecodeDate(dts,sye,smo,sda)
  else
    DecodeDate(FStartDate,sye,smo,sda);

  if dte <= FEndDate then
    DecodeDate(dte,eye,emo,eda)
  else
    DecodeDate(FEndDate,eye,emo,eda);

  if (smo <> emo) then
  begin
    with FPlanner.Items.DBItem do
    begin
      Assign(FPlanner.DefaultItem);

      if dts >= FStartDate then
        ItemStartTime := dts
      else
        ItemStartTime := FStartDate;

      if dte <= FEndDate then
        ItemEndTime := EndOfMonth(dts)
      else
        ItemEndTime := FEndDate;

      FixedPos := True;
      FixedPosition := True;
      FixedSize := True;

      RealTime := True;
      ItemRealStartTime := dts;
      ItemRealEndTime := dte;

      if KeyField <> '' then
        DBKey := D.FieldByName(KeyField).AsString;
      if SubjectField <> '' then
        CaptionText := D.FieldByName(SubjectField).AsString;
      if NotesField <> '' then
        Text.Text := D.FieldByName(NotesField).AsString;

      if FReadOnly then
      begin
        FixedPos := True;
        FixedSize := True;
        ReadOnly := True;
      end;

      while (smo < emo) do
      begin
        dts := EndOfMonth(dts) + 1;
        smo := NextMonth(smo);
        with FPlanner.CloneItem(FPlanner.Items.DBItem) do
        begin
          ItemStartTime := dts;
          if smo = emo then
            ItemEndTime := dte
          else
            ItemEndTime := EndOfMonth(dts);

          RealTime := True;
          ItemRealStartTime := dt;
          ItemRealEndTime := dte;
        end;
      end;
    end;
  end
  else
  begin
    with FPlanner.Items.DBItem do
    begin
      Assign(FPlanner.DefaultItem);

      if dts >= FStartDate then
        ItemStartTime := dts
      else
        ItemStartTime := FStartDate;

      if dte <= FEndDate then
        ItemEndTime := dte
      else
        ItemEndTime := FEndDate;

      RealTime := True;
      ItemRealStartTime := dts;
      ItemRealEndTime := dte;

      if KeyField <> '' then
        DBKey := D.FieldByName(KeyField).AsString;
      if SubjectField <> '' then
        CaptionText := D.FieldByName(SubjectField).AsString;
      if NotesField <> '' then
        Text.Text := D.FieldByName(NotesField).AsString;

      if FReadOnly then
      begin
        FixedPos := True;
        FixedSize := True;
        ReadOnly := True;
      end;

    end;
  end;

  if Assigned(FOnFieldsToItem) then
    FOnFieldsToItem(Self, D.Fields, FPlanner.Items.DBItem);
end;

procedure TTIWDBMultiMonthSource.ReadDBItems;
var
  D: TDataSet;
  B: TBookMark;
  dt,dts,dte: TDateTime;
  plIt: TPlannerItem;
  Accept: Boolean;
  smo,sye,sda: word;
  emo,eye,eda: word;

begin
  if IsUpdating then Exit;
  if not CheckDataSet then Exit;

  {$IFDEF TMSCODESITE}
  SendMsg('READ DB ITEMS : '+FPlanner.Name);
  {$ENDIF}

  BeginUpdate;

  if Assigned(FOnSetFilter) then
    FOnSetFilter(Self);

  D := DataSource.DataSet;

  D.DisableControls;

  B := D.GetBookMark;

  D.First;

  ConfigurePlanner;

  FPlanner.Items.BeginUpdate;

  while not D.Eof do
  begin
    Accept := True;

    FieldsToTime(D.Fields,dts,dte);
    dt := dts;

    if (dts <= FEndDate) and
       (dte >= FStartDate) and
       Accept and Assigned(FOnAccept) then
    begin
      FOnAccept(Self,D.Fields,Accept);
    end;

    if (dts <= FEndDate) and
       (dte >= FStartDate) and Accept then
    begin
      plIt := FPlanner.Items.FindKey(D.FieldByName(KeyField).AsString);

      if not Assigned(plIt) then
      begin
        {$IFDEF TMSCODESITE}
        SendMsg(FPlanner.Name+' Add:'+D.FieldByName(KeyField).AsString);
        {$ENDIF}
        plIt := FPlanner.Items.Add;
      end;

      if dts >= FStartDate then
        DecodeDate(dts,sye,smo,sda)
      else
        DecodeDate(FStartDate,sye,smo,sda);

      if dte <= FEndDate then
        DecodeDate(dte,eye,emo,eda)
      else
        DecodeDate(FEndDate,eye,emo,eda);

      if (smo <> emo) then
      begin
        with plIt do
        begin
          Assign(FPlanner.DefaultItem);

          if dts >= FStartDate then
            ItemStartTime := dts
          else
            ItemStartTime := FStartDate;

          if dte <= FEndDate then
            ItemEndTime := EndOfMonth(dts)
          else
            ItemEndTime := FEndDate;

          FixedPos := True;
          FixedSize := True;
          FixedPosition := True;

          RealTime := True;

          ItemRealStartTime := dts;
          ItemRealEndTime := dte;

          if KeyField <> '' then
            DBKey := D.FieldByName(KeyField).AsString;
          if SubjectField <> '' then
            CaptionText := D.FieldByName(SubjectField).AsString;
          if NotesField <> '' then
            Text.Text := D.FieldByName(NotesField).AsString;

          if FReadOnly then
          begin
            FixedPos := True;
            FixedSize := True;
            ReadOnly := True;
          end;

          while (smo < emo) do
          begin
            dts := EndOfMonth(dts) + 1;
            smo := NextMonth(smo);

            with FPlanner.CloneItem(plIt) do
            begin
              ItemStartTime := dts;
              if smo = emo then
                ItemEndTime := dte
              else
                ItemEndTime := EndOfMonth(dts);

              RealTime := True;
              ItemRealStartTime := dt;
              ItemRealEndTime := dte;

            end;
          end;

        end;
      end
      else
      begin
        with plIt do
        begin
          Assign(FPlanner.DefaultItem);

          if dts >= FStartDate then
            ItemStartTime := dts
          else
            ItemStartTime := FStartDate;

          if dte <= FEndDate then
            ItemEndTime := dte
          else
            ItemEndTime := FEndDate;

          RealTime := True;

          ItemRealStartTime := dts;
          ItemRealEndTime := dte;


          if KeyField <> '' then
            DBKey := D.FieldByName(KeyField).AsString;
          if SubjectField <> '' then
            CaptionText := D.FieldByName(SubjectField).AsString;
          if NotesField <> '' then
            Text.Text := D.FieldByName(NotesField).AsString;

          if FReadOnly then
          begin
            FixedPos := True;
            FixedSize := True;
            ReadOnly := True;
          end;

        end;
      end;
      if Assigned(FOnFieldsToItem) then
        FOnFieldsToItem(Self, D.Fields, plIt);

    end;
    D.Next;
  end;

  D.GotoBookMark(B);
  D.EnableControls;
  D.FreeBookMark(B);

  if Assigned(FOnResetFilter) then
    FOnResetFilter(Self);

  EndUpdate;

  FPlanner.Items.EndUpdate;

  if Assigned(OnItemsRead) then
    OnItemsRead(Self);  
  
  {$IFDEF TMSCODESITE}
  SendMsg('DONE READ DB ITEMS');
  {$ENDIF}
end;

procedure TTIWDBMultiMonthSource.SetNumberOfMonths(const Value: Integer);
begin
  FNumberOfMonths := Value;
  if CheckDataSet then
    UpdatePlanner;
end;

procedure TTIWDBMultiMonthSource.SetStartMonth(const Value: Integer);
begin
  if (Value <= 0) or (Value > 12) then
    Exit;

  FStartMonth := Value;
  if CheckDataSet then
    UpdatePlanner;
end;

procedure TTIWDBMultiMonthSource.SetYear(const Value: Integer);
begin
  FYear := Value;
  { reload items to reflect the new year }
  if CheckDataSet then
    UpdatePlanner;
end;

procedure TTIWDBMultiMonthSource.SynchDBItems;
var
  D: TDataSet;
  B: TBookMark;
  dts,dte: TDateTime;
  plIt: TPlannerItem;
  i: Integer;

begin
  if IsUpdating then Exit;
  if not CheckDataSet then Exit;
  if not Assigned(FPlanner) then Exit;

  BeginUpdate;

  if Assigned(FOnSetFilter) then
    FOnSetFilter(Self);

  D := DataSource.DataSet;
  B := D.GetBookMark;
  D.First;

  for i := 1 to FPlanner.Items.Count do
    FPlanner.Items[i - 1].Synched := False;

  while not D.Eof do
  begin
    FieldsToTime(D.Fields,dts,dte);

    if (dts <= FEndDate) and
       (dte >= FStartDate) then
    begin
      plIt := FPlanner.Items.FindKey(D.FieldByName(KeyField).AsString);
      if Assigned(plIt) then
        plIt.Synched := True;
    end;
    D.Next;
  end;

  for i := FPlanner.Items.Count downto 1 do
  begin
    if not FPlanner.Items[i - 1].Synched then
      FPlanner.Items[i - 1].Free;
  end;

  D.GotoBookMark(B);
  D.FreeBookMark(B);

  if Assigned(FOnResetFilter) then
    FOnResetFilter(Self);

  EndUpdate;

end;

procedure TTIWDBMultiMonthSource.UpdatePlanner;
var
  Ey,Em: Integer;

begin
  FPlanner.Positions := FNumberOfMonths;
  FPlanner.Mode.Month := FStartMonth;
  FPlanner.Mode.Year := FYear;
  FPlanner.Mode.PlannerType := plMultiMonth;

  FStartDate := EncodeDate(FYear,FStartMonth,1);

//  Ey := FYear + (FStartMonth + FNumberOfMonths - 1) div 12;

  Ey := FYear;

  Em := (FStartMonth + FNumberOfMonths - 1);

  while Em > 12 do
  begin
    Em := Em - 12;
    Inc(Ey);
  end;

  FEndDate := EncodeDate(Ey,Em,DaysInMonth(Em,Ey));

  ConfigurePlanner;
  FPlanner.Items.BeginUpdate;
  ClearDBItems;
  ReadDBItems;
  FPlanner.Items.EndUpdate;
end;

procedure TTIWDBMultiMonthSource.WriteDBItem;
var
  D: TDataSet;
  B: TBookMark;
begin
  if IsUpdating then Exit;

  if not CheckDataSet then Exit;

  {$IFDEF TMSCODESITE}
  SendMsg('write selected here?');
  {$ENDIF}

  if Assigned(FOnUpdateItem) then
  begin
    BeginUpdate;
    FOnUpdateItem(Self,FPlanner.Items.DBItem);
    EndUpdate;
    if not FUpdateByQuery then
      Exit;
  end;

  BeginUpdate;

  D := DataSource.DataSet;

  B := D.GetBookMark;

  {$IFNDEF TMSDOTNET}
  D.Locate(KeyField,FPlanner.Items.DBItem.DBKey,[]);
  {$ENDIF}

  {$IFDEF TMSDOTNET}
  D.Locate(KeyField,Variant(FPlanner.Items.DBItem.DBKey),[]);
  {$ENDIF}

  D.Edit;

  TimeToFields(D.Fields,FPlanner.Items.DBItem.ItemRealStartTime,
    FPlanner.Items.DBItem.ItemRealEndTime);

  if (SubjectField <> '') and
     (not D.FieldByName(SubjectField).CanModify) then
    D.FieldByName(SubjectField).AsString := FPlanner.Items.DBItem.CaptionText;

  if (NotesField <> '') and
     (not D.FieldByName(NotesField).CanModify) then
    D.FieldByName(NotesField).AsString := FPlanner.Items.DBItem.ItemText;

  if Assigned(FOnItemToFields) then
    FOnItemToFields(Self, D.Fields, FPlanner.Items.DBItem);

  ReadDBItem;
  FPlanner.Items.Select(FPlanner.Items.DBItem);

  try
    D.Post;
  finally
    D.GotoBookMark(B);
    D.FreeBookMark(B);
    EndUpdate;
  end;

end;

{ TTIWDBTimeLineSource }

procedure TTIWDBTimeLineSource.AddDBItem;
var
  D: TDataSet;
  B: TBookMark;

begin
  if not CheckDataSet then Exit;

  {$IFDEF TMSCODESITE}
  SendMsg('add selected here?');
  {$ENDIF}

  if Assigned(FOnInsertItem) then
  begin
    BeginUpdate;
    FOnInsertItem(Self,FPlanner.Items.DBItem);
    EndUpdate;
    Exit;
  end;

  BeginUpdate;

  D := DataSource.DataSet;

  B := D.GetBookMark;

  D.Append;

  if not AutoIncKey then
  begin
    D.FieldByName(KeyField).AsString := CreateKey;
    FPlanner.Items.DBItem.DBKey := D.FieldByName(KeyField).AsString;
  end;

  TimeToFields(D.Fields,FPlanner.Items.DBItem.ItemStartTime,
    FPlanner.Items.DBItem.ItemEndTime);


  if (ResourceField <> '') then
  begin
    ResToFields(D.Fields,FPlanner.Items.DBItem.ItemPos);
  end;

  if SubjectField <> '' then
    D.FieldByName(SubjectField).AsString := FPlanner.Items.DBItem.CaptionText;

  if NotesField <> '' then
    D.FieldByName(NotesField).AsString := FPlanner.Items.DBItem.ItemText;

  if Assigned(FOnItemToFields) then
    FOnItemToFields(Self, D.Fields, FPlanner.Items.DBItem);

  try
    D.Post;
    if AutoIncKey then
      FPlanner.Items.DBItem.DBKey := D.FieldByName(KeyField).AsString;
  finally
    D.GotoBookMark(B);
    D.FreeBookMark(B);
    EndUpdate;
  end;
end;

procedure TTIWDBTimeLineSource.ConfigurePlanner;
var
  i: Integer;
  ResName: string;
begin
  FPlanner.Positions := NumberOfResources;

  if FAutoHeaderUpdate then
  begin
    FPlanner.Header.Captions.Clear;
    FPlanner.Header.Captions.Add('');

    for i := 1 to NumberOfResources do
    begin
      ResName := ResourceMap.PositionName(i - 1);
      if Assigned(FOnGetResourceName) then
        FOnGetResourceName(Self,i,ResName);
      if ResName <> '' then
      begin
        if FPlanner.Header.Captions.Count <= i then
          FPlanner.Header.Captions.Add(ResName)
        else
          FPlanner.Header.Captions.Strings[i] := ResName;
      end;
    end;
  end;
  FPlanner.Display.CurrentPosFrom := -1;
  FPlanner.Display.CurrentPosTo := -1;
end;

constructor TTIWDBTimeLineSource.Create(AOwner: TComponent);
begin
  inherited;
  FStartDate := Int(Now);
  FEndDate := Int(Now) + 3;
  FNumberOfResources := 3;
end;

procedure TTIWDBTimeLineSource.DeleteDBItem(APlanner: TTIWWebPlanner);
var
  D: TDataSet;
begin
  inherited;
  {$IFDEF TMSCODESITE}
  SendMsg('delete selected here?');
  {$ENDIF}

  BeginUpdate;
  D := DataSource.DataSet;

  {$IFNDEF TMSDOTNET}
  D.Locate(KeyField,FPlanner.Items.DBItem.DBKey,[]);
  {$ENDIF}

  {$IFDEF TMSDOTNET}
  D.Locate(KeyField,Variant(FPlanner.Items.DBItem.DBKey),[]);
  {$ENDIF}

  D.Delete;
  EndUpdate;
end;

procedure TTIWDBTimeLineSource.GotoDBItem;
var
  D: TDataSet;
begin
  inherited;
  if IsUpdating then Exit;
  if not CheckDataSet then Exit;
  if not Assigned(FPlanner) then Exit;

  BeginUpdate;
  D := DataSource.DataSet;

  {$IFNDEF TMSDOTNET}
  D.Locate(KeyField,FPlanner.Items.DBItem.DBKey,[]);
  {$ENDIF}

  {$IFDEF TMSDOTNET}
  D.Locate(KeyField,Variant(FPlanner.Items.DBItem.DBKey),[]);
  {$ENDIF}

  EndUpdate;
end;

procedure TTIWDBTimeLineSource.ItemChanged(DBKey: string);
var
  plIt: TPlannerItem;
  dts,dte: TDateTime;
  D: TDataSet;

begin
  inherited;
  if IsUpdating then Exit;
  if (DBKEY='') then Exit;
  if not Assigned(FPlanner) then Exit;

  BeginUpdate;

  {$IFDEF TMSCODESITE}
  SendMsg(FPlanner.Name+' Itemchanged: '+DBKEY);
  {$ENDIF}

  plIt := FPlanner.Items.FindKey(DBKey);

  if Assigned(plIt) then
  begin
    {$IFDEF TMSCODESITE}
    SendMsg('Found it : '+ plIt.Name);
    {$ENDIF}
    FPlanner.Items.DBItem := plIt;
    ReadDBItem;

    FPlanner.Items.Select(plIt);
  end
  else
  begin
    {$IFDEF TMSCODESITE}
    SendMsg('Add it to '+inttostr(FPlanner.Items.Count)+' items');
    {$ENDIF}

    if not CheckDataSet then Exit;

    D := DataSource.DataSet;

    FieldsToTime(D.Fields,dts,dte);

    if (dts < FPlanner.Mode.PeriodEndDate) and
       (dte > FPlanner.Mode.PeriodStartDate) then
    begin
      plIt := FPlanner.Items.Add;
      plIt.DBKey := DBKey;
      FPlanner.Items.DBItem := plIt;
      ReadDBItem;
    end;
  end;

  EndUpdate;
end;

procedure TTIWDBTimeLineSource.Next;
begin
  inherited;
end;

procedure TTIWDBTimeLineSource.PlannerChanged;
begin
  inherited;
  UpdatePlanner;
end;

function TTIWDBTimeLineSource.PosToRes(Pos: Integer): Integer;
begin
  Result := Pos;
end;

procedure TTIWDBTimeLineSource.Prev;
begin
  inherited;
end;

procedure TTIWDBTimeLineSource.ReadDBItem;
var
  D: TDataSet;
  dts, dte: TDateTime;
  ResourcePos: Integer;
  Accept: Boolean;

begin
  if not CheckDataSet then Exit;

  {$IFDEF TMSCODESITE}
  SendMsg('read selected');
  {$ENDIF}

  D := DataSource.DataSet;

//  B := D.GetBookMark;
//  D.Locate(KeyField,APlanner.Items.Selected.DBKey,[]);

  FPlanner.RemoveClones(FPlanner.Items.DBItem);

  FieldsToTime(D.Fields,dts,dte);

  with FPlanner.Items.DBItem do
  begin
    Assign(FPlanner.DefaultItem);

    ItemStartTime := dts;
    ItemEndTime := dte;

    if KeyField <> '' then
      DBKey := D.FieldByName(KeyField).AsString;
    if SubjectField <> '' then
      CaptionText := D.FieldByName(SubjectField).AsString;
    if NotesField <> '' then
      Text.Text := D.FieldByName(NotesField).AsString;

    if (ResourceField <> '') then
    begin
      FieldsToRes(D.Fields, ResourcePos, Accept);
      ItemPos := ResourcePos;
    end;

    if FReadOnly then
    begin
      FixedPos := True;
      FixedSize := True;
      ReadOnly := True;
    end;
  end;

  if Assigned(FOnFieldsToItem) then
    FOnFieldsToItem(Self, D.Fields, FPlanner.Items.DBItem);
end;

procedure TTIWDBTimeLineSource.ReadDBItems;
var
  D: TDataSet;
  B: TBookMark;
  dts,dte: TDateTime;
  plIt: TPlannerItem;
  Accept: Boolean;
  ResourcePos: Integer;

begin
  if IsUpdating then Exit;
  if not CheckDataSet then Exit;

  {$IFDEF TMSCODESITE}
  SendMsg('READ DB ITEMS : '+FPlanner.Name);
  {$ENDIF}

  BeginUpdate;

  if Assigned(FOnSetFilter) then
    FOnSetFilter(Self);

  D := DataSource.DataSet;

  D.DisableControls;

  B := D.GetBookMark;

  D.First;

  ConfigurePlanner;

  FPlanner.Items.BeginUpdate;

  {$IFDEF TMSDEBUG}
  outputdebugstring(pchar('datezone:'+formatdatetime('dd/mm/yyyy hh:nn:ss',FStartDate)+' to ' + formatdatetime('dd/mm/yyyy hh:nn:ss',FEndDate)));
  {$ENDIF}

  while not D.Eof do
  begin
    Accept := True;

    FieldsToTime(D.Fields,dts,dte);

    if (dts <= FEndDate) and
       (dte >= FStartDate) and
       Accept and Assigned(FOnAccept) then
    begin
      FOnAccept(Self,D.Fields,Accept);
    end;

    {$IFDEF TMSDEBUG}
    outputdebugstring(pchar(formatdatetime('dd/mm/yyyy hh:nn:ss',dts)+' to ' + formatdatetime('dd/mm/yyyy hh:nn:ss',dte)));
    {$ENDIF}

    if (int(dts) <= int(FEndDate)) and
       (int(dte) >= int(FStartDate)) and Accept then
    begin
      plIt := FPlanner.Items.FindKey(D.FieldByName(KeyField).AsString);

      if not Assigned(plIt) then
      begin
        {$IFDEF TMSCODESITE}
        SendMsg(FPlanner.Name+' Add:'+D.FieldByName(KeyField).AsString);
        {$ENDIF}
        plIt := FPlanner.Items.Add;
      end;

      with plIt do
      begin
        Assign(FPlanner.DefaultItem);

        ItemStartTime := dts;
        ItemEndTime := dte;

        if KeyField <> '' then
           DBKey := D.FieldByName(KeyField).AsString;

        if SubjectField <> '' then
          CaptionText := D.FieldByName(SubjectField).AsString;
          
        if NotesField <> '' then
          Text.Text := D.FieldByName(NotesField).AsString;

        if (ResourceField <> '') then
        begin
          FieldsToRes(D.Fields, ResourcePos, Accept);
          ItemPos := ResourcePos;
        end;

        if FReadOnly then
        begin
          FixedPos := True;
          FixedSize := True;
          ReadOnly := True;
        end;
      end;

      if Assigned(FOnFieldsToItem) then
        FOnFieldsToItem(Self, D.Fields, plIt);

    end;
    D.Next;
  end;

  D.GotoBookMark(B);
  D.EnableControls;
  D.FreeBookMark(B);

  if Assigned(FOnResetFilter) then
    FOnResetFilter(Self);

  EndUpdate;

  FPlanner.Items.EndUpdate;

  if Assigned(OnItemsRead) then
    OnItemsRead(Self);  

  {$IFDEF TMSCODESITE}
  SendMsg('DONE READ DB ITEMS');
  {$ENDIF}
end;

procedure TTIWDBTimeLineSource.SetEndDate(const Value: TDateTime);
begin
  FEndDate := Value;
  UpdatePlanner;
end;

procedure TTIWDBTimeLineSource.SetNumberOfResources(const Value: Integer);
begin
  FNumberOfResources := Value;
  UpdatePlanner;
end;

procedure TTIWDBTimeLineSource.SetStartDate(const Value: TDateTime);
begin
  FStartDate := Value;
  UpdatePlanner;
end;

procedure TTIWDBTimeLineSource.SynchDBItems;
var
  D: TDataSet;
  B: TBookMark;
  dts,dte: TDateTime;
  plIt: TPlannerItem;
  i: Integer;

begin
  if IsUpdating then Exit;
  if not CheckDataSet then Exit;
  if not Assigned(FPlanner) then Exit;

  BeginUpdate;

  if Assigned(FOnSetFilter) then
    FOnSetFilter(Self);

  D := DataSource.DataSet;
  B := D.GetBookMark;
  D.First;

  for i := 1 to FPlanner.Items.Count do
    FPlanner.Items[i - 1].Synched := False;

  while not D.Eof do
  begin
    FieldsToTime(D.Fields,dts,dte);

    if (dts <= FEndDate) and
       (dte >= FStartDate) then
    begin
      plIt := FPlanner.Items.FindKey(D.FieldByName(KeyField).AsString);
      if Assigned(plIt) then
        plIt.Synched := True;
    end;
    D.Next;
  end;

  for i := FPlanner.Items.Count downto 1 do
  begin
    if not FPlanner.Items[i - 1].Synched then
      FPlanner.Items[i - 1].Free;
  end;

  D.GotoBookMark(B);
  D.FreeBookMark(B);

  if Assigned(FOnResetFilter) then
    FOnResetFilter(Self);

  EndUpdate;
end;

procedure TTIWDBTimeLineSource.UpdatePlanner;
begin
  if Assigned(FPlanner) then
  begin

    FPlanner.Mode.PlannerType := plTimeLine;

    FPlanner.Mode.TimeLineStart := int(FStartDate);

    FPlanner.Positions := FNumberOfResources;

    if FPlanner.Display.DisplayUnit > 0 then
      FPlanner.Display.DisplayEnd := ((Round(1 + FEndDate - FStartDate)) * MININDAY) div FPlanner.Display.DisplayUnit - 1;
  end;   
end;

procedure TTIWDBTimeLineSource.WriteDBItem;
var
  D: TDataSet;
  B: TBookMark;
  ResPos: Integer;
begin
  if IsUpdating then Exit;

  if not CheckDataSet then Exit;

  {$IFDEF TMSCODESITE}
  SendMsg('write selected here?');
  {$ENDIF}

  if Assigned(FOnUpdateItem) then
  begin
    BeginUpdate;
    FOnUpdateItem(Self,FPlanner.Items.DBItem);
    EndUpdate;
    if not FUpdateByQuery then
      Exit;
  end;

  BeginUpdate;

  D := DataSource.DataSet;

  B := D.GetBookMark;

  {$IFNDEF TMSDOTNET}
  D.Locate(KeyField,FPlanner.Items.DBItem.DBKey,[]);
  {$ENDIF}

  {$IFDEF TMSDOTNET}
  D.Locate(KeyField,Variant(FPlanner.Items.DBItem.DBKey),[]);
  {$ENDIF}

  D.Edit;

  TimeToFields(D.Fields,FPlanner.Items.DBItem.ItemStartTime,
    FPlanner.Items.DBItem.ItemEndTime);

  if (SubjectField <> '') and
     D.FieldByName(SubjectField).CanModify then
    D.FieldByName(SubjectField).AsString := FPlanner.Items.DBItem.CaptionText;

  if (NotesField <> '') and
     D.FieldByName(NotesField).CanModify then
    D.FieldByName(NotesField).AsString := FPlanner.Items.DBItem.ItemText;

  if (ResourceField <> '') then
  begin
    ResPos := PosToRes(FPlanner.Items.DBItem.ItemPos);

    ResToFields(D.FIelds, ResPos);
  end;

  if Assigned(FOnItemToFields) then
    FOnItemToFields(Self, D.Fields, FPlanner.Items.DBItem);

  ReadDBItem;

  FPlanner.Items.Select(FPlanner.Items.DBItem);

  try
    D.Post;
  finally
    D.GotoBookMark(B);
    D.FreeBookMark(B);
    EndUpdate;
  end;

end;

{ TTIWDBHalfDayPeriodSource }

procedure TTIWDBHalfDayPeriodSource.PlannerChanged;
begin
  inherited;
  UpdatePlanner;
end;

procedure TTIWDBHalfDayPeriodSource.UpdatePlanner;
begin
  if Assigned(FPlanner) then
  begin
    FPlanner.Display.DisplayStart := 0;
    FPlanner.Display.DisplayEnd := 2 * Round(Int(FEndDate) - Int(FStartDate));

    FPlanner.Mode.PlannerType := plHalfDayPeriod;

    FPlanner.Mode.BeginUpdate;
    FPlanner.Mode.PeriodEndDate := FEndDate;
    FPlanner.Mode.PeriodStartDate := FStartDate;
    FPlanner.Mode.EndUpdate;
    FPlanner.Items.BeginUpdate;
    ClearDBItems;
    ReadDBItems;
    FPlanner.Items.EndUpdate;
  end;
end;

{ TTIWDBDisjunctDaySource }

procedure TTIWDBDisjunctDaySource.ReadDBItems;
var
  D: TDataSet;
  B: TBookMark;
  dts,dte: TDateTime;
  plIt: TPlannerItem;
  Accept: Boolean;
  DatePos: Integer;

begin
  if IsUpdating then Exit;
  if not CheckDataSet then Exit;

  {$IFDEF TMSCODESITE}
  SendMsg(FPlanner.Name+' READ DB ITEMS : '+FPlanner.Name);
  {$ENDIF}

  BeginUpdate;

  if Assigned(FOnSetFilter) then
    FOnSetFilter(Self);

  D := DataSource.DataSet;

  D.DisableControls;

  B := D.GetBookMark;

  D.First;

  ConfigurePlanner;

  FPlanner.Items.BeginUpdate;

  while not D.Eof do
  begin
    FieldsToTime(D.Fields,dts,dte);

    {$IFDEF TMSCODESITE}
    SendMsg(FPlanner.Name+' READ ITEM '+D.FieldByName(KeyField).AsString);
    SendMsg(FormatDateTime('dd/mm/yyyy hh:nn',FDay)+'->'+FormatDateTime('dd/mm/yyyy hh:nn',FDay+Span));
    {$ENDIF}

    Accept := False;
    
    DatePos := Dates.DatePos(dts);

    // date based item acception
    if DatePos >= 0 then
    begin
      Accept := True;
      if Assigned(FOnAccept) then
        FOnAccept(Self,D.Fields,Accept);
    end;

    if Accept then
    begin
      plIt := FPlanner.Items.FindKey(D.FieldByName(KeyField).AsString);

      if not Assigned(plIt) then
      begin
        {$IFDEF TMSCODESITE}
        SendMsg(FPlanner.Name+' ADD ITEM '+D.FieldByName(KeyField).AsString +
          ' ' +DateToStr(dts) + ' ' +DateToStr(dte));
        {$ENDIF}
        plIt := FPlanner.Items.Add;
      end;

      with plIt do
      begin
        FPlanner.RemoveClones(plIt);

        // set time here correct

        ItemRealStartTime := dts;
        ItemRealEndTime := dte;

        ItemStartTime := dts;
        ItemEndTime := dte;

        InHeader := dte - dts > 1;
        RealTime := true;

        ItemPos := DatePos;

        if KeyField <> '' then
          DBKey := D.FieldByName(KeyField).AsString;
        if SubjectField <> '' then
          CaptionText := D.FieldByName(SubjectField).AsString;
        if NotesField <> '' then
          Text.Text := D.FieldByName(NotesField).AsString;

        if FReadOnly then
        begin
          FixedPos := True;
          FixedSize := True;
          ReadOnly := True;
        end;
      end;

      if Assigned(FOnFieldsToItem) then
        FOnFieldsToItem(Self, D.Fields, plIt);
    end;

    D.Next;
  end;

  D.GotoBookMark(B);

  D.EnableControls;

  D.FreeBookMark(B);

  if Assigned(FOnResetFilter) then
    FOnResetFilter(Self);

  EndUpdate;

  FPlanner.Items.EndUpdate;

  if Assigned(OnItemsRead) then
    OnItemsRead(Self);


  {$IFDEF TMSCODESITE}
  SendMsg('DONE READ DB ITEMS');
  {$ENDIF}
end;

procedure TTIWDBDisjunctDaySource.WriteDBItem;
var
  D: TDataSet;
  B: TBookMark;
  DTE, DTS: TDateTime;
  Day: TDateTime;

begin
  if IsUpdating then Exit;

  if not CheckDataSet then Exit;

  {$IFDEF TMSCODESITE}
  SendMsg('write selected here?');
  {$ENDIF}

  if Assigned(FOnUpdateItem) then
  begin
    BeginUpdate;
    FOnUpdateItem(Self,FPlanner.Items.DBItem);
    EndUpdate;
    if not FUpdateByQuery then
      Exit;
  end;

  BeginUpdate;

  D := DataSource.DataSet;

  B := D.GetBookMark;

  {$IFNDEF TMSDOTNET}
  D.Locate(KeyField,FPlanner.Items.DBItem.DBKey,[]);
  {$ENDIF}

  {$IFDEF TMSDOTNET}
  D.Locate(KeyField,Variant(FPlanner.Items.DBItem.DBKey),[]);
  {$ENDIF}

  D.Edit;

  Day := Int(Dates.PosDate (FPlanner.Items.DBItem.ItemPos));

  if FPlanner.Items.DBItem.RealTime then
  begin
    DTS := Day + Frac(FPlanner.Items.DBItem.ItemRealStartTime);
    DTE := Day + Frac(FPlanner.Items.DBItem.ItemRealEndTime);
  end
  else
  begin
    DTS := Day + Frac(FPlanner.Items.DBItem.ItemStartTime);
    DTE := Day + Frac(FPlanner.Items.DBItem.ItemEndTime);
  end;

  {$IFDEF TMSCODESITE}
  SendMsg('writedb:'+formatdatetime('dd/mm/yyyy hh:nn',dts)+'-'+formatdatetime('dd/mm/yyyy hh:nn',dte));
  {$ENDIF}

  FPlanner.Items.DBItem.RealTime := False;

  TimeToFields(D.Fields,DTS,DTE);

  if (SubjectField <> '') and
     (D.FieldByName(SubjectField).CanModify) then
    D.FieldByName(SubjectField).AsString := FPlanner.Items.DBItem.CaptionText;

  if (NotesField <> '') and
     (D.FieldByName(NotesField).CanModify) then
    D.FieldByName(NotesField).AsString := FPlanner.Items.DBItem.ItemText;

  if Assigned(FOnItemToFields) then
    FOnItemToFields(Self, D.Fields, FPlanner.Items.DBItem);

  try
    D.Post;
  finally
    D.GotoBookMark(B);
    D.FreeBookMark(B);
    EndUpdate;
  end;

end;

procedure TTIWDBDisjunctDaySource.AddDBItem;
var
  D: TDataSet;
  B: TBookMark;
  DTS, DTE: TDateTime;
  Day: TDateTime;

begin
  inherited;

  if not CheckDataSet then Exit;

  {$IFDEF TMSCODESITE}
  SendMsg('add selected here?');
  {$ENDIF}

  if Assigned(FOnInsertItem) then
  begin
    BeginUpdate;
    FOnInsertItem(Self,FPlanner.Items.DBItem);
    EndUpdate;
    Exit;
  end;

  BeginUpdate;

  D := DataSource.DataSet;

  B := D.GetBookMark;

  D.Append;

  if not AutoIncKey then
  begin
    D.FieldByName(KeyField).AsString := CreateKey;
    FPlanner.Items.DBItem.DBKey := D.FieldByName(KeyField).AsString;
  end;

  Day := Int(Dates.PosDate(FPlanner.Items.DBItem.ItemPos));

  DTS := Day + Frac(FPlanner.Items.DBItem.ItemStartTime);
  DTE := Day + Frac(FPlanner.Items.DBItem.ItemEndTime);

  FPlanner.Items.DBItem.ItemRealStartTime := DTS;
  FPlanner.Items.DBItem.ItemRealEndTime := DTE;

  TimeToFields(D.Fields,DTS,DTE);

  if SubjectField <> '' then
    D.FieldByName(SubjectField).AsString := FPlanner.Items.DBItem.CaptionText;

  if NotesField <> '' then
    D.FieldByName(NotesField).AsString := FPlanner.Items.DBItem.ItemText;

  if Assigned(FOnItemToFields) then
    FOnItemToFields(Self, D.Fields, FPlanner.Items.DBItem);

  try
    D.Post;
    if AutoIncKey then
      FPlanner.Items.DBItem.DBKey := D.FieldByName(KeyField).AsString;
  finally
    D.GotoBookMark(B);
    D.FreeBookMark(B);
    EndUpdate;
  end;
end;

procedure TTIWDBDisjunctDaySource.DeleteDBItem(APlanner: TTIWWebPlanner);
var
  D: TDataSet;
begin
  inherited;
  if not CheckDataSet then Exit;

  {$IFDEF TMSCODESITE}
  SendMsg('delete selected here?');
  {$ENDIF}

  if Assigned(FOnDeleteItem) then
  begin
    BeginUpdate;
    FOnDeleteItem(Self,APlanner.Items.DBItem);
    EndUpdate;
    Exit;
  end;

  BeginUpdate;
  D := DataSource.DataSet;

  {$IFNDEF TMSDOTNET}
  D.Locate(KeyField,FPlanner.Items.DBItem.DBKey,[]);
  {$ENDIF}

  {$IFDEF TMSDOTNET}
  D.Locate(KeyField,Variant(FPlanner.Items.DBItem.DBKey),[]);
  {$ENDIF}

  D.Delete;
  EndUpdate;
end;


procedure TTIWDBDisjunctDaySource.ItemChanged(DBKey: string);
var
  plIt: TPlannerItem;
  dt, dtE: TDateTime;
  D: TDataSet;

begin
  inherited;
  if IsUpdating then Exit;
  if (DBKEY='') then Exit;
  if not Assigned(FPlanner) then Exit;

  BeginUpdate;

  {$IFDEF TMSCODESITE}
  SendMsg(FPlanner.Name+' Itemchanged: '+DBKEY);
  {$ENDIF}

  plIt := FPlanner.Items.FindKey(DBKey);

  if Assigned(plIt) then
  begin
    {$IFDEF TMSCODESITE}
    SendMsg('Found it : '+ plIt.Name);
    {$ENDIF}

    FPlanner.Items.DBItem := plIt;
    ReadDBItem;

    FPlanner.Items.Select(plIt);
  end
  else
  begin
    {$IFDEF TMSCODESITE}
    SendMsg('Add it to '+inttostr(FPlanner.Items.Count)+' items');
    {$ENDIF}

    if not CheckDataSet then Exit;

    D := DataSource.DataSet;
    FieldsToTime(D.Fields,dt,dtE);

    if Dates.HasDate(dt) then
    begin
      plIt := FPlanner.Items.Add;
      plIt.DBKey := DBKey;
      FPlanner.Items.DBItem := plIt;
      ReadDBItem;
    end;

  end;

  EndUpdate;
end;

procedure TTIWDBDisjunctDaySource.ReadDBItem;
var
  D: TDataSet;
  dts,dte: TDateTime;
  DatePos: Integer;

begin
  if not CheckDataSet then Exit;

  {$IFDEF TMSCODESITE}
  SendMsg('read selected');
  {$ENDIF}

  D := DataSource.DataSet;

  FPlanner.RemoveClones(FPlanner.Items.DBItem);

  ConfigurePlanner;

  with FPlanner.Items.DBItem do
  begin
    FieldsToTime(D.Fields,dts,dte);

    {$IFDEF TMSCODESITE}
    SendMsg('read:'+formatdatetime('dd/mm/yyyy hh:nn',dts)+'-'+formatdatetime('dd/mm/yyyy hh:nn',dte));
    {$ENDIF}

    ItemStartTime := dts;
    ItemEndTime := dte;
    ItemRealStartTime := dts;
    ItemRealEndTime := dte;

    InHeader := dte - dts > 1;

    DatePos := Dates.DatePos(dts);

    // date based item acception
    if DatePos >= 0 then
    begin
      FPlanner.RemoveClones(FPlanner.Items.DBItem);

      ItemPos := DatePos;

      if SubjectField <> '' then
        CaptionText := D.FieldByName(SubjectField).AsString;

      if NotesField <> '' then
        Text.Text := D.FieldByName(NotesField).AsString;

      if FReadOnly then
      begin
        FixedPos := True;
        FixedSize := True;
        ReadOnly := True;
      end;

      if Assigned(FOnFieldsToItem) then
        FOnFieldsToItem(Self, D.Fields, FPlanner.Items.DBItem);
    end
    else
    begin
      FPlanner.Items.DBItem.Free;
    end;
  end;

end;

procedure TTIWDBDisjunctDaySource.SynchDBItems;
var
  D: TDataSet;
  B: TBookMark;
  dt, dtE: TDateTime;
  plIt: TPlannerItem;
  i: Integer;

begin
  if IsUpdating then Exit;
  if not CheckDataSet then Exit;
  if not Assigned(FPlanner) then Exit;

  BeginUpdate;

  if Assigned(FOnSetFilter) then
    FOnSetFilter(Self);

  D := DataSource.DataSet;
  B := D.GetBookMark;
  D.First;

  for i := 1 to FPlanner.Items.Count do
    FPlanner.Items[i - 1].Synched := False;

  while not D.Eof do
  begin
    FieldsToTime(D.Fields,dt,dtE);

    if Dates.HasDate(dt) then
    begin
      plIt := FPlanner.Items.FindKey(D.FieldByName(KeyField).AsString);
      if Assigned(plIt) then
        plIt.Synched := True;
    end;

    D.Next;
  end;

  for i := FPlanner.Items.Count downto 1 do
  begin
    if not FPlanner.Items[i - 1].Synched then
      FPlanner.Items[i - 1].Free;
  end;

  D.GotoBookMark(B);
  D.FreeBookMark(B);

  if Assigned(FOnResetFilter) then
    FOnResetFilter(Self);

  EndUpdate;
end;

procedure TTIWDBDisjunctDaySource.GotoDBItem;
var
  D: TDataSet;
begin
  inherited;
  if IsUpdating then Exit;
  if not CheckDataSet then Exit;
  if not Assigned(FPlanner) then Exit;

  BeginUpdate;
  D := DataSource.DataSet;

  {$IFNDEF TMSDOTNET}
  D.Locate(KeyField,FPlanner.Items.DBItem.DBKey,[]);
  {$ENDIF}

  {$IFDEF TMSDOTNET}
  D.Locate(KeyField,Variant(FPlanner.Items.DBItem.DBKey),[]);
  {$ENDIF}

  EndUpdate;
end;

constructor TTIWDBDisjunctDaySource.Create(AOwner: TComponent);
begin
  inherited;
  // Initialize on US date format
  FDateFormat := 'mm/dd/yyyy';
  FDates := TDateCollection.Create;
end;

procedure TTIWDBDisjunctDaySource.PlannerChanged;
begin
  inherited;
  if Assigned(FPlanner) then
    FPlanner.Mode.PlannerType := plDay;
end;

procedure TTIWDBDisjunctDaySource.ConfigurePlanner;
var
  i: Integer;
begin
  if not Assigned(FPlanner) then
    Exit;

  FPlanner.Positions := Dates.Count;

  if FAutoHeaderUpdate then
  begin
    for i := 1 to Dates.Count do
      FPlanner.Header.Captions.Strings[i] := FormatDateTime(DateFormat, Dates[i - 1].Date);
  end;
end;

procedure TTIWDBDisjunctDaySource.MapItemTimeOnPlanner(APlannerItem: TPlannerItem);
var
  dts,dte: TDateTime;

begin
  with APlannerItem do
  begin
    RealTime := True;
    dts := ItemRealStartTime;
    dte := ItemRealEndTime;
    InHeader := dte - dts > 1;
    ItemPos := Dates.DatePos(dts);
  end;
end;


destructor TTIWDBDisjunctDaySource.Destroy;
begin
  FDates.Free;
  inherited;
end;

procedure TTIWDBDisjunctDaySource.SetDates(const Value: TDateCollection);
begin
  FDates.Assign(Value);
end;

function TTIWDBDisjunctDaySource.PosToDay(Pos: Integer): TDateTime;
begin
  Result := Dates.PosDate(Pos);
end;


{ TTIWDBActiveDaySource }

procedure TTIWDBActiveDaySource.ReadDBItems;
var
  D: TDataSet;
  B: TBookMark;
  dts,dte: TDateTime;
  plIt: TPlannerItem;
  Span, j: Integer;
  Accept: Boolean;
  ResourcePos,DatePos: Integer;
  Zone: TDateTime;
  dis, die: TDateTime;
  crossflg: Boolean;

begin
  if IsUpdating then Exit;
  if not CheckDataSet then Exit;

  {$IFDEF TMSCODESITE}
  SendMsg(FPlanner.Name+' READ DB ITEMS : '+FPlanner.Name);
  {$ENDIF}

  BeginUpdate;

  if Assigned(FOnSetFilter) then
    FOnSetFilter(Self);

  D := DataSource.DataSet;

  D.DisableControls;

  B := D.GetBookMark;

  D.First;

  ConfigurePlanner(Span);

  j := FPlanner.Display.DisplayUnit * FPlanner.Display.DisplayStart;
  j := j mod 1440;

  Zone := EncodeTime(j div 60,j mod 60,0,0);


  j := FPlanner.Display.DisplayUnit * FPlanner.Display.DisplayStart;
  j := j mod 1440;
  dis := EncodeTime(j div 60, j mod 60,0,0);

  j := FPlanner.Display.DisplayUnit * FPlanner.Display.DisplayEnd;
  crossflg := j > 1440;
  j := j mod 1440;
  die := EncodeTime(j div 60, j mod 60,0,0);

  FPlanner.Items.BeginUpdate;

  while not D.Eof do
  begin
    FieldsToTime(D.Fields,dts,dte);

    {$IFDEF TMSCODESITE}
    SendMsg(FPlanner.Name+' READ ITEM '+D.FieldByName(KeyField).AsString);
    SendMsg(FormatDateTime('dd/mm/yyyy hh:nn',FDay)+'->'+FormatDateTime('dd/mm/yyyy hh:nn',FDay+Span));
    {$ENDIF}

    Accept := True;

    // resource based item acception
    if (ResourceField <> '') and
       (Mode in [dmMultiResource,dmMultiDayRes,dmMultiResDay]) and
       (dte >= FDay + Zone) and (dts < FDay + Span + Zone) then
    begin
      FieldsToRes(D.Fields, ResourcePos, Accept);
    end;

    if (Int(dte) = Int(dts)) and not crossflg and ((Frac(dte) < Frac(dis)) or (Frac(dts) > Frac(die))) then
      Accept := False;

    // date based item acception
    if (dte >= FDay + Zone) and (dts < FDay + Span + Zone) and Accept and Assigned(FOnAccept) then
    begin
      FOnAccept(Self,D.Fields,Accept);
    end;

    DatePos := IDayToPos(dts);

    if Accept then
      Accept := DatePos <> -1;

    if (dte >= FDay + Zone) and (dts < FDay + Span + Zone) and Accept then
    begin
      plIt := FPlanner.Items.FindKey(D.FieldByName(KeyField).AsString);

      if not Assigned(plIt) then
      begin
        {$IFDEF TMSCODESITE}
        SendMsg(FPlanner.Name+' ADD ITEM '+D.FieldByName(KeyField).AsString +
          ' ' +DateToStr(dts) + ' ' +DateToStr(dte));
        {$ENDIF}
        plIt := FPlanner.Items.Add;
      end;

      with plIt do
      begin
        FPlanner.RemoveClones(plIt);

        // set time here correct

        ItemRealStartTime := dts;
        ItemRealEndTime := dte;

        ItemStartTime := dts;
        ItemEndTime := dte;

        InHeader := dte - dts > 1;
        RealTime := true;

        // map to start of day
        if (Int(dts) < Int(FDay)) and not InHeader then
          ChangeCrossing;

        ItemPos := CalcItemPos(DatePos,ResourcePos);

        if KeyField <> '' then
          DBKey := D.FieldByName(KeyField).AsString;
        if SubjectField <> '' then
          CaptionText := D.FieldByName(SubjectField).AsString;
        if NotesField <> '' then
          Text.Text := D.FieldByName(NotesField).AsString;

        if FReadOnly then
        begin
          FixedPos := True;
          FixedSize := True;
          ReadOnly := True;
        end;
      end;

      if Assigned(FOnFieldsToItem) then
        FOnFieldsToItem(Self, D.Fields, plIt);

      if plIt.InHeader then
      begin
        if dts < Day then
          dts := Day;

        for j := 1  to Round(Int(dte) - Int(dts)) do
        begin
          if (j + plIt.ItemPos < FPlanner.Positions) and (dts + j < Day + Span) then
            with FPlanner.CloneItem(plIt) do
            begin
              DatePos := DatePos + 1;
              ItemPos := CalcItemPos(DatePos,ResourcePos);
            end;
        end;
      end;
    end;

    D.Next;
  end;

  D.GotoBookMark(B);

  D.EnableControls;

  D.FreeBookMark(B);

  if Assigned(FOnResetFilter) then
    FOnResetFilter(Self);

  EndUpdate;

  FPlanner.Items.EndUpdate;

  if Assigned(OnItemsRead) then
    OnItemsRead(Self);


  {$IFDEF TMSCODESITE}
  SendMsg('DONE READ DB ITEMS');
  {$ENDIF}
end;

procedure TTIWDBActiveDaySource.SetDay(const Value: TDateTime);
begin
  if FDay <> Value then
  begin
    FDay := Int(Value);
    ClearDBItems;
    ReadDBItems;
  end;
end;

procedure TTIWDBActiveDaySource.Next;
var
  i: Integer;
begin
  if not Assigned(FPlanner) then
    Exit;

  i := 1;

  while (DayOfWeek(FDay + i) in FPlanner.InActive) do
  begin
    inc(i);
  end;

  FDay := FDay + i;
  inherited;
end;

procedure TTIWDBActiveDaySource.Prev;
var
  i: Integer;
begin
  if not Assigned(FPlanner) then
    Exit;

  i := -1;

  while (DayOfWeek(FDay + i) in FPlanner.InActive) do
  begin
    dec(i);
  end;

  FDay := FDay - i;
  inherited;
end;

procedure TTIWDBActiveDaySource.WriteDBItem;
var
  D: TDataSet;
  B: TBookMark;
  DTE, DTS: TDateTime;
  ResPos: Integer;

begin
  if IsUpdating then Exit;

  if not CheckDataSet then Exit;

  {$IFDEF TMSCODESITE}
  SendMsg('write selected here?');
  {$ENDIF}

  if Assigned(FOnUpdateItem) then
  begin
    BeginUpdate;
    FOnUpdateItem(Self,FPlanner.Items.DBItem);
    EndUpdate;
    if not FUpdateByQuery then
      Exit;
  end;

  BeginUpdate;

  D := DataSource.DataSet;

  B := D.GetBookMark;

  {$IFNDEF TMSDOTNET}
  D.Locate(KeyField,FPlanner.Items.DBItem.DBKey,[]);
  {$ENDIF}

  {$IFDEF TMSDOTNET}
  D.Locate(KeyField,Variant(FPlanner.Items.DBItem.DBKey),[]);
  {$ENDIF}

  D.Edit;

  if FPlanner.Items.DBItem.RealTime then
  begin
    DTS := FPlanner.Items.DBItem.ItemRealStartTime;
    DTE := FPlanner.Items.DBItem.ItemRealEndTime;
  end
  else
  begin
    DTS := FPlanner.Items.DBItem.ItemStartTime;
    DTE := FPlanner.Items.DBItem.ItemEndTime;
  end;

  {$IFDEF TMSCODESITE}
  SendMsg('writedb:'+formatdatetime('dd/mm/yyyy hh:nn',dts)+'-'+formatdatetime('dd/mm/yyyy hh:nn',dte));
  {$ENDIF}

  FPlanner.Items.DBItem.RealTime := False;

  //  if DTE < DTS then
  //    DTE := DTE + 1;

  TimeToFields(D.Fields,DTS,DTE);

  if (ResourceField <> '') and (Mode in [dmMultiResource,dmMultiDayRes,dmMultiResDay]) then
  begin
    ResPos := PosToRes(FPlanner.Items.DBItem.ItemPos);

    ResToFields(D.Fields, ResPos);
  end;

  if (SubjectField <> '') and
     (D.FieldByName(SubjectField).CanModify) then
    D.FieldByName(SubjectField).AsString := FPlanner.Items.DBItem.CaptionText;

  if (NotesField <> '') and
     (D.FieldByName(NotesField).CanModify) then
    D.FieldByName(NotesField).AsString := FPlanner.Items.DBItem.ItemText;

  if Assigned(FOnItemToFields) then
    FOnItemToFields(Self, D.Fields, FPlanner.Items.DBItem);

  try
    D.Post;
  finally
    D.GotoBookMark(B);
    D.FreeBookMark(B);
    EndUpdate;
  end;
end;


procedure TTIWDBActiveDaySource.SetMode(const Value: TDBDayMode);
begin
  if FMode <> Value then
  begin
    FMode := Value;
    case FMode of
    dmMultiDay:
      if Assigned(FPlanner) then
      begin
        FPlanner.PositionGroup := 0;
        FPlanner.Positions := NumberOfDays;
      end;
    dmMultiResource:
      if Assigned(FPlanner) then
      begin
        FPlanner.PositionGroup := 0;
        FPlanner.Positions := NumberOfResources;
      end;
    end;
    ClearDBItems;
    ReadDBItems;
  end;
end;

procedure TTIWDBActiveDaySource.AddDBItem;
var
  D: TDataSet;
  B: TBookMark;
  DTS, DTE: TDateTime;
  ResPos: Integer;

begin
  inherited;

  if not CheckDataSet then Exit;


  {$IFDEF TMSCODESITE}
  SendMsg('add selected here?');
  {$ENDIF}

  if Assigned(FOnInsertItem) then
  begin
    BeginUpdate;
    FOnInsertItem(Self,FPlanner.Items.DBItem);
    EndUpdate;
    Exit;
  end;

  BeginUpdate;

  D := DataSource.DataSet;

  B := D.GetBookMark;

  D.Append;

  if not AutoIncKey then
  begin
    D.FieldByName(KeyField).AsString := CreateKey;
    FPlanner.Items.DBItem.DBKey := D.FieldByName(KeyField).AsString;
  end;

  DTS := FPlanner.Items.DBItem.ItemStartTime;
  DTE := FPlanner.Items.DBItem.ItemEndTime;

  FPlanner.Items.DBItem.ItemRealStartTime := DTS;
  FPlanner.Items.DBItem.ItemRealEndTime := DTE;

  TimeToFields(D.Fields,DTS,DTE);

  if (ResourceField <> '') and (Mode in [dmMultiResource,dmMultiDayRes,dmMultiResDay]) then
  begin
    ResPos := PosToRes(FPlanner.Items.DBItem.ItemPos);
    ResToFields(D.Fields, ResPos);
  end;

  if SubjectField <> '' then
    D.FieldByName(SubjectField).AsString := FPlanner.Items.DBItem.CaptionText;

  if NotesField <> '' then
    D.FieldByName(NotesField).AsString := FPlanner.Items.DBItem.ItemText;

  if Assigned(FOnItemToFields) then
    FOnItemToFields(Self, D.Fields, FPlanner.Items.DBItem);

  try
    D.Post;
    if AutoIncKey then
      FPlanner.Items.DBItem.DBKey := D.FieldByName(KeyField).AsString;
  finally
    D.GotoBookMark(B);
    D.FreeBookMark(B);
    EndUpdate;
  end;
end;

procedure TTIWDBActiveDaySource.DeleteDBItem(APlanner: TTIWWebPlanner);
var
  D: TDataSet;
begin
  inherited;
  if not CheckDataSet then Exit;

  {$IFDEF TMSCODESITE}
  SendMsg('delete selected here?');
  {$ENDIF}

  if Assigned(FOnDeleteItem) then
  begin
    BeginUpdate;
    FOnDeleteItem(Self,APlanner.Items.DBItem);
    EndUpdate;
    Exit;
  end;

  BeginUpdate;
  D := DataSource.DataSet;

  {$IFNDEF TMSDOTNET}
  D.Locate(KeyField,FPlanner.Items.DBItem.DBKey,[]);
  {$ENDIF}

  {$IFDEF TMSDOTNET}
  D.Locate(KeyField,Variant(FPlanner.Items.DBItem.DBKey),[]);
  {$ENDIF}

  D.Delete;
  EndUpdate;
end;


procedure TTIWDBActiveDaySource.ItemChanged(DBKey: string);
var
  plIt: TPlannerItem;
  dt, dtE: TDateTime;
  Span: Integer;
  D: TDataSet;

begin
  inherited;
  if IsUpdating then Exit;
  if (DBKEY='') then Exit;
  if not Assigned(FPlanner) then Exit;

  BeginUpdate;

  {$IFDEF TMSCODESITE}
  SendMsg(FPlanner.Name+' Itemchanged: '+DBKEY);
  {$ENDIF}

  plIt := FPlanner.Items.FindKey(DBKey);

  if Assigned(plIt) then
  begin
    {$IFDEF TMSCODESITE}
    SendMsg('Found it : '+ plIt.Name);
    {$ENDIF}

    FPlanner.Items.DBItem := plIt;
    ReadDBItem;

    FPlanner.Items.Select(plIt);
  end
  else
  begin
    {$IFDEF TMSCODESITE}
    SendMsg('Add it to '+inttostr(FPlanner.Items.Count)+' items');
    {$ENDIF}

    if not CheckDataSet then Exit;

    D := DataSource.DataSet;
    FieldsToTime(D.Fields,dt,dtE);

    if Mode = dmMultiResource then
      Span := 1
    else
      Span := FPlanner.Positions;


    if (dt > FDay) and (dt < FDay + Span) then
    begin
      plIt := FPlanner.Items.Add;
      plIt.DBKey := DBKey;
      FPlanner.Items.DBItem := plIt;
      ReadDBItem;
    end;

  end;

  EndUpdate;
end;

procedure TTIWDBActiveDaySource.ReadDBItem;
var
  D: TDataSet;
  dts,dte: TDateTime;
  ResourcePos,DatePos,j: Integer;
  Accept: Boolean;
  Zone: TDateTime;
  Span: Integer;

begin
  if not CheckDataSet then Exit;

  {$IFDEF TMSCODESITE}
  SendMsg('read selected');
  {$ENDIF}

  D := DataSource.DataSet;

  FPlanner.RemoveClones(FPlanner.Items.DBItem);

  j := FPlanner.Display.DisplayUnit * FPlanner.Display.DisplayStart;
  j := j mod 1440;

  Zone := EncodeTime(j div 60,j mod 60,0,0);

  case Mode of
  dmMultiDay,dmMultiDayRes,dmMultiResDay: Span := NumberOfDays;
  dmMultiResource: Span := 1;
  end;

  ConfigurePlanner(Span);

  with FPlanner.Items.DBItem do
  begin
    FieldsToTime(D.Fields,dts,dte);

    {$IFDEF TMSCODESITE}
    SendMsg('read:'+formatdatetime('dd/mm/yyyy hh:nn',dts)+'-'+formatdatetime('dd/mm/yyyy hh:nn',dte));
    {$ENDIF}

    ItemStartTime := dts;
    ItemEndTime := dte;
    ItemRealStartTime := dts;
    ItemRealEndTime := dte;

    InHeader := dte - dts > 1;

    // resource based item acception
    if (ResourceField <> '') and
       (Mode in [dmMultiResource,dmMultiDayRes,dmMultiResDay]) and
       (dte >= FDay + Zone) and (dts < FDay + Span + Zone) then
    begin
      FieldsToRes(D.Fields,ResourcePos, Accept);
    end;

    // date based item acception
    if (dte >= FDay + Zone) and (dts < FDay + Span + Zone) then
    begin
      FPlanner.RemoveClones(FPlanner.Items.DBItem);

      DatePos := IDayToPos(dts);

      ItemPos := CalcItemPos(DatePos,ResourcePos);

      if SubjectField <> '' then
        CaptionText := D.FieldByName(SubjectField).AsString;

      if NotesField <> '' then
        Text.Text := D.FieldByName(NotesField).AsString;

      if FReadOnly then
      begin
        FixedPos := True;
        FixedSize := True;
        ReadOnly := True;
      end;

      if InHeader then
      begin
        if dts < Day then
          dts := Day;

        for j := 1  to Round(Int(dte) - Int(dts)) do
        begin
          if (j + ItemPos < FPlanner.Positions) and (dts + j < Day + Span) then
            with FPlanner.CloneItem(FPlanner.Items.DBItem) do
            begin
              DatePos := DatePos + 1;
              ItemPos := CalcItemPos(DatePos,ResourcePos);
            end;
        end;
      end;


      if Assigned(FOnFieldsToItem) then
        FOnFieldsToItem(Self, D.Fields, FPlanner.Items.DBItem);
    end
    else
    begin
      FPlanner.Items.DBItem.Free;
    end;
  end;

end;

procedure TTIWDBActiveDaySource.SynchDBItems;
var
  D: TDataSet;
  B: TBookMark;
  dt, dtE: TDateTime;
  plIt: TPlannerItem;
  Span,i: Integer;

begin
  if IsUpdating then Exit;
  if not CheckDataSet then Exit;
  if not Assigned(FPlanner) then Exit;

  BeginUpdate;

  if Assigned(FOnSetFilter) then
    FOnSetFilter(Self);

  D := DataSource.DataSet;
  B := D.GetBookMark;
  D.First;

  if Mode = dmMultiResource then
    Span := 1
  else
  begin
    Span := FPlanner.Positions;
  end;

  for i := 1 to FPlanner.Items.Count do
    FPlanner.Items[i - 1].Synched := False;

  while not D.Eof do
  begin
    FieldsToTime(D.Fields,dt,dtE);

    if (dt > FDay) and (dt < FDay + Span) then
    begin
      plIt := FPlanner.Items.FindKey(D.FieldByName(KeyField).AsString);
      if Assigned(plIt) then
        plIt.Synched := True;
    end;

    D.Next;
  end;

  for i := FPlanner.Items.Count downto 1 do
  begin
    if not FPlanner.Items[i - 1].Synched then
      FPlanner.Items[i - 1].Free;
  end;

  D.GotoBookMark(B);
  D.FreeBookMark(B);

  if Assigned(FOnResetFilter) then
    FOnResetFilter(Self);

  EndUpdate;
end;

procedure TTIWDBActiveDaySource.GotoDBItem;
var
  D: TDataSet;
begin
  inherited;
  if IsUpdating then Exit;
  if not CheckDataSet then Exit;
  if not Assigned(FPlanner) then Exit;

  BeginUpdate;
  D := DataSource.DataSet;

  {$IFNDEF TMSDOTNET}
  D.Locate(KeyField,FPlanner.Items.DBItem.DBKey,[]);
  {$ENDIF}

  {$IFDEF TMSDOTNET}
  D.Locate(KeyField,Variant(FPlanner.Items.DBItem.DBKey),[]);
  {$ENDIF}

  EndUpdate;
end;

constructor TTIWDBActiveDaySource.Create(AOwner: TComponent);
begin
  inherited;
  // Initialize on US date format
  FDateFormat := 'mm/dd/yyyy';
  FNumberOfDays := 7;
  FNumberOfResources := 1;
end;

procedure TTIWDBActiveDaySource.PlannerChanged;
begin
  inherited;
  if Assigned(FPlanner) then
    FPlanner.Mode.PlannerType := plDay;
end;

function TTIWDBActiveDaySource.PosToDay(Pos: Integer): TDateTime;
begin
  case Mode of
  dmMultiDay:
    Result := IPosToDay(Pos);
  dmMultiResource:
    Result := Day;
  dmMultiDayRes:
    Result := IPosToDay(Pos mod NumberOfDays);
  dmMultiResDay:
    Result := IPosToDay(Pos div NumberOfResources);
  else
    Result := FDay;
  end;
end;

function TTIWDBActiveDaySource.PosToRes(Pos: Integer): Integer;
begin
  case Mode of
  dmMultiDay:
    Result := 0;
  dmMultiResource:
    Result := Pos;
  dmMultiDayRes:
    Result := Pos div NumberOfDays;
  dmMultiResDay:
    Result := Pos mod NumberOfResources;
  else
    Result := 0;
  end;
end;


procedure TTIWDBActiveDaySource.SetNumberOfDays(const Value: Integer);
begin
  if Value <= 0 then
    Exit;

  if FNumberOfDays <> Value then
  begin
    FNumberOfDays := Value;
    ClearDBItems;
    ReadDBItems;
  end;
end;

procedure TTIWDBActiveDaySource.SetNumberOfResources(const Value: Integer);
begin
  if Value <= 0 then
    Exit;
  if FNumberOfResources <> Value then
  begin
    FNumberOfResources := Value;
    ClearDBItems;
    ReadDBItems;
  end;
end;

procedure TTIWDBActiveDaySource.ConfigurePlanner(var Span: Integer);
var
  i,j,k,dx: Integer;
  ResName: string;
begin
  if not Assigned(FPlanner) then
    Exit;

  case Mode of
  dmMultiResource:
    begin
      FPlanner.Positions := NumberOfResources;
      Span := 1;
      if FAutoHeaderUpdate then
      begin
        FPlanner.Header.Captions.Clear;
        FPlanner.Header.Captions.Add('');

        for i := 1 to NumberOfResources do
        begin
          ResName := ResourceMap.PositionName(i - 1);
          if Assigned(FOnGetResourceName) then
            FOnGetResourceName(Self,i,ResName);
          if ResName <> '' then
          begin
            if FPlanner.Header.Captions.Count <= i then
              FPlanner.Header.Captions.Add(ResName)
            else
              FPlanner.Header.Captions.Strings[i] := ResName;
          end;
        end;
      end;
      FPlanner.Display.CurrentPosFrom := -1;
      FPlanner.Display.CurrentPosTo := -1;
    end;
  dmMultiDay:
    begin
      FPlanner.Positions := NumberOfDays; // ie. nr. of active days

      Span := DaysToSpan(NumberOfDays);

      if FAutoHeaderUpdate then
      begin
        FPlanner.Header.Captions.Clear;
        FPlanner.Header.Captions.Add('');

        j := 1;
        for i := 1 to NumberOfDays do
        begin
          if FPlanner.Header.Captions.Count <= i then
            FPlanner.Header.Captions.Add(FormatDateTime(FDateFormat,DayInSpan(i - 1)))
          else
            FPlanner.Header.Captions.Strings[j] := FormatDateTime(FDateFormat,DayInSpan(i - 1));
        end;
      end;

      dx := Round(Int(Now) - Int(Day));
      FPlanner.Display.CurrentPosFrom := dx;
      FPlanner.Display.CurrentPosTo := dx;
    end;
  dmMultiDayRes:
    begin
      FPlanner.Positions := NumberOfResources * NumberOfDays;
      FPlanner.PositionGroup := NumberOfDays;
      Span := DaysToSpan(NumberOfDays);

      if FAutoHeaderUpdate then
      begin
        FPlanner.Header.Captions.Clear;
        FPlanner.Header.Captions.Add('');
        FPlanner.Header.GroupCaptions.Clear;
        FPlanner.Header.GroupCaptions.Add('');

        for i := 1 to NumberOfResources do
        begin
          for j := 1 to NumberOfDays do
          begin
             k := j + (i - 1) * NumberOfDays;

             if FPlanner.Header.Captions.Count <= k then
               FPlanner.Header.Captions.Add(FormatDateTime(FDateFormat,DayInSpan(j - 1)))
             else
               FPlanner.Header.Captions.Strings[k] := FormatDateTime(FDateFormat,DayInSpan(j - 1));
          end;

          ResName := ResourceMap.PositionName(i - 1);
          if Assigned(FOnGetResourceName) then
            FOnGetResourceName(Self,i,ResName);

          if FPlanner.Header.GroupCaptions.Count < i then
            FPlanner.Header.GroupCaptions.Add(ResName)
          else
            FPlanner.Header.GroupCaptions[i - 1] := ResName;
        end;
      end;
      FPlanner.Display.CurrentPosFrom := -1;
      FPlanner.Display.CurrentPosTo := -1;
    end;
  dmMultiResDay:
    begin
      FPlanner.PositionGroup := NumberOfResources;
      FPlanner.Positions := NumberOfResources * NumberOfDays;
      Span := DaysToSpan(NumberOfDays);

      if FAutoHeaderUpdate then
      begin
        FPlanner.Header.Captions.Clear;
        FPlanner.Header.Captions.Add('');
        FPlanner.Header.GroupCaptions.Clear;
        FPlanner.Header.GroupCaptions.Add('');

        for i := 1 to NumberOfDays do
        begin
          if FPlanner.Header.GroupCaptions.Count < i then
            FPlanner.Header.GroupCaptions.Add(FormatDateTime(FDateFormat,DayInSpan(i - 1)))
          else
            FPlanner.Header.GroupCaptions[i - 1] := FormatDateTime(FDateFormat,DayInSpan(i - 1));
        end;

        for i := 1 to NumberOfDays do
          for j := 1 to NumberOfResources do
          begin
            ResName := ResourceMap.PositionName(j - 1);
            if Assigned(FOnGetResourceName) then
              FOnGetResourceName(Self,j,ResName);
            k := j + (i - 1) * NumberOfResources;
            if FPlanner.Header.Captions.Count <= k then
              FPlanner.Header.Captions.Add(ResName)
            else
              FPlanner.Header.Captions.Strings[k] := ResName;
          end;
      end;
      dx := Round(Int(Now) - Int(Day));
      FPlanner.Display.CurrentPosFrom := dx * NumberOfResources;
      FPlanner.Display.CurrentPosTo := (dx + 1) * NumberOfResources - 1;
    end;
  end;

end;

function TTIWDBActiveDaySource.CalcItemPos(DatePos, ResourcePos: Integer): Integer;
begin
  case Mode of
  dmMultiDay:
    Result := DatePos;
  dmMultiResource:
    Result := ResourcePos;
  dmMultiResDay:
    Result := ResourcePos + NumberOfResources * DatePos;
  dmMultiDayRes:
    Result := ResourcePos * NumberOfDays + DatePos;
  else
    Result := DatePos;
  end;
end;


procedure TTIWDBActiveDaySource.MapItemTimeOnPlanner(APlannerItem: TPlannerItem);
var
  dts,dte: TDateTime;
  j, DatePos: Integer;
begin
  with APlannerItem do
  begin
    RealTime := True;
    dts := ItemRealStartTime;
    dte := ItemRealEndTime;

    // map to start of day
    if Int(dts) < Int(FDay) then
      ChangeCrossing;

    InHeader := dte - dts > 1;

    DatePos := Round(Int(dts - FDay));

    ItemPos := CalcItemPos(DatePos,ItemPos);

    if APlannerItem.InHeader then
    begin
      for j := 1  to round(Int(dte) - Int(dts)) do
        begin
          if j + APlannerItem.ItemPos < FPlanner.Positions then
            with FPlanner.CloneItem(APlannerItem) do
              ItemPos := ItemPos + j;
        end;
    end;
  end;
end;


function TTIWDBActiveDaySource.DaysToSpan(Days: Integer): Integer;
var
  i,j: Integer;

begin
  Result := Days;

  if not Assigned(FPlanner) then
    Exit;

  j := 0;

  for i := 1 to Days do
  begin
    while (DayOfWeek(Day + i - 1 + j) in FPlanner.InActive) do
    begin
      inc(Result);
      inc(j);
    end;
  end;
end;

function TTIWDBActiveDaySource.DayInSpan(ADay: Integer): TDateTime;
var
  i,j: Integer;
  flg: Boolean;
begin
  flg := False;

  for i := 0 to 6 do
  begin
    if not (i in FPlanner.InActive) then
      Flg := True;
  end;

  if not Flg then
    raise Exception.Create('No active days selected in planner');

  Result := Day;
  j := 0;

  for i := 0 to ADay do
  begin
    while (DayOfWeek(Day + i + j) in FPlanner.InActive) do
    begin
      Result := Result + 1;
      inc(j);
    end;
  end;

  Result := Result + ADay;
end;

function TTIWDBActiveDaySource.IDayToPos(ADate: TDateTime): Integer;
var
  i: Integer;
begin
  Result := -1;

  for i := 1 to NumberOfDays do
  begin
    if int(IPosToDay(i - 1)) = int(ADate) then
    begin
      Result := i - 1;
      Break;
    end;
  end;
end;

function TTIWDBActiveDaySource.IPosToDay(ADay: Integer): TDateTime;
begin
  Result := DayInSpan(ADay);
end;

{ TResourceMap }

function TResourceMap.Add: TResourceMapItem;
begin
  Result := TResourceMapItem(inherited Add);
end;

constructor TResourceMap.Create(AOwner: TComponent);
begin
  inherited Create(TResourceMapItem);
  FOwner := AOwner;
end;

function TResourceMap.GetItem(Index: Integer): TResourceMapItem;
begin
  Result := TResourceMapItem(inherited Items[Index]);
end;

function TResourceMap.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

function TResourceMap.Insert(index: Integer): TResourceMapItem;
begin
  Result := TResourceMapItem(inherited Insert(Index));
end;

function TResourceMap.PositionName(Value: Integer): string;
var
  i: integer;
begin
  Result := '';

  for i := 1 to Count do
  begin
    if Items[i - 1].PositionIndex = Value then
    begin
      Result := Items[i - 1].DisplayName;
      Break;
    end;
  end;

end;

function TResourceMap.PositionToRes(value: Integer): Integer;
var
  i: integer;
begin
  Result := -1;

  for i := 1 to Count do
  begin
    if Items[i - 1].PositionIndex = Value then
    begin
      Result := Items[i - 1].ResourceIndex;
      Break;
    end;
  end;
end;

function TResourceMap.ResToPosition(value: Integer): Integer;
var
  i: integer;
begin
  Result := -1;

  for i := 1 to Count do
  begin
    if Items[i - 1].ResourceIndex = Value then
    begin
      Result := Items[i - 1].PositionIndex;
      Break;
    end;
  end;

end;

procedure TResourceMap.SetItem(Index: Integer;
  const Value: TResourceMapItem);
begin
  inherited Items[Index] := Value;
end;

{ TResourceMapItem }

constructor TResourceMapItem.Create(Collection: TCollection);
begin
  inherited;
  FResourceIndex := Collection.Count - 1;
  FPositionIndex := Collection.Count - 1;
end;

{ TDateCollection }

function TDateCollection.Add: TDateItem;
begin
  Result := TDateItem(inherited Add);
end;

constructor TDateCollection.Create;
begin
  inherited Create(TDateItem);
end;

function TDateCollection.DatePos(ADate: TDateTime): Integer;
var
  i: Integer;
begin
  Result := -1;
  for i := 1 to Count do
  begin
    if int(ADate) = int(Items[i - 1].Date) then
    begin
      Result := i - 1;
      Break;
    end;
  end;
end;

function TDateCollection.GetItem(Index: Integer): TDateItem;
begin
  Result := TDateItem(inherited Items[Index]);
end;

function TDateCollection.HasDate(ADate: TDateTime): Boolean;
var
  i: Integer;
begin
  Result := False;
  for i := 1 to Count do
  begin
    if int(ADate) = int(Items[i - 1].Date) then
    begin
      Result := True;
      Break;
    end;
  end;
end;

function TDateCollection.Insert(index: Integer): TDateItem;
begin
  Result := TDateItem(inherited Insert(Index));
end;

function TDateCollection.PosDate(Pos: Integer): TDateTime;
begin
  Result := Items[Pos].Date;
end;

procedure TDateCollection.SetItem(Index: Integer; const Value: TDateItem);
begin
  inherited Items[Index] := Value;
end;

initialization
  {$IFDEF TMSCODESITE}
  CodeSite.Clear;
  {$ENDIF}
end.
