{***************************************************************************}
{ TMS IntraWeb Planner Components                                           }
{ for Delphi & C++Builder                                                   }
{                                                                           }
{ written by TMS Software                                                   }
{            Copyright  2002 - 2017                                        }
{            Email : info@tmssoftware.com                                   }
{            Web : http://www.tmssoftware.com                               }
{                                                                           }
{ The source code is given as is. The author is not responsible             }
{ for any possible damage done due to the use of this code.                 }
{ The component can be freely used in any application. The complete         }
{ source code remains property of the author and may not be distributed,    }
{ published, given or sold in any form as such. No parts of the source      }
{ code can be included in any other component or application without        }
{ written authorization of the author.                                      }
{***************************************************************************}

unit IWWebPlanner;

interface

{----------------------------------------------}
{  I M P O R T A N T   N O T I C E             }
{                                              }
{ If there is a compiler error for a missing   }
{ file TMSDEFS.INC, please consult the install }
{ notes to use the appropriate file.           }
{----------------------------------------------}

{$I TMSDEFS.INC}


{$R IWWEBPLAN.RES}

uses
  Windows, Classes, Graphics, Menus, SysUtils, IWFileReference,
  IWControl, IWWebPlanImg, IWHTMLTag, IWFont, Dialogs, IWTypes, IWGlobal,
  IWColor, IWRenderContext, IWXMLTag, IWApplication, TypInfo

  {$IFDEF TMSIW121}
  //IWBrowserBase,
  , IW.Browser.Browser, IW.Browser.InternetExplorer, IW.Browser.Chrome,
  IW.Browser.Safari, IW.Browser.Firefox, IW.Browser.Android,
  IW.Browser.SafariMobile, IW.Browser.Other, IW.Browser.SearchEngine,
  IW.Browser.Webkit
  {$ENDIF}

  {$IFDEF TMSIW14}
  , IW.CacheStream
  {$ENDIF}

  {$IFDEF FREEWARE}
  {$IFDEF TMSIW51}
  , IWKlooch
  {$ELSE}
  {$IFNDEF DELPHI7_LVL}
  , IWKlooch
  {$ENDIF}
  {$ENDIF}
  {$ENDIF}
  ;
const
  {$IFDEF FREEWARE}
  FWNOTICE = 'TMS IntraWeb Planner Components evaluation';
  {$ENDIF}
  MININDAY = 1440;

  MAJ_VER = 3; // Major version nr.
  MIN_VER = 2; // Minor version nr.
  REL_VER = 5; // Release nr.
  BLD_VER = 0; // Build nr.
  DATE_VER = 'April, 2017'; // Month version

  // version history
  // 2.3.0.1 : Fixed : issue with setting Items.Selected
  // 2.3.1.0 : Improved : use of WebPlanner on IntraWeb regions
  // 2.5.0.0 : New : IntraWeb XI support
  //           New : Async Events
  // 2.6.0.0 : New : RAD Studio XE2 support
  // 2.6.5.0 : New : OnAsyncItemMoveEx event
  //	         New public property: RelativeItemMove
  //           New public function: CellToAbsTime(Index)
  //           Improved : Item moving
  // 2.7.0.0 : New : IntraWeb 12.1 support
  // 2.7.0.1 : Fixed : issue with removing floating PlannerItem when Allow is set to false in OnAsyncItemMoveEx event
  //           Fixed : issue with moving a PlannerItem when the Planner is inside an IWRegion and RelativeItemMove is true
  // 2.7.0.2 : Fixed : issue with resizing from the start time of a PlannerItem when the Planner is inside an IWRegion
  // 2.7.0.3 : Improved : Header caption handling / resource name handling of TTIWDBCustomPeriodSource has been made
	//           consistent with TTIWDBDaySource
	//           Fixed : issue with rendering items in non-IE browsers
  // 2.8.0.0 : New : RAD Studio XE3 support
  //           New : IntraWeb 14 support
  // 2.9.0.0 : New : Support for Delphi XE4 & C++Builder XE4
  //           Fixed : using a background image now also works when SideBar.Position is set to spTop
  // 2.9.1.0 : New : AutoCreateOnSelect property
  //           New : Support for Internet Explorer 10
  //           New : Gradient colors support for Chrome / FireFox / Safari
  // 3.0.0.0 : New : Support for Delphi XE5 & C++Builder XE5
  // 3.0.1.0 : New : Support for Internet Explorer 11
  //           Improved : HTML5 Doctype compatibility
  // 3.1.0.0 : New : Support for Delphi XE6 & C++Builder XE6
  // 3.1.1.0 : New : Support for non-default IW 14 versions in Delphi XE6 & C++Builder XE6
  // 3.2.0.0 : New : Support for Delphi XE7 & C++Builder XE7
  // 3.2.0.1 : Fixed : Issue with cell selection in FireFox
  // 3.2.0.2 : Fixed : Issue with using bold and italic font at the same time
  // 3.2.1.0 : New : Support for Delphi XE8 & C++Builder XE8
  // 3.2.2.0 : New : Support for Delphi 10 Seattle & C++Builder 10 Seattle
  // 3.2.4.0 : New : Support for Delphi 10.1 Berlin & C++Builder 10.1 Berlin
  // 3.2.5.0 : New : Support for Delphi 10.2 Tokyo & C++Builder 10.2 Tokyo
  //           Fixed : Issue with updating SelItemBegin, SelItemEnd, SelItemPos when selecting a single cell

type
  {$IFDEF TMSIW11}
  TIWComponent40Context = TIWCompContext;
  TIWBaseHTMLComponentContext = TIWCompContext;
  TIWBaseComponentContext = TIWCompContext;
  {$ENDIF}

  {$IFNDEF TMSIW11}
  TIWBaseComponentContext = TIWBaseHTMLComponentContext;
  {$ENDIF}

  TTIWCustomWebPlanner = class;
  TPlannerItem = class;

  TIWItemEvent = procedure(Sender: TObject; Item: TPlannerItem) of object;

  TIWItemCreateClick = procedure (Sender: TObject; FromBegin, ToEnd, Position: Integer) of object;
  TIWHeaderClick = procedure (Sender: TObject; Position : Integer) of object;
  TIWItemMoveClick = procedure(Sender: TObject; Item: TPlannerItem; FromBegin, FromEnd, FromPos, ToBegin, ToEnd, ToPos: Integer) of object;
  TIWItemSizeClick = procedure(Sender: TObject; Item: TPlannerItem; Position, FromBegin, FromEnd, ToBegin, ToEnd: Integer) of object;

  TItemMoveEvent = procedure(Sender: TObject; Item: TPlannerItem;
    FromBegin, FromEnd, FromPos, ToBegin, ToEnd, ToPos: Integer) of object;

  TPlannerGetSideBarLines = procedure (Sender: TObject;
  Index, Position: Integer; var HourString, MinuteString, AmPmString: string) of object;

  TPlannerPlanTimeToStrings = procedure (Sender: TObject; MinutesValue: Cardinal;
  var HoursString, MinutesString, AmPmString: string) of object;

  TIWDBTemplateEvent = procedure(Sender : TObject; APlannerItem : TPlannerItem; Field : string; var DBField : string) of object;

  TPlannerItemChangeClick = procedure (Sender: TObject; Item: TPlannerItem) of object;
  TPlannerItemClick = procedure (Sender: TObject; ItemIndex : Integer; Item: TPlannerItem; var ShowEditor : Boolean) of object;
  TPlannerDeleteClick = procedure (Sender: TObject; Item: TPlannerItem) of object;
  TPlannerItemCaptionChange = procedure (Sender: TObject; Item: TPlannerItem) of object;

  TIWGradientDirection = (gdHorizontal, gdVertical);
  TPlannerEditType = (edInplace, edPopup);

  TIWSelectionMode = (smMultiSelect, smSingleSelect);
  TIWPlannerDblClick = procedure(Sender : TObject; Index, Position : Integer) of object;

  TSideBarOrientation = (soHorizontal, soVertical);

  TCaptionHint = (chCaptionNotes, chCaption, chNone);

  THeaderPosition = (hpTop, hpBottom, hpBoth);

  THourType = (ht24hrs, ht12hrs, htAMPM0, htAMPM1, htAMPMOnce);

  TIWImageType = (imgPNG, imgJPEG, imgGIF, imgBMP);

  TByteSet = set of Byte;

  TDateTimeObject = class(TObject)
  private
    FDT: TDateTime;
  public
    property DT: TDateTime read FDT write FDT;
  end;

  THints = class(TPersistent)
  private
    FOwner: TComponent;
    FHintCancelItem: string;
    FHintConfirmDialog: string;
    FHintDeleteItem: String;
    FHintEditItem: string;
    FHintUpdateItem: string;
    FCaptionHint : TCaptionHint;
    procedure SetCaptionHint(const Value : TCaptionHint);
    procedure SetHintCancelItem(const Value: String);
    procedure SetHintConfirmDialog(const Value: String);
    procedure SetHintDeleteItem(const Value: String);
    procedure SetHintEditItem(const Value: String);
    procedure SetHintUpdateItem(const Value: String);
  public
    constructor Create(AOwner: TComponent);
    destructor Destroy; override;
  published
    property CaptionHint : TCaptionHint read FCaptionHint write SetCaptionHint;
    property HintCancelItem: string read FHintCancelItem write SetHintCancelItem;
    property HintConfirmDialog: string read FHintConfirmDialog write SetHintConfirmDialog;
    property HintDeleteItem: string read FHintDeleteItem write SetHintDeleteItem;
    property HintEditItem: string read FHintEditItem write SetHintEditItem;
    property HintUpdateItem: string read FHintUpdateItem write SetHintUpdateItem;
  end;

  TChangeCaption = class(TPersistent)
  private
    FOwner : TTIWCustomWebPlanner;
    FShowDialogWindow : boolean;
    FShowPromptWindow : boolean;
    FTextDialogWindow : string;
    FTextPromptWindow : string;
    FEnterCaptionAfterItemCreate : Boolean;
    procedure SetShowDialogWindow(const Value : Boolean);
    procedure SetShowPromptWindow(const Value : Boolean);
    procedure SetTextDialogWindow(const Value : string);
    procedure SetTextPromptWindow(const Value : string);
    procedure SetEnterCaptionAfterItemCreate(const Value : Boolean);
  public
    constructor Create(AOwner : TTIWCustomWebPlanner);
    destructor Destroy; override;
  published
    property ShowDialogWindow : boolean read FShowDialogWindow write SetShowDialogWindow;
    property ShowPromptWindow : boolean read FShowPromptWindow write SetShowPromptWindow;
    property TextDialogWindow : string read FTextDialogWindow write SetTextDialogWindow;
    property TextPromptWindow : string read FTextPromptWindow write SetTextPromptWindow;
    property EnterCaptionAfterItemCreate : boolean read FEnterCaptionAfterItemCreate write SetEnterCaptionAfterItemCreate;
  end;

  TPopupEdit = class(TPersistent)
  private
    FOwner : TTIWCustomWebPlanner;
    FSubjectCaption : string;
    FNotesCaption : string;
    FPopupBorderColor : TIWColor;
    FPopupBackColor : TIWColor;
    FPopupHeight : integer;
    FPopupWidth : integer;
    FBackgroundImage : TIPicture;
    FFont : TIWFont;
    procedure SetSubjectCaption(const Value : string);
    procedure SetNotesCaption(const Value : string);
    procedure SetPopupBorderColor(const Value : TIWColor);
    procedure SetPopupBackColor(const Value : TIWColor);
    procedure SetPopupHeight(const Value : integer);
    procedure SetPopupWidth(const Value :integer);
    procedure SetBackgroundImage(const Value : TIPicture);
    procedure SetFont(const Value : TIWFont);
  public
    constructor Create(AOwner: TTIWCustomWebPlanner);
    destructor Destroy; override;
  published
    property SubjectCaption : string read FSubjectCaption write SetSubjectCaption;
    property NotesCaption : string read FNotesCaption write SetNotesCaption;
    property BorderColor : TIWColor read FPopupBorderColor write SetPopupBorderColor;
    property BackColor : TIWColor read FPopupBackColor write SetPopupBackColor;
    property BackgroundImage : TIPicture read FBackgroundImage write SetBackgroundImage;
    property Font : TIWFont read FFont write SetFont;
    property PopupHeight : integer read FPopupHeight write SetPopupHeight;
    property PopupWidth : integer read FPopupWidth write SetPopupWidth;
  end;

  TGlyphs = class(TPersistent)
  private
    FOwner: TComponent;
    FCancelItem: TBitmap;
    FDeleteItem: TBitmap;
    FEditItem: TBitmap;
    FUpdateItem: TBitmap;
    FCrossingStart : TBitmap;
    FCrossingEnd : TBitmap;
    FCancelItemFile : string;
    FCrossingEndFile : string;
    FCrossingStartFile : string;
    FDeleteItemFile : string;
    FEditItemFile : string;
    FUpdateItemFile : string;
    procedure SetCancelItemFile(const Value : string);
    procedure SetCrossingEndFile(const Value : string);
    procedure SetCrossingStartFile(const Value : string);
    procedure SetDeleteItemFile(const Value : string);
    procedure SetEditItemFile(const Value : string);
    procedure SetUpdateItemFile(const Value : string);
    procedure SetCancelItem(const Value: TBitmap);
    procedure SetDeleteItem(const Value: TBitmap);
    procedure SetEditItem(const Value: TBitmap);
    procedure SetUpdateItem(const Value: TBitmap);
    procedure SetCrossingStart(const Value : TBitmap);
    procedure SetCrossingEnd(const Value : TBitmap);
  public
    constructor Create(AOwner: TComponent);
    destructor Destroy; override;
  published
    property CancelItem: TBitmap read FCancelItem write SetCancelItem;
    property CancelItemFileName : string read FCancelItemFile write SetCancelItemFile;
    property CrossingStart : TBitmap read FCrossingStart write SetCrossingStart;
    property CrossingStartFileName : string read FCrossingStartFile write SetCrossingStartFile;
    property CrossingEnd : TBitmap read FCrossingEnd write SetCrossingEnd;
    property CrossingEndFileName : string read FCrossingEndFile write SetCrossingEndFile;
    property DeleteItem: TBitmap read FDeleteItem write SetDeleteItem;
    property DeleteItemFileName : string read FDeleteItemFile write SetDeleteItemFile;
    property EditItem: TBitmap read FEditItem write SetEditItem;
    property EditItemFileName : string read FEditItemFile write SetEditItemFile;
    property UpdateItem: TBitmap read FUpdateItem write SetUpdateItem;
    property UpdateItemFileName : string read FUpdateItemFile write SetUpdateItemFile;
  end;

  TBands = class(TPersistent)
  private
    FOwner: TTIWCustomWebPlanner;
    FShow: Boolean;
    FActiveSecondary: TIWColor;
    FNonActivePrimary: TIWColor;
    FNonActiveSecondary: TIWColor;
    FActivePrimary: TIWColor;
    procedure SetActivePrimary(const Value: TIWColor);
    procedure SetActiveSecondary(const Value: TIWColor);
    procedure SetNonActivePrimary(const Value: TIWColor);
    procedure SetNonActiveSecondary(const Value: TIWColor);
    procedure SetShow(const Value: Boolean);
  public
    constructor Create(AOwner:TTIWCustomWebPlanner);
  published
    property Show: Boolean read FShow write SetShow default False;
    property ActivePrimary: TIWColor read FActivePrimary write SetActivePrimary;
    property ActiveSecondary: TIWColor read FActiveSecondary write SetActiveSecondary;
    property NonActivePrimary: TIWColor read FNonActivePrimary write SetNonActivePrimary;
    property NonActiveSecondary: TIWColor read FNonActiveSecondary write SetNonActiveSecondary;
  end;

  TPositionProp = class(TCollectionItem)
  private
    FActiveStart: Integer;
    FActiveEnd: Integer;
    FColorNonActive: TIWColor;
    FColorActive: TIWColor;
    FMinSelection: Integer;
    FMaxSelection: Integer;
    FColorNoSelect: TIWColor;
    FUse: Boolean;
    procedure SetActiveEnd(const Value: Integer);
    procedure SetActiveStart(const Value: Integer);
    procedure SetColorActive(const Value: TIWColor);
    procedure SetColorNonActive(const Value: TIWColor);
    procedure SetMaxSelection(const Value: Integer);
    procedure SetMinSelection(const Value: Integer);
    procedure SetColorNoSelect(const Value: TIWColor);
    procedure SetUse(const Value: Boolean);
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
  published
    property ActiveStart: Integer read FActiveStart write SetActiveStart;
    property ActiveEnd: Integer read FActiveEnd write SetActiveEnd;
    property ColorActive: TIWColor read FColorActive write SetColorActive;
    property ColorNonActive: TIWColor read FColorNonActive write SetColorNonActive;
    property MinSelection: Integer read FMinSelection write SetMinSelection;
    property MaxSelection: Integer read FMaxSelection write SetMaxSelection;
    property ColorNoSelect: TIWColor read FColorNoSelect write SetColorNoSelect;
    property Use: Boolean read FUse write SetUse;
  end;

  TPositionProps = class(TCollection)
  private
    FOwner: TTIWCustomWebPlanner;
    function GetItem(Index: Integer): TPositionProp;
    procedure SetItem(Index: Integer; const Value: TPositionProp);
  protected
  {$IFNDEF VER90}
    function GetOwner: TPersistent; override;
  {$ENDIF}
  public
    constructor Create(AOwner: TTIWCustomWebPlanner);
    function Add: TPositionProp;
    function Insert(Index: Integer): TPositionProp;
    property Items[Index: Integer]: TPositionProp read GetItem write SetItem; default;
  end;

  TPlannerIntList = class(TList)
  private
    FOnChange: TNotifyEvent;
    FPlannerItem: TPlannerItem;
    procedure SetInteger(Index: Integer; Value: Integer);
    function GetInteger(Index: Integer): Integer;
  public
    constructor Create(Value: TPlannerItem);
    property Items[Index: Integer]: Integer read GetInteger write SetInteger;
    default;
    procedure Add(Value: Integer);
    procedure Delete(Index: Integer);
  public
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

  TDateTimeList = class(TList)
  private
    function GetDT(index: Integer): TDateTime;
    procedure SetDT(index: Integer; const Value: TDateTime);
  public
    property Items[index: Integer]: TDateTime read GetDT write SetDT; default;
    procedure Add(Value: TDateTime);
    procedure Insert(Index: Integer; Value: TDateTime);
    procedure Delete(Index: Integer);
    procedure Clear; override;
  end;

  TCaptionType = (ctNone, ctText, ctTime, ctTimeText);

  TItemRelationShip = (irParent,irChild);

  TFindTextParameter = (fnMatchCase,fnMatchFull,fnMatchRegular,fnMatchStart,
    fnAutoGoto,fnIgnoreHTMLTags,fnBackward,fnCaptionText,fnText);

  TFindTextParams = set of TFindTextParameter;

  TSideBarPosition = (spLeft, spRight, spTop, spLeftRight);

  TPlannerSideBar = class(TPersistent)
  private
    FOwner: TTIWCustomWebPlanner;
    FAlignment: TAlignment;
    FShowOccupied: Boolean;
    FGradientDirection: TIWGradientDirection;
    FOccupiedColor: TIWColor;
    FOccupiedColorTo: TIWColor;
    FPosition: TSideBarPosition;
    FOrientation: TSideBarOrientation;
    Fwidth: Integer;
    FBackground: TIWColor;
    FBackgroundTo: TIWColor;
    FShowDayName: Boolean;
    FDateTimeFormat: string;
    FSideBarFont: TIWFont;
    FOccupiedFont: TIWFont;
    FShowHint: boolean;
    FVisible: Boolean;
    function GetOrientation: TSideBarOrientation;
    procedure SetPosition(const Value: TSideBarPosition);
    procedure SetShowDayName(const Value: Boolean);
    procedure SetGradientDirection(const Value: TIWGradientDirection);
    procedure SetDateTimeFormat(const Value: String);
    procedure SetSideBarFont(const Value: TIWFont);
    procedure SetOccupiedFont(const Value: TIWFont);
    procedure SetAlignment(const Value: TAlignment);
    procedure SetShowOccupied(const Value: Boolean);
    procedure SetBackground(const Value: TIWColor);
    procedure SetBackgroundTo(const Value: TIWColor);
    procedure SetOccupiedColor(const Value: TIWColor);
    procedure SetOccupiedColorTo(const Value: TIWColor);
    procedure SetShowHint(const Value: Boolean);
    procedure SetVisible(const Value: Boolean);
    procedure SetWidth(const Value: Integer);
  public
    constructor Create(AOwner: TTIWCustomWebPlanner);
    destructor Destroy; override;
    property Orientation: TSideBarOrientation read GetOrientation write FOrientation;
    function GetUsedWidth: Integer;
  published
    property Alignment: TAlignment read FAlignment write SetAlignment;
    property Background: TIWColor read FBackground write SetBackground;
    property BackgroundTo: TIWColor read FBackgroundTo write SetBackgroundTo;
    property DateTimeFormat: String read FDateTimeFormat write SetDateTimeFormat;
    property Font: TIWFont read FSideBarFont write SetSideBarFont;
    property GradientDirection: TIWGradientDirection read FGradientDirection write SetGradientDirection;
    property OccupiedColor: TIWColor read FOccupiedColor write SetOccupiedColor;
    property OccupiedColorTo: TIWColor read FOccupiedColorTo write SetOccupiedColorTo;
    property OccupiedFont: TIWFont read FOccupiedFont write SetOccupiedFont;
    property ShowDayName: Boolean read FShowDayName write SetShowDayName;
    property ShowHint : Boolean read FShowHint write SetShowHint;
    property ShowOccupied: Boolean read FShowOccupied write SetShowOccupied;
    property Position: TSideBarPosition read FPosition write SetPosition;
    property Visible: Boolean read FVisible write SetVisible;
    property Width: Integer read FWidth write SetWidth;
  end;

  TPlannerDisplay = class(TPersistent)
  private
    FOwner: TTIWCustomWebPlanner;
    FDisplayStart: Integer;
    FDisplayScale: Integer;
    FDisplayEnd: Integer;
    FDisplayUnit: Integer;
    FActiveEnd: Integer;
    FActiveStart: Integer;
    FColorNonActive: TIWColor;
    FColorActive: TIWColor;
    FActiveStartPrecis: Integer;
    FActiveEndPrecis: Integer;
    FDisplayStartPrecis: Integer;
    FDisplayEndPrecis: Integer;
    FScaleToFit: Boolean;
    FOldScale: Integer;
    FDisplayOffset: Integer;
    FUpdateCount: Integer;
    FDisplayText: Integer;
    FCurrentPosFrom: Integer;
    FCurrentPosTo: Integer;
    procedure SetDisplayStart(const Value: Integer);
    procedure SetDisplayEnd(const Value: Integer);
    procedure SetDisplayScale(const Value: Integer);
    procedure SetDisplayUnit(const Value: Integer);
    procedure SetDisplayOffset(const Value: Integer);
    procedure UpdatePlanner;
    procedure SetActiveEnd(const Value: Integer);
    procedure SetActiveStart(const Value: Integer);
    procedure SetColorActive(const Value: TIWColor);
    procedure SetColorNonActive(const Value: TIWColor);
    procedure SetScaleToFit(const Value: Boolean);
    procedure SetDisplayText(const Value: Integer);
    procedure SetCurrentPosFrom(const Value: Integer);
    procedure SetCurrentPosTo(const Value: Integer);
  protected
    procedure InitPrecis;
    procedure AutoScale;
  public
    constructor Create(AOwner: TTIWCustomWebPlanner);
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    procedure BeginUpdate;
    procedure EndUpdate;
    function GetDisplay : integer;
  published
    property ActiveStart: Integer read FActiveStart write SetActiveStart;
    property ActiveEnd: Integer read FActiveEnd write SetActiveEnd;
    property CurrentPosFrom: Integer read FCurrentPosFrom write SetCurrentPosFrom;
    property CurrentPosTo: Integer read FCurrentPosTo write SetCurrentPosTo;
    property DisplayStart: Integer read FDisplayStart write SetDisplayStart;
    property DisplayEnd: Integer read FDisplayEnd write SetDisplayEnd;
    property DisplayOffset: Integer read FDisplayOffset write SetDisplayOffset;
    property DisplayScale: Integer read FDisplayScale write SetDisplayScale;
    property DisplayUnit: Integer read FDisplayUnit write SetDisplayUnit;
    property DisplayText: Integer read FDisplayText write SetDisplayText;
    property ColorActive: TIWColor read FColorActive write SetColorActive;
    property ColorNonActive: TIWColor read FColorNonActive write SetColorNonActive;
    property ScaleToFit: Boolean read FScaleToFit write SetScaleToFit;
  end;

  TWeekDays = class(TPersistent)
  private
    FSat: Boolean;
    FSun: Boolean;
    FMon: Boolean;
    FTue: Boolean;
    FWed: Boolean;
    FThu: Boolean;
    FFri: Boolean;
    FChanged: TNotifyEvent;
    procedure SetSat(const Value: Boolean);
    procedure SetSun(const Value: Boolean);
    procedure SetMon(const Value: Boolean);
    procedure SetTue(const Value: Boolean);
    procedure SetWed(const Value: Boolean);
    procedure SetThu(const Value: Boolean);
    procedure SetFri(const Value: Boolean);
    procedure Changed;
  public
    constructor Create;
  published
    property Sat: Boolean read FSat write SetSat;
    property Sun: Boolean read FSun write SetSun;
    property Mon: Boolean read FMon write SetMon;
    property Tue: Boolean read FTue write SetTue;
    property Wed: Boolean read FWed write SetWed;
    property Thu: Boolean read FThu write SetThu;
    property Fri: Boolean read FFri write SetFri;
    property OnChanged: TNotifyEvent read FChanged write FChanged;
  end;

  TPlannerType = (plDay, plWeek, plMonth, plDayPeriod, plHalfDayPeriod, plMultiMonth,
    plCustom, plTimeLine, plCustomList);

  TPlannerMode = class(TPersistent)
  private
    FClip: Boolean;
    FOwner: TTIWCustomWebPlanner;
    FPlannerType: TPlannerType;
    FWeekStart: Integer;
    FYear: Integer;
    FMonth: Integer;
    FDateTimeFormat: string;
    FPeriodStartDay: Integer;
    FPeriodStartMonth: Integer;
    FPeriodStartYear: Integer;
    FPeriodEndDay: Integer;
    FPeriodEndMonth: Integer;
    FPeriodEndYear: Integer;
    FUpdateCount: Integer;
    FTimeLineStart: TDateTime;
    procedure SetMonth(const Value: Integer);
    procedure SetPlannerType(const Value: TPlannerType);
    procedure SetWeekStart(const Value: Integer);
    procedure SetYear(const Value: Integer);
    procedure SetDateTimeFormat(const Value: string);
    procedure SetPeriodStartDay(const Value: Integer);
    procedure SetPeriodStartMonth(const Value: Integer);
    procedure SetPeriodStartYear(const Value: Integer);
    procedure SetPeriodEndDay(const Value: Integer);
    procedure SetPeriodEndMonth(const Value: Integer);
    procedure SetPeriodEndYear(const Value: Integer);
    function GetPeriodEndDate: TDateTime;
    function GetPeriodStartDate: TDateTime;
    function GetStartOfMonth: TDateTime;
    procedure UpdatePeriod;
    procedure SetPeriodEndDate(const Value: TDateTime);
    procedure SetPeriodStartDate(const Value: TDateTime);
    procedure SetTimeLineStart(const Value: TDateTime);
  public
    constructor Create(AOwner: TTIWCustomWebPlanner);
    destructor Destroy; override;
    property PeriodStartDate: TDateTime read GetPeriodStartDate write SetPeriodStartDate;
    property PeriodEndDate: TDateTime read GetPeriodEndDate write SetPeriodEndDate;
    property StartOfMonth: TDateTime read GetStartOfMonth;
    procedure BeginUpdate;
    procedure EndUpdate;
    procedure Assign(Source: TPersistent); override;
  published
    property Clip: Boolean read FClip write FClip;
    property DateTimeFormat: string read FDateTimeFormat write SetDateTimeFormat;
    property Month: Integer read FMonth write SetMonth;
    property PeriodStartDay: Integer read FPeriodStartDay write SetPeriodStartDay;
    property PeriodStartMonth: Integer read FPeriodStartMonth write SetPeriodStartMonth;
    property PeriodStartYear: Integer read FPeriodStartYear write SetPeriodStartYear;
    property PeriodEndDay: Integer read FPeriodEndDay write SetPeriodEndDay;
    property PeriodEndMonth: Integer read FPeriodEndMonth write SetPeriodEndMonth;
    property PeriodEndYear: Integer read FPeriodEndYear write SetPeriodEndYear;
    property PlannerType: TPlannerType read FPlannerType write SetPlannerType;
    property TimeLineStart: TDateTime read FTimeLineStart write SetTimeLineStart;
    property WeekStart: Integer read FWeekStart write SetWeekStart;
    property Year: Integer read FYear write SetYear;
  end;

  TPlannerHeader = class(TPersistent)
  private
    FGroupCaptions: TStringList;
    FOwner: TTIWCustomWebPlanner;
    FCaptions: TStringList;
    FGradientDirection: TIWGradientDirection;
    FVisible: Boolean;
    FCaptionHeight: Integer;
    FColor: TIWColor;
    FBackgroundTo: TIWColor;
    FHeaderFont: TIWFont;
    FAlignment: TAlignment;
    FLineColor: TIWColor;
    FHeaderItemWidth : Integer;
    FPosition : THeaderPosition;
    procedure SetHeaderItemWidth(const Value : Integer);
    procedure SetAlignment(const Value: TAlignment);
    procedure SetCaptions(const Value: TStringList);
    procedure SetGroupCaptions(const Value: TStringList);
    procedure SetGradientDirection(const Value: TIWGradientDirection);
    procedure SetCaptionHeight(const Value: Integer);
    procedure SetVisible(const Value: Boolean);
    procedure SetColor(const Value: TIWColor);
    procedure SetHeaderFont(const Value: TIWFont);
    procedure SetBackgroundTo(const Value: TIWColor);
    procedure SetLineColor(const Value: TIWColor);
    procedure SetHeaderPosition(const Value : THeaderPosition);
  public
    constructor Create(AOwner: TTIWCustomWebPlanner);
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
  published
    property Alignment: TAlignment read FAlignment write SetAlignment;
    property Background: TIWColor read FColor write SetColor;
    property BackgroundTo: TIWColor read FBackgroundTo write SetBackgroundTo;
    property Captions: TStringList read FCaptions write SetCaptions;
    property CaptionHeight: Integer read FCaptionHeight write SetCaptionHeight;
    property Font: TIWFont read FHeaderFont write SetHeaderFont;
    property GradientDirection: TIWGradientDirection read FGradientDirection write SetGradientDirection;
    property GroupCaptions: TStringList read FGroupCaptions write SetGroupCaptions;
    property HeaderItemWidth : integer read FHeaderItemWidth write SetHeaderItemWidth;
    property LineColor: TIWColor read FLineColor write SetLineColor;
    property Position : THeaderPosition read FPosition write SetHeaderPosition;
    property Visible: Boolean read FVisible write SetVisible;
  end;
  TBackgroundDisplay = (bdTile,bdFixed);

  TPlannerItem = class(TCollectionItem)
  private
    FAllowDelete : Boolean;
    FTemplate: TStringList;
    FTag: Integer;
    FId: Integer;
    FObject: TObject;
    FAlignment: TAlignment;
    FAttachement: string;
    FEditMode: Boolean;
    FCaptionType: TCaptionType;
    FCaptionText: string;
    FCaptionGradientDirection : TIWGradientDirection;
    FText: TStringList;
    FItemBegin: Integer;
    FItemEnd: Integer;
    FItemBeginPrecis: Integer;
    FItemEndPrecis: Integer;
    FItemFullBegin: Integer;
    FImageID: Integer;
    FInHeader: Boolean;
    FColor: TIWColor;
    FFixedPos: Boolean;
    FFixedSize: Boolean;
    FFixedPosition: Boolean;
    FReadOnly: Boolean;
    FPlanner: TTIWCustomWebPlanner;
    FItemPos: Integer;
    FConflicts: Integer;
    FConflictPos: Integer;
    FVisible: Boolean;
    FFocus: Boolean;
    FName: string;
    FOnEditModal: TNotifyEvent;
    FLayer: Integer;
//    FShowCaption: Boolean;
    FTrackColor: TIWColor;
    FHint: string;
    FImageIndexList: TPlannerIntList;
    FItemStartTime: TDateTime;
    FItemEndTime: TDateTime;
    FItemRealStartTime: TDateTime;
    FItemRealEndTime: TDateTime;
    FAllowOverlap: Boolean;
    FBackGround: Boolean;
    FFont: TIWFont;
    FLinkedItem: TPlannerItem;
    FIsCurrent: Boolean;
    FCaptionAlign: TAlignment;
    FCaptionBkg: TIWColor;
    FCaptionBkgTo: TIWColor;
    FCaptionFont: TIWFont;
    FCaptionHeight: Integer;
    FSelected: Boolean;
    FSelectColor: TIWColor;
    FOwnsItemObject: Boolean;
    FRepainted: Boolean;
    FBeginExt: Integer;
    FEndExt: Integer;
    FPopupMenu: TPopupMenu;
    FShowSelection: Boolean;
    FDBTag: Integer;
    FDBKey: string;
    FSynched: Boolean;
    FURL: string;
    FEndOffset: Integer;
    FBeginOffset: Integer;
    FChanged: Boolean;
    FDoExport: Boolean;
    FRealTime: Boolean;
    FFlashOn: Boolean;
    FFlashing: Boolean;
    FUniformBkg: Boolean;
    FParentIndex: Integer;
    FClipped: Boolean;
    FRelationShip: TItemRelationShip;
    FNonDBItem: Boolean;
    FTransparent: Boolean;
    FSelectFontColor: TIWColor;
    FPopupEdit: Boolean;
    procedure SetTemplate(const Value : TStringList);
    procedure SetColor(const Value: TIWColor);
    procedure SetTrackColor(const Value: TIWColor);
    procedure SetLayer(const Value: Integer);
    procedure SetItemEnd(const Value: Integer);
    procedure SetItemBegin(const Value: Integer);
    procedure SetText(const Value: TStringList);
    procedure SetAlignment(const Value: TAlignment);
    procedure SetAllowOverlap(const Value: Boolean);
    procedure SetAllowDelete(const Value : Boolean);
    procedure SetCaptionType(const Value: TCaptionType);
    procedure SetCaptionText(const Value: string);
//    procedure SetShowCaption(const Value: Boolean);
    procedure SetImageID(const Value: Integer);
    procedure SetItemPos(const Value: Integer);
    procedure SetVisible(const Value: Boolean);
    function GetVisible: Boolean;
    procedure SetFocus(const Value: Boolean);
    procedure SetFont(const Value: TIWFont);
    procedure SetBackground(const Value: Boolean);
    procedure ReOrganize;
    procedure FontChange(Sender: TObject);
    procedure ImageChange(Sender: TObject);
    procedure TextChange(Sender: TObject);
    procedure SetItemEndTime(const Value: TDateTime);
    procedure SetItemStartTime(const Value: TDateTime);
    function GetItemEndTime: TDateTime;
    function GetItemStartTime: TDateTime;
    function GetItemEndTimeStr: string;
    function GetItemStartTimeStr: string;
    function GetItemSpanTimeStr: string;
    procedure SetIsCurrent(const Value: Boolean);
    procedure SetItemRealEndTime(const Value: TDateTime);
    procedure SetItemRealStartTime(const Value: TDateTime);
    procedure SetCaptionAlign(const Value: TAlignment);
    procedure SetCaptionBkg(const Value: TIWColor);
    procedure SetCaptionBkgTo(const Value: TIWColor);
    procedure SetCaptionFont(const Value: TIWFont);
    procedure SetSelectColor(const Value: TIWColor);
    procedure SetSelected(const Value: Boolean);
    procedure SetObject(const Value: TObject);
    procedure SetInHeader(const Value: Boolean);
    procedure SetItemBeginPrecis(const Value: Integer);
    procedure SetItemEndPrecis(const Value: Integer);
    procedure SetCaptionGradientDirection(const Value : TIWGradientDirection);
    procedure SetPopupMenu(const Value: TPopupMenu);
    procedure SetShowSelection(const Value: Boolean);
    procedure SetTimeTag;
    procedure GetTimeTag;
    function GetItemText: string;
    procedure SetAttachement(const Value: string);
    function GetItemRealEndTime: TDateTime;
    function GetItemRealStartTime: TDateTime;
    procedure SetURL(const Value: string);
    function GetStrippedItemText: string;
    procedure SetFlashing(const Value: Boolean);
    function GetParentItem: TPlannerItem;
    function GetCanEdit: Boolean;
    procedure SetSelectFontColor(const Value: TIWColor);
    procedure SetCaptionHeight(const Value: Integer);
  protected
{$IFNDEF VER90}
    function GetDisplayName: string; override;
{$ENDIF VER90}
    procedure UpdateWnd;
    procedure Repaint;
    procedure CalcConflictRect(var Rect: TRect; Width, Height: Integer;
      Position: Boolean);
    property IsCurrent: Boolean read FIsCurrent write SetIsCurrent;
    function GetCaptionHeight: Integer;
    function GetItemRect: TRect;
    function GetVisibleSpan: Integer;
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    procedure AssignEx(Source: TPersistent);
    function Planner: TTIWCustomWebPlanner; virtual;
    procedure Changed; virtual;
    procedure ScrollInView;
    procedure EnsureFullVisibility;
    procedure Edit;
    procedure PopupEdit;
    procedure MoveItem(NewBegin,NewEnd,NewPos: Integer; var NewOffset: Integer);
    procedure SizeItem(NewBegin,NewEnd: Integer);
    procedure DefOrganize;
    procedure Update;
    procedure ChangeCrossing;
    function GetCaptionTimeString: string;
    function GetCaptionString: string;
    property ItemObject: TObject read FObject write SetObject;
    property ID: Integer read FId write FId;
    property BeginOffset: Integer read FBeginOffset write FBeginOffset;
    property EndOffset: Integer read FEndOffset write FEndOffset;
    property Clipped: Boolean read FClipped;
    property Conflicts: Integer read FConflicts;
    property ConflictPos: Integer read FConflictPos;
    property Focus: Boolean read FFocus write SetFocus;
    property ImageIndexList: TPlannerIntList read FImageIndexList write FImageIndexList;
    property ItemBeginPrecis: Integer read FItemBeginPrecis write SetItemBeginPrecis;
    property ItemEndPrecis: Integer read FItemEndPrecis write SetItemEndPrecis;
    property ItemStartTime: TDateTime read GetItemStartTime write SetItemStartTime;
    property ItemEndTime: TDateTime read GetItemEndTime write SetItemEndTime;
    property ItemRealStartTime: TDateTime read GetItemRealStartTime write SetItemRealStartTime;
    property ItemRealEndTime: TDateTime read GetItemRealEndTime write SetItemRealEndTime;
    property ItemStartTimeStr: string read GetItemStartTimeStr;
    property ItemEndTimeStr: string read GetItemEndTimeStr;
    property ItemSpanTimeStr: string read GetItemSpanTimeStr;
    property ItemText: string read GetItemText;
    property StrippedItemText: string read GetStrippedItemText;
    property LinkedItem: TPlannerItem read FLinkedItem write FLinkedItem;
    property Owner: TTIWCustomWebPlanner read FPlanner;
    property Repainted: Boolean read FRepainted write FRepainted;
    property DBKey: string read FDBKey write FDBKey;
    property Synched: Boolean read FSynched write FSynched;
    property DoExport: Boolean read FDoExport write FDoExport;
    property RealTime: Boolean read FRealTime write FRealTime;
    property Flashing: Boolean read FFlashing write SetFlashing;
    property FlashOn: Boolean read FFlashOn write FFlashOn;
    property ParentItem: TPlannerItem read GetParentItem;
    property ParentIndex: Integer read FParentIndex write FParentIndex;
    property RelationShip: TItemRelationShip read FRelationShip write FRelationShip;
    property NotEditable: Boolean read GetCanEdit;
    property NonDBItem: Boolean read FNonDBItem write FNonDBItem;
    property Transparent: Boolean read FTransparent write FTransparent;
    property ItemRect: TRect read GetItemRect;
    property EditMode: Boolean read FEditMode write FEditMode;
  published
    property Alignment: TAlignment read FAlignment write SetAlignment;
    property AllowDelete : Boolean read FAllowDelete write SetAllowDelete;
    property AllowOverlap: Boolean read FAllowOverlap write SetAllowOverlap;
    property Attachement: string read FAttachement write SetAttachement;
    property Background: Boolean read FBackGround write SetBackground;
    property CaptionAlign: TAlignment read FCaptionAlign write SetCaptionAlign;
    property CaptionBkg: TIWColor read FCaptionBkg write SetCaptionBkg;
    property CaptionBkgTo: TIWColor read FCaptionBkgTo write SetCaptionBkgTo default clNone;
    property CaptionFont: TIWFont read FCaptionFont write SetCaptionFont;
    property CaptionGradientDirection : TIWGradientDirection read FCaptionGradientDirection write SetCaptionGradientDirection;
    property CaptionHeight: Integer read FCaptionHeight write SetCaptionHeight;
    property CaptionType: TCaptionType read FCaptionType write SetCaptionType;
    property CaptionText: string read FCaptionText write SetCaptionText;
    property Color: TIWColor read FColor write SetColor;
    property DBTag: Integer read FDBTag write FDBTag;
    property FixedPos: Boolean read FFixedPos write FFixedPos;
    property FixedPosition: Boolean read FFixedPosition write FFixedPosition;
    property FixedSize: Boolean read FFixedSize write FFixedSize;
    property Font: TIWFont read FFont write SetFont;
    property Hint: string read FHint write FHint;
    property ImageID: Integer read FImageID write SetImageID;
    property InHeader: Boolean read FInHeader write SetInHeader;
    property ItemBegin: Integer read FItemBegin write SetItemBegin;
    property ItemEnd: Integer read FItemEnd write SetItemEnd;
    property ItemPos: Integer read FItemPos write SetItemPos;
    property Layer: Integer read FLayer write SetLayer;
    property Name: string read FName write FName;
    property OnEditModal: TNotifyEvent read FOnEditModal write FOnEditModal; //##jpl
    property OwnsItemObject: Boolean read FOwnsItemObject write FOwnsItemObject;
    property PopupMenu: TPopupMenu read FPopupMenu write SetPopupMenu;
    property ReadOnly: Boolean read FReadOnly write FReadOnly;
    property SelectColor: TIWColor read FSelectColor write SetSelectColor;
    property SelectFontColor: TIWColor read FSelectFontColor write SetSelectFontColor;
    property Selected: Boolean read FSelected write SetSelected;
//    property ShowCaption: Boolean read FShowCaption write SetShowCaption;
    property ShowSelection: Boolean read FShowSelection write SetShowSelection default True;
    property Tag: Integer read FTag write FTag;
    property Template: TStringList read FTemplate write SetTemplate;
    property Text: TStringList read FText write SetText;
    property TrackColor: TIWColor read FTrackColor write SetTrackColor;
    property UniformBkg: Boolean read FUniformBkg write FUniformBkg default True;
    property URL: string read FURL write SetURL;
    property Visible: Boolean read GetVisible write SetVisible default True;
  end;

  TPlannerItems = class(TCollection)
  private
    FOwner: TTIWCustomWebPlanner;
    FSelected: TPlannerItem;
    FDBItem: TPlannerItem;
    FUpdateCount: Integer;
    FFindIndex: Integer;
    FChanging: Boolean;
    function GetItem(Index: Integer): TPlannerItem;
    procedure SetItem(Index: Integer; Value: TPlannerItem);
    function GetSelCount: Integer;
  protected
    function GetOwner: TPersistent; override;
    procedure ClearSelectedRepaints(ItemBegin,ItemPos: Integer);
    function NumItem(ItemBegin, ItemEnd, ItemPos: Integer): TPoint;
    function NumItemPos(ItemBegin, ItemEnd, ItemPos: Integer): TPoint;
    function NumItemPosStart(ItemBegin, ItemPos: Integer): Integer;
    function FindItem(ItemBegin, ItemPos: Integer): TPlannerItem;
    function FindItemInHeader(ItemPos: Integer): TPlannerItem;
    function FindItemInHeaderEx(ItemPos,SubPos: Integer): TPlannerItem;
    function FindItemIdx(ItemBegin: Integer): TPlannerItem;
    function FindItemPos(ItemBegin, ItemPos, ItemSubPos: Integer): TPlannerItem;
    function FindBkgPos(ItemBegin, ItemPos, ItemSubPos: Integer): TPlannerItem;
    function FindItemPosIdx(ItemBegin, ItemPos, ItemSubPos: Integer): TPlannerItem;
    function FindItemHeaderPosIdx(ItemPos, ItemSubPos: Integer): TPlannerItem;
    function FindItemIndex(ItemBegin, ItemPos, ItemSubIdx: Integer): TPlannerItem;
    function QueryItem(Item: TPlannerItem; ItemBegin, ItemPos: Integer): TPlannerItem;
    function FocusItem(ItemBegin, ItemPos, ItemSubPos: Integer; Control: Boolean): TPlannerItem;
    procedure SetCurrent(ItemCurrent: Integer);
    procedure ClearRepaints;
    procedure SetTimeTags;
    procedure GetTimeTags;
    function MatchItem(Item:TPlannerItem; s:string; Param:TFindTextParams): Boolean;
    function MaxItemsInPos(Position: Integer):Integer;
    function MaxItemsInHeader(Position: Integer):Integer;
    function MaxItemsInPosAtRow(Position, Row: Integer): Integer;
  public
    function GetItemClass: TCollectionItemClass; virtual;
    constructor Create(AOwner: TTIWCustomWebPlanner);
    function HasItem(ItemBegin, ItemEnd, ItemPos: Integer): Boolean;
    function HasItemEx(AItem: TPlannerItem;ItemBegin, ItemEnd, ItemPos: Integer): Boolean;
    function NumConflicts(var ItemBegin, ItemEnd: Integer; ItemPos: Integer): Integer;
    function FindFirst(ItemBegin, ItemEnd, ItemPos: Integer): TPlannerItem;
    function FindNext(ItemBegin, ItemEnd, ItemPos: Integer): TPlannerItem;
    function FindBackground(ItemBegin, ItemEnd,ItemPos: Integer): TPlannerItem;
    function FindKey(DBKey: string): TPlannerItem;
    function HeaderFirst(ItemPos: Integer): TPlannerItem;
    function HeaderNext(ItemPos: Integer): TPlannerItem;
    function FindText(StartItem:TPlannerItem;s: string; Param: TFindTextParams):TPlannerItem;
    function Add: TPlannerItem;
    function Insert(Index: Integer): TPlannerItem;
    property Items[Index: Integer]: TPlannerItem read GetItem write SetItem; default;
    function SelectNext: TPlannerItem;
    function SelectPrev: TPlannerItem;
    procedure UnSelect;
    procedure UnSelectAll;
    procedure Select(Item: TPlannerItem);
    property Selected: TPlannerItem read FSelected write FSelected;
    property DBItem: TPlannerItem read FDBItem write FDBItem;
    function InVisibleLayer(Layer: Integer): Boolean;
    procedure ClearConflicts;
    procedure SetConflicts;
    procedure ItemChanged(Item: TPlannerItem);
    function ItemsAtPosition(Pos: Integer): Integer;
    function ItemsAtIndex(Idx: Integer): Integer;
    procedure BeginUpdate; override;
    procedure EndUpdate; override;
    procedure ResetUpdate;
    procedure ClearPosition(Position: Integer);
    procedure ClearLayer(Layer: Integer);
    procedure ClearAll;
    procedure ClearDB;
    procedure OffsetItems(Offset: Integer);
    procedure MoveAll(DeltaPos, DeltaBegin: Integer);
    procedure MoveSelected(DeltaPos, DeltaBegin: Integer);
    procedure SizeAll(DeltaStart, DeltaEnd: Integer);
    procedure SizeSelected(DeltaStart, DeltaEnd: Integer);
    property Changing: Boolean read FChanging write FChanging;
  published
    property SelCount: Integer read GetSelCount;
  end;

  TIWWebPlannerClientEvents = class(TPersistent)
  private
    FEditDone: TStringList;
    FDeleteClick: TStringList;
    FSelect: TStringList;
    FEditStart: TStringList;
    procedure SetEditDone(const Value: TStringList);
    procedure SetDeleteClick(const Value: TStringList);
    procedure SetEditStart(const Value: TStringList);
    procedure SetSelect(const Value: TStringList);
  public
    constructor Create;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
  published
    property EditDone: TStringList read FEditDone write SetEditDone;
    property DeleteClick: TStringList read FDeleteClick write SetDeleteClick;
    property EditStart: TStringList read FEditStart write SetEditStart;
    property Select: TStringList read FSelect write SetSelect;
  end;

  TMoveProperties = record
    MouseFrom: integer;
    MouseTo: integer;
    FromBegin: integer;
    FromEnd: integer;
    FromPos: integer;
    ToBegin: integer;
    ToEnd: integer;
    ToPos: integer;
  end;

  TIWAsyncIndexEvent = procedure (Sender: TObject; EventParams: TStringList; Index: Integer) of object;
  TIWAsyncItemEvent = procedure (Sender: TObject; EventParams: TStringList;
    ItemIndex : Integer; Item: TPlannerItem) of object;
  TIWAsyncItemEditEvent = procedure (Sender: TObject; EventParams: TStringList;
    var Allow: boolean; ItemIndex : Integer; Item: TPlannerItem) of object;
  TIWAsyncItemMoveClick = procedure(Sender: TObject; EventParams: TStringList;
    var Allow: Boolean; Item: TPlannerItem; FromBegin, FromEnd, FromPos, ToBegin, ToEnd, ToPos: Integer) of object;
  TIWAsyncItemMoveExClick = procedure(Sender: TObject; EventParams: TStringList;
    var Allow: Boolean; Item: TPlannerItem; MoveProperties: TMoveProperties; var ShowDialog: boolean; var DialogText: string) of object;
  TIWAsyncItemSizeClick = procedure(Sender: TObject; EventParams: TStringList;
    var Allow: Boolean; Item: TPlannerItem; Position, FromBegin, FromEnd, ToBegin,
    ToEnd: Integer) of object;
  TIWAsyncPlannerDeleteClick = procedure (Sender: TObject; EventParams: TStringList; var Allow: Boolean; Item: TPlannerItem) of object;

  TTIWCustomWebPlanner = class(TIWControl)
  private
    FAlertOutsidePlanner: String;
    FShowAlertOutsidePlanner: boolean;
    FBackground: TIPicture;
    FBands: TBands;
    FBorderColor: TIWColor;
    FBorderSize: integer;
    FBorderCollapsed: Boolean;
    FChangeCaption : TChangeCaption;
    FClientEvents: TIWWebPlannerClientEvents;
    FDayNames: TStringList;
    FDefaultItem: TPlannerItem;
    FDefaultItems: TPlannerItems;
    FDisplay: TPlannerDisplay;
    FDTList: TDateTimeList;
    FEnableSelection : Boolean;
    FEditType : TPlannerEditType;
    FFont: TIWFont;
    FGlyphs: TGlyphs;
    FHints: THints;
    FHourType: THourType;
    FInactive: TByteSet;
    FItemGap: Integer;
    FItems: TPlannerItems;
    FLayer: Integer;
    FLinkUpdate: Boolean;
    FMode: TPlannerMode;
    FMultiSelect: Boolean;
    FNumberOfHeaderItems: Integer;
    FOnItemCreate: TIWItemCreateClick;
    FOnItemMove: TIWItemMoveClick;
    FOnItemSize : TIWItemSizeClick;
    FOnHeaderClick : TIWHeaderClick;
    FOnItemCaptionChange : TPlannerItemCaptionChange;
    FOnItemChange : TPlannerItemChangeClick;
    FOnItemEdit : TPlannerItemChangeClick;
    FOnPlannerGetSideBarLines: TPlannerGetSideBarLines;
    FOnPlanTimeToStrings: TPlannerPlanTimeToStrings;
    FOnItemDelete : TPlannerDeleteClick;
    FOnItemClick: TPlannerItemClick;
    FOverlap: Boolean;
    FOverlapColor: TIWColor;
    FPlannerHeader: TPlannerHeader;
    FPositionGroup: Integer;
    FPositionProps: TPositionProps;
    FPopupEdit : TPopupEdit;
    FPositions: Integer;
    FRow: Integer;
    FSHowConfirm: Boolean;
    FSideBar: TPlannerSidebar;
    FStreamPersistentTime: Boolean;
    FTempValue: string;
    FTopLeftCell: string;
    FHiddenItems:string;
    FHiddenPopups : string;
    FSelectionColor: TIWColor;
    FInActiveDays: TWeekDays;
    FSelItemBegin: Integer;
    FSelItemEnd: Integer;
    FSelPosition: Integer;
    FSelItemCaption : string;
    FSelectedIndex: Integer;
    FIsEditing: Boolean;
    FEditIndex: Integer;
    FLoading: Boolean;
    FOnFailedInsert : TIWItemCreateClick;
    FOnFailedMove: TIWItemMoveClick;
    FOnFailedSize: TIWItemSizeClick;
    FViewOnly : Boolean;
    FAutoInsDel: Boolean;
    FAutoCreateOnSelect: Boolean;
    FSelectionMode : TIWSelectionMode;
    FOnPlannerDblClick : TIWPlannerDblClick;
    FOnItemCreated: TIWItemEvent;
    FOnItemDeleted: TIWItemEvent;
    FOnTemplateData: TIWDBTemplateEvent;
    FInTemplate : Boolean;
    FPlannerInEditing : boolean;
    FRelativeItemMove: Boolean;
    FHeaderClickIndex: integer;
    FResizeIndex: integer;
    FResizeType: integer;
    FResizeValue: integer;
    FResizeAllow: boolean;
    FMoveIndex: integer;
    FMoveAllow: boolean;
    FShowDialog: boolean;
    FDialogText: string;
    FMoveBegin: integer;
    FMoveEnd: integer;
    FMovePos: integer;
    FDeleteIndex: integer;
    FDeleteAllow: boolean;
    FEditStart: boolean;
    FEditDone: boolean;
    FOnAsyncHeaderClick: TIWAsyncIndexEvent;
    FOnAsyncItemClick: TIWAsyncItemEvent;
    FOnAsyncEditStart: TIWAsyncItemEditEvent;
    FOnAsyncEditDone: TIWAsyncItemEditEvent;
    FOnAsyncItemSize: TIWAsyncItemSizeClick;
    FOnAsyncItemMove: TIWAsyncItemMoveClick;
    FOnAsyncItemMoveEx: TIWAsyncItemMoveExClick;
    FOnAsyncItemDelete: TIWAsyncPlannerDeleteClick;

    procedure SetSelectionMode(const Value : TIWSelectionMode);
    procedure SetViewOnly(const Value : Boolean);
    procedure GetSideBarLines(Index, Position: Integer; var Line1, Line2, Line3: string; var HS: Boolean);
    procedure SetBackgroundImage(const Value: TIPicture);
    procedure SetBorderColor(const Value: TIWColor);
    procedure SetBorderSize(const Value: integer);
    procedure SetBorderCollapsed(const Value: Boolean);
    procedure SetChangeCaption(const Value : TChangeCaption);
    procedure SetClientEvents(const Value: TIWWebPlannerClientEvents);
    procedure SetDayNames(const Value: TStringList);
    procedure SetDefaultItem(const Value: TPlannerItem);
    procedure SetDisplay(const Value: TPlannerDisplay);
    procedure SetEnableSelection(const Value : Boolean);
    procedure SetEditType(const Value : TPlannerEditType);
    procedure SetFont(const Value: TIWFont);
    procedure SetGlyphs(const Value: TGlyphs);
    procedure SetPopupEdit(const Value : TPopupEdit);
    procedure SetHints(const Value: THints);
    procedure SetHourType(const Value: THourType);
    procedure SetHeader(const Value: TPlannerHeader);
    procedure SetItems(const Value: TPlannerItems);
    procedure SetMode(const Value: TPlannerMode);
    procedure SetPositionProps(const Value: TPositionProps);
    procedure SetShowConfirm(const Value: Boolean);
    procedure SetSideBar(const Value: TPlannerSidebar);
    procedure SetPositionGroup(const Value: Integer);
    procedure SetPositions(const Value: Integer);
    procedure SetOverlapColor(const Value: TIWColor);
    procedure StartEdit(APlannerItem: TPlannerItem);
    procedure SetAlertOutsidePlanner(const value: string);
    procedure SetShowAlertOutsidePlanner(const value: boolean);
    procedure SetInTemplate(const Value : Boolean);
    procedure StopEdit;
    procedure RenderPopup( APlannerItem : TPlannerItem);
    function HTMLClr(AColor: TIWColor): string;
    function HTMLAlign(AAlignment: TAlignment): string;
    function ColorToStyle(AColor1, AColor2: TIWColor; ADirection: TIWGradientDirection; AContext : TIWBaseComponentContext): string;
    function ColorToHtmlHexBgColor(const Value: TIWColor): string;
    //Modified - Support '<,>,&' in notes and caption of item
    function HTMLAdapt(const Value : String) : string;
    procedure SetInActiveDays(const Value: TWeekDays);
    procedure InActiveChanged(Sender: TObject);
    procedure SetRelativeItemMove(const Value: Boolean);
    //function GetCacheDir:string;
  protected
    procedure InvalidateX;
    function AbsTimeToStr(DateTime: TDateTime): string; virtual;
    function CreateItems: TPlannerItems; virtual;
    function ItemCall(Param: String; Index: Integer): string;
    procedure GetCellBrush(Pos,Index: Integer; ABrush:TBrush);
    function GetCellColor(Pos,Index: Integer): TIWColor;
    property ItemGap: Integer read FItemGap write FItemGap;
    property LinkUpdate: Boolean read FLinkUpdate write FLinkUpdate;
    procedure DoCancel; virtual;
    procedure DoPost; virtual;
    function RenderToHTMLCol(AContext: TIWBaseComponentContext):string;
    function RenderToHTMLRow(AContext: TIWBaseComponentContext):string;
    function RenderCellToHTMLCol(APlannerItem: TPlannerItem; RowIndex,PosIndex,MaxPos,TotCol,ColIndex,SubColspan: Integer;
      AContext: TIWBaseComponentContext): string;
    function RenderCellToHTMLRow(APlannerItem: TPlannerItem; RowIndex,PosIndex,MaxPos,TotCol,ColIndex,SubColspan: Integer;
      AContext: TIWBaseComponentContext): string;
    function RenderItemToHTML(APlannerItem: TPlannerItem; RowIndex, MaxPos: integer;
      Hidden,Orientation: Boolean; TotalHeightOfItem : Integer; AContext: TIWBaseComponentContext): string;
    function RenderSideBar(RowIndex, ColIndex: Integer; AContext: TIWBaseComponentContext): string;
    property StreamPersistentTime: Boolean read FStreamPersistentTime write FStreamPersistentTime;
    procedure SetValue(const Value:string); override;
    procedure Submit(const Value: string); override;
    procedure ItemMoved(FromBegin, FromEnd, FromPos, ToBegin, ToEnd, ToPos: Integer); virtual;
    procedure ItemSized(FromBegin, FromEnd, ToBegin, ToEnd: Integer); virtual;
    procedure ItemEdited(Item: TPlannerItem); virtual;
    procedure ItemChanged(Item : TPlannerItem); virtual;
    procedure ItemSelected(Item: TPlannerItem); virtual;
    function RenderCSSStyle(PlannerMode : string; AContext : TIWBaseComponentContext) : string;
    procedure DoOnAsyncHeaderClick(AParams: TStringList);
    procedure DoOnAsyncItemClick(AParams: TStringList);
    procedure DoOnAsyncEditStart(AParams: TStringList);
    procedure DoOnAsyncEditDone(AParams: TStringList);
    procedure DoOnAsyncItemSize(AParams: TStringList);
    procedure DoOnAsyncItemMove(AParams: TStringList);
    procedure DoOnAsyncItemDelete(AParams: TStringList);
  public
    function CellToAbsTime(Index: integer): TDateTime; virtual;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Loaded; override;
    function IsCurPos(APosition: Integer): Boolean;
    function CloneItem(Item: TPlannerItem): TPlannerItem; virtual;
    function CloneItemAtSelection(Item: TPlannerItem): TPlannerItem; virtual;
    function CreateItem:TPlannerItem; virtual;
    function CreateItemAtSelection:TPlannerItem; virtual;
    procedure FreeItem(APlannerItem:TPlannerItem); virtual;
    function GetDayName(WeekDay: Integer): string; virtual;
    function IndexToTime(Index: Integer): TDateTime; virtual;
    function PlanTimeToStr(MinutesValue: Cardinal): string;
    procedure PlanTimeToStrings(MinutesValue: Cardinal; var HoursString, MinutesString, AmPmString: string);
    function PosToDay(Pos: Integer): TDateTime; virtual;
    procedure ProcessTemplateField(APlannerItem : TPlannerItem; fld:string; var DBFld:string); virtual;
    property RelativeItemMove: Boolean read FRelativeItemMove write SetRelativeItemMove default false;
    property InActive: TByteSet read FInActive;
    procedure IWPaint; override;
    function RenderHTML(AContext: TIWBaseComponentContext): TIWHTMLTag; override;
    procedure RemoveClones(Item: TPlannerItem);
    function TimeToIndex(DT: TDateTime): Integer; virtual;
    procedure UpdateItem(APlannerItem:TPlannerItem); virtual;
    function GetVersionNr: Integer; virtual;
    function GetVersionString: string; virtual;
    procedure MovePosition(FromPos, ToPos: Integer);
    procedure DeletePosition(Position: Integer);
    procedure InsertPosition(Position: Integer);
    procedure SaveToStream(Stream: TStream);
    procedure LoadFromStream(Stream: TStream);
    procedure InsertFromStream(Stream: TStream);
    procedure SavePositionToStream(Stream: TStream; Position: Integer);
    procedure LoadPositionFromStream(Stream: TStream; Position: Integer);
    procedure SaveLayerToStream(Stream: TStream; Layer: Integer);
    procedure LoadLayerFromStream(Stream: TStream; Layer: Integer);
    procedure SaveToFile(FileName: string);
    procedure LoadFromFile(FileName: string);
    procedure InsertFromFile(FileName: string);
    procedure SavePositionToFile(FileName: string; Position: Integer);
    procedure LoadPositionFromFile(FileName: string; Position: Integer);
    procedure SaveLayerToFile(FileName: string; Layer: Integer);
    procedure LoadLayerFromFile(FileName: string; Layer: Integer);

    property AutoInsDel: Boolean read FAutoInsDel write FAutoInsDel;
    property AutoCreateOnSelect: Boolean read FAutoCreateOnSelect write FAutoCreateOnSelect;
    property SelectedIndex : Integer read FSelectedIndex write FSelectedIndex;
    property SelItemBegin: Integer read FSelItemBegin write FSelItemBegin;
    property SelItemCaption : string read FSelItemCaption write FSelItemCaption;
    property SelItemEnd: Integer read FSelItemEnd write FSelItemEnd;
    property SelPosition: Integer read FSelPosition write FSelPosition;
    property AlertOutsidePlanner: String read FAlertOutsidePlanner write SetAlertOutsidePlanner;
    property Background: TIPicture read FBackground write SetBackgroundImage;
    property Bands: TBands read FBands write FBands;
    property BorderColor: TIWColor read FBorderColor write SetBorderColor;
    property BorderCollapsed: Boolean read FBorderCollapsed write SetBorderCollapsed;
    property BorderSize: integer read FBorderSize write SetBorderSize;
    property ChangeCaption : TChangeCaption read FChangeCaption write SetChangeCaption;
    property ClientEvents: TIWWebPlannerClientEvents read FClientEvents write SetClientEvents;
    property DateTimeList: TDateTimeList read FDTList write FDTList;
    property DayNames: TStringList read FDayNames write SetDayNames;
    property DefaultItem: TPlannerItem read FDefaultItem write SetDefaultItem;
    property Display: TPlannerDisplay read FDisplay write SetDisplay;
    property EditType : TPlannerEditType read FEditType write SetEditType;
    property EnableSelection : Boolean read FEnableSelection write SetEnableSelection;
    property Font: TIWFont read FFont write SetFont;
    property Glyphs: TGlyphs read FGlyphs write SetGlyphs;
    property Header: TPlannerHeader read FPlannerHeader write SetHeader;
    property Hints: THints read FHints write SetHints;
    property HourType: THourType read FHourType write SetHourType;
    property InActiveDays: TWeekDays read FInActiveDays write SetInActiveDays;
    property InTemplate : Boolean read FInTemplate write SetInTemplate;
    property Items: TPlannerItems read FItems write SetItems;
    property Layer: Integer read FLayer write FLayer;
    property Mode: TPlannerMode read FMode write SetMode;
    property MultiSelect: Boolean read FMultiSelect write FMultiSelect;
    property OnFailedInsert : TIWItemCreateClick read FOnFailedInsert write FOnFailedInsert;
    property OnFailedMove: TIWItemMoveClick read FOnFailedMove write FOnFailedMove;
    property OnFailedSize: TIWItemSizeClick read FOnFailedSize write FOnFailedSize;
    property OnItemCaptionChange : TPlannerItemCaptionChange read FOnItemCaptionChange write FOnItemCaptionChange;
    property OnItemChange : TPlannerItemChangeClick read FOnItemChange write FOnItemChange;
    property OnItemCreate: TIWItemCreateClick read FOnItemCreate write FOnItemCreate;
    property OnItemCreated: TIWItemEvent read FOnItemCreated write FOnItemCreated;
    property OnItemClick: TPlannerItemClick read FOnItemClick write FOnItemClick;
    property OnItemDelete: TPlannerDeleteClick read FOnItemDelete write FOnItemDelete;
    property OnItemDeleted: TIWItemEvent read FOnItemDeleted write FOnItemDeleted;
    property OnItemEdit : TPlannerItemChangeClick read FOnItemEdit write FOnItemEdit;
    property OnItemMove: TIWItemMoveClick read FOnItemMove write FOnItemMove;
    property OnItemSize: TIWItemSizeClick read FOnItemSize write FOnItemSize;
    property OnHeaderClick : TIWHeaderClick read FOnHeaderClick write FOnHeaderClick;
    property OnPlannerDblClick : TIWPlannerDblClick read FOnPlannerDblClick write FOnPlannerDblClick;
    property OnPlannerGetSideBarLines: TPlannerGetSideBarLines read FOnPlannerGetSideBarLines write FOnPlannerGetSideBarLines;
    property OnPlanTimeToStrings: TPlannerPlanTimeToStrings read FOnPlanTimeToStrings write FOnPlanTimeToStrings;
    property OnTemplateData: TIWDBTemplateEvent read FOnTemplateData write FOnTemplateData;
    property Overlap: Boolean read FOverlap write FOverlap;
    property OverlapColor: TIWColor read FOverlapColor write SetOverlapColor;
    property PopupEdit : TPopupEdit read FPopupEdit write SetPopupEdit;
    property PositionProps: TPositionProps read FPositionProps write SetPositionProps;
    property Positions: Integer read FPositions write SetPositions;
    property PositionGroup: Integer read FPositionGroup write SetPositionGroup;
    property SelectionColor: TIWColor read FSelectionColor write FSelectionColor;
    property SelectionMode : TIWSelectionMode read FSelectionMode write SetSelectionMode;
    property SideBar: TPlannerSidebar read FSideBar write SetSideBar;
    property ShowAlertOutsidePlanner : Boolean read FShowAlertOutsidePlanner write SetShowAlertOutsidePlanner;
    property ShowConfirm: Boolean read FShowConfirm write SetShowConfirm;
    property TopLeftCell : string read FTopLeftCell write FTopLeftCell;
    property ViewOnly : Boolean read FViewOnly write SetViewOnly;
    function RenderAsync(AContext: TIWBaseHTMLComponentContext): TIWXMLTag; override;
    property OnAsyncHeaderClick: TIWAsyncIndexEvent read FOnAsyncHeaderClick write FOnAsyncHeaderClick;
    property OnAsyncItemClick: TIWAsyncItemEvent read FOnAsyncItemClick write FOnAsyncItemClick;
    property OnAsyncEditStart: TIWAsyncItemEditEvent read FOnAsyncEditStart write FOnAsyncEditStart;
    property OnAsyncEditDone: TIWAsyncItemEditEvent read FOnAsyncEditDone write FOnAsyncEditDone;
    property OnAsyncItemSize: TIWAsyncItemSizeClick read FOnAsyncItemSize write FOnAsyncItemSize;
    property OnAsyncItemMove: TIWAsyncItemMoveClick read FOnAsyncItemMove write FOnAsyncItemMove;
    property OnAsyncItemMoveEx: TIWAsyncItemMoveExClick read FOnAsyncItemMoveEx write FOnAsyncItemMoveEx;
    property OnAsyncItemDelete: TIWAsyncPlannerDeleteClick read FOnAsyncItemDelete write FOnAsyncItemDelete;
  end;

  TTIWWebPlanner = class(TTIWCustomWebPlanner)
  published
    property AlertOutsidePlanner;
    property AutoInsDel;
    property AutoCreateOnSelect;
    property Background;
    property Bands;
    property BorderColor;
    property BorderSize;
    property BorderCollapsed;
    property ChangeCaption;
    property ClientEvents;
    property DayNames;
    property DefaultItem;
    property Display;
    property EditType;
    property EnableSelection;
    property Font;
    property Glyphs;
    property Header;
    property Hints;
    property HourType;
    property InActiveDays;
    property InTemplate;
    property Items;
    property Layer;
    property Mode;
    property Overlap;
    property OverlapColor;
    property PopupEdit;
    property PositionGroup;
    property PositionProps;
    property Positions;
    property SelectionColor;
    property SelectionMode;
    property ShowAlertOutsidePlanner;
    property ShowConfirm;
    property SideBar;
    property TopLeftCell;
    property OnItemCreate;
    property OnItemCreated;
    property OnFailedInsert;
    property OnFailedMove;
    property OnFailedSize;
    property OnHeaderClick;
    property OnItemClick;
    property OnItemCaptionChange;
    property OnItemChange;
    property OnItemDelete;
    property OnItemDeleted;
    property OnItemEdit;
    property OnItemMove;
    property OnItemSize;
    property OnPlannerDblClick;
    property OnPlannerGetSideBarLines;
    property OnPlanTimeToStrings;
    property OnTemplateData;
    property ViewOnly;
    property OnAsyncHeaderClick;
    property OnAsyncItemClick;
    property OnAsyncEditStart;
    property OnAsyncEditDone;
    property OnAsyncItemSize;
    property OnAsyncItemMove;
    property OnAsyncItemMoveEx;
    property OnAsyncItemDelete;
  end;

  TPlannerIO = class(TComponent)
    FItems: TPlannerItems;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Items: TPlannerItems read FItems write FItems;
  end;

  TPlannerItemIO = class(TComponent)
    FItem: TPlannerItem;
  public
    constructor CreateItem(AOwner: TPlannerItems);
    destructor Destroy; override;
  published
    property Item: TPlannerItem read FItem write FItem;
  end;

  //IW121
  TTMSBrowser=(
     brUnknown
     , brNetscape7
     , brIE, brGecko, brOpera, brSafari, brChrome
     , brIE4, brNetscape6, brNetscape4, brOther, brHTML32Test
     , brWMLTest, brXHTMLMPTest, brWAP
   );

  function IsBrowserIe(AContext: TIWBaseComponentContext): boolean; overload;
  function IsBrowserIe(AContext: TIWPageContext40): boolean; overload;

  function IsBrowserIe10(AContext: TIWBaseComponentContext): boolean; overload;
  function IsBrowserIe10(AContext: TIWPageContext40): boolean; overload;

  function GetTMSBrowser(AContext: TIWBaseComponentContext): TTMSBrowser; overload;
  function GetTMSBrowser(AContext: TIWPageContext40): TTMSBrowser; overload;
  //END IW121

  function HTMLStrip(s: string):string;
  function MatchStr(s1,s2:string;DoCase:Boolean): Boolean;
  function DaysInMonth(mo, ye: word): word;

implementation

uses
  IWServerControllerBase, Math, IWAppForm

  {$IFDEF TMSIW14}
 , IWImageUtils //required for IWAppCache
  {$ENDIF}
  ;

const
  ShortDaySun = 'Sun';
  ShortDayMon = 'Mon';
  ShortDayTue = 'Tue';
  ShortDayWed = 'Wed';
  ShortDayThu = 'Thu';
  ShortDayFri = 'Fri';
  ShortDaySat = 'Sat';
  CStyleLF = '\n';
  RegularCRLF = #13#10;
  HtmlBreak = '<BR>';
  RtfStart = '{\';
  HtmlEndTagStart = '</';
  HtmlNonBreakingSpace = '&nbsp;';
  DaysPerWeek = 7;
  FirstDayOfWeek = 1;
  Percent = '%';

type
  {$IFNDEF DELPHI_UNICODE}
  TCharSet = set of char;
  {$ENDIF}

  {$IFDEF DELPHI_UNICODE}
  TCharSet = array of char;
  {$ENDIF}

{$I DELPHIXE.INC}

{$IFDEF TMSIW121}
function Get121Browser(xbrowser: Tbrowser):TTmsBrowser;
begin
  if xBrowser is TInternetExplorer then
    result := BrIE
  else if xBrowser is TChrome then
    result := BrChrome
  else if xBrowser is TFireFox then
    result := BrGecko
  else if xBrowser is TSafari then
    result := BrSafari
  else
    result:=brOther;
end;
{$ELSE}
function GetPre121Browser(xBrowser:integer):TTmsBrowser;
begin
  case xBrowser of
    0: result := brunknown;
    1: result := BrNetscape7;
    2: result := BrIE;
    3: result := brgecko;
    4: result := brOpera;
    5: result := brsafari;
    6: result := brchrome;
    7: result := brIE4;
    8: result := brNetscape6;
    9: result := brNetScape4;
    10: result := brHTML32Test;
    11: result := brWMLTest;
    12: result := brXHTMLMPTest;
    13: result := brWap;
  else
    result:=brunknown;
  end;
end;
{$ENDIF}

function GetTMSBrowser(AContext: TIWBaseComponentContext):TTMSBrowser;
begin
  {$IFDEF TMSIW121}
  Result := Get121Browser(aContext.webApplication.Browser);
  {$ELSE}
  result := GetPre121Browser(integer(AContext.webapplication.browser));
  {$ENDIF}
end;

function GetTMSBrowser(AContext: TIWPageContext40):TTMSBrowser;overload;
begin
  {$IFDEF TMSIW121}
  Result := Get121Browser(aContext.webApplication.Browser);
  {$ELSE}
  result := GetPre121Browser(integer(AContext.webapplication.Browser));
  {$ENDIF}
end;

function IsBrowserIe(AContext: TIWBaseComponentContext):boolean;
begin
  result := GetTmsBrowser(aContext) = BrIe;
end;

function IsBrowserIe(AContext: TIWPageContext40):boolean;
begin
  result := GetTmsBrowser(aContext) = BrIe;
end;

function IsBrowserIe10(AContext: TIWBaseComponentContext):boolean;
begin
  result := false;
  if GetTmsBrowser(aContext) = BrIe then
  begin
    {$IFDEF TMSIW121}
    result := AContext.WebApplication.Browser.Version >= 10;
    {$ELSE}
//    result := AContext.WebApplication.BrowserSubType = brsIE10;
    result := GetEnumName(TypeInfo(TIWBrowserSubtype), Integer(AContext.WebApplication.BrowserSubType)) = 'brsIE10';
    {$ENDIF}
  end;
end;

function IsBrowserIe10(AContext: TIWPageContext40):boolean;
begin
  result := false;
  if GetTmsBrowser(aContext) = BrIe then
  begin
    {$IFDEF TMSIW121}
    result := AContext.WebApplication.Browser.Version >= 10;
    {$ELSE}
//    result := AContext.WebApplication.BrowserSubType = brsIE10;
    result := GetEnumName(TypeInfo(TIWBrowserSubtype), Integer(AContext.WebApplication.BrowserSubType)) = 'brsIE10';
    {$ENDIF}
  end;
end;

function GetCacheDirEx(Control:TIWCustomControl): string;
begin
  {$IFNDEF TMSIW14}
  Result := GServerController.UserCacheURL;
  {$ENDIF}
end;

function GetNewCacheFile(APrefix: string; AUserCache: Boolean): string;
begin
{$IFNDEF TMSIW14}
  Result := TIWServerControllerBase.NewCacheFile(APrefix,AUserCache);
{$ENDIF}
end;

{ TIWAppCache }

{$IFDEF TMSIW14}
function GraphicToCacheFile(AGraphic: TGraphic;
  const ImgType: TIWImageType): string;
var
  xCacheStream: TCacheStream;
  xContentType: string;
  ASession: TIWApplication;
begin
  case imgType of
    imgPNG: xContentType := 'image/png';
    imgJPEG: xContentType := 'image/jpeg';
    imgGIF: xContentType := 'image/gif';
    imgBMP: xContentType := 'image/bmp';
  end;

  ASession := GGetWebApplicationThreadVar();
//  xCacheStream := TCacheStream.Create(ASession);
  xCacheStream := TCacheStream.Create(ASession, TCacheType.ctSession);
//  xCacheStream := TCacheStream.Create(ASession, TCacheType.ctApp);
  xCacheStream.ContentType := xContentType;
  Result := xCacheStream.Href;

  try
    AGraphic.SaveToStream(xCacheStream.Stream);
  finally
    xCacheStream.Free;
  end;
end;

function StreamToCacheFile(AStream: TMemoryStream;
  const ImgType: TIWImageType): string;
var
  xCacheStream: TCacheStream;
  xContentType: string;
  ASession: TIWApplication;
begin
  case imgType of
    imgPNG: xContentType := 'image/png';
    imgJPEG: xContentType := 'image/jpeg';
    imgGIF: xContentType := 'image/gif';
    imgBMP: xContentType := 'image/bmp';
  end;

  ASession := GGetWebApplicationThreadVar();
  xCacheStream := TCacheStream.Create(ASession);
  xCacheStream.ContentType := xContentType;
  Result := xCacheStream.Href;

  try
    AStream.Position := 0;
    AStream.SaveToStream(xCacheStream.Stream);
  finally
    xCacheStream.Free;
  end;
end;

{$ENDIF}

{$IFNDEF TMSIW14}
function GraphicToCacheFile(AGraphic: TGraphic;
  const ImgType: TIWImageType): string;
begin
  case ImgType of
    imgJPEG: Result := GetNewCacheFile('jpeg',true);
    imgGIF: Result := GetNewCacheFile('gif',true);
    imgBMP: Result := GetNewCacheFile('bmp',true)
  end;
  AGraphic.SaveToFile(Result);

  Result := GServerController.UserCacheURL + ExtractFilename(Result);
end;

function StreamToCacheFile(AStream: TMemoryStream;
  const ImgType: TIWImageType): string;
begin
  case ImgType of
    imgJPEG: Result := GetNewCacheFile('jpeg',true);
    imgGIF: Result := GetNewCacheFile('gif',true);
    imgBMP: Result := GetNewCacheFile('bmp',true)
  end;

  AStream.SaveToFile(Result);
  Result := GServerController.UserCacheURL + ExtractFilename(Result);
end;

{$ENDIF}

function Limit(Value,vmin,vmax: integer): Integer;
begin
  Result := Value;
  if Value < vmin then
    Result := vmin;
  if Value > vmax then
    Result := vmax;
end;

function Clip(const Value: Integer; const RangeLow, RangeHigh: Integer): Integer;
begin
  if Value > RangeHigh then
    Result := RangeHigh
  else if Value < RangeLow then
    Result := RangeLow
  else
    Result := Value;
end;

function VarPos(su,s:string;var Respos:Integer):Integer;
begin
  Respos := Pos(su,s);
  Result := Respos;
end;

function HTMLStrip(s: string):string;
var
  TagPos: integer;
begin
  Result := '';
  // replace line breaks by linefeeds
  while (pos('<BR>',uppercase(s))>0) do s := StringReplace(s,'<BR>',chr(13)+chr(10),[rfIgnoreCase]);
  while (pos('<HR>',uppercase(s))>0) do s := StringReplace(s,'<HR>',chr(13)+chr(10),[rfIgnoreCase]);

  {remove all other tags}
  while (VarPos('<',s,TagPos) > 0) do
  begin
    Result := Result + Copy(s,1,TagPos-1);
    if (VarPos('>',s,TagPos)>0) then
      Delete(s,1,TagPos)
    else
      Break;
  end;
  Result := Result + s;
end;

function IsDate(s:string;var dt:TDateTime):boolean;
var
  su: string;
  da,mo,ye: word;
  err: Integer;
  dp,mp,yp,vp: Integer;
begin
  Result := False;

  su := UpperCase(ShortDateFormat);
  dp := pos('D',su);
  mp := pos('M',su);
  yp := pos('Y',su);

  da := 0;
  mo := 0;
  ye := 0;

  if VarPos(DateSeparator,s,vp) > 0 then
  begin
    su := Copy(s,1,vp - 1);

    if (dp < mp) and
       (dp < yp) then
       Val(su,da,err)
    else
    if (mp < dp) and
       (mp < yp) then
       Val(su,mo,err)
    else
    if (yp < mp) and
       (yp < dp) then
       Val(su,ye,err);

    if err <> 0 then Exit;
    Delete(s,1,vp);

    if VarPos(DateSeparator,s,vp) > 0 then
    begin
      su := Copy(s,1,vp - 1);

      if ((dp > mp) and (dp < yp)) or
         ((dp > yp) and (dp < mp)) then
         Val(su,da,err)
      else
      if ((mp > dp) and (mp < yp)) or
         ((mp > yp) and (mp < dp)) then
         val(su,mo,err)
      else
      if ((yp > mp) and (yp < dp)) or
         ((yp > dp) and (yp < mp)) then
         Val(su,ye,err);

      if err <> 0 then Exit;
      Delete(s,1,vp);

      if (dp > mp) and
         (dp > yp) then
         Val(s,da,err)
      else
      if (mp > dp) and
         (mp > yp) then
         Val(s,mo,err)
      else
      if (yp > mp) and
         (yp > dp) then
         Val(s,ye,err);

      if err <> 0 then Exit;
      if da > 31 then Exit;
      if mo > 12 then Exit;

      Result := True;

      try
        dt := EncodeDate(ye,mo,da);
      except
        Result := False;
      end;
    end;
  end;
end;

function PlannerGetIdCol(Planner: TTIWCustomWebPlanner; Index, Position: Integer): string;
var
  Line1, Line2, Line3: string; // HoursString, MinutesString, AmPmString
  HS: Boolean;
begin
  Planner.GetSideBarLines(Index, Position, Line1, Line2, Line3, HS);

  if Planner.Mode.PlannerType in [plDay,plTimeLine] then
  begin
    Result := Format('%s%s%s', [Line1, TimeSeparator, Line2]);
    if Line3 <> '' then
      Result := Format('%s %s', [Result, Line3]);
  end
  else
  begin
    Result := Line1 + Line2 + Line3;
  end;
end;

function Matches(s0a,s1a:PChar):boolean;
const
  larger = '>';
  smaller = '<';
  logand  = '&';
  logor   = '^';
  asterix = '*';
  qmark = '?';
  negation = '!';
  null = #0;

var
  matching,done: Boolean;
  len: longint;
  s0,s1,s2,s3: PChar;
  oksmaller,oklarger,negflag: Boolean;
  compstr:array[0..255] of Char;
  flag1,flag2,flag3: Boolean;
  equal: Boolean;
  n1,n2: Double;
  code1,code2: Integer;
  dt1,dt2: TDateTime;
begin
  oksmaller := True;
  oklarger := True;
  flag1 := False;
  flag2 := False;
  flag3 := False;
  negflag := False;
  equal := False;

  { [<>] string [&|] [<>] string }

  s2 := StrPos(s0a,larger);

  if s2 <> nil then
  begin
    inc(s2);
    if (s2^='=') then
    begin
      Equal := True;
      inc(s2);
    end;

    while (s2^ = ' ') do inc(s2);

    s3 := s2;
    len := 0;
    while (s2^ <> ' ') and (s2^ <> NULL) and (s2^ <> '&') and (s2^ <> '|')  do
    begin
      inc(s2);
      inc(len);
    end;

    StrLCopy(compstr,s3,len);

    Val(s1a,n1,code1);
    Val(compstr,n2,code2);

    if (code1 = 0) and (code2 = 0) then {both are numeric types}
    begin
      if equal then
        oklarger := n1 >= n2
      else
        oklarger := n1 > n2;
    end
    else
    begin
      if IsDate(StrPas(compstr),dt2) and IsDate(StrPas(s1a),dt1) then
      begin
        if equal then
         oklarger := dt1 >= dt2
        else
         oklarger := dt1 > dt2;
      end
      else
      begin
        if equal then
         oklarger := StrLComp(compstr,s1a,255) <= 0
        else
         oklarger := StrLComp(compstr,s1a,255) < 0;
      end;
    end;
    flag1 := True;
  end;

  equal := False;
  s2 := StrPos(s0a,smaller);
  if s2 <> nil then
  begin
    inc(s2);
    if (s2^ = '=') then
    begin
      equal := True;
      inc(s2);
    end;

    while s2^ = ' ' do inc(s2);
    s3 := s2;
    len := 0;
    while (s2^ <> ' ') and (s2^ <> NULL) and (s2^ <> '&') and (s2^ <> '|') do
    begin
      inc(s2);
      inc(len);
    end;

    StrLCopy(compstr,s3,len);

    Val(s1a,n1,code1);
    Val(compstr,n2,code2);

    if (code1 = 0) and (code2 = 0) then //both are numeric types
    begin
      if equal then
        oksmaller := n1 <= n2
      else
        oksmaller := n1 < n2;
    end
    else
    begin
      //check for dates here ?}
      if IsDate(StrPas(compstr),dt2) and IsDate(StrPas(s1a),dt1) then
      begin
        if equal then
          oksmaller := dt1 <= dt2
        else
          oksmaller := dt1 < dt2;
      end
      else
      begin
        if equal then
          oksmaller := StrLComp(compstr,s1a,255) >= 0
        else
          oksmaller := StrLComp(compstr,s1a,255) > 0;
      end;
    end;
    flag2 := True;
  end;

  s2 := StrPos(s0a,negation);
  if s2 <> nil then
  begin
    inc(s2);
    while s2^ = ' ' do inc(s2);
    s3 := s2;
    len := 0;
    while (s2^ <> ' ') and (s2^ <> NULL) and (s2^ <> '&') and (s2^ <> '|') do
    begin
      inc(s2);
      inc(len);
    end;
    StrLCopy(compstr,s3,len);
    flag3 := True;
  end;

  if flag3 then
  begin
    if StrPos(s0a,larger) = nil then
      flag1 := flag3;
    if StrPos(s0a,smaller) = nil then
      flag2 := flag3;
  end;

  if StrPos(s0a,logor) <> nil then
    if flag1 or flag2 then
    begin
      Matches := oksmaller or oklarger;
      Exit;
    end;

  if StrPos(s0a,logand) <> nil then
    if flag1 and flag2 then
    begin
      Matches := oksmaller and oklarger;
      Exit;
    end;

  if ((StrPos(s0a,larger) <> nil) and oklarger) or
     ((StrPos(s0a,smaller) <> nil) and oksmaller) then
  begin
    Matches := True;
    Exit;
  end;

  s0 := s0a;
  s1 := s1a;

  matching:=True;

  done := (s0^ = NULL) and (s1^ = NULL);

  while not done and matching do
  begin
    case s0^ of
    qmark:
      begin
        matching := s1^ <> NULL;
        if matching then
        begin
          inc(s0);
          inc(s1);
        end;
      end;
    negation:
      begin
         negflag := True;
         inc(s0);
      end;
    asterix:
      begin
        repeat
          inc(s0)
        until s0^ <> asterix;
        len := StrLen(s1);
        inc(s1,len);
        matching := matches(s0,s1);
        while (len >= 0) and not matching do
        begin
          dec(s1);
           dec(len);
           matching := Matches(s0,s1);
        end;
        if matching then
        begin
          s0 := StrEnd(s0);
          s1 := StrEnd(s1);
        end;
      end;
      else
      begin
        matching := s0^ = s1^;
        if matching then
        begin
          inc(s0);
          inc(s1);
        end;
      end;
    end;
    Done := (s0^ = NULL) and (s1^ = NULL);
  end;

  if negflag then
    Matches := not matching
  else
    Matches := matching;
end;

{$IFNDEF DELPHI_UNICODE}
function FirstChar(Charset:TCharSet;s:string):char;
var
  i:Integer;
begin
  i := 1;
  Result := #0;
  while i <= Length(s) do
  begin
    if s[i] in Charset then
    begin
      Result := s[i];
      Break;
    end;
    Inc(i);
  end;
end;
{$ENDIF}

{$IFDEF DELPHI_UNICODE}
function FirstChar(Charset:TCharSet;s:string):char;
var
  i:Integer;
  q: Integer;

  function InArray(ch: char): boolean;
  var
    j: integer;
  begin
    result := false;
    for j := 0 to High(CharSet) - 1 do
    begin
      if ch = CharSet[j] then
      begin
        result := true;
        break;
      end;
    end;
  end;


begin
  i := 1;
  q := 0;
  Result := #0;

  while i <= Length(s) do
  begin
    if s[i] = '"' then
      inc(q);

    if (InArray(s[i])) and not odd(q) then
    begin
      Result := s[i];
      Break;
    end;
    Inc(i);
  end;
end;
{$ENDIF}


function ReplaceCLFWith(const Value: string; const Replacement: string): string;
var
  CLFPos: Integer;
begin
  Result := Value;
  CLFPos := Pos(CStyleLF, Result);
  while CLFPos > 0 do
  begin
    Result := Copy(Result, 1, CLFPos - 1) +
      RegularCRLF +
      Copy(Result, CLFPos + 2, Length(Result));
    CLFPos := Pos(CStyleLF, Result);
  end;
end;

function CLToBR(Value: string): string;
begin
  Result := ReplaceCLFWith(Value, HtmlBreak);
end;

function MatchStr(s1,s2:string;DoCase:Boolean): Boolean;
var
  ch,lastop: Char;
  sep: Integer;
  res,newres: Boolean;
  {$IFDEF DELPHI_UNICODE}
  CharArray: TCharSet;
  {$ENDIF}


begin
 {remove leading & trailing spaces}
  s1 := Trim(s1);
 {remove spaces between multiple filter conditions}
  while VarPos(' &',s1,sep) > 0 do Delete(s1,sep,1);
  while VarPos(' ;',s1,sep) > 0 do Delete(s1,sep,1);
  while VarPos(' ^',s1,sep) > 0 do Delete(s1,sep,1);
  while VarPos('& ',s1,sep) > 0 do Delete(s1,sep+1,1);
  while VarPos('; ',s1,sep) > 0 do Delete(s1,sep+1,1);
  while VarPos('^ ',s1,sep) > 0 do Delete(s1,sep+1,1);

  LastOp := #0;
  Res := True;

  {$IFDEF DELPHI_UNICODE}
  SetLength(CharArray,5);
  CharArray[1] := ';';
  CharArray[2] := '^';
  CharArray[3] := '&';
  {$ENDIF}

  repeat
    {$IFDEF DELPHI_UNICODE}
    ch := FirstChar(CharArray,s1);
    {$ENDIF}

    {$IFNDEF DELPHI_UNICODE}
    ch := FirstChar(['(',';','^','&','|'],s1);
    {$ENDIF}

    //ch := FirstChar([   ';','^','&'],s1);

    {extract first part of filter}
    if ch <> #0 then
    begin
      VarPos(ch,s1,sep);
      NewRes := MatchStr(Copy(s1,1,sep-1),s2,DoCase);
      Delete(s1,1,sep);

      if LastOp = #0 then
        Res := NewRes
      else
        case LastOp of
        ';','^':Res := Res or NewRes;
        '&':Res := Res and NewRes;
        end;

      LastOp := ch;
     end;

  until ch = #0;

  if DoCase then
    NewRes := Matches(PChar(s1),PChar(s2))
  else
    NewRes := Matches(PChar(AnsiUpperCase(s1)),PChar(AnsiUpperCase(s2)));

  if LastOp = #0 then
    Res := NewRes
  else
    case LastOp of
    ';','^':Res := Res or NewRes;
    '&':Res := Res and NewRes;
    end;

  Result := Res;
end;

function DaysInMonth(mo, ye: word): word;
const
  ADaysInMonth: array[1..13] of word = (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31, 29);
begin
  if mo <> 2 then
    DaysInMonth := ADaysInMonth[mo]
  else
  begin
    if (ye mod 4 = 0) then DaysInMonth := 29
    else
      DaysInMonth := 28;
    if (ye mod 100 = 0) then DaysInMonth := 28;
    if (ye mod 400 = 0) then DaysInmonth := 29;
  end;
end;

{ TTIWCustomWebPlanner }

procedure TTIWCustomWebPlanner.DoOnAsyncHeaderClick(AParams: TStringList);
var
  Temp, strSelection, strParam, strHeaderIndex: string;
begin

  Temp := AParams.GetText;

  if (pos('|',Temp) <> 0) then
    Delete(Temp,1,pos('|',Temp));
  FTempValue := Temp;

  strSelection := FTempValue;
  Delete(strSelection,1,pos('|',strSelection));
  Delete(strSelection,1,pos('|',strSelection));

  //Async Parameters
  Delete(strSelection,1,pos('|',strSelection));
  strParam := Copy(strSelection,1,1);

  if strParam = 'H' then
  begin
    //Set the OnAsyncHeaderClick index parameter
    strHeaderIndex := Copy(strSelection,2,pos('|',strSelection) - 2);
    if strHeaderIndex <> '' then
      FHeaderClickIndex := StrToInt(strHeaderIndex);
  end;

  if Assigned(OnAsyncHeaderClick) then
    OnAsyncHeaderClick(Self, AParams, FHeaderClickIndex);
end;

procedure TTIWCustomWebPlanner.DoOnAsyncItemClick(AParams: TStringList);
begin
  if Assigned(OnAsyncItemClick) then
    OnAsyncItemClick(Self, AParams, SelectedIndex, Items[SelectedIndex]);
end;

procedure TTIWCustomWebPlanner.DoOnAsyncEditStart(AParams: TStringList);
begin
  FEditStart := true;

  if Assigned(OnAsyncEditStart) then
    OnAsyncEditStart(Self, AParams, FEditStart, SelectedIndex, Items[SelectedIndex]);
end;

procedure TTIWCustomWebPlanner.DoOnAsyncEditDone(AParams: TStringList);
begin
  FEditDone := true;

  if Assigned(OnAsyncEditDone) then
    OnAsyncEditDone(Self, AParams, FEditDone, SelectedIndex, Items[SelectedIndex]);
end;


procedure TTIWCustomWebPlanner.DoOnAsyncItemSize(AParams: TStringList);
var
  ResizeItem : TPlannerItem;
  Temp, strSelection, strParam, TempValue: string;
begin

  Temp := AParams.GetText;

  if (pos('|',Temp) <> 0) then
    Delete(Temp,1,pos('|',Temp));
  FTempValue := Temp;

  strSelection := FTempValue;
  Delete(strSelection,1,pos('|',strSelection));
  Delete(strSelection,1,pos('|',strSelection));

  //Async Parameters
  Delete(strSelection,1,pos('|',strSelection));
  strParam := Copy(strSelection,1,1);

  TempValue := strSelection;

    //Set the OnAsyncItemSize parameters
    if (pos('ResizeStart',TempValue) <> 0 ) then
    begin
      Delete(TempValue,1,15);

      FResizeIndex := StrToInt(Copy(TempValue,1,pos('R',TempValue)-1));
      Delete(TempValue,1,pos('R',TempValue));
      FResizeValue := StrToInt(Copy(TempValue,pos('*',TempValue)+1,pos('|', TempValue) - 3));
      FResizeType := 1;
    end
    else if (pos('ResizeEnd',TempValue) <> 0 ) then
    begin
      Delete(TempValue,1,13);

      FResizeIndex := StrToInt(Copy(TempValue,1,pos('R',TempValue)-1));
      Delete(TempValue,1,pos('R',TempValue));
      FResizeValue := StrToInt(Copy(TempValue,pos('*',TempValue)+1,pos('|', TempValue) - 3));
      FResizeType := 0;
    end;

  if Assigned(OnAsyncItemSize) then
  begin

    ResizeItem := Items[FResizeIndex];

    if FResizeType = 1 then
      OnAsyncItemSize(Self, AParams, FResizeAllow, ResizeItem, ResizeItem.ItemPos,
        ResizeItem.ItemBegin, ResizeItem.ItemEnd, FResizeValue, ResizeItem.ItemEnd)
    else if FResizeType = 0 then
      OnAsyncItemSize(Self, AParams, FResizeAllow, ResizeItem, ResizeItem.ItemPos,
        ResizeItem.ItemBegin, ResizeItem.ItemEnd, ResizeItem.ItemBegin, FResizeValue);
  end;
end;

procedure TTIWCustomWebPlanner.DoOnAsyncItemMove(AParams: TStringList);
var
  MoveItem, Item : TPlannerItem;
  Temp, strSelection, strParam, TempValue: string;
  OldBeginOffset, MouseFrom: integer;
  MoveProperties: TMoveProperties;
begin

  Temp := AParams.GetText;
  MouseFrom := -1;

  if (pos('|',Temp) <> 0) then
    Delete(Temp,1,pos('|',Temp));
  FTempValue := Temp;

  strSelection := FTempValue;
  Delete(strSelection,1,pos('|',strSelection));
  Delete(strSelection,1,pos('|',strSelection));

  //Async Parameters
  Delete(strSelection,1,pos('|',strSelection));
  strParam := Copy(strSelection,1,1);

  TempValue := strSelection;

  //Set the OnAsyncItemMove parameters
  if (pos('Move',TempValue) <> 0) then
  begin
    TempValue := strSelection;
    FMoveIndex := StrToInt(Copy(TempValue,9,pos('*',TempValue) - 9));
    Delete(TempValue,1,pos('*',TempValue));

    FMovePos := StrToInt(Copy(TempValue,1,pos('*',TempValue) - 1));

    if (FMovePos >= Positions) or (FMoveIndex >= FItems.Count) then
      FMoveAllow := false
    else
      FMoveAllow := true;

    Delete(TempValue,1,pos('*',TempValue));

    FMoveBegin := StrToInt(Copy(TempValue,1,pos('*',TempValue) - 1));

    Delete(TempValue,1,pos('*',TempValue));

    MouseFrom := StrToInt(Copy(TempValue,1,pos('|',TempValue) - 1));

    Item := FItems[FMoveIndex];

    FMoveBegin := FMoveBegin - Item.BeginOffset;

    OldBeginOffset := Item.BeginOffset;
    if (FMoveBegin < 0) then
    begin
      Item.BeginOffset := - FMoveBegin;
      FMoveBegin := 0;
    end
    else
      Item.BeginOffset := 0;

    FMoveEnd := Item.ItemEnd - Item.ItemBegin;
    FMoveEnd := FMoveEnd + FMoveBegin + OldBeginOffset - Item.BeginOffset;
    FMoveEnd := FMoveEnd + Item.EndOffset;

    if (FMoveEnd > Display.DisplayEnd - Display.DisplayStart + 1) then
    begin
      if ((FMoveEnd + Display.DisplayStart) * Display.DisplayUnit >= MinInDay) then
      begin
        if (Mode.PlannerType = plDay) then
          Item.InHeader := true;
      end;
        Item.EndOffset := FMoveEnd - (Display.DisplayEnd - Display.DisplayStart + 1);
        FMoveEnd := Display.DisplayEnd - Display.DisplayStart + 1;
    end
    else
      Item.EndOffset := 0;

    if (PositionProps.Count > FMovePos) then
    begin
      if ((PositionProps[FMovePos].MinSelection > 0) or (PositionProps[FMovePos].MaxSelection > 0)) then
      begin
        if (FMoveBegin < PositionProps[FMovePos].MinSelection) then
        begin
          FMoveEnd := FMoveEnd + (PositionProps[FMovePos].MinSelection - FMoveBegin);
          FMoveBegin := PositionProps[FMovePos].MinSelection;
        end;

        if (FMoveEnd > PositionProps[FMovePos].MaxSelection) then
        begin
          FMoveBegin := FMoveBegin - (FMoveEnd - PositionProps[FMovePos].MaxSelection) + 1;
          FMoveEnd := PositionProps[FMovePos].MaxSelection + 1;
        end;
      end;
    end;

    if FItems[FMoveIndex].FixedPosition then
      FMovePos := FItems[FMoveIndex].ItemPos;
  end;

  if Assigned(OnAsyncItemMove) then
  begin
    MoveItem := Items[FMoveIndex];

    OnAsyncItemMove(Self, AParams, FMoveAllow, MoveItem, MoveItem.ItemBegin,
      MoveItem.ItemEnd, MoveItem.ItemPos,FMoveBegin, FMoveEnd, FMovePos);
  end;


 if Assigned(OnAsyncItemMoveEx) then
  begin
    MoveItem := Items[FMoveIndex];

    FShowDialog := false;

    FDialogText := 'Confirm to move the event ' + MoveItem.CaptionText + ' from ' + AbsTimeToStr(MoveItem.ItemStartTime);

    if MoveItem.ItemPos <> FMovePos then
    begin
      FDialogText := FDialogText + ' for ';
      if MoveItem.ItemPos < FPlannerHeader.Captions.Count - 1 then
        FDialogText := FDialogText + FPlannerHeader.Captions[MoveItem.ItemPos + 1]
      else
        FDialogText := FDialogText + 'Position ' + IntToStr(MoveItem.ItemPos + 1);
    end;

    FDialogText := FDialogText + ' to ' + AbsTimeToStr(CellToAbsTime(FMoveBegin));

    if MoveItem.ItemPos <> FMovePos then
    begin
      FDialogText := FDialogText + ' for ';
      if FMovePos < FPlannerHeader.Captions.Count - 1 then
        FDialogText := FDialogText + FPlannerHeader.Captions[FMovePos + 1]
      else
        FDialogText := FDialogText + 'Position ' + IntToStr(FMovePos + 1);
    end;

    FDialogText := FDialogText + '?';

    if (MouseFrom >= 0) and (FSideBar.Orientation = soHorizontal) and (RelativeItemMove) then
    begin
      MoveProperties.MouseFrom := MoveItem.ItemBegin + (MouseFrom - FMoveBegin);
      MoveProperties.MouseTo := MouseFrom;
    end
    else
    begin
      MoveProperties.MouseFrom := MoveItem.ItemBegin;
      MoveProperties.MouseTo := FMoveBegin;
    end;
    MoveProperties.FromBegin := MoveItem.ItemBegin;
    MoveProperties.FromEnd := MoveItem.ItemEnd;
    MoveProperties.FromPos := MoveItem.ItemPos;
    MoveProperties.ToBegin := FMoveBegin;
    MoveProperties.ToEnd := FMoveEnd;
    MoveProperties.ToPos := FMovePos;

    OnAsyncItemMoveEx(Self, AParams, FMoveAllow, MoveItem, MoveProperties, FShowDialog, FDialogText);
  end;
end;

procedure TTIWCustomWebPlanner.DoOnAsyncItemDelete(AParams: TStringList);
var
  DeleteItem : TPlannerItem;
  Temp, strSelection, strParam, TempValue: string;
begin

  Temp := AParams.GetText;

  if (pos('|',Temp) <> 0) then
    Delete(Temp,1,pos('|',Temp));
  FTempValue := Temp;

  strSelection := FTempValue;
  Delete(strSelection,1,pos('|',strSelection));
  Delete(strSelection,1,pos('|',strSelection));

  //Async Parameters
  Delete(strSelection,1,pos('|',strSelection));
  strParam := Copy(strSelection,1,1);

  TempValue := strSelection;

  if (pos('Delete',TempValue) <> 0) then
  begin
    Delete(TempValue,1,6);
    FDeleteIndex := StrToInt(Copy(TempValue,1,pos('|',TempValue)-1));
  end;

  if Assigned(OnAsyncItemDelete) then
  begin

    DeleteItem := Items[FDeleteIndex];

    OnAsyncItemDelete(Self, AParams, FDeleteAllow, DeleteItem);
  end;
end;

function TTIWCustomWebPlanner.RenderAsync(AContext: TIWBaseHTMLComponentContext): TIWXMLTag;
var
 js : string;
begin
  Result := TIWXMLTag.CreateTag('control');
  try
    Result.AddStringParam('id', HTMLName);
    Result.AddStringParam('type', 'TTIWWEBPLANNER');
    RenderAsyncCommonProperties(AContext, Result);

    if FEditStart then
    begin
      FEditStart := false;

        js :=
          '<![CDATA['
          + 'SubmitClick('#39+HTMLName+#39','#39+ 'Select' + IntToStr(FSelectedIndex) + #39',false);'
          +']]>';

        GGetWebApplicationThreadVar.CallBackResponse.AddJavaScriptToExecute(js);
    end;

    if FEditDone then
    begin
      FEditDone := false;

        js :=
          '<![CDATA['
          + 'SubmitClick('#39 + HTMLName + #39',"UpdEdit' + IntToStr(FSelectedIndex) + '",false)'#13
          +']]>';

        GGetWebApplicationThreadVar.CallBackResponse.AddJavaScriptToExecute(js);
    end;

    if FResizeAllow and (FResizeIndex > -1) and (FResizeIndex > -1) and (FResizeType > -1) then
    begin
      if FResizeType = 1 then
      begin
        js :=
          '<![CDATA['
            + 'SubmitClick('#39+HTMLName+#39',"ResizeStartITEM' + IntToStr(FResizeIndex)
            + 'RT*' + IntToStr(FResizeValue) + '",false)'
          +']]>';

        GGetWebApplicationThreadVar.CallBackResponse.AddJavaScriptToExecute(js);
      end
      else if FResizeType = 0 then
      begin
        js :=
          '<![CDATA['
            + 'SubmitClick('#39+HTMLName+#39',"ResizeEndITEM' + IntToStr(FResizeIndex)
            + 'RD*' + IntToStr(FResizeValue) + '",false)'
          +']]>';

        GGetWebApplicationThreadVar.CallBackResponse.AddJavaScriptToExecute(js);
      end;

      FResizeIndex := -1;
      FResizeValue := -1;
      FResizeType := -1;

    end
    else
    begin
      FResizeAllow := true;
      FResizeIndex := -1;
      FResizeValue := -1;
      FResizeType := -1;
    end;

      if FMoveIndex > -1 then
      begin
        js := '<![CDATA[';

        if Assigned(OnAsyncItemMoveEx) then
          js := js
            + '  if (El(tempDiv + "D"))'#13
            + '    El(tempDiv + "D").style.visibility = "hidden";'#13;

        if FMoveAllow then
        begin
          if FShowDialog then
            js := js + 'if (confirm("' + FDialogText + '"))'#13;

          js := js
              + '  SubmitClick('#39+HTMLName+#39',"MoveITEM' + IntToStr(FMoveIndex)
              + '*' + IntToStr(FMovePos) + '*' + IntToStr(FMoveBegin) + '",false)';
        end;

        js := js +']]>';

        GGetWebApplicationThreadVar.CallBackResponse.AddJavaScriptToExecute(js);

        FMoveIndex := -1;
    end
    else
    begin
      FMoveAllow := false;
      FMoveIndex := -1;
    end;

    if FDeleteAllow then
    begin
      if FDeleteIndex > -1 then
      begin

        js :=
          '<![CDATA['
            + 'SubmitClick('#39+HTMLName+#39',"Delete' + IntToStr(FDeleteIndex) + '",false)'
          +']]>';

        GGetWebApplicationThreadVar.CallBackResponse.AddJavaScriptToExecute(js);

        FDeleteIndex := -1;
      end
    end
    else
    begin
      FDeleteAllow := false;
    end;

  except
    FreeAndNil(Result);
  end;
end;

constructor TTIWCustomWebPlanner.Create(AOwner: TComponent);
begin
  inherited;
  FBackground := TIPicture.Create;
  FBands := TBands.Create(Self);
  FDayNames := TStringList.Create;
  FDayNames.Add(ShortDaySun);
  FDayNames.Add(ShortDayMon);
  FDayNames.Add(ShortDayTue);
  FDayNames.Add(ShortDayWed);
  FDayNames.Add(ShortDayThu);
  FDayNames.Add(ShortDayFri);
  FDayNames.Add(ShortDaySat);
  FDisplay := TPlannerDisplay.Create(Self);
  FEnableSelection := true;
  FEditType := edPopup;
  FDTList := TDateTImeList.Create;
  FFont := TIWFont.Create;
  FItemGap := 10;
//  FItems := TPlannerItems.Create(Self);
  FItems := CreateItems;
  FMode := TPlannerMode.Create(Self);
  FLinkUpdate := False;
  FPopupEdit := TPopupEdit.Create(Self);
  FPlannerHeader := TPlannerHeader.Create(Self);
  FPositionProps := TPositionProps.Create(Self);
  FAutoInsDel := True;
  FAutoCreateOnSelect := True;

  {$IFNDEF TMSIW121}
  SetRenderSize(True);
  {$ENDIF}
//  FRenderSize := True;
//  FSupportsSubmit := True;
//  FSupportsInput := True;

  FSideBar := TPlannerSideBar.Create(Self);
  FSideBar.Width := 40;
  FPositions := 3;
  FClientEvents := TIWWebPlannerClientEvents.Create;
  FDefaultItems := CreateItems;
  FDefaultItem := FDefaultItems.Add;
  FDefaultItem.CaptionType := ctTimeText;
  FDefaultItem.CaptionBkg := clNavy;
  FDefaultItem.CaptionBkgTo := clBlue;
  FDefaultItem.Color := clWhite;
  FDefaultItem.FAllowOverlap := true;
  FDefaultItem.FAllowDelete := true;
//  FDefaultItem.ShowCaption := true;
  FDefaultItem.FVisible := true;
  FDefaultItem.FFont.FontName := 'Verdana';
  FDefaultItem.FFont.Size := 10;
  FDefaultItem.CaptionFont.FontName := 'Verdana';
  FDefaultItem.CaptionFont.Color := clWhite;
  FDefaultItem.CaptionFont.Style := [fsBold];
  FDefaultItem.CaptionFont.Size := 8;
  FDefaultItem.FFont.Style := [fsItalic];
  FGlyphs := TGlyphs.Create(Self);
  FHints := THints.Create(Self);

  if (csDesigning in ComponentState) and not ((csReading in Owner.ComponentState) or (csLoading in Owner.ComponentState)) then
  begin
    FHints.HintCancelItem := 'Cancel item';
    FHints.HintConfirmDialog := 'Do you want to add an item here?';
    FHints.HintDeleteItem := 'Delete item';
    FHints.FHintEditItem := 'Click here to make the item editable';
    FHints.FHintUpdateItem := 'Update item';
  end;

  FShowConfirm := true;
  FBorderSize := 1;
  FBorderCollapsed := True;
  FSelectionColor := clHighLight;
  FInActiveDays := TWeekDays.Create;
  FInActiveDays.OnChanged := InActiveChanged;
  FOverlap := true;
  FOverlapColor := clWhite;
  Width := 500;
  FAlertOutsidePlanner := 'An item cannot be dropped outside the planner.';
  FShowAlertOutsidePlanner := true;
  FSelectedIndex := -1;
  FChangeCaption := TChangeCaption.Create(Self);
  FViewOnly := false;
  FSelectionMode := smMultiSelect;
  FHints.CaptionHint := chCaption;
  Self.Height:= 500;

  FEditIndex := -1;

  FHeaderClickIndex := -1;
  FResizeType := -1;
  FResizeAllow := true;
  FResizeIndex := -1;
  FResizeValue := -1;
  FMoveAllow := false;
  FMoveIndex := -1;
  FMoveBegin := -1;
  FMoveEnd := -1;
  FMovePos := -1;
  FEditStart := false;
  FEditDone := false;
  FDeleteIndex := -1;
  FDeleteAllow := true;
end;

destructor TTIWCustomWebPlanner.Destroy;
begin
  FBackground.Free;
  FBands.Free;
  FChangeCaption.Free;
  FClientEvents.Free;
  FDayNames.Free;
  FDefaultItem.Free;
  FDefaultItem := nil;
  FDefaultItems.Free;
  FDisplay.Free;
  FDTList.Free;
  FGlyphs.Free;
  FPopupEdit.Free;
  FHints.Free;
  FItems.Free;
  FFont.Free;
  FMode.Free;
  FPositionProps.Free;
  FSideBar.Free;
  FInActiveDays.Free;
  FPlannerHeader.Free;
  inherited;
end;

procedure TTIWCustomWebPlanner.DoCancel;
begin
end;

procedure TTIWCustomWebPlanner.DoPost;
begin
end;

procedure TTIWCustomWebPlanner.ProcessTemplateField(APlannerItem : TPlannerItem; fld:string; var DBFld:string);
begin
  if Assigned(OnTemplateData) then
    OnTemplateData(self,APlannerItem,fld,DBFld);
end;

procedure TTIWcustomWebPlanner.SetPopupEdit(const value : TPopupEdit);
begin
  FPopupEdit.Assign(Value);
end;

procedure TTIWCustomWebPlanner.SetAlertOutsidePlanner(const value: string);
begin
  FAlertOutsidePlanner := Value;
end;

procedure TTIWCustomWebPlanner.SetPositionProps(const Value: TPositionProps);
begin
  FPositionProps.Assign(Value);
end;

procedure TTIWCustomWebPlanner.SetDisplay(const Value: TPlannerDisplay);
begin
  FDisplay := Value;
end;

procedure TTIWCustomWebPlanner.SetSideBar(const Value: TPlannerSidebar);
begin
  FSideBar := Value;
end;

procedure TTIWCustomWebPlanner.SetItems(const Value: TPlannerItems);
begin
  FItems.Assign(Value);
end;

procedure TTIWCustomWebPlanner.SetOverlapColor(const Value: TIWColor);
begin
  FOverlapColor := Value;
end;

procedure TTIWCustomWebPlanner.SetDefaultItem(const Value: TPlannerItem);
begin
  FDefaultItem.Assign(Value);
end;

procedure TTIWCustomWebPlanner.SetFont(const Value: TIWFont);
begin
  FFont.Assign(Value);
end;

procedure TTIWCustomWebPlanner.SetHints(const Value: THints);
begin
  FHints.Assign(Value);
end;

procedure TTIWCustomWebPlanner.SetShowAlertOutsidePlanner(const Value: Boolean);
begin
  FShowAlertOutsidePlanner := Value;
end;

procedure TTIWCustomWebPlanner.SetMode(const Value: TPlannerMode);
begin
  FMode := Value;
end;

procedure TTIWCustomWebPlanner.SetBorderCollapsed(const Value: Boolean);
begin
  FBorderCollapsed := Value;
end;

procedure TTIWCustomWebPlanner.SetPositionGroup(const Value: Integer);
begin
  FPositionGroup := Value;
  InvalidateX;
end;

procedure TTIWCustomWebPlanner.Submit(const Value: String);
var
  temp,s,ItBegin,ItEnd,ItPos,ItToMove,ItToResize,ChangedValue: integer;
  textItem, CaptionItem, TempValue, TempV: string;
  Item,plIt : TPlannerItem;
  OldBeginOffset : integer;
  ShowEditor : boolean;
  FOldPos,FOldBegin,FOldEnd: integer;
begin

  if (pos('Delete',Value) <> 0) then
  begin
    temp := StrToInt(Copy(Value,7,length(Value)-6));

    if Assigned(OnItemDelete) then
      OnItemDelete(Self,FItems[temp]);

    if AutoInsDel then
    begin
      if Assigned(OnItemDeleted) then
        OnItemDeleted(Self,FItems[temp]);
      FreeItem(FItems[temp]);
      //FItems[temp].Free;
    end;
    Exit;
  end;

  if Pos('PostRow',Value) > 0 then
  begin
    DoPost;
    Exit;
  end;

  if Pos('CancelRow',Value) > 0 then
  begin
    DoCancel;
    Exit;
  end;

  if (pos('Select', Value) <> 0) then
  begin
    Items.Select(FItems[FSelectedIndex]);
    StopEdit;

    if FSelectedIndex > -1 then
    begin
      if Assigned(FOnItemClick) then
      begin
        ShowEditor := true;
        FOnItemClick(Self,FSelectedIndex,FItems[FSelectedIndex],ShowEditor);
        if (ShowEditor) and (FSelectedIndex > -1) then
        begin
          FEditIndex := FSelectedIndex;
          StartEdit(FItems[FSelectedIndex]);
        end;
      end
      else
      begin
        if (FSelectedIndex > -1) then
        begin
          FEditIndex := FSelectedIndex;
          StartEdit(FItems[FSelectedIndex]);
        end;
      end;
    end;

    Exit;
  end;

  if (pos('ChangeCaption', Value) <> 0) then
  begin
    TempV := Copy(Value,19,Length(Value) - 18);
    Temp := StrToInt(TempV);
    TempV := FTempValue;
    CaptionItem := '';

    if (pos('|',TempV) <> 0) then
       Delete(TempV,1,pos('|',TempV));

    if (pos('|',TempV) <> 0) then
      Delete(TempV,1,pos('|',TempV));

    if (pos('|',TempV) <> 0) then
      Delete(TempV,1,pos('|',TempV));

    if (pos('|',TempV) <> 0) then
    begin
      Delete(TempV,1,pos('|',TempV));
      CaptionItem := TempV;
    end;

    FItems[Temp].CaptionText := CaptionItem;

    if Assigned(OnItemCaptionChange) then
      OnItemCaptionChange(Self,FItems[Temp]);
    Exit;
  end;

  if (pos('PopupEd', Value) <> 0) then
  begin
    TextItem := '';
    CaptionItem := '';

    if (FSelectedIndex > -1) then
    begin
      for s:=0 to FItems[FSelectedIndex].Text.Count-1 do
        FItems[FSelectedIndex].Text.Delete(0);

      TempV := FTempValue;
      if (pos('|',TempV) <> 0) then
         Delete(TempV,1,pos('|',TempV));

      if (pos('|',TempV) <> 0) then
        Delete(TempV,1,pos('|',TempV));

      if (pos('|',TempV) <> 0) then
        Delete(TempV,1,pos('|',TempV));

      if (pos('|',TempV) <> 0) then
      begin
        TextItem := Copy(TempV,1,pos('|',TempV) - 1);

        while (pos(#13#10,TextItem) <> 0) do
        begin
          FItems[FSelectedIndex].Text.Add(Copy(TextItem, 0, pos(#13#10,TextItem) - 1));
          Delete(TextItem,1,pos(#13#10, TextItem));
        end;

        Delete(TempV,1,pos('|',TempV));
        CaptionItem := TempV;
      end;

      FItems[FSelectedIndex].Text.Add(TextItem);
      FItems[FSelectedIndex].CaptionText := CaptionItem;

      if Assigned(OnItemEdit) then
        OnItemEdit(Self, FItems[FSelectedIndex]);

      ItemChanged(FItems[FSelectedIndex]);

      Exit;
    end;
  end;

  if (pos('UpdEdit', Value) <> 0) then
  begin
    TextItem := '';

    FSelectedIndex := StrToInt(Copy(Value,8,length(Value)-7));

    for s:=0 to FItems[FSelectedIndex].Text.Count-1 do
      FItems[FSelectedIndex].Text.Delete(0);

    TempV := FTempValue;

    if (pos('|',TempV) <> 0) then
       Delete(TempV,1,pos('|',TempV));

    if (pos('|',TempV) <> 0) then
      Delete(TempV,1,pos('|',TempV));

    if (pos('|',TempV) <> 0) then
      Delete(TempV,1,pos('|',TempV));

    if (pos('|',TempV) <> 0) then
    begin
      TextItem := Copy(TempV,1,pos('|',TempV) - 1);

      while(pos(#13#10,TextItem) <> 0) do
      begin
        FItems[FSelectedIndex].Text.Add(Copy(TextItem,0,pos(#13#10,TextItem) - 1));
        Delete(TextItem,1,pos(#13#10, TextItem));
      end;
      Delete(TempV,1,pos('|',TempV));
      CaptionItem := TempV;
    end
    else
    begin
      TextItem := TempV;

      while(pos(#13#10,TextItem) <> 0) do
      begin
        FItems[FSelectedIndex].Text.Add(Copy(TextItem,0,pos(#13#10,TextItem) - 1));
        Delete(TextItem,1,pos(#13#10, TextItem));
      end;
    end;

    FItems[FSelectedIndex].Text.Add(TextItem);
    FItems[FSelectedIndex].EditMode := false;

    if Assigned(OnItemEdit) then
      OnItemEdit(Self, FItems[FSelectedIndex]);

    ItemChanged(FItems[FSelectedIndex]);

    Exit;
  end;

  if (pos('CancelEdit', Value) <> 0) then
  begin
    temp := StrToInt(Copy(Value,11,length(Value)-10));
    FItems[temp].FEditMode := false;
    FEditIndex := -1;
    Exit;
  end;

  if (pos('AddSingleI',Value) <> 0 ) then
  begin
    TempValue := FTempValue;
    TempV := Copy(TempValue,2,pos('|',TempValue)-2);
    SelItemBegin := StrToInt(TempV);
    SelItemEnd := SelItemBegin + 1;
    Delete(TempValue,1,pos('|',TempValue));
    if (pos('|',TempValue) <> 0) then
    begin
      TempV := Copy(TempValue,2,pos('|',TempValue) - 2);
      SelPosition := StrToInt(TempV);
    end
    else
      SelPosition := 0;

    OnPlannerDblClick(Self,SelItemBegin, SelPosition);
  end;

  if (pos('AddItem',Value) <> 0 ) then
  begin
    CaptionItem := DefaultItem.CaptionText;
    TempValue := FTempValue;
    TempV := Copy(TempValue,2,pos('|',TempValue)- 2);
    SelItemBegin := StrToInt(TempV);

    Delete(TempValue,1,pos('|',TempValue));
    TempV := Copy(TempValue,2,pos('|', TempValue) - 2);
    SelItemEnd := StrToInt(TempV ) + 1;

    Delete(TempValue, 1, pos('|', TempValue));
    if (pos('|',TempValue) <> 0) then
    begin
      TempV := Copy(TempValue, 2 , pos('|', TempValue) - 2);
      Delete(TempValue, 1 , pos('|', TempValue));
      Delete(TempValue, 1 , pos('|', TempValue));
      if (TempValue <> '') then
        CaptionItem := TempValue;
    end
    else
      TempV := Copy(TempValue, 2 , Length(TempValue) - 1);

    SelPosition := StrToInt(TempV);
    SelItemCaption := CaptionItem;

    if Assigned(OnItemCreate) then
      OnItemCreate(Self, SelItemBegin, SelItemEnd, SelPosition);

    if AutoInsDel then
    begin
      plIt := CreateItemAtSelection;
      if Assigned(OnItemCreated) then
        OnItemCreated(Self,plIt);
    end;
  end;

  if (pos('Move',Value) <> 0) then
  begin
    TempValue := Value;
    ItToMove := StrToInt(Copy(TempValue,9,pos('*',TempValue) - 9));
    Delete(TempValue,1,pos('*',TempValue));

    ItPos := StrToInt(Copy(TempValue,1,pos('*',TempValue) - 1));
    if ItPos >= Positions then
    begin
      raise Exception.Create('Invalid position after item move');
    end;

    if ItToMove >= FItems.Count then
      raise Exception.Create('Invalid item index for move');

    Delete(TempValue,1,pos('*',TempValue));

    ItBegin := StrToInt(Copy(TempValue,1,length(TempValue)));

    Item := FItems[ItToMove];

    ItBegin := ItBegin - Item.BeginOffset;

    OldBeginOffset := Item.BeginOffset;
    if (ItBegin < 0) then
    begin
      Item.BeginOffset := - ItBegin;
      ItBegin := 0;
    end
    else
      Item.BeginOffset := 0;

    ItEnd := Item.ItemEnd - Item.ItemBegin;
    ItEnd := ItEnd + ItBegin + OldBeginOffset - Item.BeginOffset;
    ItEnd := ItEnd + Item.EndOffset;

    if (ItEnd > Display.DisplayEnd - Display.DisplayStart + 1) then
    begin
      if ((ItEnd + Display.DisplayStart) * Display.DisplayUnit >= MinInDay) then
      begin
        if (Mode.PlannerType = plDay) then
          Item.InHeader := true;
      end;
        Item.EndOffset := ItEnd - (Display.DisplayEnd - Display.DisplayStart + 1);
        ItEnd := Display.DisplayEnd - Display.DisplayStart + 1;
    end
    else
      Item.EndOffset := 0;

    if (PositionProps.Count > ItPos) then
    begin
      if ((PositionProps[ItPos].MinSelection > 0) or (PositionProps[ItPos].MaxSelection > 0)) then
      begin
        if (ItBegin < PositionProps[ItPos].MinSelection) then
        begin
          ItEnd := ItEnd + (PositionProps[ItPos].MinSelection - ItBegin);
          ItBegin := PositionProps[ItPos].MinSelection;
        end;

        if (ItEnd > PositionProps[ItPos].MaxSelection) then
        begin
          ItBegin := ItBegin - (ItEnd - PositionProps[ItPos].MaxSelection) + 1;
          ItEnd := PositionProps[ItPos].MaxSelection + 1;
        end;
      end;
    end;

    if FItems.HasItemEx(FItems[ItToMove],ItBegin,ItEnd,ItPos) then
    begin
      if Assigned(OnFailedMove) then
        OnFailedMove(Self, FItems[ItToMove],FItems[ItToMove].ItemBegin, FItems[ItToMove].ItemEnd, FItems[ItToMove].ItemPos,ItBegin,ItEnd,ItPos);
      Exit;
    end;

    if FItems[ItToMove].FixedPosition then
      ItPos := FItems[ItToMove].ItemPos;

    FOldPos := FItems[ItToMove].ItemPos;
    FOldBegin := FItems[ItToMove].ItemBegin;
    FOldEnd := FItems[ItToMove].ItemEnd;

    FItems[ItToMove].ItemBegin := ItBegin;
    FItems[ItToMove].ItemEnd := ItEnd;
    FItems[ItToMove].ItemPos := ItPos;

    if Assigned(OnItemMove) then
      OnItemMove(Self, FItems[ItToMove],FOldBegin, FOldEnd, FOldPos,ItBegin,ItEnd,ItPos);

    ItemChanged(FItems[ItToMove]);
  end;

  if (pos('Resize',Value) <> 0 ) then
  begin
    TempValue := Value;

    if pos('ResizeStart',Value) <> 0 then
      Delete(TempValue,1,15)
    else
      Delete(TempValue,1,13);

    ItToResize := StrToInt(Copy(TempValue,1,pos('R',TempValue)-1));
    Delete(TempValue,1,pos('R',TempValue));
    ChangedValue := StrToInt(Copy(TempValue,pos('*',TempValue)+1,length(TempValue)));

    if (pos('ResizeStart',Value) <> 0) then
    begin
      if FItems.HasItemEx(FItems[ItToResize],ChangedValue,FItems[ItToResize].ItemEnd,FItems[ItToResize].ItemPos) then
      begin
        if Assigned(OnFailedSize) then
          OnFailedSize(Self, FItems[ItToResize],FItems[ItToResize].ItemPos,FItems[ItToResize].ItemBegin,FItems[ItToResize].ItemEnd,ChangedValue,FItems[ItToResize].ItemEnd);
        Exit;
      end;

      FItems[ItToResize].ItemBegin := ChangedValue;

      if Assigned(FOnItemSize) then
        OnItemSize(Self, FItems[ItToResize],FItems[ItToResize].ItemPos,FItems[ItToResize].ItemBegin,FItems[ItToResize].ItemEnd,ChangedValue,FItems[ItToResize].ItemEnd);
    end
    else
    begin
      if (FItems[ItToResize].ItemBegin = ChangedValue) then
        ChangedValue := ChangedValue + 1;

      if FItems.HasItemEx(FItems[ItToResize],FItems[ItToResize].ItemBegin,ChangedValue,FItems[ItToResize].ItemPos) then
      begin
        if Assigned(OnFailedSize) then
          OnFailedSize(Self, FItems[ItToResize],FItems[ItToResize].ItemPos,FItems[ItToResize].ItemBegin,FItems[ItToResize].ItemEnd,FItems[ItToResize].ItemBegin,ChangedValue);
        Exit;
      end;

      FItems[ItToResize].ItemEnd := ChangedValue;

      if Assigned(FOnItemSize) then
        OnItemSize(Self, FItems[ItToResize],FItems[ItToResize].ItemPos,FItems[ItToResize].ItemBegin,FItems[ItToResize].ItemEnd,FItems[ItToResize].ItemBegin,ChangedValue);
    end;
    ItemChanged(FItems[ItToResize]);
  end;

  if (pos('HeaderClick',Value) <> 0 ) then
  begin
    TempValue := Value;
    Delete(TempValue,1,pos('#',TempValue));
    if Assigned(OnHeaderClick) then
      OnHeaderClick(Self,StrToInt(TempValue));
  end;
end;

procedure TTIWCustomWebPlanner.SetGlyphs(const Value: TGlyphs);
begin
  FGlyphs.Assign(Value);
end;

procedure TTIWCustomWebPlanner.SetBackgroundImage(const Value: TIPicture);
begin
end;

procedure TTIWCustomWebPlanner.SetBorderColor(const Value: TIWColor);
begin
  FBorderColor := Value;
end;

procedure TTIWCustomWebPlanner.SetBorderSize(const Value: integer);
begin
  FBorderSize := Value;
end;

procedure TTIWCustomWebPlanner.SetClientEvents(const Value: TIWWebPlannerClientEvents);
begin
  FClientEvents.Assign(Value);
end;

procedure TTIWCustomWebPlanner.GetSideBarLines(Index, Position: Integer; var Line1,
  Line2, Line3: string;var HS: Boolean);
var
  Mins: Integer;
  dt: TDatetime;
begin
  Line1 := '';
  Line2 := '';
  Line3 := '';

  case FMode.FPlannerType of
    plDay,plTimeLine:
      begin
        Mins := (Index + FDisplay.DisplayStart) * FDisplay.DisplayUnit
          + FDisplay.DisplayOffset;

        HS := (Mins div 60) <> ((Mins - FDisplay.DisplayUnit) div 60);

        PlanTimeToStrings(Mins, Line1, Line2, Line3);
      end;
    plWeek:
      begin
        if FSideBar.ShowDayName then
          Line2 := GetDayName(Index - FMode.WeekStart);
        if FSideBar.FDateTimeFormat <> '' then
          Line2 := Line2 + ' ' + FormatDateTime(FSideBar.FDateTimeFormat,
          FMode.StartOfMonth + Index);
      end;
    plMonth:
      begin
        if FSideBar.ShowDayName then
          Line1 := GetDayName(DayOfWeek(Index + FMode.StartOfMonth));
        Line2 := FormatDateTime(FSideBar.FDateTimeFormat,
          FMode.StartOfMonth + Index);
      end;
    plDayPeriod:
      begin
        if FSideBar.ShowDayName then
          Line1 := GetDayName(DayOfWeek(Index + FMode.PeriodStartDate));
        Line2 := FormatDateTime(FSideBar.DateTimeFormat,
          FMode.PeriodStartDate + Index);
      end;
    plHalfDayPeriod:
    begin
      if FSideBar.ShowDayName then
        Line1 := GetDayName(DayOfWeek(Trunc(Index / 2) + FMode.PeriodStartDate));
      Line2 := FormatDateTime(FSideBar.DateTimeFormat,
        Trunc(FMode.PeriodStartDate + Index / 2));
      if Odd(Index) then
      Line3 := 'PM' else Line3 := 'AM';
    end;
    plMultiMonth:
    begin
      if Index < 31 then
        Line2 := IntTostr(Index + 1);
    end;
    plCustom, plCustomList:
    begin
      dt := IndexToTime(Index);
      Line1 := FormatDateTime(FSidebar.DateTimeFormat,dt);
    end;
  end;
  if Assigned(FOnPlannerGetSideBarLines) then
  begin
    FOnPlannerGetSideBarLines(Self, Index, Position, Line1, Line2, Line3);
    HS := Line1 <> '';
  end;
end;

function TTIWCustomWebPlanner.CreateItem: TPlannerItem;
begin
  Result := Items.Add;
end;

function TTIWCustomWebPlanner.CreateItems: TPlannerItems;
begin
  Result := TPlannerItems.Create(Self);
end;

function TTIWCustomWebPlanner.AbsTimeToStr(DateTime: TDateTime): string;
begin
  case Mode.PlannerType of
    plDay:
      Result := TimeToStr(DateTime);
    plWeek:
      Result := DateTimeToStr(DateTime);
    plMonth:
      Result := DateTimeToStr(DateTime);
    plDayPeriod:
      Result := TimeToStr(DateTime);
    plHalfDayPeriod:
      Result := TimeToStr(DateTime);
    plMultiMonth:
      Result := DateTimeToStr(DateTime);
    plCustom, plCustomList:
      Result := TimeToStr(DateTime);
  end;
end;

function TTIWCustomWebPlanner.CellToAbsTime(Index: integer): TDateTime;
var
  res: Integer;
  dtStart: TDateTime;
  flg: Boolean;
begin

  dtStart := 0;

  case Mode.PlannerType of
    plDay:
      begin
        res := ((Index + Display.DisplayStart) * Display.DisplayUnit)
          + Display.DisplayOffset;

        flg := False;
        while res >= 1440 do
        begin
          res := res - 1440;
          flg := True;
        end;

        if (res div 60 < 24) then
          dtStart := PosToDay(SelPosition) + EncodeTime(res div 60, res mod 60,
            0, 0);

        res := res + Display.DisplayUnit;

        while res >= 1440 do
          res := res - 1440;

        if flg then
        begin
          dtStart := dtStart + 1;
        end;

      end;
    plWeek:
      begin
        dtStart := Mode.StartOfMonth + Index;
      end;
    plMonth:
      begin
        dtStart := Mode.StartOfMonth + Index;
      end;
    plDayPeriod:
      begin
        dtStart := Mode.PeriodStartDate + Index;
      end;
    plHalfDayPeriod:
      begin
        dtStart := Mode.PeriodStartDate + Index / 2;
      end;
    plMultiMonth:
      begin
        dtStart := EncodeDate(Mode.Year, Mode.Month, 1);

        if Index < DaysInMonth(Mode.Month, Mode.Year) then
          dtStart := EncodeDate(Mode.Year, Mode.Month, Index + 1);
      end;
    plCustom, plCustomList:
      begin
        dtStart := IndexToTime(Index);
      end;
  end;

  result := dtStart;
end;

function TTIWCustomWebPlanner.CloneItem(Item: TPlannerItem): TPlannerItem;
begin
  Result := Items.Add;
  Result.Assign(Item);
  Result.RelationShip := irChild;
  Item.RelationShip := irParent;
  Item.ParentIndex := Item.Index;
  Result.ParentIndex := Item.Index;
end;

procedure TTIWCustomWebPlanner.SetViewOnly(const Value : Boolean);
begin
  FViewOnly := Value;
end;

procedure TTIWCustomWebPlanner.RemoveClones(Item: TPlannerItem);
var
  i, idx: Integer;
begin
  idx := Item.ParentIndex;
  i := 0;
  while (i < Items.Count) do
  begin
    if (Items[i].ParentIndex = idx) and (Items[i].RelationShip = irChild) then
      Items[i].Free
    else
      inc(i);
  end;
end;

function TTIWCustomWebPlanner.CreateItemAtSelection: TPlannerItem;
var
  NoCreation : boolean;
begin
  NoCreation := Items.HasItem(SelItemBegin, SelItemEnd, SelPosition);

  if not NoCreation then
  begin
    Result := Items.Add;
    Result.ItemBegin := SelItemBegin;
    Result.ItemEnd := SelItemEnd;
    Result.ItemPos := SelPosition;

    if (ChangeCaption.EnterCaptionAfterItemCreate) then
      Result.CaptionText := SelItemCaption;
  end
  else
  begin
    if Assigned(OnFailedInsert) then
      OnFailedInsert(Self,SelItemBegin, SelItemEnd,SelPosition);

    Result := nil;
  end;
end;

function TTIWCustomWebPlanner.GetCellColor(Pos,Index: Integer): TIWColor;
var
  ABrush: TBrush;
begin
  ABrush := TBrush.Create;
  GetCellBrush(Pos,Index,ABrush);
  Result := ABrush.Color;
  ABrush.Free;
end;

procedure TTIWCustomWebPlanner.GetCellBrush(Pos,Index: Integer; ABrush: TBrush);
var
  DateTime: TDateTime;
  ActiveStart,ActiveEnd,M,Y: Integer;
  ColorActive,ColorNonActive: TIWColor;
  UsePP: Boolean;

begin
  UsePP := (PositionProps.Count > Pos) and (Pos >= 0);

  if UsePP then
    UsePP := PositionProps.Items[Pos].Use;

  if UsePP then
  begin
    ColorActive := PositionProps.Items[Pos].ColorActive;
    ColorNonActive := PositionProps.Items[Pos].ColorNonActive;
    ActiveStart := PositionProps.Items[Pos].ActiveStart;
    ActiveEnd := PositionProps.Items[Pos].ActiveEnd;
  end
  else
  begin
    ColorActive := Self.Display.ColorActive;
    ColorNonActive := Self.Display.ColorNonActive;
    ActiveStart := FDisplay.ActiveStart;
    ActiveEnd := FDisplay.ActiveEnd;
  end;

  if FBands.Show then
  begin
    if Odd(Index) then
      ColorActive := FBands.ActivePrimary
    else
      ColorActive := FBands.ActiveSecondary;

    if Odd(Index) then
      ColorNonActive := FBands.NonActivePrimary
    else
      ColorNonActive := FBands.NonActiveSecondary;
  end;

  ABrush.Style := bsSolid;
  ABrush.Color := ColorActive;

  case FMode.FPlannerType of
    plDay:
      begin
        if (Index < ActiveStart) or (Index >= ActiveEnd) then
          ABrush.Color := ColorNonActive;
      end;
    plWeek:
      begin
        if ((Index - FMode.WeekStart) mod DaysPerWeek) in [0, 1] then
          ABrush.Color := ColorNonActive;
      end;
    plMonth:
      begin
        DateTime := Mode.StartOfMonth + Index;
        if DayOfWeek(DateTime) in FInactive then
          ABrush.Color := ColorNonActive;
      end;
    plDayPeriod:
      begin
        DateTime := Mode.PeriodStartDate + Index;
        if DayOfWeek(DateTime) in FInactive then
          ABrush.Color := ColorNonActive;
      end;
    plHalfDayPeriod:
      begin
        DateTime := Mode.PeriodStartDate + Index / 2;
        if DayOfWeek(DateTime) in FInactive then
          ABrush.Color := ColorNonActive;
      end;
    plMultiMonth:
      begin
        M := Mode.Month + Pos;
        Y := Mode.Year;
        while M > 12 do
        begin
          Dec(M,12);
          Inc(Y);
        end;

        if (Index < DaysInMonth(M,Y)) and (Index >= 0) then
        begin
          DateTime := EncodeDate(Y,M,Index + 1);
          if DayOfWeek(DateTime) in FInactive then
            ABrush.Color := ColorNonActive;
        end;
      end;
    plCustom, plCustomList:
      begin
      end;
    plTimeLine:
      begin
      end;
  end;

  if UsePP then
  begin
    if (PositionProps.Items[Pos].MinSelection > 0) then
    begin
      if  (Index < PositionProps.Items[Pos].MinSelection) then
        ABrush.Color := PositionProps.Items[Pos].ColorNoSelect;
    end;

    if (PositionProps.Items[Pos].MaxSelection > 0) then
    begin
      if  (Index > PositionProps.Items[Pos].MaxSelection) then
        ABrush.Color := PositionProps.Items[Pos].ColorNoSelect;
    end;
  end;
end;

function TTIWCustomWebPlanner.CloneItemAtSelection(Item: TPlannerItem): TPlannerItem;
begin
  Result := CloneItem(Item);
end;

procedure TTIWCustomWebPlanner.PlanTimeToStrings(MinutesValue: Cardinal;
  var HoursString: string; var MinutesString: string; var AmPmString: string);
var
  Hours, Minutes: Integer;

  function ChooseNonEmpty(s1,s2:string):string;
  begin
    if s2 = '' then
      Result := s1
    else
      Result := s2;
  end;

begin
  Hours := MinutesValue div 60;
  Minutes := MinutesValue mod 60;
  Hours := Hours mod 24;

  MinutesString := Format('%.2d', [Minutes]);

  case HourType of
  ht24hrs: AmPmString := '';
  ht12hrs: if Hours > 12 then
             Hours := Hours - 12;
  htAMPM0:
    begin
      if Hours >= 12 then
        AmPmString := ChooseNonEmpty('PM',TimePMString)
      else
        AmPmString := ChooseNonEmpty('AM',TimeAMString);

      if Hours > 12 then
        Hours := Hours - 12
    end;
  htAMPM1:
    begin
      if Hours >= 13 then
        AmPmString := ChooseNonEmpty('PM', TimePMString)
      else
        AmPmString := ChooseNonEmpty('AM', TimeAMString);

      if Hours > 12 then
        Hours := Hours - 12
    end;
  htAMPMOnce:
    begin
      If Minutes = 0 then
      begin
        AMPMString := '';
        if Hours >= 12 then
          MinutesString := ChooseNonEmpty('PM',TimePMString)
        else
          MinutesString := ChooseNonEmpty('AM',TimeAMString);
      end;
      if Hours > 12 then
        Hours := Hours - 12
    end;
  end;

  HoursString := Format('%d', [Hours]);

  if Assigned(OnPlanTimeToStrings) then
    OnPlanTimeToStrings(Self, MinutesValue, HoursString, MinutesString, AmPmString);
end;

procedure TTIWCustomWebPlanner.FreeItem(APlannerItem: TPlannerItem);
begin
  APlannerItem.ParentItem.Free;
end;

function TTIWCustomWebPlanner.GetDayName(WeekDay: Integer): string;
begin
  if (WeekDay > DaysPerWeek) then
    WeekDay := WeekDay mod DaysPerWeek;

  while (WeekDay < FirstDayOfWeek) do
    WeekDay := WeekDay + DaysPerWeek;

  if (WeekDay <= FDayNames.Count) and (WeekDay > 0) then
    Result := FDayNames.Strings[WeekDay - 1]
  else
    case WeekDay of
    1: Result := ShortDaySun;
    2: Result := ShortDayMon;
    3: Result := ShortDayTue;
    4: Result := ShortDayWed;
    5: Result := ShortDayThu;
    6: Result := ShortDayFri;
    7: Result := ShortDaySat;
  end;
end;

function TTIWCustomWebPlanner.PlanTimeToStr(
  MinutesValue: Cardinal): string;
var
  Hours, Minutes: word;
  Value: String;
begin
  Hours := MinutesValue div 60;
  Minutes := MinutesValue mod 60;
  Hours := Hours mod 24;

  case HourType of
    ht24hrs :
    begin
      Value := TimeToStr(EncodeTime(Hours,Minutes,0,0));
      Value := Copy(Value,1,length(Value) - 3);
    end;
    ht12hrs:
    begin
      if (Hours > 12) then
        Hours := Hours - 12;

      Value := TimeToStr(EncodeTime(Hours,Minutes,0,0));
      Value := Copy(Value,1,length(Value) - 3);
    end;

    htAMPM0, htAMPMOnce:
    begin
      if (Hours >= 12) then
      begin
        Hours := Hours - 12;
        Value := TimeToStr(EncodeTime(Hours,Minutes,0,0));
        Value := Copy(Value,1,length(Value) - 3);
        Value := Value + ' PM';
      end
      else
      begin
        Value := TimeToStr(EncodeTime(Hours,Minutes,0,0));
        Value := Copy(Value,1,length(Value) - 3);
        Value := Value + ' AM';
      end;
    end;

    htAMPM1:
    begin
      if (Hours > 12) then
      begin
        Hours := Hours - 12;
        Value := TimeToStr(EncodeTime(Hours,Minutes,0,0));
        Value := Copy(Value,1,length(Value) - 3);
        Value := Value + ' PM';
      end
      else
      begin
        Value := TimeToStr(EncodeTime(Hours,Minutes,0,0));
        Value := Copy(Value,1,length(Value) - 3);
        Value := Value + ' AM';
      end;
    end;
  end;
  Result := Value;
end;

function TTIWCustomWebPlanner.IsCurPos(APosition: Integer): Boolean;
begin
  Result := True;
  if (Display.CurrentPosFrom = -1) or (Display.CurrentPosTo = -1) then
    Exit;

  Result := (APosition >= Display.CurrentPosFrom) and
            (APosition <= Display.CurrentPosTo);
end;

function TTIWCustomWebPlanner.PosToDay(Pos: Integer): TDateTime;
begin
  Result := 0;
end;

function TTIWCustomWebPlanner.IndexToTime(Index: Integer): TDateTime;
begin
  Result := 0;

  if Mode.PlannerType = plCustomList then
  begin
    if Index < FDTList.Count then
      Result := FDTList.Items[Index];
  end;
end;

function TTIWCustomWebPlanner.TimeToIndex(DT: TDateTime): Integer;
var
  Idx: Integer;
begin
  Result := 0;
  if Mode.PlannerType = plCustomList then
  begin
    Idx := 0;
    while Idx < FDTList.Count do
    begin
      if FDTList.Items[Idx] >= DT then
      begin
        Result := Idx;
        Break;
      end
      else
       Inc(Idx);
    end;
  end;
end;

procedure TTIWCustomWebPlanner.UpdateItem(APlannerItem: TPlannerItem);
begin
end;

function FontSizeToHtmlFontSize(FontSize: Integer): string;
begin
  case FontSize of
    1..8: Result := '"1"';
    9, 10: Result := '"2"';
    11, 12: Result := '"3"';
    13, 14: Result := '"4"';
    15..18: Result := '"5"';
    19..24: Result := '"6"';
  else
    Result := '"2"';
  end;
end;

function FontToHtmlFont(const Font: TIWFont): string;
begin
  Result := Format('<Font face="%s" size=%s>',
    [Font.FontName, FontSizeToHtmlFontSize(Font.Size)]);
end;

function EmbedInHtmlTag(const Value: string; const Tag: string): string;
begin
  Result := Format('<%s>%s</%s>', [Tag, Value, Tag]);
end;

function EmbedInHtmlFontStyles(const Value: string; const FontStyle: TFontStyles): string;
begin
  Result := Value;
  if fsBold in FontStyle then
    Result := EmbedInHtmlTag(Result, 'B');
  if fsItalic in FontStyle then
    Result := EmbedInHtmlTag(Result, 'I');
  if fsUnderline in FontStyle then
    Result := EmbedInHtmlTag(Result, 'U');
end;

function EmbedInHtmlOptions(const Value: string; const FontStyle: TFontStyles): string;
begin
  Result := EmbedInHtmlFontStyles(Value, FontStyle);
end;

type
  TIWColorRecord = record
    RedValue: Byte;    //  clRed = TIWColor($0000FF);   Low byte
    GreenValue: Byte;  //  clLime = TIWColor($00FF00);  Middle byte
    BlueValue: Byte;   //  clBlue = TIWColor($FF0000);  High byte
    SystemValue: Byte; //  becomes zero when calling ColorToRgb
  end;

function TTIWCustomWebPlanner.ColorToHtmlHexBgColor(const Value: TIWColor): string;
begin
  Result := ' BGCOLOR="' + HTMLClr(Value) + '"';
end;

//Modified - Support '<,>,&' in notes and caption of item
function TTIWCustomWebPlanner.HTMLAdapt(const Value : String) : string;
var
  res : string;
begin
  res := StringReplace(Value,'&','&amp;',[rfIgnoreCase,rfReplaceAll]);
  res := StringReplace(res,'<','&lt;',[rfIgnoreCase,rfReplaceAll]);
  res := StringReplace(res,'>','&gt;',[rfIgnoreCase,rfReplaceAll]);
  Result := res;
end;

procedure TTIWCustomWebPlanner.StartEdit(APlannerItem: TPlannerItem);
begin
  APlannerItem.FEditMode := true;
end;

procedure TTIWCustomWebPlanner.StopEdit;
var
  i: integer;
begin
  for i:= 0 to FItems.Count-1 do
    FItems[i].FEditMode := false;
  FEditIndex := -1;
end;

procedure TTIWCustomWebPlanner.SetDayNames(const Value: TStringList);
begin
  if Assigned(Value) then
    FDayNames.Assign(Value);
end;

procedure TTIWCustomWebPlanner.SetHourType(const Value: THourType);
begin
  if FHourType <> Value then
  begin
    FHourType := Value;
  end;
end;

procedure TTIWcustomWebPlanner.SetEditType(const Value : TPlannerEditType);
begin
  FEditType := Value;
end;

procedure TTIWcustomWebPlanner.SetSelectionMode(const Value : TIWSelectionMode);
begin
  FSelectionMode := Value;
end;

procedure TTIWCustomWebPlanner.SetEnableSelection(const Value : Boolean);
begin
  FEnableSelection := Value;
end;

procedure TTIWCustomWebPlanner.SetShowConfirm(const Value: Boolean);
begin
  FShowConfirm := Value;
end;

procedure TTIWCustomWebPlanner.SetInTemplate(const Value : Boolean);
begin
  FInTemplate := Value;
end;

function TTIWCustomWebPlanner.ItemCall(Param: String; Index: Integer): String;
begin
  if Param = 'Delete' then
  begin
    Result := '"return SubmitClick('#39+HTMLName+#39','#39+ 'Delete' + IntToStr(Index) + #39',false)"'
  end
  else if Param = 'Select' then
  begin
    Result := Result + '"return SubmitClick('#39+HTMLName+#39','#39+ 'Select' + IntToStr(Index) + #39',false)"';
  end;
end;

function TTIWCustomWebPlanner.RenderItemToHTML(APlannerItem: TPlannerItem; RowIndex, MaxPos: Integer;
  Hidden,Orientation: Boolean; TotalHeightOfItem : integer; AContext: TIWBaseComponentContext): string;
var
  rtf,HtmlBackGroundColor,HtmlFontColor,TArea,FontColor,BorderCollapse: string;
  ImgName,UpdateItem,CancelItem: string;
  HeightRow,WidthColumn,i,WidthUpdateItem,WidthCancelItem: Integer;
  WidthSubColumn: Integer;
  CellWidth: String;
  CellHeight,CellHeightShowCap: Integer;
  CellNof: String;
  ItemHeight: String;
  CH,WidthColGlyph,WidthColNoGlyph, BorderS: Integer;
  AlignText, Text, TextDelete, CaptionText, temp,CellDiv : string;
  Color,ColorTo : TIWColor;

  EndFont : string;
  ItemBackColor, ItemForeColor : TIWColor;
  CellPropsCaption, HintCaption : string;
  OffsetImg, CellFontCaption : string;
  WidthDeleteItem, WidthGlyphsCaption : integer;
  SpanOnClick, SpanEditClick, EndSpan : string;
  WidthEditItem : integer;
  CellEditItem, CellDeleteItem, FontClr, ForeColor, CellPropsItem : string;
  CellFontItem, ItemText, CodeMoveSpan, CodeSelect, CodeCursorSpan, ChangeCap : string;
  CountOfLines : integer;
  HeightItem, DivHeightItem, TitleCaption, HintEdit : string;
  WidthCaptionCol : integer;
  CellUpdate, CellCancel, HeightDef : string;
  TextAreaForeColor, TextAreaBackColor : TIWColor;
  backgroundCell : string;

  ItText : string;
  lines : integer;
  k : integer;

  CaptionTime : string;
  CaptionTimeWidth : integer;

begin
  EndFont := '</font>';
  ItemBackColor := clWhite;
  ItemForeColor := clBlack;

  if (APlannerItem.Color <> clNone) then
    ItemBackColor := APlannerItem.Color;

  if (APlannerItem.Font.Color <> clNone) then
    ItemForeColor := APlannerItem.Font.Color;

  if IsBrowserIe(AContext) then
  begin
    BorderCollapse := 'style="border-collapse:collapse"';
    BorderS := 2 * BorderSize
  end
  else
  begin
    BorderCollapse := '';
    BorderS := 4 * BorderSize;
  end;

  if (Display.DisplayScale > 0) then
    HeightRow := Display.DisplayScale
  else
    HeightRow := 16;

  if not IsBrowserIe(AContext) and not (Orientation) then
    HeightRow := HeightRow - 1;

  if (Positions > 0) then
  begin
    if Overlap then
      WidthColumn := ((Width - SideBar.GetUsedWidth - BorderS) div Positions) - 10
    else
      WidthColumn := (Width - SideBar.GetUsedWidth - BorderS - Positions) div Positions;
  end
  else
  begin
    WidthColumn := (Width - SideBar.Width - 2 * BorderSize);
  end;

  if APlannerItem.Conflicts = 1 then
    WidthSubColumn := WidthColumn - 2
  else
    WidthSubColumn := (WidthColumn - 2 * MaxPos) div MaxPos;

  if (Orientation) then
  begin
    if (Overlap) then
      CellHeight := (((Height - SideBar.GetUsedWidth - 2 * BorderSize) div Positions) - 10 - MaxPos + 1) div MaxPos
    else
      CellHeight := (((Height - SideBar.GetUsedWidth - 2 * BorderSize) div Positions) - MaxPos) div MaxPos;

    if (APlannerItem.Conflicts = 1) then
      CellHeight := CellHeight * MaxPos;

    if (APlannerItem.InHeader) then
    begin
      WidthColumn := Header.HeaderItemWidth;
      WidthSubColumn := Widthcolumn - 2;
    end
    else
    begin
      if (APlannerItem.ItemEnd > APlannerItem.ItemBegin + 1) then
      begin
        WidthColumn := ((HeightRow) * (APlannerItem.ItemEnd - APlannerItem.ItemBegin));
      end
      else
        WidthColumn := (HeightRow-1);

      WidthSubColumn := WidthColumn;
    end;
  end
  else
  begin
    if not APlannerItem.FixedPosition then
      CellHeight := (Min(Display.DisplayEnd,APlannerItem.ItemEnd) - APlannerItem.ItemBegin) * HeightRow - 4
    else
      CellHeight := (Min(Display.DisplayEnd,APlannerItem.ItemEnd) - APlannerItem.ItemBegin) * HeightRow - 3;
  end;

  CH := APlannerItem.CaptionHeight;
  CellWidth := ' width="' + IntToStr(WidthSubColumn) + 'px"';
  ItemHeight := ' style="height:' + IntToStr(CellHeight) + 'px;"';

  CellDiv := '<div style="overflow:hidden;height: ' + IntToStr(CellHeight) + 'px;width: '
    + IntToStr(WidthSubColumn) + 'px">';

  if not APlannerItem.UniformBkg then
  begin
    Color := APlannerItem.CaptionBkg;
    if (APlannerItem.CaptionBkgTo = clNone) then
      ColorTo := Color
    else
      ColorTo := APlannerItem.CaptionBkgTo;
  end
  else
  begin
    if ((FSelectedIndex > -1) and (APlannerItem = Items[FSelectedIndex]) and (APlannerItem.ShowSelection) and (APlannerItem.SelectColor <> clNone)) then
      Color := APlannerItem.SelectColor
    else
    begin
      if (APlannerItem.Color <> clNone) then
        Color := APlannerItem.Color
      else
        Color := clWhite;
    end;

    ColorTo := Color;
  end;

//  if not IsBrowserIe(AContext) then
//    ColorTo := Color;

  Text := '';
  for i := 1 to APlannerItem.Text.Count do
  begin
    Text := Text + APlannerItem.Text.Strings[i-1];
    if (i < APlannerItem.Text.Count) then
      Text := Text + #13;
  end;

  Text := StringReplace(Text,'<br>',#13,[rfIgnoreCase,rfReplaceAll]);

  //Collect all properties of the itemheader.
  CellPropsCaption := '';
  CellPropsCaption := CellPropsCaption
    + ' height="' + IntToStr(CH) + ' px" style="' + HTMLAlign(APlannerItem.CaptionAlign) + ';';

  CellPropsCaption := CellPropsCaption
    + ColorToStyle(Color,ColorTo, APlannerItem.CaptionGradientDirection, AContext) + ';"';

  if (APlannerItem.CaptionType = ctTimeText) or (APlannerItem.CaptionType = ctText) then
    //Modified - Support '<,>,&' in notes and caption of item
    CaptionText := HtmlAdapt(APlannerItem.CaptionText);

  CaptionTime := '';
  CaptionTimeWidth := 0;
  if (APlannerItem.CaptionType = ctTime) or (APlannerItem.CaptionType = ctTimeText) then
  begin
    CaptionTime := APlannerItem.GetCaptionTimeString;
    CaptionTimeWidth := Round(Length(CaptionTime) * APlannerItem.CaptionFont.Size * 0.8);
  end;

  HintCaption := '';

	case Hints.CaptionHint of
    chCaption :
      HintCaption := CaptionText;
    chCaptionNotes :
    begin
      itText := '';
			lines := 0;
      for k := 0 to APlannerItem.Text.Count - 1 do
      begin
        lines := lines + 1;

        if (lines < APlannerItem.Text.Count) then
          ItText := ItText + APlannerItem.Text[k] + #13
        else
          ItText := ItText + APlannerItem.Text[k];
      end;

      if (ItText <> '') then
        HintCaption := CaptionText + #13#10 + #13#10 + ItText;
      end;
  else
    HintCaption := '';
  end;

//Modified - Caption
//  if (APlannerItem.ShowCaption) then
  if (APlannerItem.CaptionType <> ctNone) then
    CellHeightShowCap := TotalHeightOfItem - CH + 4
  else
    CellHeightShowCap := TotalHeightOfItem + 4;
//End Modified - Caption

  OffsetImg := '';
  if ((APlannerItem.BeginOffset > 0) and not(APlannerItem.InHeader)) then
  begin
    if (Glyphs.CrossingStartFileName <> '') then
      OffsetImg := '<img src="' + Glyphs.CrossingStartFileName + '">&nbsp;'
    else
    begin
      if not (Glyphs.CrossingStart.Empty) then
      begin
//        ImgName := TIWServerControllerBase.NewCacheFile('bmp', true);
//        FGlyphs.CrossingStart.SaveToFile(ImgName);
//        OffsetImg := '<img src="' + GetCacheDirEx(Self) + ExtractFilename(ImgName) + '">&nbsp;';

        ImgName := GraphicToCacheFile(FGlyphs.CrossingStart, imgBMP);
        OffsetImg := '<img src="' + ImgName + '">&nbsp;';
      end
      else
      begin
        if (Orientation) then
          OffsetImg := '<font face="Webdings">3&nbsp;</font>'
        else
          OffsetImg := '<font face="Webdings">5&nbsp;</font>';
      end;
    end;
  end;

  if ((APlannerItem.EndOffset > 0) and not(APlannerItem.InHeader)) then
  begin
    if (Glyphs.CrossingEndFileName <> '') then
      OffsetImg := OffsetImg + '<img src="' + Glyphs.CrossingEndFileName + '">&nbsp;'
    else
    begin
      if not (Glyphs.CrossingEnd.Empty) then
      begin
//        ImgName := TIWServerControllerBase.NewCacheFile('bmp', true);
//        FGlyphs.CrossingEnd.SaveToFile(ImgName);
//        OffsetImg := OffsetImg + '<img src="' + GetCacheDirEx(Self) + ExtractFilename(ImgName) + '">&nbsp;';
        ImgName := GraphicToCacheFile(FGlyphs.CrossingEnd, imgBMP);
        OffsetImg := OffsetImg + '<img src="' + ImgName + '">&nbsp;';
      end
      else
      begin
        if (Orientation) then
          OffsetImg := OffsetImg + '<font face="Webdings">4&nbsp;</font>'
        else
          OffsetImg := OffsetImg + '<font face="Webdings">6&nbsp;</font>';
      end;
    end;
  end;

  if (APlannerItem.CaptionFont.FontName = '') then
    APlannerItem.CaptionFont.FontName := 'Verdana';

  if (APlannerItem.CaptionFont.Size = 0) then
    APlannerItem.CaptionFont.Size := 8;

  CellFontCaption := '<font face="' + APlannerItem.CaptionFont.FontName + '" color="'
    + HTMLClr(APlannerItem.CaptionFont.Color) + '" style="font-size:' + IntToStr(APlannerItem.CaptionFont.Size) + 'pt;';

  if (fsBold in APlannerItem.CaptionFont.Style) then
    CellFontCaption := CellFontCaption
      + ' font-weight : bold;';

  if (fsItalic in APlannerItem.CaptionFont.Style) then
    CellFontCaption := CellFontCaption
      + ' font-style : italic;';

  CellFontCaption := CellFontCaption + '">'#13;

  WidthGlyphsCaption := 0;

	if not (APlannerItem.ReadOnly) and not (APlannerItem.Background) and not (ViewOnly) then
  begin
		if (Hidden) then
			EndSpan := ''
		else
    begin
      if (FPlannerInEditing) then
      begin
        SpanOnClick := '';
        SpanEditClick := '';
      end
      else
      begin

			SpanOnClick := '<span style="cursor:pointer;cursor:hand" onclick="'
			 + 'ItemCall(''Delete'',' + IntToStr(APlannerItem.Index) + ',''null'',''null'',''null'',''null'')' + '">'#13;

			SpanEditClick := '<span style="cursor:pointer;cursor:hand" onclick="'
						+ 'ItemCall(''SelectFC'',''' + IntToStr(APlannerItem.Index) + ''',''' + HTMLClr(APlannerItem.SelectColor) + ''','''
						+ HTMLClr(APlannerItem.SelectFontColor)
						+ ''',''' + HTMLClr(ItemBackColor) + ''',''' + HTMLClr(ItemForeColor) + ''');">'#13;
      end;
		end;

    WidthEditItem := 0;

    if (EditType = edPopup) then
    begin
      if (WidthSubColumn - 15 < 15) then
				WidthEditItem := WidthSubColumn - 17
			else

       if (Glyphs.EditItem.Empty) and (Glyphs.EditItemFileName = '') then
        begin
          WidthEditItem := 15;

          CellEditItem := CellEditItem
              + '<td title="' + Hints.HintEditItem + '" valign="top" bgcolor="' + HTMLClr(ColorTo) + '"'
              + ' style="'
              + HTMLAlign(APlannerItem.CaptionAlign) + ';"'
              + ' height="' + IntToStr(CH) + 'px" width="' + IntToStr(WidthEditItem) + 'px">'#13
              + '<div style="overflow:hidden;height:' + IntToStr(CH) + 'px;width:' + IntToStr(WidthEditItem)
              + 'px">'#13	+ SpanEditClick + '<font face="Webdings" color="' + HTMLClr(APlannerItem.CaptionFont.Color)
              + '">' + ' ' + EndFont + #13 + EndSpan + '</div></td>'#13;
        end
	  	else
  		begin
        if (Glyphs.EditItemFileName <> '') then
        begin
          WidthEditItem := 17;
          if (WidthEditItem > WidthSubColumn) then
            WidthEditItem := WidthSubColumn - 2;

          ImgName := Glyphs.EditItemFileName;
          ImgName := GetCacheDirEx(Self) + ExtractFilename(ImgName);
        end
        else
        begin
          WidthEditItem := FGlyphs.EditItem.Width + 2;

          if (WidthEditItem > WidthSubColumn) then
	  	  		WidthEditItem := WidthSubColumn - 2;

//          ImgName := TIWServerControllerBase.NewCacheFile('bmp',true);
//          FGlyphs.EditItem.SaveToFile(ImgName);
            ImgName := GraphicToCacheFile(FGlyphs.EditItem, imgBMP);
        end;

	  		CellDeleteItem := CellDeleteItem
		  	  + '<td title="' + Hints.HintEditItem + '" valign="top" bgcolor="' + HTMLClr(ColorTo) + '"'
					+ 'style="' + HTMLAlign(APlannerItem.CaptionAlign) + ';"'
					+ ' height="' + IntToStr(CH) + 'px" width="' + IntToStr(WidthEditItem - 2) + 'px">'
					+ '<div $$ style="overflow:hidden;height:' + IntToStr(CH) + 'px;width:' + IntToStr(WidthEditItem - 2)
          + 'px">'#13	+ SpanEditClick + CellFontCaption + '<img src="' + ImgName + '">'
          + EndFont	+ '</span></div></td>'#13;
      end;
    end;

    if (APlannerItem.AllowDelete) then
    begin
      if (Glyphs.DeleteItem.Empty) and (Glyphs.DeleteItemFileName = '') then
      begin
        WidthDeleteItem := 15;
		  	  if (WidthDeleteItem > WidthSubColumn) then
			  	  WidthDeleteItem := WidthSubColumn - 2;

  			  CellDeleteItem := CellDeleteItem
	  					+ '<td title="' + Hints.HintDeleteItem + '" valign="top" bgcolor="' + HTMLClr(ColorTo) + '"'
		  				+ ' style="'
			  			+ HTMLAlign(APlannerItem.CaptionAlign) + ';"'
				  		+ ' height="' + IntToStr(CH) + 'px" width="' + IntToStr(WidthDeleteItem - 2) + 'px">'#13
  						+ '<div style="overflow:hidden;height:' + IntToStr(CH) + 'px;width:' + IntToStr(WidthDeleteItem - 2)
	  					+ 'px">'#13	+ SpanOnClick + '<font face="Webdings" color="' + HTMLClr(APlannerItem.CaptionFont.Color)
		  				+ '">' + 'r' + EndFont + #13 + EndSpan + '</div></td>'#13;
      end
	  	else
  		begin
        if (Glyphs.DeleteItemFileName <> '') then
        begin
          WidthDeleteItem := 17;
          if (WidthDeleteItem > WidthSubColumn) then
            WidthDeleteItem := WidthSubColumn - 2;

          ImgName := Glyphs.DeleteItemFileName;
          ImgName := GetCacheDirEx(Self) + ExtractFilename(ImgName);
        end
        else
        begin
          WidthDeleteitem := FGlyphs.DeleteItem.Width + 2;

          if (WidthDeleteItem > WidthSubColumn) then
	  	  		WidthDeleteItem := WidthSubColumn - 2;

//          ImgName := TIWServerControllerBase.NewCacheFile('bmp',true);
//          FGlyphs.DeleteItem.SaveToFile(ImgName);
            ImgName := GraphicToCacheFile(FGlyphs.DeleteItem, imgBMP);
        end;

	  		CellDeleteItem := CellDeleteItem
		  	  + '<td title="' + Hints.HintDeleteItem + '" valign="top" bgcolor="' + HTMLClr(ColorTo) + '"'
					+ 'style="' + HTMLAlign(APlannerItem.CaptionAlign) + ';"'
					+ ' height="' + IntToStr(CH) + 'px" width="' + IntToStr(WidthDeleteItem - 2) + 'px">'
					+ '<div style="overflow:hidden;height:' + IntToStr(CH) + 'px;width:' + IntToStr(WidthDeleteItem - 2) + 'px">'#13
					+ SpanOnClick + CellFontCaption + '<img src="' + ImgName + '">' + EndFont
					+ '</span></div></td>'#13;
      end;
    end
    else
    begin
      WidthDeleteItem := 15;
  	    if (WidthDeleteItem > WidthSubColumn) then
		  	  WidthDeleteItem := WidthSubColumn - 2;

        BackgroundCell := ' bgcolor="' + HTMLClr(ColorTo) + '" ';

//        if IsBrowserIe(AContext) then
        begin
          if ((APlannerItem.CaptionBkg <> APlannerItem.CaptionBkgTo) and (APlannerItem.CaptionBkgTo <> clNone) and (APlannerItem.CaptionGradientDirection = gdVertical)) then
//            BackgroundCell := 'style="FILTER: progid:DXImageTransform.Microsoft.Gradient(GradientType=0, StartColorStr='#39+HTMLClr(APlannerItem.CaptionBkg)+#39',EndColorStr='#39+HTMLClr(APlannerItem.CaptionBkgTo )+#39')" ';
              BackgroundCell := 'style="' + ColorToStyle(APlannerItem.CaptionBkg, APlannerItem.CaptionBkgTo, APlannerItem.CaptionGradientDirection,AContext) + '"';
        end;

  			CellDeleteItem := CellDeleteItem
//	  		+ '<td valign="top" bgcolor="' + HTMLClr(ColorTo) + '"'
	  			+ '<td valign="top" ' + BackgroundCell
		  		+ ' style="'
			  	+ HTMLAlign(APlannerItem.CaptionAlign) + ';"'
					+ ' height="' + IntToStr(CH) + 'px" width="' + IntToStr(WidthDeleteItem - 2) + 'px">'#13
  				+ '<div style="overflow:hidden;height:' + IntToStr(CH) + 'px;width:' + IntToStr(WidthDeleteItem - 2)
	  			+ 'px">&nbsp;</div></td>'#13;
    end;

    WidthGlyphsCaption := WidthDeleteItem;

    if (EditType = edPopup) then
      WidthGlyphsCaption := WidthGlyphsCaption + WidthEditItem + 2;
  end;

	FontClr := '';

	if ((FSelectedIndex > -1) and (APlannerItem = Items[FSelectedIndex]) and (APlannerItem.ShowSelection) and (APlannerItem.SelectFontColor <> clNone)) then
		FontClr := ' color:' + HTMLClr(APlannerItem.SelectFontColor) + ';'
	else
		FontClr := ' color:' + HTMLClr(APlannerItem.Font.Color) + ';';

	CellPropsItem := '';

	if ((FSelectedIndex > -1) and (APlannerItem = Items[FSelectedIndex]) and (APlannerItem.ShowSelection)) then
  begin
		if (APlannerItem.SelectColor = clNone) then
			CellPropsItem := CellPropsItem
					  + ' bgColor="' + HTMLClr(clInfoBk) + '" style="cursor:pointer;cursor:hand;' + HTMLAlign(APlannerItem.Alignment) + '; bgcolor:' + HTMLClr(clInfoBk) + ';'
						+ FontClr + '"'
		else
			CellPropsItem := CellPropsItem
						+ ' bgColor="' + HTMLClr(APlannerItem.SelectColor)
						+ '" style="cursor:pointer;cursor:hand;' + HTMLAlign(APlannerItem.Alignment) + '; bgcolor:' + HTMLClr(APlannerItem.SelectColor) + ';'
						+ FontClr + '"';
  end
	else
	begin
		if (APlannerItem.Color = clNone) then
			CellPropsItem := CellPropsItem
					  + ' bgColor="' + HTMLClr(clWhite) + '" style="cursor:pointer;cursor:hand;' + HTMLAlign(APlannerItem.Alignment) + '; bgcolor:' + HTMLClr(clWhite) + ';'
						+ FontClr + '"'
    else
      CellPropsItem := CellPropsItem
						+ ' bgColor="' + HTMLClr(APlannerItem.Color)
						+ '" style="cursor:pointer;cursor:hand;' + HTMLAlign(APlannerItem.Alignment) + '; bgcolor:' + HTMLClr(APlannerItem.Color) + ';'
						+ FontClr + '"';
  end;

	if (APlannerItem.Font.FontName = '') then
		APlannerItem.Font.FontName := 'Verdana';

  if (APlannerItem.Font.Size = 0) then
		APlannerItem.Font.Size := 8;

	CellPropsItem := CellPropsItem
				+ ' colspan="2"';

	CellFontItem := '<font face="' + APlannerItem.Font.FontName + '"'
				+ ' style="font-size:' + IntToStr(APlannerItem.Font.Size) + 'pt;';

	if (fsBold in APlannerItem.Font.Style) then
  	CellFontItem := CellFontItem
					+ ' font-weight:bold;';

  if (fsItalic in APlannerItem.Font.Style) then
		CellFontItem := CellFontItem
    			+ ' font-style:italic;';

	CellFontItem := CellFontItem + '">'#13;

	ItemText := '';
	CountOfLines := 0;

 for i := 0 to APlannerItem.Text.Count-1 do
  begin
    CountOfLines := CountOfLines + 1;

    //Modified - Support '<,>,&' in notes and caption of item
  	if (CountOfLines < APlannerItem.Text.Count) then
			ItemText := ItemText + HtmlAdapt(APlannerItem.Text[i]) + '<br>'
		else
			ItemText := ItemText + HtmlAdapt(APlannerItem.Text[i]);
  end;

	rtf := '<table border="0" bordercolor="black" cellspacing="0" cellpadding="0" width=100%' + BorderCollapse + '>'#13;

	if not (APlannerItem.EditMode)  or (ViewOnly) then
  begin
		CodeMoveSpan := '';
	  CodeSelect := '';
    CodeCursorSpan := '';

    if (FPlannerInEditing) then
      CodeSelect := ''
    else
    begin
  		if (not (APlannerItem.ReadOnly) and not (Hidden) and not (APlannerItem.Background) and not (ViewOnly)) then
      begin
		  	if (APlannerItem.ShowSelection) then
			  	CodeSelect := ' onclick="'
							+ 'ItemCall(''Select'',''' + IntToStr(APlannerItem.Index) + ''',''' + HTMLClr(APlannerItem.SelectColor) + ''','''
							+ HTMLClr(APlannerItem.SelectFontColor)
							+ ''',''' + HTMLClr(ItemBackColor) + ''',''' + HTMLClr(ItemForeColor) + ''')";'
  			else
	  			CodeSelect := ' onclick="'
  						+ 'ItemCall(''Select'',''' + IntToStr(APlannerItem.Index) + ''',''null'',''null'',''null'',''null'')"' + ';';
      end
	  	else
		  		CodeSelect := '';
    end;

    if (FPlannerInEditing) then
    begin
      CodeMoveSpan := '';
      EndSpan := '';
      CodeCursorSpan := '';
    end
    else
    begin
  		if ((APlannerItem.InHeader) or (APlannerItem.FixedPos) or (Hidden) or (APlannerItem.Background) or (ViewOnly)) then
      begin
        if (APlannerItem.InHeader) or (APlannerItem.FixedPos) then
        begin
    		  ChangeCap := 'False';

	    		if ( not (APlannerItem.ReadOnly) and not (APlannerItem.CaptionType = ctNone) and not (APlannerItem.CaptionType = ctTime)) then
		    		ChangeCap := 'True';

          CodeMoveSpan := 'onMouseDown="HeaderChangeCap(''ITEM' + IntToStr(APlannerItem.Index) + ''',''' + ChangeCap + ''',event)";'#13;
          EndSpan := '</span>'#13;
          CodeCursorSpan := '<span ' + CodeMoveSpan + '>'#13;
        end
        else
        begin
    		  CodeMoveSpan := '';
 		  	  EndSpan := '';
  		  	CodeCursorSpan := '';
        end;
	  	end
		  else
      begin
	  	  ChangeCap := 'False';

		  	if ( not (APlannerItem.ReadOnly) and not (APlannerItem.CaptionType = ctNone) and not (APlannerItem.CaptionType = ctTime)) then
			  	ChangeCap := 'True';

  			CodeMoveSpan := 'onMouseUp="mUp(event)"'
          + ' onMouseMove="ieP('''',event)"'
					+ ' onMouseDown="startMove(''ITEM' + IntToStr(APlannerItem.Index) + ''',''' + ChangeCap + ''',event);"'#13;

		  	EndSpan := '</span>'#13;

			  CodeCursorSpan := '<span style="cursor:pointer;cursor:move" ' + CodeMoveSpan + '>'#13;
      end;
    end;

  	HeightItem := '';
    DivHeightItem := '';

		if not (APlannerItem.InHeader) then
    begin
  		HeightItem := 'height="' + IntToStr(CellHeightShowCap - 5) + 'px"';
			DivHeightItem := 'height:' + IntToStr(CellHeightShowCap - 5) + 'px;';
    end;

//Modified - Caption
		//if (APlannerItem.ShowCaption) then
    if (APlannerItem.CaptionType <> ctNone) then
//End Modified - Caption
    begin
		  TitleCaption := '';

			if (HintCaption <> '') then
				TitleCaption := ' title="' + HintCaption + '"';

			WidthCaptionCol := WidthSubColumn - WidthGlyphsCaption - 2;

			if (WidthCaptionCol < 0) then
				WidthCaptionCol := 0;

      if (APlannerItem.InHeader) and (Orientation) then
        HeightDef := ' valign="top" text-align="top" height="' + IntToStr(CellHeightShowCap) + 'px" '
      else
        HeightDef := ' ';

      if (OffsetImg <> '') then
      begin
        rtf := rtf
						+ '<tr>'#13
						+ '<td width="' + IntToStr(WidthCaptionCol) + 'px" ' + TitleCaption + ' id="ITEM' + IntToStr(APlannerItem.Index) + '" '
            + CellPropsCaption + '>'#13
						+ CodeCursorSpan
						+ CellFontCaption

						+ '<div style="overflow:hidden; width:' + IntToStr(WidthCaptionCol) + 'px; height:'
            + IntToStr(CH) + 'px">'#13
            + '<table cellspacing="0" cellpadding="0" height="' + IntToStr(CH) + 'px" width="' + IntToStr(WidthCaptionCol) + 'px">'#13
            + '<tr><td height="' + IntToStr(CH) + 'px" width="10px" style="text-align:left">'
            + '<div style="overflow:hidden; width:' + IntToStr(10) + 'px; height:'
						+ IntToStr(CH) + 'px">'#13;
            if (CaptionTime <> '') then
            begin
              if (CaptionText = '') then
                CaptionTimeWidth := WidthCaptionCol - 12;

              if (CaptionTimeWidth > WidthCaptionCol - 12) then
                CaptionTimeWidth := WidthCaptionCol - 12;

              rtf := rtf
     						+ CellFontCaption + OffsetImg + EndFont + '</div></td><td width="' + IntToStr(CaptionTimeWidth) + 'px" style="' + HTMLAlign(APlannerItem.CaptionAlign) + '">'
                + CellFontCaption
                + '<div style="overflow:hidden;width:' + IntToStr(CaptionTimeWidth) + 'px;height:'
                + IntToStr(CH) + 'px">' + CaptionTime + '</div>' + EndFont + '</td>'
                + '<td style="' + HTMLAlign(APlannerItem.CaptionAlign) + '">' + CellFontCaption;
            end
            else
            begin
                rtf := rtf
      						+ CellFontCaption + OffsetImg + EndFont + '</div></td><td width="' + IntToStr(WidthCaptionCol - 10 - CaptionTimeWidth - 10) + 'px" style="'
                  + HTMLAlign(APlannerItem.CaptionAlign) + '">' + CellFontCaption;
            end;

          rtf := rtf
            + '<div id="DIVITEM' + IntToStr(APlannerItem.Index) + '" style="overflow:hidden; width:' + IntToStr(WidthCaptionCol - 10 - CaptionTimeWidth - 12) + 'px; height:'
						+ IntToStr(CH) + 'px">' + CaptionText + '</div>' + EndFont + '</td></tr></table>'#13
            + '</div>'#13 + EndSpan + EndFont + '</td>'#13
						+ CellEditItem + CellDeleteItem + '</tr>'#13;
      end
      else
      begin
        rtf := rtf
						+ '<tr>'#13
						+ '<td width="' + IntToStr(WidthCaptionCol) + '" ' + TitleCaption + ' id="ITEM' + IntToStr(APlannerItem.Index) + '" '
            + CellPropsCaption + '>'#13
						+ CodeCursorSpan
						+ CellFontCaption;
            if (CaptionTime <> '') then
            begin
              if (CaptionText = '') then
                CaptionTimeWidth := WidthCaptionCol -2;

              if (CaptionTimeWidth > WidthCaptionCol-2) then
                CaptionTimeWidth := WidthCaptionCol-2;

              rtf := rtf
                + '<div style="overflow:hidden;width:' + IntToStr(WidthCaptionCol) + 'px; height: ' + IntToStr(CH) + 'px">'#13
                + '<table cellspacing="0" cellpadding="0" border="0" height="' + IntToStr(CH) + 'px" width="' + IntToStr(WidthCaptionCol) + 'px">'#13
                + '<tr><td height="' + IntToStr(CH) + 'px" width="' + IntToStr(CaptionTimeWidth) + 'px" style="' + HTMLAlign(APlannerItem.CaptionAlign) + '">'
                + CellFontCaption + '<div style="overflow:hidden; width:'
                + IntToStr(CaptionTimeWidth- 2) + 'px; height:'
  				  		+ IntToStr(CH) + 'px">'#13
                + CaptionTime + '</div>' + EndFont + '</td><td width="' + IntToStr(WidthCaptionCol - CaptionTimeWidth)
                + 'px" style="' + HTMLAlign(APlannerItem.CaptionAlign) + '">' + CellFontCaption;
            end;

         rtf := rtf
            + '<div id="DIVITEM' + IntToStr(APlannerItem.Index) + '" style="overflow:hidden; width:' + IntToStr(WidthCaptionCol - CaptionTimeWidth) + 'px; height:'
						+ IntToStr(CH) + 'px">' + CaptionText;
      if(CaptionTime <> '') then
        rtf := rtf
            + '</div>' + EndFont + '</td></tr></table>'#13;

      rtf := rtf
            + '</div>'#13 + EndSpan + EndFont + '</td>'#13
						+ CellEditItem + CellDeleteItem + '</tr>'#13;
       end;

			HintEdit := '';

      if APlannerItem.Hint <> '' then
      begin
    		HintEdit := 'title="' + APlannerItem.Hint + '"'
      end
			else if ( not (APlannerItem.ReadOnly) and (Hints.HintEditItem <> '') and not (APlannerItem.Background) and not (ViewOnly)) then
      begin
  				HintEdit := 'title="' + Hints.HintEditItem + '"';
      end;

			rtf := rtf
						+ '<tr>'#13
						+ '<td ' + HeightDef + ' colspan="3" ' + HeightItem + ' id="IT' + IntToStr(APlannerItem.Index) + '"' + HintEdit + CodeSelect + CellPropsItem + '>'
						+ CellFontItem + '<div id="NOTES_' + IntToStr(APlannerItem.Index) + '" style="overflow:hidden;width:'
						+ IntToStr(WidthSubColumn) + 'px;' + DivHeightItem + '">' + ItemText + '</div>' + EndFont + '</td></tr></table>';
    end
		else
		begin
			OffsetImg := '';

		  HintEdit := '';

      if APlannerItem.Hint <> '' then
      begin
    		HintEdit := 'title="' + APlannerItem.Hint + '"'
      end
			else if ( not (APlannerItem.ReadOnly) and (Hints.HintEditItem <> '') and not (APlannerItem.Background)) then
      begin
  				HintEdit := 'title="' + Hints.HintEditItem + '"';
      end;

			rtf := rtf
						+ '<tr>'#13
						+ '<td colspan="3" ' + HeightItem + ' id="IT' + IntToStr(APlannerItem.Index) + '"' + HintEdit + CodeSelect + CellPropsItem + '>'
						+ CellFontItem + '<div id="NOTES_' + IntToStr(APlannerItem.Index) + '" style="overflow:hidden;width:'
						+ IntToStr(WidthSubColumn) + 'px;' + DivHeightItem + '">' + OffsetImg + ItemText + '</div>' + EndFont + '</td></tr></table>';
    end;
  end
  else
  begin
	  CellUpdate := '';
    CellCancel := '';

    if (Glyphs.UpdateItem.Empty) and (Glyphs.UpdateItemFileName = '') then
    begin
			WidthUpdateItem := 15;

			if (WidthUpdateItem > WidthSubColumn) then
				WidthUpdateItem := WidthSubColumn - 2;

			CellUpdate := CellUpdate
  					+ '<td title="' + Hints.HintUpdateItem + '" valign="top" '
						+ ' bgcolor="' + HTMLClr(ColorTo) + '"'
						+ ' style="' + HTMLAlign(APlannerItem.CaptionAlign) + ';"'
						+ ' height="' + IntToStr(CH) + 'px" width="' + IntToStr(WidthUpdateItem) + 'px">'#13
						+ '<div style="overflow:hidden;height:' + IntToStr(CH) + 'px;width:' + IntToStr(WidthUpdateItem) + 'px">'#13
						+ '<span style="cursor:pointer;cursor:hand" onclick="UpdEdit(this,' + IntToStr(APlannerItem.Index) + ')">'#13
						+ '<font face="Webdings" color="' + HTMLClr(APlannerItem.CaptionFont.Color) + '">' + 'a' + EndFont + '</span></div></td>';
    end
  	else
  	begin
      if (Glyphs.UpdateItemFileName <> '') then
      begin
        WidthUpdateItem := 17;

  			if (WidthUpdateItem > WidthSubColumn) then
		  		WidthUpdateItem := WidthSubColumn - 2;

        ImgName := Glyphs.UpdateItemFileName;
        ImgName := GetCacheDirEx(Self) + ExtractFilename(ImgName);
      end
      else
      begin
	  		WidthUpdateItem := Glyphs.UpdateItem.Width;

  			if (WidthUpdateItem > WidthSubColumn) then
		  		WidthUpdateItem := WidthSubColumn - 2;

//        ImgName := TIWServerControllerBase.NewCacheFile('bmp', true);
//        FGlyphs.UpdateItem.SaveToFile(ImgName);
          ImgName := GraphicToCacheFile(FGlyphs.UpdateItem, imgBMP);
      end;

			CellUpdate := CellUpdate
   					+ '<td title="' + Hints.HintUpdateItem + '" valign="top" '
						+ ' bgcolor="' + HTMLClr(ColorTo) + '"'
						+ ' style="' + HTMLAlign(APlannerItem.CaptionAlign) + ';"'
						+ ' height="' + IntToStr(CH) + 'px" width="' + IntToStr(WidthUpdateItem) + 'px">'#13
						+ '<div style="overflow:hidden;height:' + IntToStr(CH) + 'px;width:' + IntToStr(WidthUpdateItem) + 'px">'#13
						+ '<span style="cursor:pointer;cursor:hand" onclick="UpdEdit(this,' + IntToStr(APlannerItem.Index) + ')">'#13
					  + '<img src="' + ImgName + '"></span></div></td>'#13;
    end;

    if (Glyphs.CancelItem.Empty) and (Glyphs.CancelItemFileName = '') then
    begin
			WidthCancelItem := 15;

			if (WidthCancelItem + WidthUpdateItem > WidthSubColumn) then
				WidthCancelItem := WidthSubColumn - WidthUpdateItem;

			CellCancel := CellCancel
   					+ '<td title="' + Hints.HintCancelItem + '" valign="top" '
						+ ' bgcolor="' + HTMLClr(ColorTo) + '"'
						+ ' style="' + HTMLAlign(APlannerItem.CaptionAlign) + ';"'
						+ ' height="' + IntToStr(CH) + 'px" width="' + IntToStr(WidthCancelItem) + 'px">'#13
						+ '<div style="overflow:hidden;height:' + IntToStr(CH) + 'px;width:' + IntToStr(WidthCancelItem) + 'px">'#13
						+ '<span style="cursor:pointer;cursor:hand" onclick="CancelEdit(this,' + IntToStr(APlannerItem.Index) + ')">'#13
						+ '<font face="Webdings" color="' + HTMLClr(APlannerItem.CaptionFont.Color) + '">' + 'r' + EndFont + '</span></div></td>';
    end
		else
    begin
      if (Glyphs.CancelItemFileName <> '') then
      begin
        WidthCancelItem := 17;

 			  if (WidthCancelItem + WidthUpdateItem > WidthSubColumn) then
		  		WidthCancelItem := WidthSubColumn - WidthUpdateItem;

        ImgName := Glyphs.CancelItemFileName;
        ImgName := GetCacheDirEx(Self) + ExtractFilename(ImgName);
      end
      else
      begin
  			WidthCancelItem := Glyphs.CancelItem.Width;

			  if (WidthCancelItem + WidthUpdateItem > WidthSubColumn) then
		  		WidthCancelItem := WidthSubColumn - WidthUpdateItem;

//        ImgName := TIWServerControllerBase.NewCacheFile('bmp', true);
//        FGlyphs.CancelItem.SaveToFile(ImgName);
        ImgName := GraphicToCacheFile(FGlyphs.CancelItem, imgBMP);
      end;

			CellCancel := CellCancel
   					+ '<td title="' + Hints.HintCancelItem + '" valign="top" '
						+ ' bgcolor="' + HTMLClr(ColorTo) + '"'
						+ ' style="' + HTMLAlign(APlannerItem.CaptionAlign) + ';"'
						+ ' height="' + IntToStr(CH) + 'px" width="' + IntToStr(WidthCancelItem) + 'px">'#13
						+ '<div style="overflow:hidden;height:' + IntToStr(CH) + 'px;width:' + IntToStr(WidthCancelItem) + 'px">'#13
						+ '<span style="cursor:pointer;cursor:hand" onclick="CancelEdit(this,' + IntToStr(APlannerItem.Index) + ')">'#13
					  + '<img src="' + ImgName + '"></span></div></td>'#13;
    end;

    TArea := '';

    if (APlannerItem.ShowSelection) then
    begin
			TextAreaForeColor := APlannerItem.SelectFontColor;
			TextAreaBackColor := APlannerItem.SelectColor;
    end
		else
		begin
		  if (APlannerItem.Font.Color <> clNone) then
				TextAreaForeColor := APlannerItem.Font.Color
			else
				TextAreaForeColor := clBlack;

			if (APlannerItem.Color <> clNone) then
				TextAreaBackColor := APlannerItem.Color
			else
				TextAreaBackColor := clWhite;
    end;

    if APlannerItem.InHeader then
      TArea := '<tr><td bgcolor="' + HTMLClr(TextAreaBackColor) + '" colspan="3">'
             + '<div style="overflow:hidden; width:' + IntToStr(WidthSubColumn) + 'px">'
             + '<textarea id="EDITITEM' + IntToStr(APlannerItem.Index) + '" style="resize:none;border:0; width:'+ IntToStr(WidthSubColumn) + 'px; height: '
             + IntToStr(HeightRow * 3) + 'px; font-family:' + APlannerItem.Font.FontName + '; font-size:' + IntToStr(APlannerItem.Font.Size)
             + 'pt; color:' + HTMLClr(TextAreaForeColor) + '; background:' + HTMLClr(TextAreaBackColor) + '">'
             + Text + '</textarea></div></td></tr></table>'#13
    else
      TArea := '<tr><td width="' + IntToStr(WidthSubColumn) + 'px" height="' + IntToStr(CellHeight - CH)
             + 'px" bgcolor="' + HTMLClr(TextAreaBackColor) + '" colspan="3">'
             + '<div style="overflow:hidden;width:'
             + IntToStr(WidthSubColumn) + 'px; height:' + IntToStr(CellHeight - CH) + 'px">' + '<textarea id="EDITITEM'
             + IntToStr(APlannerItem.Index) + '" style="resize:none; border:0; height:'
             + IntToStr(CellHeight - CH) + 'px; width:' + IntToStr(WidthSubColumn - 2) + 'px; font-family:' + APlannerItem.Font.FontName + '; font-size:'
             + IntToStr(APlannerItem.Font.Size) + 'pt; color:'
             + HTMLClr(TextAreaForeColor) + '; background:' + HTMLClr(TextAreaBackColor) + '">' + Text + '</textarea></div></td></tr></table>';

  	if (APlannerItem.InHeader) then
			rtf := '<table cellspacing="0" cellpadding="0"' + CellWidth + BorderCollapse + '>'#13
		else
			rtf := '<table cellspacing="0" cellpadding="0"' + ItemHeight + CellWidth + BorderCollapse + '>'#13;

    WidthCaptionCol := WidthSubColumn - (WidthUpdateItem + WidthCancelItem + 2);

		if (WidthCaptionCol < 0) then
			WidthCaptionCol := 0;

 	  rtf := rtf
					+ '<tr>'#13
					+ '<td height="' + IntToStr(CH) + 'px" width="' + IntToStr(WidthCaptionCol)
					+ 'px" ' + 'style="'
         + ColorToStyle(Color,ColorTo,APlannerItem.CaptionGradientDirection,AContext)
         + '">'#13
         + CellFontCaption + '<div style="overflow:hidden; width:' + IntToStr(WidthCaptionCol)
  			 + 'px; height:' + IntToStr(CH) + 'px">' + OffsetImg + CaptionText + '</div>' + EndFont + '</td>' + CellUpdate
				 + CellCancel + TArea;
  end;
	Result := rtf;
end;

procedure TTIWCustomWebPlanner.RenderPopup(APlannerItem : TPlannerItem);
var
  FontPopup, EndFont, PopupItemFont, ContentPopupWindow : string;
  BackgroundPopupWindow, ImgName, PropertiesTA : string;

begin
  if (PopupEdit.Font.FontName = '') then
		PopupEdit.Font.FontName := 'Verdana';

	if (PopupEdit.Font.Size = 0) then
 	  PopupEdit.Font.Size := 10;

  if (PopupEdit.Font.Color = clNone) then
		PopupEdit.Font.Color := clBlack;

	FontPopup := '<font color="' + HTMLClr(PopupEdit.Font.Color) + '" face="' + PopupEdit.Font.FontName + '"'
				+ ' style="font-size:' + IntToStr(PopupEdit.Font.Size) + 'px;';

	if (fsBold in PopupEdit.Font.Style) then
  	FontPopup := FontPopup
					+ ' font-weight:bold;';

  if (fsItalic in PopupEdit.Font.Style) then
		FontPopup := FontPopup
					+ ' font-style:italic;';

	FontPopup := FontPopup + '">'#13;

  EndFont := '</font>';

  PopupItemFont := 'style="resize:none;width:' + IntToStr(PopupEdit.PopupWidth - 10) + 'px;color:' + HTMLClr(PopupEdit.Font.Color) + ';font-family:' + PopupEdit.Font.FontName
        + ';font-size:' + IntToStr(PopupEdit.Font.Size) + 'pt;';
  PropertiesTA := PopupItemFont + 'height:' + IntToStr(Round(PopupEdit.PopupHeight * 0.3))  + 'px';

	if (fsBold in PopupEdit.Font.Style) then
	 	PopupItemFont := PopupItemFont
					+ 'font-weight:bold;';

	if (fsItalic in PopupEdit.Font.Style) then
 	  PopupItemFont := PopupItemFont
    			+ 'font-style:italic;';

	PopupItemFont := PopupItemFont
  			+ '"';

	BackgroundPopupWindow := '';

	if (PopupEdit.BackgroundImage.Empty) then
  begin
  	if (PopupEdit.BackColor = clNone) then
			BackgroundPopupWindow := 'bgColor="white"'
		else
			BackgroundPopupWindow := 'bgColor="' + HTMLClr(PopupEdit.BackColor) + '"';
	end
	else
  begin
//    ImgName := TIWServerControllerBase.NewCacheFile('bmp', true);
//    FPopupEdit.BackgroundImage.SaveToFile(ImgName);
    ImgName := GraphicToCacheFile(FPopupEdit.BackgroundImage, imgBMP);
		BackgroundPopupWindow := 'background="' + ImgName + '"';
  end;

	ContentPopupWindow := '';
	ContentPopupWindow := ContentPopupWindow

				+ '<table ' + BackgroundPopupWindow + ' width=100% height=100%>'
        + '<tr><td colspan="2">'
				+  FontPopup + PopupEdit.SubjectCaption + EndFont + '</td></tr>'
				+ '<tr>'
        + '<td colspan="2"><input ' + PopupItemFont + ' id="TF_' + IntToStr(APlannerItem.Index)
        + '" type="text" style="width:' + IntToStr(PopupEdit.PopupWidth - 10) + 'px;" value="" name="CaptionInput_'
				+ IntToStr(APlannerItem.Index) + '"></td></tr>'
				+ '<tr><td colspan="2">' + FontPopup + PopupEdit.NotesCaption + EndFont + '</td></tr>'#13
				+ '<tr>'
        + '<td colspan="2"><textarea ' + PropertiesTA + '" id="TA_'
        + IntToStr(APlannerItem.Index)+ '" ></textarea></td></tr>'
				+ '<tr><td width=52% align="right">'#13
        + '<Button style="font-family:' + PopupEdit.Font.FontName + '" onclick="ConfirmPopupEdit('''
        + IntToStr(APlannerItem.Index) + ''')">Ok</button></td><td><button style="font-family:' + PopupEdit.Font.FontName
        + '" onclick="CancelPopupEdit(''' + IntToStr(APlannerItem.Index) + ''')">Cancel</button></td></tr></table>';

			FHiddenPopups := FHiddenPopups
				+ '<div id="POPUP_ITEM_' + IntToStr(APlannerItem.Index) + '"'
				+ ' style="width:' + IntToStr(PopupEdit.PopupWidth) + 'px;height:' + IntToStr(PopupEdit.PopupHeight) + 'px;z-index:101;top:0;left:0;border: 2px outset ' + HTMLClr(PopupEdit.BorderColor) + '; overflow:hidden;'
				+ ' position:absolute;visibility:hidden;">' + ContentPopupWindow + '</div>'#13;
end;

function TTIWCustomWebPlanner.RenderCSSStyle(PlannerMode : string; AContext : TIWBaseComponentContext) : string;
var
  HtmlText : string;
  HtmlAlignment : string;
begin
  HtmlText := '';
  HtmlAlignment := '';

  if (Header.Font.FontName = '') then
    Header.Font.FontName := 'Verdana';

  if (Header.Font.Size = 0) then
    Header.Font.Size := 8;

  if (Header.Font.Color = clNone) then
    Header.Font.Color := clBlack;

  HtmlText := HtmlText
    + '<style>'#13
    + '#SPC {'#13
    + '    overflow:hidden;'#13
    + '    width:8px;'#13
    + '}'#13
    + '#SPS {'#13
    + '    cursor;pointer;cursor:default;'#13
    + '}'#13



    + '#' + HTMLName + 'Caption {'#13
    + '                          color : ' + HTMLClr(Header.Font.Color) + ';'#13
    + '                          font-size: ' + IntToStr(Header.Font.Size) + 'pt;'#13
    + '                          font-family: ' + Header.Font.FontName + ';'#13;

  if (fsBold in Header.Font.Style) then
  begin
    HtmlText := HtmlText
      + '                          font-weight: bold;'#13;
  end;

  if (fsItalic in Header.Font.Style) then
  begin
    HtmlText := HtmlText
      + '                         font-style: italic;'#13;
  end;

  if assigned(OnHeaderClick) or assigned(OnAsyncHeaderClick) then
  begin
    HtmlText := HtmlText
      + '                        text-decoration : underline;'#13;

    if IsBrowserIE(AContext) then
      HtmlText := HtmlText
        + '                      cursor : hand;'#13
    else
      HtmlText := HtmlText
        + '                      cursor : pointer;'#13
        + '                      cursor : hand;'#13;
  end;

  HtmlAlignment := HtmlAlign(Header.Alignment);

  if (HtmlAlignment <> '') then
  begin
    HtmlText := HtmlText
      + '                         ' + HtmlAlignment + ';'#13;
  end;

  HtmlText := HtmlText
    + '                           border-color : ' + HtmlClr(Header.LineColor) + ';'#13;

  if (PlannerMode = 'vertical') then
  begin
    HtmlText := HtmlText
      + '                         height: ' + IntToStr(Header.CaptionHeight) + 'px;'#13
      + '                         ' + ColorToStyle(Header.Background, Header.BackgroundTo, Header.GradientDirection, AContext) + ';'#13
      + '}'#13;
  end
  else
  begin
    HtmlText := HtmlText
      + '                         width: ' + IntToStr(Header.CaptionHeight) + 'px;'#13
      + '                         ' + ColorToStyle(Header.Background, Header.BackgroundTo, Header.GradientDirection, AContext) + ';'#13
      + '}'#13;
  end;

  HtmlText := HtmlText
    + '#' + HTMLName + 'CaptionTL {'#13
    + '                          color : ' + HTMLClr(Header.Font.Color) + ';'#13
    + '                          font-size: ' + IntToStr(Header.Font.Size) + 'pt;'#13
    + '                          font-family: ' + Header.Font.FontName + ';'#13;

  if (fsBold in Header.Font.Style) then
  begin
    HtmlText := HtmlText
      + '                          font-weight: bold;'#13;
  end;

  if (fsItalic in Header.Font.Style) then
  begin
    HtmlText := HtmlText
      + '                         font-style: italic;'#13;
  end;

  HtmlAlignment := HtmlAlign(Header.Alignment);

  if (HtmlAlignment <> '') then
  begin
    HtmlText := HtmlText
      + '                         ' + HtmlAlignment + ';'#13;
  end;

  HtmlText := HtmlText
    + '                           border-color : ' + HtmlClr(Header.LineColor) + ';'#13;

  if (PlannerMode = 'vertical') then
  begin
    HtmlText := HtmlText
      + '                         height: ' + IntToStr(Header.CaptionHeight) + 'px;'#13
      + '                         ' + ColorToStyle(Header.Background, Header.BackgroundTo, Header.GradientDirection, AContext) + ';'#13
      + '}'#13;
  end
  else
  begin
    HtmlText := HtmlText
      + '                         width: ' + IntToStr(Header.CaptionHeight) + 'px;'#13
      + '                         ' + ColorToStyle(Header.Background, Header.BackgroundTo, Header.GradientDirection, AContext) + ';'#13
      + '}'#13;
  end;

  if (SideBar.Font.FontName = '') then
    SideBar.Font.FontName := 'Verdana';

  if (SideBar.Font.Size = 0) then
    SideBar.Font.Size := 8;

  if (SideBar.Font.Color = clNone) then
    SideBar.Font.Color := clBlack;

  HtmlText := HtmlText
    + '#' + HTMLName + 'SideBar {'#13
    + '                          color:' + HTMLClr(SideBar.Font.Color) + ';'#13
    + '                          ' + ColorToStyle(SideBar.Background, SideBar.BackgroundTo, SideBar.GradientDirection, AContext) + ';'#13
    + '                          font-size: ' + IntToStr(SideBar.Font.Size) + 'pt;'#13
    + '                          font-family: ' + SideBar.Font.FontName + ';'#13;

  if (fsBold in SideBar.Font.Style) then
  begin
    HtmlText := HtmlText
      + '                        font-weight: bold;'#13;
  end;

  if (fsItalic in SideBar.Font.Style) then
  begin
    HtmlText := HtmlText
      + '                        font-style: italic;'#13;
  end;

  HtmlAlignment := HtmlAlign(SideBar.Alignment);

  if (HtmlAlignment <> '') then
  begin
    HtmlText := HtmlText
      + '                        ' + HtmlAlignment + ';'#13;
  end;

  if (SideBar.OccupiedFont.FontName = '') then
    SideBar.OccupiedFont.FontName := 'Verdana';

  if (SideBar.OccupiedFont.Size = 0) then
    SideBar.OccupiedFont.Size := 8;

  if (SideBar.OccupiedFont.Color = clNone) then
    SideBar.OccupiedFont.Color := clBlack;

  HtmlText := HtmlText
    + '}'#13;

  //Rendering SideBar when occupied.
  HtmlText := HtmlText
    + '#' + HTMLName + 'SideBarOccupied {'#13
    + '                          color:' + HTMLClr(SideBar.OccupiedFont.Color) + ';'#13
    + '                          ' + ColorToStyle(SideBar.OccupiedColor, SideBar.OccupiedColorTo, SideBar.GradientDirection, AContext) + ';'#13
    + '                          font-size: ' + IntToStr(SideBar.OccupiedFont.Size) + 'pt;'#13
    + '                          font-family: ' + SideBar.OccupiedFont.FontName + ';'#13;

  if (fsBold in SideBar.OccupiedFont.Style) then
  begin
    HtmlText := HtmlText
      + '                        font-weight: bold;'#13;
  end;

  if (fsItalic in SideBar.OccupiedFont.Style) then
  begin
    HtmlText := HtmlText
      + '                        font-style: italic;'#13;
  end;

  HtmlAlignment := HtmlAlign(SideBar.Alignment);

  if (HtmlAlignment <> '') then
  begin
    HtmlText := HtmlText
      + '                        ' + HtmlAlignment + ';'#13;
  end;

  HtmlText := HtmlText
    + '}'#13

    //Rendering cells of the SideBar when occupied.
    + '#' + HTMLName + 'SideBarTDO {'#13
    + '                             color : ' + HTMLClr(SideBar.OccupiedFont.Color) + ';'#13
    + '                             font-size: ' + IntToStr(SideBar.OccupiedFont.Size) + 'pt;'#13
    + '                             font-family: ' + SideBar.OccupiedFont.FontName + ';'#13;

  if (fsBold in SideBar.OccupiedFont.Style) then
  begin
    HtmlText := HtmlText
      + '                           font-weight: bold;'#13;
  end;

  if (fsItalic in SideBar.OccupiedFont.Style) then
  begin
    HtmlText := HtmlText
      + '                           font-style: italic;'#13;
  end;

  HtmlAlignment := HtmlAlign(SideBar.Alignment);

  if (HtmlAlignment <> '') then
  begin
    HtmlText := HtmlText
      + '                        ' + HtmlAlignment + ';'#13;
  end;

  HtmlText := HtmlText
    + '}'#13

    //Rendering cells of the SideBar when not occupied.
    + '#' + HTMLName + 'SideBarTD {'#13
    + '                             color : ' + HTMLClr(SideBar.Font.Color) + ';'#13
    + '                             font-size: ' + IntToStr(SideBar.Font.Size) + 'pt;'#13
    + '                             font-family: ' + SideBar.Font.FontName + ';'#13;

  if (fsBold in SideBar.Font.Style) then
  begin
    HtmlText := HtmlText
      + '                           font-weight: bold;'#13;
  end;

  if (fsItalic in SideBar.Font.Style) then
  begin
    HtmlText := HtmlText
      + '                           font-style: italic;'#13;
  end;

  HtmlAlignment := HtmlAlign(SideBar.Alignment);

  if (HtmlAlignment <> '') then
  begin
    HtmlText := HtmlText
      + '                        ' + HtmlAlignment + ';'#13;
  end;

  HtmlText := HtmlText
    + '}'#13;

  HtmlText := HtmlText
    + '</style>'#13;

  Result := HtmlText;
end;

{$WARNINGS OFF}
function TTIWCustomWebPlanner.RenderCellToHTMLCol(APlannerItem: TPlannerItem; RowIndex,PosIndex,MaxPos,TotCol, ColIndex,SubColspan: Integer; AContext : TIWBaseComponentContext): string;
var
  rtf,HtmlAlignment, ColSpan, HtmlText, HtmlBackGroundColor: string;
  BorderCol: string;
  GotSpan,ItemAtPos: boolean;
  HeightRow: Integer;
  WidthColumn, WidthSubColumn: Integer;
  idx,mp, RowSpan, HeightCell: Integer;
  CellID, ItemID: string;
  CellWidth: string;
  CellHeight: string;
  Tmp: string;
  CellColor: TIWColor;

  PosPropIndex : integer;
  Prop : TPositionProp;
  ItemBackColor : TIWColor;
  TopResize, BottomResize : string;
  DblCl : string;
begin
  HtmlText := '';
  CellWidth := '';
  CellHeight := '';

  if (Display.DisplayScale > 0) then
    HeightRow := Display.DisplayScale
  else
    HeightRow := 16;

  GotSpan := False;

  if (Positions > 0) then
  begin
    if (Overlap) then
      WidthColumn := ((Width - SideBar.GetUsedWidth - 2 * BorderSize) div Positions) - 10
    else
      WidthColumn := (Width - SideBar.GetUsedWidth - 2 * BorderSize - Positions) div Positions;
  end
  else
    WidthColumn := (Width - SideBar.GetUsedWidth - 2 * BorderSize);

  mp := Items.MaxItemsInPosAtRow(ColIndex - 1,RowIndex);

  PosPropIndex := ColIndex - 1;

  if (PositionProps.Count > PosPropIndex) then
  begin
    Prop := PositionProps[PosPropIndex];

    if ((Prop.MinSelection > 0) or (Prop.MaxSelection > 0)) then
    begin
      if ((RowIndex < Prop.MinSelection) or (RowIndex > Prop.MaxSelection)) then
        CellID := 'R' + IntToStr(RowIndex) + 'C' + IntToStr(ColIndex - 1) + 'NS'
      else
        CellID := 'R' + IntToStr(RowIndex) + 'C' + IntToStr(ColIndex - 1);
    end
    else
      CellID := 'R' + IntToStr(RowIndex) + 'C' + IntToStr(ColIndex - 1);
  end
  else
    CellID := 'R' + IntToStr(RowIndex) + 'C' + IntToStr(ColIndex - 1);

  if ColIndex = 0 then
    CellHeight := ' height="' + IntToStr(HeightRow) + 'px" ';

  if (MaxPos > 1) then
    WidthSubColumn := (WidthColumn - 2 * MaxPos) div Maxpos
  else
    WidthSubColumn := WidthColumn;

  if RowIndex = 0 then
    CellWidth := ' width="' + IntToStr(WidthSubColumn) + 'px" ';

  CellColor := GetCellColor(ColIndex-1,RowIndex);

  if (SubColspan > 0) then
  begin
    if not (FBackground.Empty) then
      HtmlBackgroundColor := ''
    else
      HtmlBackgroundColor := ' bgcolor="' + HTMLClr(CellColor) + '" ';

    if SubColSpan > 1 then
      ColSpan := ' colspan="' + IntToStr(SubColspan) + '"'
    else
      ColSpan := '';

    ItemAtPos := false;

    for idx := 0 to Items.Count - 1 do
    begin
      if ((Items[idx].ItemPos = ColIndex - 1) and (Items[idx].Visible) and not (Items[idx].InHeader) and (Items[idx].ItemBegin <= RowIndex)
      and (Items[idx].ItemEnd > RowIndex) and (Items.InVisibleLayer(Items[idx].Layer))) then
      begin
        ItemAtPos := true;
        Break;
      end;
    end;

    DblCl := '';
    if ((FSelectionMode = smSingleSelect) and Assigned(FOnPlannerDblClick)) then
      DblCl := ' ondblclick = "PlannerDblClick();" ';

    tmp := CellID + '" ' + CellWidth + CellHeight + ColSpan + HtmlBackGroundColor

       + DblCl
      + ' onMouseOver="mOv(''' + CellID + ''',event)" onMouseMove="ieP(''' + CellID + ''',event)"'
      + ' onMouseDown="mDwn(''' + CellID + ''',event)" '
      + ' onMouseUp="mUp(event)">'
      +  '&nbsp;</td>'#13#10;

    if ItemAtPos then
    begin
      if not (mp = 1) then
        HtmlText := HtmlText
          + '<td id="P' + tmp;
    end
    else
      HtmlText := HtmlText
        + '<td id="' + tmp;
  end
  else
  begin
    if Assigned(APlannerItem) then
    begin
      ItemID := IntToStr(APlannerItem.Index);

      if ((FSelectedIndex > -1) and (APlannerItem = Items[FSelectedIndex]) and (APlannerItem.ShowSelection)) then
        ItemBackColor := APlannerItem.SelectColor
      else
      begin
        if (APlannerItem.Color <> clNone) then
          ItemBackColor := APlannerItem.Color
        else
          ItemBackColor := clWhite;
      end;

      if (APlannerItem.Conflicts = 1) then
      begin
        WidthSubColumn := WidthColumn - 1;
        CellWidth := ' style="width:'+IntToStr(WidthSubColumn) + 'px"';
      end;

      if (MaxPos > 1) and (APlannerItem.ConflictPos = 0) and (TotCol = 1) then
        ColSpan := ' colspan="' + IntToStr(MaxPos) + '"'
      else
        ColSpan := '';

      if (APlannerItem.ItemBegin = RowIndex) then
      begin
        Rowspan := Max(1,APlannerItem.ItemEnd - APlannerItem.ItemBegin);

        if (APlannerItem.ItemEnd + Display.DisplayStart > Display.DisplayEnd) then
          RowSpan := RowSpan - ((APlannerItem.ItemEnd + Display.DisplayStart) - Display.DisplayEnd - 1); //added -1//

        BorderCol := HTMLClr(FBorderColor);

        HeightCell := HeightRow * RowSpan - 1;

        if ( not (APlannerItem.Background) and not (APlannerItem.FixedSize)) then
          HeightCell := HeightCell - 2;

        rtf := RenderItemToHtml(APlannerItem,RowIndex,MaxPos,True,false, HeightCell, AContext);

        {$IFDEF TMSDEBUG}
        outputdebugstring(pchar('RowHeight: ' + Inttostr(Display.DisplayScale) + ' Begin' + IntToStr(APlannerItem.ItemBegin) + ' End : ' + IntTostr(APlannerItem.ItemEnd) + 'is: ' + IntTostr((APlannerItem.ItemEnd - APlannerItem.ItemBegin) * Display.DisplayScale)));
        {$ENDIF}

        // render as hidden item
        if not (APlannerItem.FixedPos) and not (ViewOnly) then
        begin
            FHiddenItems := FHiddenItems
              + '<div id="ITEM' + IntToStr(APlannerItem.Index)
              + 'D" style="border:thin solid; overflow:hidden; border-color:'
              + HtmlClr(APlannerItem.TrackColor) + ';background-color:'
              + HtmlClr(ItemBackColor) + ';position:absolute; visibility:hidden;height:'
              + IntToStr((RowSpan * HeightRow) - 2) + 'px;width:'
              + IntToStr(WidthSubColumn) + 'px"' + ' onMouseMove="resizeItem(event)" onMouseUp="mUp(event)">'
              + rtf + '</div>'#13;
        end;

        if (EditType = edPopup) then
          RenderPopup(APlannerItem);

        HtmlText := HtmlText
          + '<td bgcolor="' + HTMLClr(ItemBackColor) + '" id="I' + CellID + '" onMouseMove="ieP(''' + CellID + ''',event)" width="'
          + IntToStr(WidthSubColumn) + 'px" bordercolor="' + BorderCol + '" rowspan="' + IntToStr(Rowspan) + '"' + ColSpan;

        rtf := RenderItemToHtml(APlannerItem,RowIndex,MaxPos,false,false, HeightCell,AContext);

        if ((FSelectedIndex > -1) and (APlannerItem = Items[FSelectedIndex]) and (APlannerItem.ShowSelection)) then
          HtmlBackGroundColor := ' bgcolor="' + HTMLClr(APlannerItem.SelectColor) + '" '
        else
          HtmlBackGroundColor := ' bgcolor="' + HTMLClr(APlannerItem.Color) + ' "';

        if (MaxPos > 0) and (APlannerItem.ConflictPos = 0) and (TotCol = 1) then
          GotSpan := True;

        HtmlText := HtmlText + HtmlAlignment;

        if (APlannerItem.Background) or (FPlannerInEditing) then
        begin
				  HtmlText := HtmlText
					  + '<div style="overflow:hidden;width:' + IntToStr(WidthSubColumn) + 'px; height: '
 						+ IntToStr(HeightRow * RowSpan - 1) + 'px">' + rtf + '</div></td>'#13;
        end
				else
				begin
          TopResize := '<div id="ITEM' + IntToStr(APlannerItem.Index) + 'RT" style="width:'
					  + IntToStr(WidthSubColumn + 1) + ';height:1px;visibility:hidden;cursor:pointer;cursor:n-resize;background-Color:'
						+ HTMLClr(APlannerItem.TrackColor) + ';overflow:hidden" onMouseDown="startResize(''ITEM'
						+ ItemID + 'RT'',event)">&nbsp;</div>'#13;

          BottomResize := '<div id="ITEM'
					  + IntToStr(APlannerItem.Index) + 'RD" style="width:'
						+ IntToStr(WidthSubColumn + 1) + ';height:1px;visibility:hidden;cursor:pointer;cursor:s-resize;background-Color:'
						+ HTMLClr(APlannerItem.TrackColor) + ';overflow:hidden" onMouseDown="startResize(''ITEM'
 					  + ItemID + 'RD'',event)">&nbsp;</div>'#13;

					if (APlannerItem.BeginOffset = 0) then
					  TopResize := '<div id="ITEM' + IntToStr(APlannerItem.Index) + 'RT" style="width:'
						  + IntToStr(WidthSubColumn + 1) + ';height:1px;cursor:pointer;cursor:n-resize;background-Color:'
							+ HTMLClr(APlannerItem.TrackColor) + ';overflow:hidden" onMouseDown="startResize(''ITEM'
							+ ItemID + 'RT'',event)">&nbsp;</div>'#13;

 				  if (APlannerItem.EndOffset = 0) then
					  BottomResize := '<div id="ITEM'
						  + IntToStr(APlannerItem.Index) + 'RD" style="width:'
  						+ IntToStr(WidthSubColumn + 1) + ';height:1px;cursor:pointer;cursor:s-resize;background-Color:'
  						+ HTMLClr(APlannerItem.TrackColor) + ';overflow:hidden" onMouseDown="startResize(''ITEM'
							+ ItemID + 'RD'',event)">&nbsp;</div>'#13;

					if ((FSelectedIndex > -1) and (APlannerItem = Items[FSelectedIndex]) and (APlannerItem.ShowSelection)) then
          begin
						if ((APlannerItem.ItemEnd > APlannerItem.ItemBegin) and not (GotSpan) and (APlannerItem.ConflictPos = PosIndex)
            and not (APlannerItem.InHeader)) then
            begin
							if ((APlannerItem.FixedSize) or (APlannerItem.FixedPos)) then
							  HtmlText := HtmlText
								  + ColSpan + ' onMouseUp="mUp(event)">' + '<div style="overflow:hidden;width:'
									+ IntToStr(WidthSubColumn) + 'px;height:' + IntToStr(HeightRow * RowSpan - 1) + 'px">'
									+ rtf + '</div></td>'#13
							else
              begin
							  HtmlText := HtmlText
								  + ColSpan + ' onMouseMove="ieP('''',event)" onMouseUp="mUp(event)">'
									+ TopResize + '<div style="overflow:hidden;width:' + IntToStr(WidthSubColumn)
									+ 'px;height:' + IntToStr(HeightRow * RowSpan - 3) + 'px">' + rtf + '</div>'
									+ BottomResize + '</td>'#13;
              end;
            end
						else
						begin
						  if ((APlannerItem.FixedSize) or (ViewOnly) or (APlannerItem.FixedPos) or (APlannerItem.EditMode)) then
              begin
								HtmlText := HtmlText
								  + '  onMouseUp="mUp(event)">' + '<div style="overflow:hidden;width:'
									+ IntToStr(WidthSubColumn) + 'px; height:' + IntToStr(HeightRow * RowSpan - 1) + 'px">'
									+ rtf + '</div></td>'#13;
              end
							else
							begin
								HtmlText := HtmlText
										+ ' onMouseUp="mUp(event)">'
										+ TopResize + '<div style="overflow:hidden;width:' + IntToStr(WidthSubColumn)
										+ 'px;height:' + IntToStr(HeightRow * RowSpan - 3) + 'px">' + rtf + '</div>'
										+ BottomResize + '</td>';
              end;
            end;
          end
					else
					begin
						if ((APlannerItem.ItemEnd > APlannerItem.ItemBegin) and not (GotSpan) and (APlannerItem.ConflictPos = PosIndex)
            and not (APlannerItem.InHeader)) then
            begin
							if ((APlannerItem.FixedSize) or (APlannerItem.FixedPos) or (APlannerItem.EditMode)) then
								HtmlText := HtmlText
									+ ColSpan + ' onMouseUp="mUp(event)" bordercolor="#000000">'
									+ '<div style="overflow:hidden;width: ' + IntToStr(WidthSubColumn) + 'px; height: '
									+ IntToStr(HeightRow * RowSpan - 1) + 'px">' + rtf + '</div></td>'
							else
              begin
								HtmlText := HtmlText
									+ ColSpan + ' bordercolor="#000000" onMouseMove="ieP('''',event)" '
									+ ' onMouseUp="mUp(event)">'
									+ TopResize + '<div style="overflow:hidden;width:' + IntToStr(WidthSubColumn)
									+ 'px;height:' + IntToStr(HeightRow * RowSpan - 3) + 'px">' + rtf + '</div>'
									+ BottomResize + '</td>';
							end;
            end
						else
						begin
							if ((APlannerItem.FixedSize) or (APlannerItem.FixedPos) or (APlannerItem.EditMode)) then
								HtmlText := HtmlText
											+ ' onMouseUp="mUp(event)">' + '<div style="overflow:hidden;width:'
											+ IntToStr(WidthSubColumn) + 'px; height: ' + IntToStr(HeightRow * RowSpan - 1) + 'px">'
											+ rtf + '</div></td>'
							else
							begin
								HtmlText := HtmlText
									+ ' onMouseUp="mUp(event)">'#13
									+ TopResize + '<div style="overflow:hidden;width:' + IntToStr(WidthSubColumn)
									+ 'px;height:' + IntToStr(HeightRow * RowSpan - 3) + 'px">' + rtf + '</div>'
									+ BottomResize + '</td>';
							end;
						end;
					end;
				end;
			end;
		end;
	end;

  Result := HtmlText;
end;
{$WARNINGS ON}

{ Rendering of the Planner in vertical mode }
function TTIWCustomWebPlanner.RenderToHTMLCol(AContext : TIWBaseComponentContext):string;
var
  Color, BorderColor, BorderCollapse, HtmlText, ImgName,w,sw,LineColor: String;
  ColSpan, HtmlAlignment, txt: String;
  GroupIndex, MaxPos, ColIndex, CaptionCols: Integer;
  idx, TotCol, RowIndex, PosIndex: Integer;
  WidthColumn, SubColspan, BorderS: Integer;
  APlannerItem : TPlannerItem;
  CellWidth: string;
  CellNof : string;
  SideBarLeft, SideBarRight : Boolean;
  SCellID : string;
  ColIn : integer;
  Prop : TPositionProp;

  dblCl : string;
  headerClick : string;
begin
  HtmlText := '';
  FHiddenItems := '';
  APlannerItem := nil;

  if IsBrowserIE(AContext) then
    BorderS := 2 * BorderSize
  else
    BorderS := 4 * BorderSize;

  BorderColor := HTMLClr(FBorderColor);

  SideBarLeft := false;
  if ((SideBar.Position = spLeft) or (SideBar.Position = spLeftRight)) then
    SideBarLeft := true;

  SideBarRight := false;
  if ((SideBar.Position = spRight) or (SideBar.Position = spLeftRight)) then
    SideBarRight := true;

  if IsBrowserIE(AContext) and (FBorderCollapsed) then
    bordercollapse := 'style="border-collapse:collapse"'
  else
    bordercollapse := '';

  CellNof := ' style="overflow:hidden;"';

  if FBackground.Empty then
  begin
    HtmlText := HtmlText
      + FontToHtmlFont(Self.Font) + '<table id="TABLE' + HTMLName + '" onkeydown="tableKey();"' + bordercollapse + ' border="' + IntToStr(FBorderSize)
      + '" bordercolor="' + BorderColor + '" cellpadding="0" cellspacing="0" style="font-size:2;">';
  end
  else
  begin
    if FBackground.IsGIF then
//        ImgName := TIWServerControllerBase.NewCacheFile('gif', true)
        ImgName := GraphicToCacheFile(FBackground, imgGIF)
    else
//        ImgName := TIWServerControllerBase.NewCacheFile('jpg', true);
        ImgName := GraphicToCacheFile(FBackground, imgJPEG);

//    FBackground.SaveToFile(ImgName);

    HtmlText := HtmlText
      + FontToHtmlFont(Self.Font) + '<table id="TABLE' + HTMLName + '" onkeydown="tableKey();" '
      + 'BACKGROUND="' + ImgName + '"'
      + bordercollapse + ' border="' + IntToStr(FBorderSize) + '" bordercolor="' + BorderColor + '">';
  end;

  if Positions > 0 then
  begin
    if Overlap then
      WidthColumn := ((Width - SideBar.GetUsedWidth - BorderS) div Positions) - 10
    else
      WidthColumn := (Width - SideBar.GetUsedWidth - BorderS - Positions) div Positions;
  end
  else
    WidthColumn := (Width - SideBar.Width - BorderS);

  w := 'width="' + IntToStr(WidthColumn) + 'px" ';
  sw := 'width="' + IntToStr(SideBar.Width) + 'px" ';

  HtmlText := HtmlText + RenderCSSStyle('vertical', AContext);

  if FPlannerHeader.FVisible then
  begin
    LineColor := HTMLClr(FPlannerHeader.LineColor);

    // Code to display the groupcaptions
    if (PositionGroup > 1) then
    begin
       HtmlText := HtmlText
         + '<tr height="' + IntToStr(Header.CaptionHeight) + 'px" width="' +  IntToStr(Width) + 'px">'#13;

      for ColIndex := 0 to Positions do
      begin
        GroupIndex := (ColIndex - 1) div PositionGroup;

        if ((ColIndex = 0) and (FSideBar.Visible) and (SideBarLeft)) then
        begin
          HtmlText := HtmlText
            + '<td id="' + HTMLName + 'Caption" bordercolor="'
            + LineColor + '"' + sw + '>'#13;
        end
        else if (ColIndex > 0) then
        begin
          if ((ColIndex - 1) mod PositionGroup = 0) then
          begin
            MaxPos := 0;
            for CaptionCols := 0 to PositionGroup - 1 do
              MaxPos := MaxPos + Items.MaxItemsInPos((ColIndex + CaptionCols) - 1);

            {$IFDEF TMSDEBUG}
            outputdebugstring(pchar(IntToStr(ColIndex - 1) + ':' + IntToStr(MaxPos)));
            {$ENDIF}

            if (Overlap) then
            begin
              if (MaxPos > 1) and (ColIndex > 0) then
                ColSpan := ' colspan="' + IntToStr(MaxPos + PositionGroup)+'"'
              else
                ColSpan := ' colspan="2"';
            end
            else
            begin
              if (MaxPos > 1) and (ColIndex > 0) then
                ColSpan := ' colspan="'+ IntToStr(MaxPos)+'" '
              else
                ColSpan := ' ';
            end;

            HtmlText := HtmlText
              + '<td id="' + HTMLName + 'Caption" ' + ColSpan + '>'#13;

            if (FPlannerHeader.GroupCaptions.Count >= (Positions div PositionGroup)) then
              txt := CLToBr(FPlannerHeader.GroupCaptions[GroupIndex])
            else
              txt := HtmlNonBreakingSpace;
          end;
        end;

        if ((ColIndex = 0) and (FSideBar.Visible) and (SideBarLeft) or ((ColIndex - 1) mod PositionGroup = 0)) then
        begin
          HtmlText := HtmlText
            + '<div style="overflow:hidden; height:'+ IntToStr(FPlannerHeader.CaptionHeight) +'px">' + txt + '</div>'
            + '</td>';
        end;
      end;
      if ((FSideBar.Visible) and (SideBarRight)) then
        HtmlText := HtmlText
          + '<td id="' + HTMLName + 'Caption" bordercolor="' + LineColor + '" height="' + intToStr(FPlannerHeader.CaptionHeight)
          + 'px"' + sw + '>'#13
          + '<div style="overflow:hidden; height:' + IntToStr(FPlannerHeader.CaptionHeight) +'px; width:'
          + IntToStr(SideBar.Width) + 'px">&nbsp;</div>' + '</td>'#13;

      HtmlText := HtmlText + '</tr>'#13;
    end;

    txt := '';

    if (FPlannerHeader.Position = hpTop) or (FPlannerHeader.Position = hpBoth) then
    begin
    // Code to display the Headercaptions
      HtmlText := HtmlText
        + '<tr height="' + IntToStr(Header.CaptionHeight) + 'px" width="' + IntToStr(Width) + 'px">'#13;

      for ColIndex := 0 to Positions do
      begin
        if ((ColIndex = 0) and (FSideBar.Visible) and (SideBarLeft)) then
        begin
          HtmlText := HtmlText
            + '<td valign="top" id="' + HTMLName + 'CaptionTL" bordercolor="' + LineColor + '" '
            + ' width="' + IntToStr(SideBar.Width - 2) + 'px">'#13;

          if IsBrowserIE(AContext) then
           (*
           HtmlText := HtmlText
           + '<div style="overflow:hidden;width:' + IntToStr(SideBar.Width) + 'px">&nbsp;</div></td>'#13
           *)
            HtmlText := HtmlText
              + '<div style="overflow:hidden;width:' + IntToStr(SideBar.Width) + 'px">' + TopLeftCell + '</div></td>'#13
          else
            HtmlText := HtmlText
                + '<div style="overflow:hidden;height:' + IntToStr(Header.CaptionHeight) + 'px;width:' + IntToStr(SideBar.Width - 2) + 'px">'
                + '<table width="' + IntToStr(SideBar.Width - 2) + 'px" cellpadding="0" cellspacing="0"><tr><td width=100% style="height:'
                + IntToStr(Header.CaptionHeight) + 'px;"><div style="overflow:hidden;height:' + IntToStr(Header.CaptionHeight) + 'px;width:'
  //              + IntToStr(SideBar.Width - 2) + 'px">&nbsp;</div></td></tr></table></div></td>'#13
                + IntToStr(SideBar.Width - 2) + 'px">' + TopLeftCell + '</div></td></tr></table></div></td>'#13
        end
        else if (ColIndex > 0) then
        begin
          MaxPos := Items.MaxItemsInPos(ColIndex - 1);
          {$IFDEF TMSDEBUG}
          outputdebugstring(pchar(IntToStr(colindex - 1) + ':' + IntToStr(MaxPos)));
          {$ENDIF}

          if (MaxPos > 1) and (ColIndex > 0) then
            ColSpan := ' colspan="' + IntToStr(MaxPos) + '" '
          else
            ColSpan := '';

          if IsBrowserIE(AContext) then
            CellWidth := ' width="' + IntToStr(WidthColumn) + 'px" '
          else
            CellWidth := ' width="' + IntToStr(WidthColumn - 1) + 'px" ';

          if Assigned(OnHeaderClick) then
            headerClick := ' onclick="return SubmitClick('#39 + HTMLName + #39','#39 + 'HeaderClick#' + IntToStr(ColIndex-1) + #39');" ';

          if Assigned(FOnAsyncHeaderClick) then
          begin
            headerclick := 'onclick="setToHiddenInput(''H' + IntToStr(ColIndex - 1) + ''');processAjaxEvent(''onHeaderClick'', '
              + HTMLControlImplementation.IWCLName
              + ',''' + HTMLName + '.' + 'DoOnAsyncHeaderClick' + ''','
              + 'true' + ', null, '
              + 'true' + ');"';

          end;

          Acontext.WebApplication.RegisterCallBack(HTMLName+'.DoOnAsyncHeaderClick', DoOnAsyncHeaderClick);

          HtmlText := HtmlText
            + '<td ' + headerClick + CellWidth + 'id="' + HTMLName + 'Caption" '
            + ColSpan + '>'#13;

          if FPlannerHeader.Captions.Count > ColIndex then
            txt := CLToBr(FPlannerHeader.Captions[ColIndex])
          else
            txt := HtmlNonBreakingSpace;
        end;

        if (ColIndex > 0) then
        begin
          HtmlText := HtmlText
            + '<div style="overflow:hidden; height:';

            if IsBrowserIE(AContext) then
              HtmlText := HtmlText
                + IntToStr(FPlannerHeader.CaptionHeight) +'px; width:' + IntToStr(WidthColumn) + '">'
            else
              HtmlText := HtmlText
                + IntToStr(FPlannerHeader.CaptionHeight) +'px; width:' + IntToStr(WidthColumn-1) + '">';

              HtmlText := HtmlText
                + txt + '</div>' + '</td>'#13;

            if (Overlap) then
            begin
              if IsBrowserIE(AContext) then
               HtmlText := HtmlText
                 + '<td bgcolor="' + HtmlClr(OverlapColor) + '" bordercolor="' + LineColor + '" width="8px"></td>'#13
             else
               HtmlText := HtmlText
                + '<td bgcolor="' + HtmlClr(OverlapColor) + '" bordercolor="' + LineColor + '" width="9px">'
                + '&nbsp;</td>';
            end;
          end;
        end;

        if ((FSideBar.Visible) and (SideBarRight)) then
          HtmlText := HtmlText
            + '<td id="' + HTMLName + 'Caption" bordercolor="' + LineColor + '"'
            + sw + '>'#13
            + '<div style="overflow:hidden; height:'
            + IntToStr(FPlannerHeader.CaptionHeight) +'px; width:' + IntToStr(SideBar.Width) + 'px">&nbsp;</div>'
            + '</td>'#13;

        HtmlText := HtmlText + '</tr>';
      end;
    end;


    // code to display the items in the header
    FNumberOfHeaderItems := 0;
    for idx := 0 to Items.Count-1 do
    begin
      APlannerItem := Items[idx];
      if Assigned(APlannerItem) then
      begin
        if APlannerItem.InHeader then
          FNumberOfHeaderItems := FNumberOfHeaderItems + 1;
      end;
    end;

    if FNumberOfHeaderItems > 0 then
    begin
      HtmlText := HtmlText + '<tr>'#13;

      for ColIndex := 0 to Positions do
      begin
        if ((ColIndex = 0) and (FSideBar.Visible) and (SideBarLeft)) then
          HtmlText := HtmlText
            + '<td bordercolor="' + LineColor + '" ' + sw
            + '>'#13 + '&nbsp;' + '</div></td>'#13

        else if (ColIndex > 0) then
        begin
          MaxPos := Items.MaxItemsInPos(ColIndex - 1);
          {$IFDEF TMSDEBUG}
          outputdebugstring(pchar(inttostr(ColIndex - 1) + ':' + IntToStr(MaxPos)));
          {$ENDIF}

          if (MaxPos > 1) and (ColIndex > 0) then
            ColSpan := ' colspan="' + IntToStr(MaxPos) + '"'
          else
            ColSpan := ' ';

          if Background.Empty then
            CellWidth := ' style="width:' + IntToStr(WidthColumn - 1) + 'px" '
          else
            CellWidth := ' style="width:' + IntToStr(WidthColumn - 2) + 'px" ';

          HtmlText := HtmlText
            + '<td valign="top" bordercolor="' + Color + '"' + ' width="'
            + IntToStr(WidthColumn - 1) + 'px" ';

          HtmlAlignment := HTMLAlign(FPlannerHeader.Alignment);
          HtmlText := HtmlText + ' style="' + HtmlAlignment + '" ' + ColSpan;

          if IsBrowserIE(AContext) then
            txt := ''
          else
            txt := '&nbsp;';

          for idx := 0 to Items.Count-1 do
          begin
            APlannerItem := Items[idx];
              if ((Items.InVisibleLayer(APlannerItem.Layer)) and (APlannerItem.ItemPos = ColIndex - 1) and (APlannerItem.InHeader) and (APlannerItem.Visible)) then
              begin
               txt := txt + RenderItemToHtml(APlannerItem,0,MaxPos,false,false,0, AContext);

               if (EditType = edPopup) then
                 RenderPopup(APlannerItem);
              end;
          end;

        HtmlText := HtmlText
          + '>'#13 + '<div style="overflow:hidden;width:' + IntToStr(WidthColumn - 1) + 'px">'#13
          + txt + '</div></td>'#13;

        if (Overlap) then
        begin
          if IsBrowserIE(AContext) then
            HtmlText := HtmlText
              + '<td bordercolor="' + HTMLClr(Header.LineColor) + '" bgcolor="' + HtmlClr(OverlapColor) + '" width="8px"></td>'#13
          else
            HtmlText := HtmlText
              + '<td bordercolor="' + HTMLClr(Header.LineColor) + '" bgcolor="' + HtmlClr(OverlapColor)
              + '" width="9px">&nbsp;</td>'#13;
          end;
        end;
      end;
      HtmlText := HtmlText + '</tr>'#13;
    end;

  // code to display the not-in-header-items
  w := '';
  TotCol := 0;

  for RowIndex := 0 to Display.GetDisplay - 1 do
  begin

    HtmlText := HtmlText
      + '<tr style="height:' + IntToStr(Display.DisplayScale) + 'px";>'#13;

    for ColIndex := 0 to Positions do
    begin
      if ((ColIndex = 0) and (FSideBar.Visible) and (SideBarLeft)) then
        HtmlText := HtmlText + RenderSideBar(RowIndex, ColIndex, AContext)

      else if (ColIndex > 0) then
      begin
        MaxPos := Items.MaxItemsInPos(ColIndex - 1);

        //if (RowIndex = Display.GetDisplay - 1) then
          //OutputDebugString(pchar('Col ' + IntToStr(ColIndex) + ' MaxPos :' + IntToStr(MaxPos)));

        PosIndex := 0;
        SubColspan := 0;

        while PosIndex < MaxPos do
        begin
          APlannerItem := Items.FindItem(RowIndex, ColIndex - 1);

          if Assigned(APlannerItem) then
            TotCol := APlannerItem.Conflicts
          else
            TotCol := 0;

          APlannerItem := Items.FindItemPosIdx(RowIndex, ColIndex - 1, PosIndex);
          if Assigned(APlannerItem) and APlannerItem.InHeader then
            APlannerItem := nil;

          if Assigned(APlannerItem) then
          begin
            FRow := RowIndex;
            if (SubColspan > 0) then
              HtmlText := HtmlText + RenderCellToHtmlCol(APlannerItem, RowIndex, PosIndex, MaxPos,TotCol, ColIndex,SubColspan, AContext);

            SubColspan := 0;

            HtmlText := HtmlText + RenderCellToHtmlCol(APlannerItem,RowIndex,PosIndex,MaxPos,TotCol, ColIndex,SubColspan, AContext);
          end
          else
            SubColspan := SubColspan + 1;

          PosIndex := PosIndex + 1;
        end;

        if (SubColspan > 0)  then
        begin
          HtmlText := HtmlText + RenderCellToHtmlCol(APlannerItem, RowIndex, PosIndex, MaxPos,TotCol,ColIndex,SubColspan, AContext);
        end;

        // render mini cell here
        if (Overlap) then
        begin
          ColIn := ColIndex - 1;

          if (PositionProps.Count > ColIn) then
          begin
            Prop := PositionProps[ColIn];

            if ((Prop.MinSelection > 0) or (Prop.MaxSelection > 0)) then
            begin
              if ((RowIndex < Prop.MinSelection) or (RowIndex > Prop.MaxSelection)) then
                SCellID := 'SR' + IntToStr(RowIndex) + 'C' + IntToStr(ColIn) + 'NS'
              else
                SCellID := 'SR' + IntToStr(RowIndex) + 'C' + IntToStr(ColIn);
            end
            else
              SCellID := 'SR' + IntToStr(RowIndex) + 'C' + IntToStr(ColIn);
          end
          else
            SCellID := 'SR' + IntToStr(RowIndex) + 'C' + IntToStr(ColIn);


          DblCl := '';
          if ((FSelectionMode = smSingleSelect) and Assigned(FOnPlannerDblClick))then
            DblCl := ' ondblclick="PlannerDblClick()";';

          if IsBrowserIE(AContext) then
            HtmlText := HtmlText
              //+ '<td id="' + SCellID + '" ' --> in commentaar gezet
              + '<td id="' + SCellID + '" ' +  DblCl
              + 'bgcolor="' + HtmlClr(OverlapColor) + '" width="8px" '
              + 'onMouseMove="ieP(''' + SCellID + ''',event)" '
              + 'onMouseOver="mOv(''' + SCellID + ''',event)" '
              + 'onMouseDown="mDwn(''' + SCellID + ''',event)" '
              + 'onMouseUp="mUp(event)";><span class="SPS">'#13
              + '<div class="SPC">&nbsp;</div></span></td>'#13
          else
            HtmlText := HtmlText
              + '<td id="' + SCellID + '" ' +  DblCl

              + 'bgcolor="' + HtmlClr(OverlapColor)
              + '" width="9px" onMouseMove="ieP(''' + SCellID + ''',event)" '
              + 'onMouseOver="mOv(''' + SCellID + ''',event)" '
              + 'onMouseDown="mDwn(''' + SCellID + ''',event)" '
              + 'onMouseUp="mUp(event)";>&nbsp;</td>'#13;
        end;
      end;
    end;

    if (SideBarRight) and (FSideBar.Visible) then
      HtmlText := HtmlText + RenderSideBar(RowIndex, Positions, AContext);

    HtmlText := HtmlText + '</tr>'#13;
  end;

  if (FPlannerHeader.Visible) and ((FPlannerHeader.Position = hpBottom) or (FPlannerHeader.Position = hpBoth)) then
  begin
  // Code to display the Headercaptions
    HtmlText := HtmlText
      + '<tr height="' + IntToStr(Header.CaptionHeight) + 'px" width="' + IntToStr(Width) + 'px">'#13;

    for ColIndex := 0 to Positions do
    begin
      if ((ColIndex = 0) and (FSideBar.Visible) and (SideBarLeft)) then
      begin
        HtmlText := HtmlText
          + '<td valign="top" id="' + HTMLName + 'CaptionTL" bordercolor="' + LineColor + '" '
          + ' width="' + IntToStr(SideBar.Width - 2) + 'px">'#13;
        if IsBrowserIE(AContext) then
          HtmlText := HtmlText
            + '<div style="overflow:hidden;width:' + IntToStr(SideBar.Width) + 'px">' + TopLeftCell + '</div></td>'#13
        else
          HtmlText := HtmlText
            + '<div style="overflow:hidden;height:' + IntToStr(Header.CaptionHeight) + 'px;width:' + IntToStr(SideBar.Width - 2) + 'px">'
            + '<table width="' + IntToStr(SideBar.Width - 2) + 'px" cellpadding="0" cellspacing="0"><tr><td width=100% style="height:'
            + IntToStr(Header.CaptionHeight) + 'px;"><div style="overflow:hidden;height:' + IntToStr(Header.CaptionHeight) + 'px;width:'
            + IntToStr(SideBar.Width - 2) + 'px">' + TopLeftCell + '</div></td></tr></table></div></td>'#13
      end
      else if (ColIndex > 0) then
      begin
        MaxPos := Items.MaxItemsInPos(ColIndex - 1);
        {$IFDEF TMSDEBUG}
        outputdebugstring(pchar(IntToStr(colindex - 1) + ':' + IntToStr(MaxPos)));
        {$ENDIF}

        if (MaxPos > 1) and (ColIndex > 0) then
          ColSpan := ' colspan="' + IntToStr(MaxPos) + '" '
        else
          ColSpan := '';

        if IsBrowserIE(AContext) then
          CellWidth := ' width="' + IntToStr(WidthColumn) + 'px" '
        else
          CellWidth := ' width="' + IntToStr(WidthColumn - 1) + 'px" ';

        if Assigned(OnHeaderClick) then
          headerClick := ' onclick="return SubmitClick('#39 + HTMLName + #39','#39 + 'HeaderClick#' + IntToStr(ColIndex-1) + #39');" ';

        HtmlText := HtmlText
          + '<td ' + headerClick + CellWidth + 'id="' + HTMLName + 'Caption" '
          + ColSpan + '>'#13;

        if FPlannerHeader.Captions.Count > ColIndex then
          txt := CLToBr(FPlannerHeader.Captions[ColIndex])
        else
          txt := HtmlNonBreakingSpace;
      end;

      if (ColIndex > 0) then
      begin
        HtmlText := HtmlText
          + '<div style="overflow:hidden; height:';

        if IsBrowserIE(AContext) then
          HtmlText := HtmlText
            + IntToStr(FPlannerHeader.CaptionHeight) +'px; width:' + IntToStr(WidthColumn) + '">'
        else
          HtmlText := HtmlText
            + IntToStr(FPlannerHeader.CaptionHeight) +'px; width:' + IntToStr(WidthColumn-1) + '">';

          HtmlText := HtmlText
            + txt + '</div>' + '</td>'#13;

        if (Overlap) then
        begin
          if IsBrowserIE(AContext) then
            HtmlText := HtmlText
              + '<td bgcolor="' + HtmlClr(OverlapColor) + '" bordercolor="' + LineColor + '" width="8px"></td>'#13
          else
            HtmlText := HtmlText
             + '<td bgcolor="' + HtmlClr(OverlapColor) + '" bordercolor="' + LineColor + '" width="9px">'
             + '&nbsp;</td>';
        end;
      end;

      if ((FSideBar.Visible) and (SideBarRight)) then
        HtmlText := HtmlText
          + '<td id="' + HTMLName + 'Caption" bordercolor="' + LineColor + '"'
          + sw + '>'#13
          + '<div style="overflow:hidden; height:'
          + IntToStr(FPlannerHeader.CaptionHeight) +'px; width:' + IntToStr(SideBar.Width) + 'px">&nbsp;</div>'
          + '</td>'#13;

    end;
    HtmlText := HtmlText + '</tr>';
  end;
  HtmlText := HtmlText + '</table></Font>';

  if (EditType = edPopup) then
    Result := HTMLText + FHiddenItems + FHiddenPopups
  else
    Result := HTMLText + FHiddenItems;
end;

function TTIWCustomWebPlanner.RenderSideBar(RowIndex, ColIndex : integer; AContext : TIWBaseComponentContext): string;
var
  Txt, CellDiv, CellDivT, IDCell, HtmlSideBar : string;
  HtmlBackGroundColor, SideBarValue, Hint, HintSideBar : String;
  SideBarAlignment, sw, BorderCollapse, CellProps, CellHeight : string;

  SideBarW, Delta, HeightRow : integer;

  Occupied : Boolean;

  APlannerItem : TPlannerItem;
begin
  HtmlSideBar := '';
  Txt := '';
  HtmlBackgroundColor := '';
  Hint := '';

  SideBarAlignment := HTMLAlign(FSideBar.Alignment);
  SideBarW := SideBar.Width - 2;
  sw := 'width="' + IntToStr(SideBarW) + 'px" ';

  if (Display.DisplayScale > 0) then
    HeightRow := Display.DisplayScale
  else
    HeightRow := 16;

  BorderCollapse := ' style="border-collapse:collapse;"';

  if not IsBrowserIE(AContext) then
  begin
    BorderCollapse := ' ';
    HeightRow  := HeightRow - 1;
  end;

  CellHeight := ' style="height:' + IntToStr(HeightRow - 1) + 'px;"'#13;
  CellDiv := '<div style="overflow:hidden;height:' + IntToStr(HeightRow - 1) + 'px">'#13;
  CellDivT := '<div style="overflow:hidden;height:' + IntToStr((HeightRow - 2) div 2) + 'px">'#13;

  Occupied := false;

  CellProps := ' cellpadding="0" cellspacing="0" ';

  if FSideBar.ShowOccupied then
  begin
    APlannerItem := Items.FindItemIdx(RowIndex);

    Occupied := (APlannerItem <> nil) and FSidebar.ShowOccupied;
  end;

  if (Occupied) then
    IDCell := 'id="' + HTMLName + 'SideBarTDO" '
  else
    IDCell := 'id="' + HTMLName + 'SideBarTD" ';

  HtmlBackGroundColor := ColorToHtmlHexBgColor(FSidebar.Background);

  Txt := PlannerGetIdCol(Self, RowIndex, ColIndex);
  HintSideBar := Txt;

  if (Display.DisplayText > 0) then
  begin
    Delta := RowIndex mod Display.DisplayText;

    if (Delta <> 0) then
      Txt := '';
  end;

  SideBarValue := Txt;

  Txt := '<table width="' + IntToStr(SideBarW) + 'px" ' + BorderCollapse + CellProps + '>'#13
      + ' <tr><td ' + IDCell + ' width=100% ' + CellHeight + '>'#13
      + ' <div style="overflow:hidden;height:' + IntToStr(HeightRow - 2) + 'px; width: ' + IntToStr(SideBarW) + 'px">'#13
      + SideBarValue + #13 + '</div></td>'#13 + '</tr></table>'#13;

  if (SideBar.ShowHint) then
    Hint := ' title="' + HintSideBar + '" ';

  if (Occupied) then
  begin
    HtmlSideBar := HtmlSideBar
      + '<td id="' + HTMLName + 'SideBarOccupied" ' + Hint + sw + '>'#13
      + '<div style="overflow:hidden;height: ' + IntToStr(HeightRow - 1) + 'px; width: '
      + IntToStr(SideBarW) + 'px">'#13 + Txt + '</div></td>'#13;
  end
  else
  begin
    HtmlSideBar := HtmlSideBar
      + '<td id="' + HTMLName + 'SideBar" ' + Hint + sw + '>'#13
      + '<div style="overflow:hidden;height: ' + IntToStr(HeightRow - 1) + 'px; width: '
      + IntToStr(SideBarW) + 'px">'#13 + Txt + '</div></td>'#13;
  end;

  Result := HtmlSideBar;
end;

{$WARNINGS OFF}
// Delphi 7 causes a false warning on return value might be undefined

{ Performs HTML rendering of Planner cell in horizontal mode }

function TTIWCustomWebPlanner.RenderCellToHTMLRow(APlannerItem: TPlannerItem; RowIndex, PosIndex, MaxPos, TotCol, ColIndex, SubColspan: Integer; AContext : TIWBaseComponentContext): string;
var
  rtf,HtmlAlignment, ColSpan, HtmlText, HtmlBackGroundColor: string;
  BorderCol: string;
  GotSpan,ItemAtPos: boolean;
  HeightRow, idx,mp, RowSpan: Integer;
  CellColor: TIWColor;
  CellID: string;
  CellHeight, CellWidth, ItemWidth, ItemWidthTable, BorderCollapse,CellDiv,CellDivTable: string;
  tmp, ItemID: string;
  HeightCell,CellH,CellW,CellWTable: Integer;
  ItemBackColor : TIWColor;
  VisibilityDiv : string;
  Prop : TPositionProp;
  PosPropIndex : integer;

  DblCl : string;
begin
  Result := '';

  if IsBrowserIE(AContext) then
    BorderCollapse := 'style="border-collapse:collapse"'
  else
    BorderCollapse := '';

  if (Display.DisplayScale > 0) then
    HeightRow := Display.DisplayScale
  else
    HeightRow := 16;

  mp := Items.MaxItemsInPosAtRow(ColIndex - 1,RowIndex);

  PosPropIndex := ColIndex - 1;

  if (PositionProps.Count > PosPropIndex) then
  begin
    Prop := PositionProps[PosPropIndex];

    if ((Prop.MinSelection > 0) or (Prop.MaxSelection > 0)) then
    begin
      if ((RowIndex < Prop.MinSelection) or (RowIndex > Prop.MaxSelection)) then
        CellID := 'R' + IntToStr(RowIndex) + 'C' + IntToStr(ColIndex - 1) + 'NS'
      else
        CellID := 'R' + IntToStr(RowIndex) + 'C' + IntToStr(ColIndex - 1);
    end
    else
      CellID := 'R' + IntToStr(RowIndex) + 'C' + IntToStr(ColIndex - 1);
  end
  else
    CellID := 'R' + IntToStr(RowIndex) + 'C' + IntToStr(ColIndex - 1);

  CellColor := GetCellColor(ColIndex - 1,RowIndex);

  if IsBrowserIE(AContext) then
    CellWidth := ' style="width:' + IntToStr(HeightRow) + 'px"'
  else
    CellWidth := ' style="width:' + IntToStr(HeightRow) + 'px"';

  if Overlap then
    HeightCell := (((Height - SideBar.GetUsedWidth - 2 * BorderSize) div Positions) - 10 - MaxPos + 1) div MaxPos
  else
    HeightCell := (((Height - SideBar.GetUsedWidth - 2 * BorderSize) div Positions) - MaxPos) div MaxPos;

  CellHeight := ' style="height: ' + IntToStr(HeightCell) + 'px;"';
  CellH := HeightCell;

  HTMLText := '';

  GotSpan := False;

  if IsBrowserIE(AContext) then
    CellDiv := '<div style="overflow:hidden;height: ' + IntToStr(HeightCell) + 'px;width:' + IntToStr(HeightRow) + 'px">'
  else
    CellDiv := '<div style="overflow:hidden;height: ' + IntToStr(HeightCell) + 'px;width:' + IntToStr(HeightRow) + 'px">';

  if SubColspan > 0 then
  begin
    if not (FBackground.Empty) then
      HtmlBackgroundColor := ''
    else
      HtmlBackgroundColor := ' bgcolor="' + HTMLClr(CellColor) + '" ';

      if SubColSpan > 1 then
        ColSpan := ' rowspan="' + IntToStr(SubColspan) + '"';

    ItemAtPos := False;
    for idx := 0 to Items.Count - 1 do
    begin
      if ((Items.InVisibleLayer(Items[idx].Layer)) and (Items[idx].ItemPos = ColIndex - 1) and(Items[idx].Visible) and not(Items[idx].InHeader) and (Items[idx].ItemBegin <= RowIndex) and (Items[idx].ItemEnd > RowIndex)) then
      begin
        ItemAtPos := true;
        Break;
      end;
    end;

    if IsBrowserIE(AContext) then
      tmp := CellID + '" width="' + IntToStr(HeightRow - 1) + 'px" height="' + IntToStr(HeightCell) + 'px" ' + ColSpan + HtmlBackGroundColor
    else
      tmp := CellID + '" width="' + IntToStr(HeightRow) + 'px" height="' + IntToStr(HeightCell) + 'px"' + ColSpan + HtmlBackGroundColor;


    DblCl := '';
    if ((FSelectionMode = smSingleSelect) and Assigned(FOnPlannerDblClick)) then
    begin
      DblCl := ' ondblclick = "PlannerDblClick();" ';
      tmp := tmp
       + DblCl;
    end;


    tmp := tmp
         + ' onMouseMove="ieP(''' + CellID + ''',event)" '
         + ' onMouseOver="mOv(''' + CellID + ''',event)" '
         + ' onMouseDown="mDwn(''' + CellID + ''',event)" '
         + ' onMouseUp="mUp(event)">'
        + '&nbsp;</td>'#13#10;

    if ItemAtPos then
    begin
      if not (mp = 1) then
        HtmlText := HtmlText + '<td id="P' + tmp;
    end
    else
      HtmlText := HtmlText + '<td id="' + tmp;
  end
  else
  begin
    if Assigned(APlannerItem) then
    begin
      ItemID := IntToStr(APlannerItem.Index);

      if ((FSelectedIndex > -1) and (APlannerItem = FItems[FSelectedIndex]) and (APlannerItem.ShowSelection)) then
        ItemBackColor := APlannerItem.SelectColor
      else
      begin
        if (APlannerItem.Color <> clNone) then
          ItemBackColor := APlannerItem.Color
        else
          ItemBackColor := clWhite;
      end;

      if APlannerItem.Conflicts = 1 then
      begin
        if Overlap then
        begin
          CellHeight := ' style="height:' + IntToStr((Height - SideBar.GetUsedWidth - 2 * BorderSize) div Positions - 10) + 'px;"';
          CellH := (Height - SideBar.GetUsedWidth - 2 * BorderSize) div Positions - 10;
        end
        else
        begin
          CellHeight := ' style="height:' + IntToStr((Height - SideBar.GetUsedWidth - 2 * BorderSize - Positions) div Positions) + 'px;"';
          CellH := (Height - SideBar.GetUsedWidth - 2 * BorderSize - Positions) div Positions;
        end;
      end;

      if APlannerItem.ItemEnd > APlannerItem.ItemBegin + 1 then
      begin
        if APlannerItem.ItemEnd + Display.DisplayStart > Display.DisplayEnd then
        begin
          if IsBrowserIE(AContext) then
          begin
            ItemWidthTable := 'style="width: ' + IntToStr(((Display.DisplayEnd - Display.DisplayStart - APlannerItem.ItemBegin) * HeightRow) - 1) + 'px"';
            CellWTable := ((Display.DisplayEnd - Display.DisplayStart + 1 - APlannerItem.ItemBegin) * HeightRow) - 1;
          end
          else
          begin
            ItemWidthTable := 'style="width: ' + IntToStr(((Display.DisplayEnd - Display.DisplayStart + 1 - APlannerItem.ItemBegin) * HeightRow) - 2) + 'px"';
            CellWTable := ((Display.DisplayEnd - Display.DisplayStart  + 1 - APlannerItem.ItemBegin) * HeightRow) - 2;
          end;

          if not APlannerItem.FixedSize then
          begin
            if IsBrowserIE(AContext) then
            begin
              ItemWidth := 'style="width: ' + IntToStr(((Display.DisplayEnd - Display.DisplayStart  + 1 - APlannerItem.ItemBegin) * HeightRow) - 3) + 'px"';
              CellW := ((Display.DisplayEnd - Display.DisplayStart  + 1 - APlannerItem.ItemBegin) * HeightRow) - 3;
            end
            else
            begin
              ItemWidth := 'style="width: ' + IntToStr(((Display.DisplayEnd - Display.DisplayStart + 1 - APlannerItem.ItemBegin) * HeightRow) - 4) + 'px"';
              CellW := ((Display.DisplayEnd - Display.DisplayStart  + 1 - APlannerItem.ItemBegin) * HeightRow) - 4;
            end;
          end
          else
          begin
            ItemWidth := ItemWidthTable;
            CellW := CellWTable;
          end;
        end
        else
        begin
          if IsBrowserIE(AContext) then
          begin
            ItemWidthTable := ' style="width:' + IntToStr((HeightRow) * (APlannerItem.ItemEnd - APlannerItem.ItemBegin) - 1) + 'px"';
            CellWTable := (HeightRow) * (APlannerItem.ItemEnd - APlannerItem.ItemBegin) - 1;
          end
          else
          begin
            ItemWidthTable := ' style="width:' + IntToStr((HeightRow) * (APlannerItem.ItemEnd - APlannerItem.ItemBegin) - 2) + 'px"';
            CellWTable := (HeightRow) * (APlannerItem.ItemEnd - APlannerItem.ItemBegin) - 2;
          end;

          if not APlannerItem.FixedSize then
          begin
            if IsBrowserIE(AContext) then
            begin
              ItemWidth := ' style="width:' + IntToStr((HeightRow) * (APlannerItem.ItemEnd - APlannerItem.ItemBegin) - 3) + 'px"';
              CellW := (HeightRow) * (APlannerItem.ItemEnd - APlannerItem.ItemBegin) - 3;
            end
            else
            begin
              ItemWidth := ' style="width:' + IntToStr((HeightRow) * (APlannerItem.ItemEnd - APlannerItem.ItemBegin) - 4) + 'px"';
              CellW := (HeightRow) * (APlannerItem.ItemEnd - APlannerItem.ItemBegin) - 4;
            end;
          end
          else
          begin
            ItemWidth := ItemWidthTable;
            CellW := CellWTable;
          end;
        end;
      end
      else
      begin
        if IsBrowserIE(AContext) then
        begin
          ItemWidthTable := ' style="width:' + IntToStr(HeightRow - 1) + 'px"';
          CellWTable := HeightRow - 1;
        end
        else
        begin
          ItemWidthTable := ' style="width:' + IntToStr(HeightRow - 2) + 'px"';
          CellWTable := HeightRow - 2;
        end;

        if not APlannerItem.FixedSize then
        begin
          if IsBrowserIE(AContext) then
          begin
            ItemWidth := ' style="width:' + IntToStr(HeightRow - 3) + 'px"';
            CellW := HeightRow - 3;
          end
          else
          begin
            ItemWidth := ' style="width:' + IntToStr(HeightRow - 4) + 'px"';
            CellW := HeightRow - 4;
          end;
        end
        else
        begin
          ItemWidth := ItemWidthTable;
          CellW := CellWTable;
        end;
      end;

      CellDiv := '<div style="overflow:hidden;height:' + IntToStr(CellH) + 'px;width:' + IntToStr(CellW) + 'px">';
      CellDivTable := '<div style="overflow:hidden;height:' + IntToStr(CellH) + 'px;width:' + IntToStr(CellWTable) + 'px">';

      if (MaxPos > 1) and (APlannerItem.ConflictPos = 0) and (TotCol = 1) then
        ColSpan := ' rowspan="' + IntToStr(MaxPos) + '"'
      else
        ColSpan := '';

      Rowspan := APlannerItem.ItemEnd - APlannerItem.ItemBegin;

      if (APlannerItem.ItemBegin = RowIndex) then
      begin
        rtf := RenderItemToHtml(APlannerItem,RowIndex,MaxPos,True,True, CellH,AContext);

        BorderCol := HTMLClr(FBorderColor);

        if not (APlannerItem.FixedPos) and not (ViewOnly) then
        begin
          FHiddenItems := FHiddenItems
              + '<div id="ITEM' + IntToStr(APlannerItem.Index)
              + 'D" style="border:thin solid; overflow:hidden; border-color:'
              + HtmlClr(APlannerItem.TrackColor) + ';background-color:'
              + HtmlClr(ItemBackColor) + '; position:absolute; visibility:hidden; height: '
              + IntToStr(CellH + 1) + 'px;width:'+ IntToStr((RowSpan * (HeightRow))) + 'px;"'
              + ' onMouseMove="resizeItem(event)" onMouseUp="mUp(event)">'
              + rtf + '</div>'
        end;

        if (EditType = edPopup) then
          RenderPopup(APlannerItem);

        HtmlText := HtmlText
          + '<td bgcolor="' + HTMLClr(ItemBackColor) + '" id="I' + CellID + '" bordercolor="' + HtmlClr(BorderColor) + '" onMouseMove="ieP(''' + CellID + ''',event)" '
          + ' onMouseUp="mUp(event)" height="' + IntToStr(CellH) + 'px" colspan="' + IntToStr(Rowspan) + '" '+  ColSpan;

        rtf := RenderItemToHtml(APlannerItem,RowIndex,MaxPos,False,True,CellH,AContext);

        HtmlAlignment := ' valign="top"' + HTMLAlign(APlannerItem.Alignment);

        if ((FSelectedIndex > -1) and (APlannerItem = Items[FSelectedIndex]) and (APlannerItem.ShowSelection)) then
          HtmlBackgroundColor := ColorToHtmlHexBgColor(APlannerItem.SelectColor)
        else
          HtmlBackGroundColor := ColorToHtmlHexBgColor(APlannerItem.Color);

        if (MaxPos > 0) and (APlannerItem.ConflictPos = 0) and (TotCol = 1) then
          GotSpan := True;

        HtmlText := HtmlText + ' width="' + IntToStr(CellWTable) + 'px" ' + HtmlBackgroundColor + HtmlAlignment;

        if ((FSelectedIndex > -1) and (APlannerItem = FItems[FSelectedIndex]) and (APlannerItem.ShowSelection)) then
        begin
          if (APlannerItem.ItemEnd > APlannerItem.ItemBegin) and not GotSpan and (APlannerItem.ConflictPos = PosIndex) and not (APlannerItem.InHeader)  then
            HtmlText := HtmlText + ColSpan + '>';
        end
        else
        begin
          if (APlannerItem.ItemEnd > APlannerItem.ItemBegin) and not GotSpan and (APlannerItem.ConflictPos = PosIndex) and not (APlannerItem.InHeader)  then
            HtmlText := HtmlText + ColSpan + ' bordercolor="#000000">'#13
          else
            HtmlText := HtmlText + '>'#13;
        end;

        if (APlannerItem.Conflicts = 1) then
          HeightCell := CellH;

        if (APlannerItem.FixedSize) or (FPlannerInEditing) or (ViewOnly) or (APlannerItem.FixedPos) or (APlannerItem.FEditMode) or (APlannerItem.Background) then

          HtmlText := HtmlText + CellDiv + rtf + '</div></td>'#13
        else
        begin
          HtmlText := HtmlText + CellDivTable
            + '<table cellspacing="0" cellpadding="0"'
            + BorderCollapse + '><tr>'#13;

          visibilityDiv := '';

          if (APlannerItem.BeginOffset <> 0) then
            visibilityDiv := 'visibility:hidden;';

          HtmlText := HtmlText
            + '<td align="left">'
            + '<div id="ITEM' + IntToStr(APlannerItem.Index)
            + 'RT" style="cursor:pointer;cursor:w-resize;background-color:'
            + HtmlClr(APlannerItem.TrackColor) + ';width:1px;overflow:hidden;' + visibilityDiv + ' height:'
            + IntToStr(HeightCell) + 'px"' + ' onMouseDown="startResize(''ITEM'
            + ItemID + 'RT'',event)">&nbsp;</div></td>'#13;

          HtmlText := HtmlText
            + '<td>' + CellDiv + rtf + '</div></td>'#13;

          visibilityDiv := '';

          if(APlannerItem.EndOffset <> 0) then
            visibilityDiv := 'visibility: hidden;';

          HtmlText := HtmlText
            + '<td align="right">'
            + '<div id="ITEM' + IntToStr(APlannerItem.Index) + 'RD" style="cursor:pointer;cursor:e-resize;background-color:' + HtmlClr(APlannerItem.TrackColor) + ';width:1px;overflow:hidden;'
            + ' ' + visibilityDiv + ' height:' + IntToStr(HeightCell) + 'px" onMouseDown="startResize(''ITEM' + ItemID + 'RD'',event)">&nbsp;</div></td>'#13;

          HtmlText := HtmlText
            + '</tr></table></div></td>'#13;
        end;
      end;
    end;
  end;

  Result := HtmlText;
end;

{$WARNINGS ON}

{ Performs HTML rendering of the Planner in horizontal mode }
function TTIWCustomWebPlanner.RenderToHTMLRow(AContext :TIWBaseComponentContext): string;
var
  RowIndex, ColIndex, StartCol: Integer;
  HtmlText, HtmlBackGroundColor, txt: string;
  APlannerItem: TPlannerItem;
  HtmlAlignment: string;
  ABrush: TBrush;
  Color,bordercollapse: string;
  HeaderWidth,HAlign: string;
  subpos,temp: integer;
  MaxPos,TotPos, CaptionCols: Integer;
  rowspan: string;
  numdown, Count, GroupIndex: Integer;
  Occupied,Placed: Boolean;
  UsedFont : TIWFont;

  NumberOfHeaderItems, idx, itemIdx, CellHeight : integer;
  HintSideBar, SideBarH, IdSideBar : string;
  BGColor : TIWColor;
  First : boolean;
  SCellID : string;
  Prop : TPositionProp;

  DblCl : string;

  RealWidth : integer;
  headerClick : string;

begin
  HtmlText := '';
  FHiddenItems := '';
  NumberOfHeaderItems := 0;

  Color := HTMLClr(FBorderColor);

  if IsBrowserIE(AContext) and (FBorderCollapsed) then
    bordercollapse := ' style="border-collapse:collapse"'
  else
    bordercollapse := '';

  HtmlText := HtmlText + RenderCSSStyle('horizontal',AContext);

  if (Header.Visible) then
    RealWidth := Display.GetDisplay * Display.DisplayScale + Header.FCaptionHeight
  else
    RealWidth := Display.GetDisplay * Display.DisplayScale;

  if IsBrowserIE(AContext) and (FBorderCollapsed) then
    HtmlText := HtmlText
      + FontToHtmlFont(Self.Font) + '<table id="TABLE' + HTMLName + '" onkeydown="tableKey();" cellspacing="0" cellpadding="0"' + bordercollapse + ' border="' + IntToStr(FBorderSize)
      + '" bordercolor="' + Color + '">'
  else
    HtmlText := HtmlText
      + FontToHtmlFont(Self.Font) + '<table width="' + IntToStr(RealWidth) + 'px" id="TABLE' + HTMLName + '" onkeydown="tableKey();" cellspacing="0" cellpadding="0"' + bordercollapse + ' border="' + IntToStr(FBorderSize)
      + '" bordercolor="' + Color + '">';

  HeaderWidth := '';
  if Header.CaptionHeight > 0 then
    HeaderWidth := ' width="' + IntToStr(Header.CaptionHeight - 2) + 'px"';

  StartCol := 1;

  for idx := 0 to FItems.Count - 1 do
  begin
		APlannerItem := FItems[idx];
		if Assigned(APlannerItem) then
    begin
      if (APlannerItem.InHeader) then
        NumberOfHeaderItems := NumberOfHeaderItems + 1;
    end;
  end;

  // sidebar is rendered here
  if FSideBar.Visible then
  begin
    if IsBrowserIE(AContext) and (FBorderCollapsed) then
      HtmlText := HtmlText + '<tr style="height:' + IntToStr(SideBar.Width) + 'px;">'
    else
      HtmlText := HtmlText + '<tr width="' + IntToStr(RealWidth) + 'px" style="height:' + IntToStr(SideBar.Width) + 'px;">';

    if (Header.Visible) then
    begin
      if (PositionGroup > 0) then
        HtmlText := HtmlText
          + '<td id="' + HTMLName + 'Caption" '  + HeaderWidth + '>&nbsp;</td>';

      if (FPlannerHeader.Position = hpTop) or (FPlannerHeader.Position = hpBoth) then
      begin
        if IsBrowserIE(AContext) then
          HtmlText := HtmlText
            + '<td id="' + HTMLName + 'CaptionTL" ' + HeaderWidth + '>' + TopLeftCell + '</td>'
        else
          HtmlText := HtmlText
            + '<td id="' + HTMLName + 'CaptionTL" ' + HeaderWidth + '><div style="width:' + IntToStr(Header.CaptionHeight -2)  + 'px">' + TopLeftCell + '</div></td>';

        StartCol := 0;
      end;
    end;

    if (NumberOfHeaderItems > 0) then
    begin
      HtmlText := HtmlText
        + '<td width="' + IntToStr(Header.CaptionHeight * 2) + 'px" id="' + HTMLName + 'Caption" ' + HeaderWidth
        + '>&nbsp;</td>'#13;
    end;

    for ColIndex := 0 to Display.GetDisplay - 1 do
    begin
      Occupied := false;

      if FSideBar.ShowOccupied then
      begin
        APlannerItem := Items.FindItemIdx(ColIndex);
        Occupied := (APlannerItem <> nil) and FSidebar.ShowOccupied;
      end;

      txt := PlannerGetIdCol(Self, ColIndex, 0);
      txt := EmbedInHtmlOptions(txt, FPlannerHeader.FHeaderFont.Style);
      HintSideBar := txt;
      SideBarH := '';
      IdSideBar := '';

      if (Occupied) then
        IdSideBar := ' id="' + HTMLName + 'SideBarOccupied"'
      else
        IdSideBar := ' id="' + HTMLName + 'SideBar"';

      if (SideBar.ShowHint) then
        SideBarH := ' title="' + HintSideBar + '" ';

      if IsBrowserIE(AContext) then
        HtmlText := HtmlText
          + '<td ' + IdSideBar + SideBarH + ' width="' + IntToStr(Display.DisplayScale - 1) + 'px">'#13
          + '<div style="overflow:hidden;width:' + IntToStr(Display.DisplayScale - 1) + 'px">'#13
      else
        HtmlText := HtmlText
          + '<td ' + IdSideBar + SideBarH + ' width="' + IntToStr(Display.DisplayScale - 2) + 'px">'#13
          + '<div style="overflow:hidden;width:' + IntToStr(Display.DisplayScale - 2) + 'px">'#13;
      HtmlText := HtmlText
        + txt + '</div></td>'#13;
    end;

    if (Header.Visible) and ((FPlannerHeader.Position = hpBottom) or (FPlannerHeader.Position = hpBoth)) then
    begin
        if IsBrowserIE(AContext) then
          HtmlText := HtmlText
            + '<td id="' + HTMLName + 'CaptionTL" ' + HeaderWidth + '>' + TopLeftCell + '</td>'
        else
          HtmlText := HtmlText
            + '<td id="' + HTMLName + 'CaptionTL" ' + HeaderWidth + '><div style="width:' + IntToStr(Header.CaptionHeight -2)  + 'px">' + TopLeftCell + '</div></td>';
    end;

    HtmlText := HtmlText + '</tr>';
  end
  else
  begin
    if Header.Visible then
      StartCol := 0;
  end;

  // render all rows
  RowIndex := 1;

  while (RowIndex <= Positions) do
  begin
    First := true;
    Placed := false;
    MaxPos := Items.MaxItemsInPos(RowIndex-1);

    MaxPos := Max(1,MaxPos);
    MaxPos := Max(MaxPos, Items.MaxItemsInHeader(RowIndex-1));

    SubPos := 1;

    while subpos <= MaxPos do
    begin
      HtmlText := HtmlText + '<tr>'#13;

      for ColIndex := StartCol to Display.GetDisplay do
      begin
        // render GroupCaption header column here
        if (PositionGroup > 0) and (ColIndex = 0)  then
        begin
          GroupIndex := (RowIndex - 1) div PositionGroup;
          if ((RowIndex mod PositionGroup = 1) and not Placed) then
          begin
            Placed := true;
            Temp := 0;
            for CaptionCols := 0 to PositionGroup - 1 do
              Temp := Temp + Max(Items.MaxItemsInPos((RowIndex + CaptionCols) - 1),Items.MaxItemsInHeader((RowIndex + CaptionCols) - 1));

            if Overlap then
              Temp := Temp + PositionGroup;

            RowSpan := ' rowspan="' + IntToStr(Temp) + '" ';

            if (Header.GroupCaptions.Count > GroupIndex) then
              txt := CLToBr(Header.GroupCaptions[GroupIndex])
            else
              txt := HtmlNonBreakingSpace;

            HAlign := 'style="' + HTMLAlign(Header.Alignment) + '" ';

            HtmlText := HtmlText
              + '<td id="' + HTMLName + 'Caption" ' + HAlign + RowSpan + HeaderWidth + '>'#13
              + '<div style="overflow:hidden;width:' + IntToStr(Header.CaptionHeight) + '">'#13
              + txt + '</div></td>'#13;
          end;
        end;

        if (ColIndex = 0) and (subpos = 1) then
        begin
          if (Header.Captions.Count > RowIndex) then
            txt := CLToBR(Header.Captions[RowIndex])
          else
            txt := HtmlNonBreakingSpace;

          txt := EmbedInHtmlOptions(txt, FSideBar.FSideBarFont.Style);

          if MaxPos > 1 then
            RowSpan := ' rowspan="' + IntToStr(MaxPos) + '" '
          else
            RowSpan := '';

          HAlign := '';
          HAlign := HTMLAlign(Header.Alignment);

          headerClick := '';

          if Assigned(OnHeaderClick) then
            headerClick := ' onclick="return SubmitClick('#39 + HTMLName + #39','#39 + 'HeaderClick#' + IntToStr(RowIndex-1) + #39');" ';

          HtmlText := HtmlText
            + '<td ' + headerClick + 'id="' + HTMLName + 'Caption" ' + HAlign + RowSpan + HeaderWidth + '>'#13
            + '<div style="overflow:hidden;width:' + IntToStr(Header.CaptionHeight) + '">'#13
            + txt + '</div></td>'#13;
        end
        else
        begin
            //rendering header items
            if ((NumberOfHeaderItems > 0) and (ColIndex = 1)) then
            begin
              APlannerItem := Items.FindItemInHeaderEx(RowIndex - 1,SubPos-1);

              if Assigned(APlannerItem) and (Items.InVisibleLayer(APlannerItem.Layer)) then
              begin
                if (Overlap) then
                  CellHeight := (((Height - SideBar.GetUsedWidth - 2 * Self.BorderSize) div Positions) - 10 - MaxPos + 1) div MaxPos
                else
                  CellHeight := (((Height - SideBar.GetUsedWidth - 2 * BorderSize) div Positions) - MaxPos) div MaxPos;

                BGColor := APlannerItem.Color;

                if (BGColor = clNone) then
                  BGColor := clWhite;

                HtmlText := HtmlText
                  + '<td height="' + IntToStr(CellHeight) + 'px" valign="top" bgcolor="' + HTMLClr(BGColor) + '">'#13
                  + '<div style="overflow:hidden;height:' + IntToStr(CellHeight) + 'px; width:'+ IntToStr(Header.HeaderItemWidth) + 'px">'#13
                  + RenderItemToHtml(APlannerItem,0,MaxPos,false,true,CellHeight, AContext)
                  + '</div></td>'#13;

                if (EditType = edPopup) then
                  RenderPopup(APlannerItem);
              end
              else
              begin
                BGColor := clWhite;

                if (First) then
                begin
                  First := false;
                  HtmlText := HtmlText
                  + '<td valign="top" rowspan="' + IntToStr(MaxPos - (FItems.MaxItemsInHeader(RowIndex - 1)))
                  + '" bgcolor="' + HTMLClr(BGColor) + '">&nbsp;</td>'#13;
                end;
             end;
            end;

          //rendering items not in header
          HtmlBackGroundColor := '';

          ABrush := TBrush.Create;
          GetCellBrush(RowIndex,ColIndex - 1,ABrush);
          HtmlBackGroundColor := HtmlClr(ABrush.Color);
          ABrush.Free;

          APlannerItem := Items.FindItem(ColIndex - 1, RowIndex - 1);

          if Assigned(APlannerItem) and not (APlannerItem.InHeader) then
          begin
            TotPos := APlannerItem.Conflicts;
          end
          else
            TotPos := 0;

          {$IFDEF TMSDEBUG}
          outputdebugstring(pchar('try to find item at subpos : '+inttostr(subpos)+':'+inttostr(colindex-1)+':'+inttostr(rowindex-1)));
          {$ENDIF}

          APlannerItem := Items.FindItemPosIdx(ColIndex - 1, RowIndex - 1,subpos - 1);

          if Assigned(APlannerItem) and not (APlannerItem.InHeader) then
          begin
            HtmlAlignment := ' valign="top"' + HTMLAlign(APlannerItem.Alignment);

            if (APlannerItem.ItemBegin = ColIndex - 1) then
            begin
              htmltext := htmltext + RenderCellToHTMLRow(APlannerItem,ColIndex-1,RowIndex,MaxPos,TotPos,RowIndex,0, AContext);
            end;
          end
          else
          begin
            if (TotPos = 0) and (subpos = 1) then
            begin
              htmltext := htmltext + RenderCellToHTMLRow(nil,ColIndex-1,RowIndex,MaxPos,TotPos,RowIndex,MaxPos, AContext);
            end
            else
            begin
              if Assigned(Items.FindItemPosIdx(ColIndex - 1, RowIndex - 1,subpos - 2)) or (subpos = 1) then
              begin
                numdown := 1;
                while not Assigned(Items.FindItemPosIdx(ColIndex - 1, RowIndex - 1,subpos - 1 + numdown)) and (numdown <= MaxPos - subpos) do
                  inc(numdown);

                htmltext := htmltext + RenderCellToHTMLRow(nil,ColIndex-1,RowIndex,MaxPos,TotPos,RowIndex,numdown, AContext)
              end;
            end;
          end;
        end;
      end;

      if (Header.Visible) and ((FPlannerHeader.Position = hpBottom) or (FPlannerHeader.Position = hpBoth)) then
      begin
        if (subpos = 1) then
        begin
          if (Header.Captions.Count > RowIndex) then
            txt := CLToBR(Header.Captions[RowIndex])
          else
            txt := HtmlNonBreakingSpace;

          txt := EmbedInHtmlOptions(txt, FSideBar.FSideBarFont.Style);

          if MaxPos > 1 then
            RowSpan := ' rowspan="' + IntToStr(MaxPos) + '" '
          else
            RowSpan := '';

          HAlign := '';
          HAlign := HTMLAlign(Header.Alignment);

          headerClick := '';

          if Assigned(OnHeaderClick) then
            headerClick := ' onclick="return SubmitClick('#39 + HTMLName + #39','#39 + 'HeaderClick#' + IntToStr(RowIndex-1) + #39');" ';

          HtmlText := HtmlText
            + '<td ' + headerClick + 'id="' + HTMLName + 'Caption" ' + HAlign + RowSpan + HeaderWidth + '>'#13
            + '<div style="overflow:hidden;width:' + IntToStr(Header.CaptionHeight) + '">'#13
            + txt + '</div></td>'#13;
        end;
      end;

      Inc(subpos);

      HtmlText := HtmlText + '</tr>'#13;
    end;

    if Overlap then
    begin
      HtmlText := HtmlText + '<tr>'#13;

      if FPlannerHeader.Visible then
      begin
        if (PositionGroup > 0) and (RowIndex mod PositionGroup = 0) then
        begin
          if IsBrowserIE(AContext) then
            HtmlText := HtmlText + '<td height="8px" bgcolor="' + HtmlClr(OverlapColor) + '">'#13
              + '<div style="overflow:hidden;height:8px">&nbsp;</div></td>'#13
          else
            HtmlText := HtmlText + '<td width="' + IntToStr(Display.DisplayScale - 2) + 'px" height="9px" bgcolor="' + HtmlClr(OverlapColor) + '">'#13
              + '<font point-size="4">&nbsp;</font></td>'#13;
        end
        else
        begin
          if IsBrowserIE(AContext) then
            HtmlText := HtmlText + '<td height="8px" bgcolor="' + HtmlClr(OverlapColor) + '">'#13
              + '<div style="overflow:hidden;height:8px">&nbsp;</div></td>'#13
          else
            HtmlText := HtmlText
              + '<td width="' + IntToStr(Display.DisplayScale - 2) + 'px" height="9px" bgcolor="' + HtmlClr(OverlapColor) + '">'
              + '<font point-size="4">&nbsp;</font></td>'#13;
        end;
      end;

      if (NumberOfHeaderItems > 0) then
      begin
        if IsBrowserIE(AContext) then
        begin
          HtmlText := HtmlText
            + '<td height="8px" bgcolor="' + HTMLClr(OverlapColor) + '">'#13
            + '<div style="overflow:hidden;height:8px">&nbsp;</div></td>'#13;
        end
        else
        begin
          HtmlText := HtmlText
            + '<td width="' + IntToStr(Display.DisplayScale - 2) + 'px" height="9px" bgcolor="' + HTMLClr(OverlapColor) + '">'#13
            + '<font point-size="4">&nbsp;</font></td>'#13;
        end;
      end;

      for ColIndex := 1 to Display.GetDisplay do
      begin
        if (PositionProps.Count > RowIndex-1) then
        begin
          Prop := PositionProps[RowIndex-1];
          if ((Prop.MinSelection > 0) or (Prop.MaxSelection > 0)) then
          begin
            if ((ColIndex - 1 < Prop.MinSelection) or (ColIndex-1 > Prop.MaxSelection)) then
              SCellID := 'SR' + IntToStr(ColIndex-1) + 'C' + IntToStr(RowIndex - 1) + 'NS'
            else
              SCellID := 'SR' + IntToStr(ColIndex-1) + 'C' + IntToStr(RowIndex - 1);
          end
          else
            SCellID := 'SR' + IntToStr(ColIndex-1) + 'C' + IntToStr(RowIndex - 1);
        end
        else
          SCellID := 'SR' + IntToStr(ColIndex-1) + 'C' + IntToStr(RowIndex - 1);

        DblCl := '';
        if ((FSelectionMode = smSingleSelect) and Assigned(FOnPlannerDblClick))then
          DblCl := ' ondblclick="PlannerDblClick()";';

        if IsBrowserIE(AContext) then
          HtmlText := HtmlText
            + '<td height="8px" ' + DblCl + ' id="' + SCellID
            + '" bgcolor="' + HtmlClr(OverlapColor) + '"'
            + ' onMouseMove="ieP(''' + SCellID + ''',event)" onMouseOver="mOv(''' + SCellID + ''',event)" onMouseDown="mDwn('''
            + SCellID
            + ''',event)" onMouseUp="mUp(event)"><span style="cursor:pointer;cursor:default"><div style="overflow:hidden;height:8px">&nbsp;</div></span></td>'
        else
          HtmlText := HtmlText
            + '<td width="' + IntToStr(Display.DisplayScale - 2) + 'px" height="7px" ' + DblCl + ' id="' + SCellID + '" bgcolor="' + HtmlClr(OverlapColor) + '"'
            + ' onMouseMove="ieP(''' + SCellID + ''',event)" onMouseOver="mOv(''' + SCellID + ''',event)"'
            + ' onMouseDown="mDwn(''' + SCellID
            + ''',event)" onMouseUp="mUp(event)"><div style="overflow:hidden;height:7px">&nbsp;</div></td>';
      end;
      HtmlText := HtmlText + '</tr>';
    end;
    Inc(RowIndex);
  end;

  HtmlText := HtmlText + '</table></Font>';
  if (EditType = edPopup) then
    Result := HTMLText + FHiddenItems + FHiddenPopups
  else
    Result := HTMLText + FHiddenItems;
end;

function TTIWCustomWebPlanner.RenderHTML(AContext : TIWBaseComponentContext): TIWHTMLTag;
var
  htmlRes: string;
  aItem : TPlannerItem;
  BColor, FColor : TIWColor;

  function MakeScript: String;
  var
    s: String;
    WidthColumn: Integer;
    HeightRow: Integer;
    i : integer;
    k : integer;
  begin

    if (Display.DisplayScale <= 0) then
      HeightRow := 16
    else
      HeightRow := Display.DisplayScale;

    if FSideBar.Orientation = soHorizontal then
    begin
      if (Overlap) then
      begin
        if IsBrowserIE(AContext) then
        begin
          if (Positions > 0) then
            WidthColumn := ((Height - SideBar.GetUsedWidth - 2 * BorderSize - Positions - (Positions * 8)) div Positions) + 8
         else
            WidthColumn := (Height - SideBar.GetUsedWidth - 2 * BorderSize);
        end
        else
        begin
          if (Positions > 0) then
            WidthColumn := ((Height - SideBar.GetUsedWidth - 2 * BorderSize - Positions - (Positions * 9)) div Positions) + 9
         else
            WidthColumn := (Height - SideBar.GetUsedWidth - 2 * BorderSize);
        end;
      end
      else
      begin
      if Positions > 0 then
        WidthColumn := ((Height - SideBar.GetUsedWidth - 2 * BorderSize - Positions) div Positions)
      else
        WidthColumn := (Height - SideBar.GetUsedWidth - 2 * BorderSize);
      end;
    end
    else
    begin
      if Positions > 0 then
        WidthColumn := ((Width - SideBar.GetUsedWidth - 2 * BorderSize - Positions) div Positions)
      else
        WidthColumn := (Width - SideBar.GetUsedWidth - 2 * BorderSize);
    end;

		if (Items.Count > 0) then
    begin
			FPlannerInEditing := false;
			for k := 0 to Items.Count - 1 do
      begin
	  		if (Items[k].EditMode) then
        begin
			   	FPlannerInEditing := true;
          Break;
        end;
      end;
    end;

    s := s
      + '<SCRIPT Language="JavaScript">'#13;

    if IsBrowserIe(AContext) then
      s := s + ' var isIE = true;'#13
    else
      s := s + ' var isIE = false;'#13;

    if IsBrowserIe10(AContext) then
      s := s + ' var isIE10 = true;'#13
    else
      s := s + ' var isIE10 = false;'#13;

    s := s
      + 'var selectedOver=false;'#13
      + 'var selectStart;'#13
      + 'var selectEnd;'#13
      + 'var ar_Colors = new Array();'#13
      + 'var ar_Ids = new Array();'#13
      + 'var selected = false;'#13
      + 'var CellsSelected=0;'#13
      + 'var column;'#13
      + 'var xDown;'#13
      + 'var yDown;'#13
      + 'var selDiv;'#13
      + 'var xUp;'#13
      + 'var yUp;'#13
      + 'var selDivLeft;'#13
      + 'var selDivTop;'#13
      + 'var widthCell;'#13
      + 'var heightCell;'#13
      + 'var allowMove = true;'#13
      + 'var leftR0C0;'#13
      + 'var topR0C0;'#13
      + 'var idToResize;'#13
      + 'var startResizeTop;'#13
      + 'var endResizeTop;'#13
      + 'var startResizeLeft;'#13
      + 'var endResizeLeft;'#13
      + 'var topBeginResize;'#13
      + 'var topEndResize;'#13
      + 'var countOfRows;'#13
      + 'var countOfColumns;'#13
      + 'var leftEndRowEndCol;'#13
      + 'var topEndRowEndCol;'#13
      + 'var widthOverlap;'#13
      + 'var widthRightSidebar;'#13
      + 'var windowX;'#13
      + 'var windowY;'#13
      + 'widthCell = parseInt(' + IntToStr(WidthColumn) + ');'#13
      + 'heightCell = parseInt(' + IntToStr(HeightRow) + ');'#13
      + 'var moveStarted;'#13
      + 'var moveLeft;'#13
      + 'var moveDelta;'#13
  		+ 'var NewSelIndex;'#13
	  	+ 'var NewSelColor;'#13
			+ 'var NewSelFontColor;'#13
			+ 'var OldSelIndex;'#13
			+ 'var OldSelColor;'#13
			+ 'var OldSelFontColor;'#13
			+ 'var OldColor;'#13
			+ 'var OldFontColor;'#13
      + 'var lastID = null;'#13
      + 'var inMultiSelect = false;'#13
      + 'var inEditing = null;'#13
      + 'var askToAdd = true;'#13
      + 'var defaultSelect = false;'#13
      + 'var edititemfocus = false;'#13
      ;

      if Overlap then
        s := s + 'widthOverlap = 8;'#13
      else
        s := s + 'widthOverlap = 0;'#13;

     s := s
      + 'countOfRows = parseInt(' + IntToStr(Display.GetDisplay) + ');'#13
      + 'countOfColumns = parseInt(' + IntToStr(Positions) + ');'#13;

     if ((FSelectedIndex <> -1) and (FSelectedIndex >= Items.Count)) then
				FSelectedIndex := -1;

     for i := 0 to Items.Count - 1 do
      begin
				aItem := Items[i];

				if ((FSelectedIndex > -1) and (aItem = Items[FSelectedIndex]) and (aItem.ShowSelection)) then
        begin
					if (aItem.Color <> clNone) then
						BColor := aItem.Color
					else
						BColor := clWhite;

					if(aItem.Font.Color <> clNone) then
						FColor := aItem.Font.Color
					else
						FColor := clBlack;

					s := s
						+ 'OldSelIndex = ''' + IntToStr(aItem.Index) + ''';'#13
						+ 'OldColor = ''' + HTMLClr(BColor) + ''';'#13
						+ 'OldFontColor = ''' + HTMLClr(FColor) + ''';'#13;
        end;
      end;

      s := s
        + 'function El(id){'#13
        + '  return(document.getElementById(id));'#13
        + '}';


      if ((FSelectionMode = smSingleSelect) and Assigned(FOnPlannerDblClick)) then
      begin
        s := s
          + ' function PlannerDblClick() {'#13
          + '   submval = FindElem('#39+HTMLName+#39').value;'#13
          + '   s_ar = submval.split("|");'#13
          + '   s_ar[1] = selectStart;'#13
          + '   s_ar[2] = column;'#13
          + '   FindElem('#39+HTMLName+#39').value = s_ar.join("|");'#13
          + '   return SubmitClick('#39+HTMLName+#39',"AddSingleI",false)'#13
          + ' }'#13;
      end;

      s := s
        {Set the selected ItemBegin, ItemEnd and ItemPosition to the hidden input field}
        + ' function setToHiddenInput(asyncparam) {'#13
        + '   submval = FindElem('#39+HTMLName+#39').value;'#13
        + '   s_ar = submval.split("|");'#13
        + '   var start = parseInt(selectStart.substring(1,selectStart.length));'#13
        + '   var end = parseInt(selectEnd.substring(1,selectEnd.length));'#13
        + '   if (start > end) {'#13
        + '     s_ar[1] = selectEnd;'#13
        + '     s_ar[2] = selectStart;'#13
        + '   }'#13
        + '   else {'#13
        + '     s_ar[1] = selectStart;'#13
        + '     s_ar[2] = selectEnd;'#13
        + '   }'#13
        + '   s_ar[3] = column;'#13
        + '   if (asyncparam)'#13
        + '     s_ar[4] = asyncparam;'#13
        + '   FindElem('#39+HTMLName+#39').value = s_ar.join("|");'#13
        + ' }'#13

        {Select the cell given by the parameters}
        + ' function SelectCell(row, col) {'#13
        //Fix: If the planner appears lower on the page, the page is scrolled
        //automatically when setting the focus to the planner. which is not good.
        //Fix: Focus must be kept on item in editmode
//        + '   if ( (navigator.appName != "Netscape") && (! edititemfocus))'#13
//        + '     document.getElementById("TABLE' + HTMLName + '").focus();'#13
        + '   id = "R" + row + "C" + col;'#13
        + '   selectStart = "R" + row;'#13
        + '   column = "C" + col;'#13
        + '   selectEnd = "R" + row;'#13
        ;

      if (EnableSelection) and not (ViewOnly) then
      begin
      s := s
        + '   selected = true;'#13
        + '   if (El(selectStart + column) != null) {'#13
        + '     ar_Colors[0] = El(selectStart + column).bgColor;'#13
        + '     El(selectStart + column).bgColor = "' + HTMLClr(SelectionColor) + '";'#13
        + '   }'#13
        + '   if (El("S" + selectStart + column) != null) {'#13
        + '     El("S" + selectStart + column).bgColor = "' + HTMLClr(SelectionColor) + '";'#13
        + '   }'#13
        + '   var start = parseInt(selectStart.substring(1,selectStart.length));'#13
        + '   var end = parseInt(selectEnd.substring(1,selectEnd.length));'#13
        + '   ar_Ids[0] = selectStart + column;'#13
        + '   CellsSelected = 1;'#13;
    end;
    s := s
        + '   setToHiddenInput();'#13
        + '   defaultSelect = true;'#13
        + ' }'#13

        {Do the navigation with the keyboard}
        + ' function tableKey() {'#13;


        if (EnableSelection) and not (ViewOnly) and not (FPlannerInEditing) then
        begin
          s := s
        //Arrow Down
        + '  if ((event.keyCode == 40) & (event.shiftKey == false)) {'#13
        + '    if (selected == true) {'#13
        + '      start = parseInt(selectStart.substring(1,selectStart.length));'#13
        + '      end = parseInt(selectEnd.substring(1,selectEnd.length));'#13
        + '      restoreColors(start,end);'#13
        + '      cleanUpArrays();'#13
        + '    }'#13;

        s := s
        + '    if (inMultiSelect) {'#13
        + '      end = start;'#13
        + '      inMultiSelect = false;'#13
        + '    }'#13;

        if (SideBar.Position <> spTop) then
        begin
          s := s
            + '    if (countOfRows > start+1) {'#13
            + '      start = start + 1;'#13
            + '      end = end + 1;'#13
            + '    }'#13;
        end
        else
        begin
          s := s
            + '    col = parseInt(column.substring(1,column.length));'#13
            + '    if (countOfColumns > col+1) {'#13
            + '      col = parseInt(column.substring(1,column.length)) + 1;'#13
            + '      column = "C" + (col);'#13
            + '    }'#13;
        end;

        s := s
        + '    selectStart = "R" + start;'#13
        + '    selectEnd = "R" + end;'#13
        + '    id =selectStart + column;'#13
        + '    if (El(id) != null) {'#13
        + '      ar_Colors[0] = El(id).bgColor;'#13
        + '      El(id).bgColor = "' + HTMLClr(SelectionColor) + '";'#13
        + '    }'#13
        + '    if (El("S" + selectStart + column) != null) {'#13
        + '      El("S" + selectStart + column).bgColor = "' + HTMLClr(SelectionColor) + '";'#13
        + '    }'#13
        + '    ar_Ids[0] = selectStart + column;'#13
        + '    CellsSelected = 1;'#13
        + '    setToHiddenInput();'#13
        + '   defaultSelect = false;'#13
        + '  }'#13

        //Arrow Up
        + '  if ((event.keyCode == 38) & (event.shiftKey == false)) {'#13
        + '    if (selected == true) {'#13
        + '      start = parseInt(selectStart.substring(1,selectStart.length));'#13
        + '      end = parseInt(selectEnd.substring(1,selectEnd.length));'#13
        + '      restoreColors(start,end);'#13
        + '      cleanUpArrays();'#13
        + '    }'#13;

        s := s
        + '    if (inMultiSelect) {'#13
        + '      end = start;'#13
        + '      inMultiSelect = false;'#13
        + '    }'#13 ;

        if (SideBar.Position <> spTop) then
        begin
          s := s
            + '    if (start > 0) {'#13
            + '      start = start - 1;'#13
            + '      end = end - 1;'#13
            + '    }'#13;
        end
        else
        begin
          s := s
            + '    col = parseInt(column.substring(1,column.length));'#13
            + '    if (col > 0) {'#13
            + '      col = parseInt(column.substring(1,column.length)) - 1;'#13
            + '    }'#13
            + '    column = "C" + col;'#13
        end;

        s := s
          + '    selectStart = "R" + start;'#13
          + '    selectEnd = "R" + end;'#13
          + '    id =selectStart + column;'#13
          + '    if (El(id) != null) {'#13
          + '      ar_Colors[0] = El(id).bgColor;'#13
          + '      El(id).bgColor = "' + HTMLClr(SelectionColor) + '";'#13
          + '    }'#13
          + '    if (El("S" + selectStart + column) != null) {'#13
          + '      El("S" + selectStart + column).bgColor = "' + HTMLClr(SelectionColor) + '";'#13
          + '    }'#13
          + '    ar_Ids[0] = selectStart + column;'#13
          + '    CellsSelected = 1;'#13
          + '    setToHiddenInput();'#13
          + '    defaultSelect = false;'#13
          + '  }'#13

        //Arrow Right
        + '  if ((event.keyCode == 39) & (event.shiftKey == false)) {'#13
        + '    if (selected == true) {'#13
        + '      start = parseInt(selectStart.substring(1,selectStart.length));'#13
        + '      end = parseInt(selectEnd.substring(1,selectEnd.length));'#13
        + '      restoreColors(start,end);'#13
        + '      cleanUpArrays();'#13
        + '    }'#13;

        s := s
        + '    if (inMultiSelect) {'#13
        + '      end = start;'#13
        + '      inMultiSelect = false;'#13
        + '    }'#13;

        if (SideBar.Position <> spTop) then
        begin
          s := s
            + '    col = parseInt(column.substring(1,column.length));'#13
            + '    if (countOfColumns > col+1) {'#13
            + '      col = parseInt(column.substring(1,column.length)) + 1;'#13
            + '    }'#13
            + '    column = "C" + col;'#13;
        end
        else
        begin
          s := s
            + '    if (countOfRows > start+1) {'#13
            + '      start = start + 1;'#13
            + '      end = end + 1;'#13
            + '    }'#13;
        end;

        s := s
          + '    selectStart = "R" + start;'#13
          + '    selectEnd = "R" + start;'#13
          + '    id =selectStart + column;'#13
          + '    if (El(id) != null) {'#13
          + '      ar_Colors[0] = El(id).bgColor;'#13
          + '      El(id).bgColor = "' + HTMLClr(SelectionColor) + '";'#13
          + '    }'#13
          + '    if (El("S" + selectStart + column) != null) {'#13
          + '      El("S" + selectStart + column).bgColor = "' + HTMLClr(SelectionColor) + '";'#13
          + '    }'#13
          + '    ar_Ids[0] = selectStart + column;'#13
          + '    CellsSelected = 1;'#13
          + '    setToHiddenInput();'#13
          + '    defaultSelect = false;'#13
          + '  }'#13

        //Arrow Left
        + '  if ((event.keyCode == 37) & (event.shiftKey == false)) {'#13
        + '    if (selected == true) {'#13
        + '      start = parseInt(selectStart.substring(1,selectStart.length));'#13
        + '      end = parseInt(selectEnd.substring(1,selectEnd.length));'#13
        + '      restoreColors(start,end);'#13
        + '      cleanUpArrays();'#13
        + '    }'#13;

        s := s
        + '    if (inMultiSelect) {'#13
        + '      end = start;'#13
        + '      inMultiSelect = false;'#13
        + '    }'#13;

        if (SideBar.Position <> spTop) then
        begin
          s := s
            + '    col = parseInt(column.substring(1,column.length));'#13
            + '    if (col > 0) {'#13
            + '      col = parseInt(column.substring(1,column.length)) - 1;'#13
            + '    }'#13
            + '    column = "C" + col;'#13;
        end
        else
        begin
          s := s
            + '    if (start > 0) {'#13
            + '      start = start - 1;'#13
            + '      end = end - 1;'#13
            + '    }'#13;
        end;

        s := s
        + '    selectStart = "R" + start;'#13
        + '    selectEnd = "R" + start;'#13
        + '    id =selectStart + column;'#13
        + '    if (El(id) != null) {'#13
        + '      ar_Colors[0] = El(id).bgColor;'#13
        + '      El(id).bgColor = "' + HTMLClr(SelectionColor) + '";'#13
        + '    }'#13
        + '    if (El("S" + selectStart + column) != null) {'#13
        + '      El("S" + selectStart + column).bgColor = "' + HTMLClr(SelectionColor) + '";'#13
        + '    }'#13
        + '    ar_Ids[0] = selectStart + column;'#13
        + '    CellsSelected = 1;'#13
        + '    setToHiddenInput();'#13
        + '    defaultSelect = false;'#13
        + '  }'#13

        //SpaceBar --> Add Item
        + '  if ((event.keyCode == 45) & (selected == true)) {'#13
        + '    submval = FindElem('#39+HTMLName+#39').value;'#13
        + '    s_ar = submval.split("|");'#13
        + '    var start = parseInt(selectStart.substring(1,selectStart.length));'#13
        + '    var end = parseInt(selectEnd.substring(1,selectEnd.length));'#13
        + '    if (start > end) {'#13
        + '      s_ar[1] = selectEnd;'#13
        + '      s_ar[2] = selectStart;'#13
        + '    }'#13
        + '    else {'#13
        + '      s_ar[1] = selectStart;'#13
        + '      s_ar[2] = selectEnd;'#13
        + '    }'#13
        + '    s_ar[3] = column;'#13
        + '    FindElem('#39+HTMLName+#39').value = s_ar.join("|");'#13
        + '    return SubmitClick('#39+HTMLName+#39',"AddItem",false)'#13
        + '  }'#13;

    if (FSelectionMode = smMultiSelect) then
    begin
      if (SideBar.Position <> spTop) then
        s := s
          + '  if ((event.shiftKey) & (event.keyCode == 40)) {'#13
      else
        s := s
          + '  if ((event.shiftKey) & (event.keyCode == 39)) {'#13;

    s := s
        + '    inMultiSelect = true;'#13
        + '    var temp;'#13
        + '    start = parseInt(selectStart.substring(1,selectStart.length));'#13
        + '    end = parseInt(selectEnd.substring(1,selectEnd.length));'#13
        + '    restoreColors(start,end);'#13
        + '    selectEnd = "R" + (end + 1) + column;'#13
        + '    temp = selectEnd.indexOf("C");'#13
        + '    selectEnd = selectEnd.substring(0,temp);'#13
        + '    end = parseInt(selectEnd.substring(1,selectEnd.length));'#13
        + '    if (start < end) {'#13
        + '      saveColorsAndIds(start+1,end);'#13
        + '    }'#13
        + '    else {'#13
        + '      saveColorsAndIds(start-1,end);'#13
        + '    }'#13
        + '    setSelectionColor(start,end);'#13
        + '    setToHiddenInput();'#13
        + '  }'#13;

    if (SideBar.Position <> spTop) then
      s := s
        + '  if ((event.shiftKey) & (event.keyCode == 38)) {'#13
    else
      s := s
        + '  if ((event.shiftKey) & (event.keyCode == 37)) {'#13;

    s := s
        + '    inMultiSelect = true;'#13
        + '    var temp;'#13
        + '    start = parseInt(selectStart.substring(1,selectStart.length));'#13
        + '    end = parseInt(selectEnd.substring(1,selectEnd.length));'#13
        + '    restoreColors(start,end);'#13
        + '    selectEnd = "R" + (end - 1) + column;'#13
        + '    temp = selectEnd.indexOf("C");'#13
        + '    selectEnd = selectEnd.substring(0,temp);'#13
        + '    end = parseInt(selectEnd.substring(1,selectEnd.length));'#13
        + '    if (start < end) {'#13
        + '      saveColorsAndIds(start+1,end);'#13
        + '    }'#13
        + '    else {'#13
        + '      saveColorsAndIds(start-1,end);'#13
        + '    }'#13
        + '    setSelectionColor(start,end);'#13
        + '    setToHiddenInput();'#13
        + '    defaultSelect = false;'#13
        + '  }'#13;
  end;
  end;
  s := s
    + ' }'#13;

{function UpdEdit}
    s := s
      + 'function UpdEdit(ctrl,itemidx) {'#13
      +  Clientevents.EditDone.Text + #13
      + '  temp = El("EDITITEM"+itemidx);'#13
      + '  submval = FindElem('#39+HTMLName+#39').value;'#13
      + '  s_ar= submval.split("|");'#13
      + '  s_ar[4] = temp.value;'#13
      + '  FindElem('#39 + HTMLName + #39').value = s_ar.join("|");'#13;

    if Assigned(FOnAsyncEditDone) then
    begin
      s := s + 'processAjaxEvent(''onEditDone'', '
        + HTMLControlImplementation.IWCLName
        + ',''' + HTMLName + '.' + 'DoOnAsyncEditDone' + ''','
        + 'true' + ', null, '
        + 'true' + ');';

      Acontext.WebApplication.RegisterCallBack(HTMLName+'.DoOnAsyncEditDone', DoOnAsyncEditDone);
    end
    else
    begin
      s := s
        + '  return SubmitClick('#39 + HTMLName + #39',"UpdEdit" + (itemidx),false)'#13
    end;

      s := s
        + '}'#13

{function CancelEdit}
      + 'function CancelEdit(ctrl,itemidx) {'#13
      + '  return SubmitClick('#39 + HTMLName + #39',"CancelEdit" + (itemidx),false)'#13
      + '}'#13

{function MouseDown}
      + 'function mDwn(id,e) {'#13;

      if (EnableSelection) and not (ViewOnly) and not (FPlannerInEditing) then
      begin
      if (EditType = edPopup) then
      begin
          s := s
          	+ ' if (inEditing != null) {'#13
						+ '   CancelPopupEdit(inEditing);'#13
						+ '   if (OldSelIndex != null) {'#13
						+ '     if (OldColor != null) {'#13
						+ '       if (El("IT" + (OldSelIndex)) != null) {'#13
						+ '         El("IT" + (OldSelIndex)).bgColor = OldColor;'#13
						+ '         El("IT" + (OldSelIndex)).style.color = OldFontColor;'#13
						+ '       }'#13
						+ '       else {'#13
						+ '		    if (El("ITEM" + (OldSelIndex)) != null) {'#13
						+ '           El("ITEM" + (OldSelIndex)).bgColor = OldColor;'#13
						+ '           El("ITEM" + (OldSelIndex)).style.color = OldFontColor;'#13
						+ '         }'#13
						+ '       }'#13
						+ '     }'#13
						+ '   }'#13
						+ '}'#13;
      end;

      s := s
       + ' if (! isIE) {'#13
       + '   window.document.captureEvents(Event.MOUSEOVER);'#13
       + ' }'#13

    + '  if (!containsName("'+HTMLName+'")) {'#13
    + ' 	 window.ChangedControls += "'+HTMLName+',";'#13
    + '}'#13
    ;

      //e.which is supported from IE9
//      if IsBrowserIE(AContext) then
//      begin
//        s := s
//          + '  if (event.button != 1) return;'#13 // handle only left mouse-clicks
//      end
//      else
//      begin
        s := s
          + '  if (e.which != 1) return;'#13;
//      end;

      if IsBrowserIE(AContext) and (not IsBrowserIe10(AContext)) then
      begin
        s := s
          + '    var r = document.selection.createRange();'#13
          + '    r.execCommand("Unselect");'#13;
      end
      else
      begin
        s := s
          + '  if (navigator.appName == "Netscape") {'#13
          + '    window.getSelection().removeAllRanges();'#13
          + '  }'#13;
      end;

         s := s
           + '  var temp;'#13
           + '  if (selected == true) {'#13
           + '    var start = parseInt(selectStart.substring(1,selectStart.length));'#13
           + '    var end = parseInt(selectEnd.substring(1,selectEnd.length));'#13
           + '    restoreColors(start,end);'#13
           + '    cleanUpArrays();'#13
           + '  }'#13

           + '  var area = id.substring(id.length - 2, id.length);'#13
           + '  if (area != "NS") {'#13
           + '  askToAdd = true;'#13

           + '    selectStart = id;'#13
           + '    temp = selectStart.indexOf("C");'#13
           + '    column = selectStart.substring(temp,selectStart.length);'#13
           + '    if (selectStart.indexOf("S") > -1) {'#13
           + '      selectStart = selectStart.substring(1,temp);'#13
           + '      if (El(selectStart + column) != null) {'#13
           + '        ar_Colors[0] = El(selectStart + column).bgColor;'#13
           + '        El(selectStart + column).bgColor = "' + HTMLClr(SelectionColor) + '";'#13
           + '      }'#13
           + '      if (El(id) != null) {'#13
           + '        El(id).bgColor = "'+HTMLClr(SelectionColor)+'";'#13
           + '      }'#13
           + '      ar_Ids[0] = selectStart + column;'#13
           + '    }'#13
           + '    else {'#13
           + '      selectStart = selectStart.substring(0,temp)'#13
           + '      if (El(id) != null) {'#13
           + '        ar_Colors[0] = El(id).bgColor;'#13
           + '        El(id).bgColor = "' + HTMLClr(SelectionColor) + '";'#13
           + '      }'#13
           + '      if (El("S" + selectStart + column) != null) {'#13
           + '        El("S" + selectStart + column).bgColor = "' + HTMLClr(SelectionColor) + '";'#13
           + '      }'#13
           + '      ar_Ids[0] = selectStart + column;'#13
           + '    }'#13
           + '    selectEnd = id;'#13
           + '    temp = selectEnd.indexOf("C");'#13
           + '    if (selectEnd.indexOf("S") > -1) {'#13
           + '      selectEnd = selectEnd.substring(1,temp)'#13
           + '    }'#13
           + '    else {'#13
           + '      selectEnd = selectEnd.substring(0,temp)'#13
           + '    }'#13
           + '    selectedOver = true;'#13
           + '    CellsSelected = 1;'#13
           + '    if (isIE)'#13
           + '    {'#13
           + '      var st;'#13
           + '      var sl;'#13
           + '      if (document.compatMode && document.compatMode != "BackCompat"){'#13
           + '      var st = document.documentElement.scrollTop;'#13
           + '      var sl = document.documentElement.scrollLeft;'#13
           + '      } else {'#13
           + '      var st = document.body.scrollTop;'#13
           + '      var sl = document.body.scrollLeft;'#13
           + '      }'#13
           + '      document.getElementById("TABLE' + HTMLName + '").focus();'#13
           + '      if (document.compatMode && document.compatMode != "BackCompat"){'#13
           + '      document.documentElement.scrollTop = st;'#13
           + '      document.documentElement.scrollLeft = sl;'#13
           + '      } else {'#13
           + '      document.body.scrollTop = st;'#13
           + '      document.body.scrollLeft = sl;'#13
           + '      }'#13
           + '    }'#13
           + '    setToHiddenInput();'#13
           + '    defaultSelect = false;'#13
           + '  }'#13;
      end
      else
      begin
        s := s
           + '  if (navigator.appName == "Netscape") {'#13
           + '    window.getSelection().removeAllRanges();'#13
           + '  }'#13
           + '  else {'#13
           + '    var r = document.selection.createRange();'#13
           + '    r.execCommand("Unselect");'#13
           + '  }'#13;
      end;
      s := s
        + '}'#13

  {function mouseOver}
      + 'function mOv(id,e) {'#13;
  if (EnableSelection) and not (ViewOnly) and not(FPlannerInEditing) then
  begin
    if IsBrowserIE(AContext) and (not IsBrowserIe10(AContext)) then
    begin
      s := s
         + '  var r = document.selection.createRange();'#13
         + '  r.execCommand("Unselect");'#13;
    end
    else
    begin
      s := s
        + '  if (navigator.appName == "Netscape") {'#13
        + '    window.getSelection().removeAllRanges();'#13
        + '  }'#13
    end;


      s := s
        + '  var end;'#13
        + '  var start;'#13;

      if (FSelectionMode = smMultiSelect) then
      begin
        s := s
          + '  if (selectedOver == true) {'#13
          + '    var area = id.substring(id.length - 2, id.length);'#13
          + '    if (area != "NS") {'#13
          + '      var temp;'#13
          + '      start = parseInt(selectStart.substring(1,selectStart.length));'#13
          + '      end = parseInt(selectEnd.substring(1,selectEnd.length));'#13
          + '      restoreColors(start,end);'#13
          + '      selectEnd = id;'#13
          + '      temp = selectEnd.indexOf("C");'#13
          + '      selectEnd = selectEnd.substring(0,temp);'#13
          + '      if (selectEnd.indexOf("S") > -1) {'#13
          + '        selectEnd = selectEnd.substring(1,temp)'#13
          + '      }'#13
          + '      else {'#13
          + '        selectEnd = selectEnd.substring(0,temp)'#13
          + '      }'#13
          + '      end = parseInt(selectEnd.substring(1,selectEnd.length));'#13
	        + '      if (id != selectStart + column) {'#13
          + '         if (start < end) {'#13
          + '          saveColorsAndIds(start+1,end);'#13
          + '        }'#13
          + '        else {'#13
          + '          saveColorsAndIds(start-1,end);'#13
          + '        }'#13
          + '      }'#13
          + '      setSelectionColor(start,end);'#13
          + '    }'#13;

      s := s
        + ' console.log("mouse over: " + start);'
          + '    setToHiddenInput();'#13;

      s := s
          + '  }'#13
          + '  if (selDiv != null) {'#13
          + '    moveItem(e);'#13
          + '  }'#13;
        end;
      end
      else
      begin
        s := s
          + '  if (navigator.appName == "Netscape") {'#13
//Chrome Fix: leave focus on item in InPlaceEdit Mode
//          + '    window.getSelection().removeAllRanges();'#13
          + '  }'#13
          + '  else {'#13
          + '    var r = document.selection.createRange();'#13
          + '    r.execCommand("Unselect");'#13
          + '  }'#13;
      end;

      s := s
        + '}'#13

{function MouseUpInItem}
      + 'function mouseUpInItem() {'#13;

      if IsBrowserIE(AContext) and (not IsBrowserIe10(AContext)) then
      begin
      s := s
        + '  var r = document.selection.createRange();'#13
        + '  r.execCommand("Unselect");'#13;
      end
      else
      begin
        s := s
          + '  if (navigator.appName == "Netscape"){'#13
          + '    window.getSelection().removeAllRanges();'#13
          + '  } else {'#13
          + '    var r = document.selection.createRange();'#13
          + '    r.execCommand("Unselect");'#13
          + '  }'#13;
      end;

   s := s
      + '  if (selDiv != null) {'#13
      + '    El(selDiv + "D").style.visibility = "hidden";'#13
      + '    selDiv = null;'#13
      + '  }'#13
      + '}'#13

{function MouseUp}
      + 'function mUp(e) {'#13
      + '  if (! isIE) {'#13
//Chrome fix: keep focus on item in inplace edit mode
      + '    window.getSelection().removeAllRanges();'#13
      + '    windowX = e.pageX;'#13
      + '    windowY = e.pageY;'#13
      + ' }'#13
      + ' else {'#13

      + '  if (document.compatMode && document.compatMode != "BackCompat"){'#13
      + '    windowX = window.event.x + document.documentElement.scrollLeft;'#13
      + '    windowY = window.event.y + document.documentElement.scrollTop;'#13
      + '  } else {'#13
      + '    windowX = window.event.x + document.body.scrollLeft;'#13
      + '    windowY = window.event.y + document.body.scrollTop;'#13
      + '  }'#13
      + ' }'#13

      + '  selectedOver = false;'#13
      + '  CellsSelected = 0;'#13
      + '  selected = true;'#13
      + '  if (idToResize != null) {'#13
      + '    var divItem = idToResize.substring(4,idToResize.indexOf("R"));'#13
      + '    El("ITEM" + divItem + "D").style.visibility = "hidden";'#13
      + '    if (idToResize.indexOf("RT") > -1 ) {'#13;

   if not (FSideBar.Orientation = soHorizontal) then
   begin
     s := s
      + '      var temp1 = parseInt(windowY - topR0C0);'#13
      + '      temp1 = parseInt(temp1 / heightCell);'#13;
   end
   else
   begin
     s := s
      + '      var temp1 = parseInt(windowX - leftR0C0);'#13
      + '      temp1 = parseInt(temp1 / heightCell);'#13;
   end;

  if Assigned(FOnAsyncItemSize) then
  begin
    s := s + 'setToHiddenInput("ResizeStart" + (idToResize) + "*" + (temp1));';

    s := s + 'processAjaxEvent(''onItemSize'', '
      + HTMLControlImplementation.IWCLName
      + ',''' + HTMLName + '.' + 'DoOnAsyncItemSize' + ''','
      + 'true' + ', null, '
      + 'true' + ');';

    Acontext.WebApplication.RegisterCallBack(HTMLName+'.DoOnAsyncItemSize', DoOnAsyncItemSize);
  end
  else
  begin
    s := s
      + '      return SubmitClick('#39+HTMLName+#39',"ResizeStart" + (idToResize) + "*" + (temp1),false)'#13
  end;

   s := s
      + '    }'#13;

   if not (FSideBar.Orientation = soHorizontal) then
   begin
     s := s
      + '    else if (windowY - startResizeTop - ' + IntToStr(Top) + ' > heightCell) {'#13
      + '           var temp1 = parseInt(windowY - topR0C0);'#13
      + '           temp1 = parseInt(temp1 / heightCell) + 1;'#13;

   end
   else
   begin
     s := s
      + '    else if (windowX - startResizeLeft - ' + IntToStr(Left) + ' > heightCell) {'#13
      + '           var temp1 = parseInt(windowX - leftR0C0);'#13
      + '           temp1 = parseInt(temp1 / heightCell) + 1;'#13;

   end;

  if Assigned(FOnAsyncItemSize) then
  begin
    s := s + 'setToHiddenInput("ResizeEnd" + (idToResize) + "*" + (temp1));';

    s := s + 'processAjaxEvent(''onItemSize'', '
      + HTMLControlImplementation.IWCLName
      + ',''' + HTMLName + '.' + 'DoOnAsyncItemSize' + ''','
      + 'true' + ', null, '
      + 'true' + ');';

    Acontext.WebApplication.RegisterCallBack(HTMLName+'.DoOnAsyncItemSize', DoOnAsyncItemSize);
  end
  else
  begin

    s := s
      + '           return SubmitClick('#39+HTMLName+#39',"ResizeEnd" + (idToResize) + "*" + (temp1),false)'#13;
  end;

   s := s
      + '    }'#13
      + '    idToResize = null;'#13
      + '  }'#13
      + '  if (selDiv != null) {'#13;

   if not Assigned(FOnAsyncItemMoveEx) then
      s := s + '    El(selDiv + "D").style.visibility = "hidden";'#13;

      s := s
      + '    tempDiv = selDiv;'#13
      + '    selDiv = null;'#13

      + '   if (moveStarted) {'#13
      + '    getCoordsR0C0();'#13
      + '    xUp = windowX;'#13
      + '    yUp = windowY;'#13
      + '    var temp1 = parseInt(xUp - leftR0C0);'#13
      + '    var stemp1 = parseInt(xUp - leftR0C0);'#13
      + '    var temp2 = parseInt(yUp - topR0C0);'#13
      + '    var stemp2 = -1;'#13;

   if FSideBar.Orientation = soHorizontal then
   begin

    if RelativeItemMove then
     s := s
      + '    temp1 = parseInt((temp1 - moveDelta) / heightCell);'#13
      + '    stemp2 = parseInt(stemp1 / heightCell);'#13
     else
     s := s
      + '    temp1 = parseInt(temp1 / heightCell);'#13;

     s := s
      + '    temp2 = parseInt(temp2 / (widthCell+1));'#13
   end
   else
   begin
     s := s
      + '    temp1 = parseInt(temp1 / (widthCell+2));'#13
      + '    temp2 = parseInt(temp2 / heightCell);'#13
   end;

  if Assigned(FOnAsyncItemMove) or Assigned(FOnAsyncItemMoveEx) then
  begin

   if FSideBar.Orientation = soHorizontal then
   begin
    if RelativeItemMove then
      s := s + 'setToHiddenInput("Move" + (tempDiv) + "*" + (temp2) + "*" + (temp1) + "*" + (stemp2));'
    else
      s := s + 'setToHiddenInput("Move" + (tempDiv) + "*" + (temp2) + "*" + (temp1) + "*-1");'
   end
   else
    s := s + 'setToHiddenInput("Move" + (tempDiv) + "*" + (temp1) + "*" + (temp2) + "*-1");';

    s := s + 'processAjaxEvent(''onItemMove'', '
      + HTMLControlImplementation.IWCLName
      + ',''' + HTMLName + '.' + 'DoOnAsyncItemMove' + ''','
      + 'true' + ', null, '
      + 'true' + ');';

    Acontext.WebApplication.RegisterCallBack(HTMLName+'.DoOnAsyncItemMove', DoOnAsyncItemMove);
  end
  else
  begin

   if FSideBar.Orientation = soHorizontal then
    s := s
      + '    return SubmitClick('#39+HTMLName+#39',"Move" + (tempDiv) + "*" + (temp2) + "*" + (temp1) ,false)'#13
   else
    s := s
      + '    return SubmitClick('#39+HTMLName+#39',"Move" + (tempDiv) + "*" + (temp1) + "*" + (temp2) ,false)'#13
  end;

   s := s
      + '    selDiv = null;'#13
      + '   }'#13
      + '  }'#13

    + '  if ((selectStart != null) & (defaultSelect == false)) {'#13;

   s := s
      + '  if (askToAdd) {'#13;

   if AutoCreateOnSelect then
   begin
     if FShowConfirm then
     begin
       s := s + '    if (confirm("'+ FHints.FHintConfirmDialog + '") == true) {'#13
        + '      AddItem();'#13
        + '    }'#13
        + '    else {'#13
        + '      askToAdd = false;'#13
        + '    }'#13;
     end
     else
     begin
       s := s + '      AddItem();'#13;
     end;
   end;

   s := s
      + ' }'#13;

   s := s
      + '  }'#13
      + '  else {'#13
      + '  }'#13
      + '  lastID = null;'#13
      + '}'#13

{function AddItem}
      + 'function AddItem() {'#13
      + '  submval = FindElem('#39+HTMLName+#39').value;'#13
      + '  s_ar = submval.split("|");'#13
      + '  var start = parseInt(selectStart.substring(1,selectStart.length));'#13
      + '  var end = parseInt(selectEnd.substring(1,selectEnd.length));'#13;

   if (ChangeCaption.EnterCaptionAfterItemCreate) and (DefaultItem.CaptionType <> ctTime) and (DefaultItem.CaptionType <> ctNone) then
   begin
     s := s
       + '   var captionText = window.prompt("' + ChangeCaption.TextPromptWindow + '","");'#13
       + '   var send = true;'#13
       + '   if (captionText == null)'#13
       + '     send = false;'#13
       + '   if (captionText == ("undefined"))'#13
       + '     send = false;'#13
       + '   if (captionText == "")'#13
       + '     send = false;'#13
       + '   if (captionText == " ")'#13
       + '     send = false;'#13
       + '   if (!send) {'#13
       + '     captionText = "";'#13
       + '   }'#13
       + '   s_ar[5] = captionText;'#13;
   end;

     s := s
      + '  if (start > end) {'#13
      + '    s_ar[1] = selectEnd;'#13
      + '    s_ar[2] = selectStart;'#13
      + '  }'#13
      + '  else {'#13
      + '    s_ar[1] = selectStart;'#13
      + '    s_ar[2] = selectEnd;'#13
      + '  }'#13
      + '  s_ar[3] = column;'#13
      + '  FindElem('#39+HTMLName+#39').value = s_ar.join("|");'#13
      + '    return SubmitClick('#39+HTMLName+#39',"AddItem",false)'#13
      + '}'#13

{function RestoreColors}
      + 'function restoreColors(start,end) {'#13
      + '  var index = null;'#13
      + '  var loopBegin;'#13
      + '  var loopEnd;'#13
      + '  if (start <= end) {'#13
      + '    loopBegin = start;'#13
      + '    loopEnd = end;'#13
      + '  }'#13
      + '  else {'#13
      + '    loopBegin = end;'#13
      + '    loopEnd = start;'#13
      + '  }'#13
      + '  for (loopBegin;loopBegin<=loopEnd;loopBegin++) {'#13
      + '    for (i=0;i<ar_Ids.length;i++) {'#13
      + '      if ((ar_Ids[i] == "R" + loopBegin + column) | (ar_Ids[i] == "SR" + loopBegin + column)){'#13
      + '        index = i;'#13
      + '        break;'#13
      + '      }'#13
      + '    }'#13
      + '    if (index > -1) {'#13
      + '      if (El("R" + loopBegin + column) != null) {'#13
      + '        El("R" + loopBegin + column).bgColor = ar_Colors[index];'#13
      + '      }'#13
      + '      if (El("SR" + loopBegin + column) != null) {'#13
      + '        El("SR" + loopBegin + column).bgColor ="'  + HtmlClr(OverlapColor) + '";'#13
      + '      }'#13
      + '    }'#13
      + '  }'#13
      + '}'#13

{function CleanUpArrays}
      + 'function cleanUpArrays() {'#13
      + '  ar_Colors = new Array();'#13
      + '  ar_Ids = new Array();'#13
      + '}'#13

{function SaveColors}
      + 'function saveColorsAndIds(start,end) {'#13
      + '  var loopBegin;'#13
      + '  var loopEnd;'#13
      + '  if (start <= end) {'#13
      + '    loopBegin = start;'#13
      + '    loopEnd = end;'#13
      + '  }'#13
      + '  else {'#13
      + '    loopBegin = end;'#13
      + '    loopEnd = start;'#13
      + '  }'#13

      + '  var exists = false;'#13
      + '  for (j=loopBegin;j<=loopEnd;j++) {'#13
      + '    for (i=0;i<ar_Ids.length;i++) {'#13
      + '      if (ar_Ids[i] == "R" + j) {'#13
      + '        exists = true;'#13
      + '        break;'#13
      + '      }'#13
      + '    }'#13
      + '    if (exists == false) {'#13
      + '      if (El("R" + j + column) != null) {'#13
      + '        ar_Colors[CellsSelected] = El("R" + j + column).bgColor;'#13
      + '        ar_Ids[CellsSelected] = "R" + j + column;'#13
      + '        CellsSelected += 1;'#13
      + '      }'#13
      + '      else {'#13
      + '        if (El("SR" + j + column) != null) {'#13
      + '          ar_Colors[CellsSelected] = El("SR" + j + column).bgColor;'#13
      + '          ar_Ids[CellsSelected] = "SR" + j + column;'#13
      + '          CellsSelected += 1;'#13
      + '        }'#13
      + '      }'#13
      + '    }'#13
      + '    else {'#13
      + '      exists = false;'#13
      + '    }'#13
      + '  }'#13
      + '  var temp = " ";'#13
      + '}'#13

{function setSelectionColor}
      + 'function setSelectionColor(start,end){'#13
      + '  var loopBegin;'#13
      + '  var loopEnd;'#13
      + '  if (start <= end) {'#13
      + '    loopBegin = start;'#13
      + '    loopEnd = end;'#13
      + '  }'#13
      + '  else {'#13
      + '    loopBegin = end;'#13
      + '    loopEnd = start;'#13
      + '  }'#13
      + '  for(loopBegin;loopBegin<=loopEnd;loopBegin++) {'#13
      + '    if ((El("R" + loopBegin + column) != null) & (El("IR" + loopBegin + column) == null)) {'#13
      + '      El("R" + loopBegin + column).bgColor = "' + HTMLClr(SelectionColor) + '";'#13
      + '    }'#13
      + '    if (El("SR" + loopBegin + column) != null) {'#13
      + '      El("SR" + loopBegin + column).bgColor = "' + HTMLClr(SelectionColor) + '";'#13
      + '    }'#13
      + '  }'#13
      + '}'#13

{function iePointerPosn}

      + 'function ieP(id,event) {'#13
// Chrome Fix: focus does not stay on InplaceEdit
//      + '    if (navigator.appName == "Netscape") '#13
//      + '      window.getSelection().removeAllRanges();'#13
      + ' if (! isIE) {'#13
      + '   window.document.captureEvents(Event.MOUSEOVER);'#13
      + ' }'#13
      + '  if (selDiv != null) {'#13
      + '    if (! isIE) {'#13
      + '      window.document.captureEvents(Event.MOUSEMOVE);'#13
      + '    }'#13
      + '      document.onmousemove = moveItem;'#13
      + '    moveItem(event);'#13
      + '  }'#13
      + '  else if (idToResize != null) {'#13
      + '    if (! isIE) {'#13
      + '      window.document.captureEvents(Event.MOUSEMOVE);'#13
      + '    }'#13
      + '    document.onmousemove = resizeItem;'#13
      + '    resizeItem(event);'#13
      + '  }'#13

      + '  if (! isIE) {'#13;
    if (SideBar.Position = spTop) then
      s := s
        + '    var temp1 = parseInt(event.pageX - leftR0C0);'#13
    else
      s := s
        + '    var temp1 = parseInt(event.pageY - topR0C0);'#13;

    s := s
      + '    temp1 = parseInt(temp1 / (heightCell));'#13
      + '    var Col = id.substring(id.indexOf("C"),id.length);'#13
      + '    id = "R" + temp1 + Col;'#13
      + '  }'#13
      + '  if (! isIE) {'#13
      + '    if ((id != "") && (id != lastID)) {'#13
      + '      lastID = id;'#13
      + '      mOv(id,event);'#13
      + '    }'#13
      + '  }'#13

      + '}'#13

{function HeaderChangeCap}
      + 'function HeaderChangeCap(id,changeCap, e){'#13;

      //e.which is supported from IE9
//      if IsBrowserIE(AContext) then
//        s := s
//          + '  if (event.button != 1) {'#13
//      else
        s := s
          + '  if (e.which != 1) {'#13;

    if ((ChangeCaption.ShowPromptWindow) and (EditType = edInplace)) then
    begin
      if (ChangeCaption.ShowDialogWindow) then
      begin
        s := s
          + '      if (changeCap.indexOf("True") >= 0) {'#13
          + '        var answer = window.confirm("' + ChangeCaption.TextDialogWindow + '");'#13
          + '        if (answer)'#13
          + '           {'#13
					+ '               var oldCaption = document.getElementById(''DIV'' + (id));'#13
					+ '               askCaption(id,oldCaption.innerHTML);'#13
				  + '           }'#13
          + '      }'#13;
      end
      else
      begin
        s := s
          + '     if (changeCap.indexOf("True") >= 0) {'#13
          + '           {'#13
					+ '               var oldCaption = document.getElementById(''DIV'' + (id));'#13
					+ '               askCaption(id,oldCaption.innerHTML);'#13
				  + '           }'#13
          + '     }'#13;
      end;
    end;

      s := s
        + '  }'#13
        + '}'#13

{function startMove}
      + 'function startMove(id,changeCap,e){'#13;

      if (EditType = edPopup) then
      begin
          s := s
          	+ ' if (inEditing != null) {'#13
						+ '   CancelPopupEdit(inEditing);'#13
						+ '   if (OldSelIndex != null) {'#13
						+ '     if (OldColor != null) {'#13
						+ '       if (El("IT" + (OldSelIndex)) != null) {'#13
						+ '         El("IT" + (OldSelIndex)).bgColor = OldColor;'#13
						+ '         El("IT" + (OldSelIndex)).style.color = OldFontColor;'#13
						+ '       }'#13
						+ '       else {'#13
						+ '		    if (El("ITEM" + (OldSelIndex)) != null) {'#13
						+ '           El("ITEM" + (OldSelIndex)).bgColor = OldColor;'#13
						+ '           El("ITEM" + (OldSelIndex)).style.color = OldFontColor;'#13
						+ '         }'#13
						+ '       }'#13
						+ '     }'#13
						+ '   }'#13
						+ '}'#13;
      end;
      //End Modified

      //Modified - Support other browsers
      //e.which is supported from IE9
//     if IsBrowserIE(AContext) then
//       s := s
//         + '  if (event.button != 1) {'#13
//     else
       s := s
         + '  if (e.which != 1) {'#13;
    //End Modified

    if ((ChangeCaption.ShowPromptWindow) and (EditType = edInplace)) then
    begin
      if (ChangeCaption.ShowDialogWindow) then
      begin
        s := s
          + '      if (changeCap.indexOf("True") >= 0) {'#13
          + '        var answer = window.confirm("' + ChangeCaption.TextDialogWindow + '");'#13
          + '        if (answer)'#13
    			//Modified - Bug double and single qoutes in CaptionText
          //+ '          askCaption(id, oldCaption);'#13
          + '           {'#13
					+ '               var oldCaption = document.getElementById(''DIV'' + (id));'#13
					+ '               askCaption(id,oldCaption.innerHTML);'#13
				  + '           }'#13
          //End Modified
          + '      }'#13;
      end
      else
      begin
        s := s
          + '     if (changeCap.indexOf("True") >= 0) {'#13
          //Modified - Bug double and single qoutes in CaptionText
          + '           {'#13
					+ '               var oldCaption = document.getElementById(''DIV'' + (id));'#13
					+ '               askCaption(id,oldCaption.innerHTML);'#13
				  + '           }'#13
          //+ '       askCaption(id, oldCaption);'#13
          //End Modified
          + '     }'#13;
      end;
    end;

      s := s
        + '  }'#13
        + '  else {'#13;

   if IsBrowserIE(AContext) then
   begin
     s := s
        + '  if (document.compatMode && document.compatMode != "BackCompat"){'#13
        + ' xDown = window.event.x + document.documentElement.scrollLeft;'#13
        + ' yDown = window.event.y + document.documentElement.scrollTop;'#13
        + '  } else {'#13
        + ' xDown = window.event.x + document.body.scrollLeft;'#13
        + ' yDown = window.event.y + document.body.scrollTop;'#13
        + '  }'#13
   end
   else
   begin
     s := s
        //Modified - Support other browsers
        //+ ' windowX = e.pageX + e.pageXOffset;'#13
        //+ ' windowY = e.pageY + e.pageYOffset;'#13
        + ' xDown = e.pageX + 5;'#13
        + ' yDown = e.pageY + 5;'#13;
        //End Modfied
   end;

   if IsBrowserIE(AContext) and (not IsBrowserIe10(AContext)) then
   begin
     s := s
        + '  var r = document.selection.createRange();'#13
        + '  r.execCommand("Unselect");'#13;
   end
   else
   begin
     s := s
        + ' window.getSelection().removeAllRanges();'#13;
   end;

   s := s
      + '  selDiv = id;'#13
      + '  if (El(id + "D") != null) {'#13
      + '    var cell = El(id + "D");'#13
      + '    selDivLeft = 0;'#13
      + '    var el = cell;'#13
      + '    do'#13
      + '      selDivLeft += el.offsetLeft'#13
      + '    while ((el = el.offsetParent));'#13
      + '    xS = xDown;'#13
      + '    yS = yDown;'#13
      + '    moveStarted = false;'#13

      + '  e = window.event || e;'#13
      + '  mEl = e.target || e.srcElement;'#13
//      + '  mEl = e.srcElement;'#13
      + '  moveLeft = mEl.offsetLeft;'#13
      + '  while (mEl = mEl.parentNode)'#13
      + '  {'#13
      + '    if (mEl.offsetLeft){'#13
      + '      if (mEl.id.length > 0)'#13
      + '      moveLeft += mEl.offsetLeft;'#13
      + '    }'#13
      + '  }'#13
      + '  moveDelta = null;'#13

//only move item when mouse cursor has moved (see function moveItem)
//      + '    El(selDiv + "D").style.visibility = "visible";'#13
//      + '    ieP(id,e);'#13
      + '    if (! isIE) {'#13
      + '      document.captureEvents(Event.MOUSEUP);'#13
      + '     }'#13
      + '    document.onmouseup = docOnMouseUp;'#13
      + '  }'#13
      + '}'#13
      + '}'#13

{function getCoordsEndRowEndColumn}
      + 'function getCoordsEndRowEndColumn() {'#13;
if (SideBar.Orientation = soHorizontal) then
begin
  s := s
      + '  leftEndRowEndCol = parseInt(' + IntToStr(Display.DisplayScale) + ') * parseInt(countOfRows) + parseInt(leftR0C0);'#13
      + '  topEndRowEndCol = parseInt(widthCell) * parseInt(' + IntToStr(Positions) + ') + parseInt(topR0C0) + parseInt(' + IntToStr(Positions) + ');'#13;
end
else
begin
  s := s
     //Modified - Optimize coords EndPlanner
     //+ ' leftEndRowEndCol = parseInt(leftR0C0) + parseInt(' + IntToStr(Positions) + ') * parseInt(widthCell);'#13
     + ' leftEndRowEndCol = parseInt(leftR0C0) + parseInt(' + IntToStr(Positions) + ') * parseInt(widthCell) + parseInt(' + IntToStr(Positions) + ');'#13
     //End Modified
     + ' topEndRowEndCol = parseInt(topR0C0) + parseInt(' + IntToStr(Display.DisplayScale) + ') * parseInt(countOfRows);'#13;
end;

  s := s
      + '}'#13

      + ' function docOnMouseUp(e) {'#13
      + '   getCoordsR0C0();'#13
      + '   getCoordsEndRowEndColumn();'#13
      + '  if (! isIE) {'#13
      //Modified - Support other browsers
      //+ '    windowX = e.pageX + e.pageXOffset;'#13
      //+ '    windowY = e.pageY + e.pageYOffset;'#13
      + '    windowX = e.pageX;'#13
      + '    windowY = e.pageY;'#13
      //End Modified
      + '  }'#13
      + '  else {'#13
      + '  if (document.compatMode && document.compatMode != "BackCompat"){'#13
      + '    windowX = window.event.x + document.documentElement.scrollLeft;'#13
      + '    windowY = window.event.y + document.documentElement.scrollTop;'#13
      + '  } else {'#13
      + '    windowX = window.event.x + document.body.scrollLeft;'#13
      + '    windowY = window.event.y + document.body.scrollTop;'#13
      + '  }'#13
      + '  }'#13
//      + '   alert(leftEndRowEndCol + ";" + topEndRowEndCol);'#13
      + '   if ((windowX > leftEndRowEndCol) | (windowX < leftR0C0) | (windowY > topEndRowEndCol)  | (windowY < topR0C0) ) {'#13
      + '     if (selDiv != null) {'#13
      + '       El(selDiv + "D").style.visibility = "hidden";'#13
      + '       selDiv = null;'#13;

   if ShowAlertOutsidePlanner then
   begin
     s := s
        + '    alert("' + FAlertOutsidePlanner + '");'#13;
   end;

   s := s
      + '     }'#13
      + '   }'#13
      + ' }'#13

{function resizeItem}
      + 'function resizeItem(e) {'#13;

  if IsBrowserIE(AContext) then
   begin
     s := s
      + '  if (document.compatMode && document.compatMode != "BackCompat"){'#13
      + '  windowX = window.event.x + document.documentElement.scrollLeft;'#13
      + '  windowY = window.event.y + document.documentElement.scrollTop;'#13
      + '  } else {'#13
      + '  windowX = window.event.x + document.body.scrollLeft;'#13
      + '  windowY = window.event.y + document.body.scrollTop;'#13
      + '  }'#13
   end
   else
   begin
     s := s
      //Modified - Support other browsers
      //+ '  windowX = e.pageX + e.pageXOffset;'#13
      //+ '  windowY = e.pageY + e.pageYOffset;'#13
      + '  windowX = e.pageX;'#13
      + '  windowY = e.pageY;'#13
      //End Modified
   end;

  if IsBrowserIE(AContext) and (not IsBrowserIe10(AContext)) then
   begin
     s := s
      + '  var r = document.selection.createRange();'#13
      + '  r.execCommand("Unselect");'#13;
   end
   else
   begin
     s := s
      + '  window.getSelection().removeAllRanges();'#13;
   end;

   s := s
      + '  if (idToResize != null) {'#13
      + '    var divItem = idToResize.substring(4,idToResize.indexOf("R"));'#13
      + '    if ((idToResize != null) & (El("ITEM" + divItem + "D") != null)) {'#13;

      if not (FSideBar.Orientation = soHorizontal) then
      begin
    s := s
      + '      if (idToResize.indexOf("RT") > -1) {'#13

      //Modified - resizing when inside an IWRegion

      + '      myEl = El("ITEM" + divItem + "D");'#13
      + '      posEl = myEl.parentNode;'#13
      + '      elTop = 0;'#13

      + '      while (posEl = posEl.parentNode)'#13
      + '      {'#13
      + '        if (posEl.offsetTop)'#13
      + '          elTop += posEl.offsetTop;'#13
      + '      }'#13

      + '        if (endResizeTop - windowY + ' + IntToStr(Top) + '> heightCell) {'#13;
      //Modified - Support use of templates
      if (InTemplate) then
        s := s
          + '          El("ITEM" + divItem + "D").style.top = (windowY - elTop) + "px";'#13
      else
        s := s
          + '          El("ITEM" + divItem + "D").style.top = (windowY - elTop - ' + IntToStr(Top) + ') + "px";'#13;

      //End Modified
      s := s
      + '          El("ITEM" + divItem + "D").style.height = (endResizeTop - windowY + ' + IntToStr(Top) + ') + "px";'#13

      + '        }'#13
      + '      }'#13
      + '      else if (windowY - startResizeTop - ' + IntToStr(Top) + ' > heightCell) {'#13
      + '        El("ITEM" + divItem + "D").style.height = (windowY - startResizeTop - ' + IntToStr(Top) + ') + "px";'#13
      + '      }'#13
      + '    }'#13
      + '  }'#13
      + '}'#13;
      end
      else
      begin
   s := s
      + '      if (idToResize.indexOf("RT") > -1) {'#13
      + '        if (endResizeLeft - windowX + ' + IntToStr(Left) + ' > ' + IntToStr(HeightRow) + ') {'#13

      + '      myEl = El("ITEM" + divItem + "D");'#13
      + '      posEl = myEl.parentNode;'#13
      + '      elLeft = 0;'#13

      + '      while (posEl = posEl.parentNode)'#13
      + '      {'#13
      + '        if (posEl.offsetLeft)'#13
      + '          elLeft += posEl.offsetLeft;'#13
      + '      }'#13

      ;
      //Modified - Support use of template
      if (InTemplate) then
        s := s
          + '            El("ITEM" + divItem + "D").style.left = (windowX - elLeft) + "px";'#13
      else
        s := s
          + '            El("ITEM" + divItem + "D").style.left = (windowX - elLeft - ' + IntToStr(Left) + ') + "px";'#13;

      //End Modified
    s := s
      + '            El("ITEM" + divItem + "D").style.width = (endResizeLeft - windowX + ' + IntToStr(Left) + ') + "px";'#13
      + '        }'#13
      + '      }'#13
      + '      else if (windowX - startResizeLeft - ' + IntToStr(Left) + ' > ' + IntToStr(HeightRow) + ') {'#13
      + '        El("ITEM" + divItem + "D").style.width = (windowX - startResizeLeft - ' + IntToStr(Left) + ') + "px";'#13
      + '      }'#13
      + '    }'#13
      + '  }'#13
      + '}'#13;
      end;

   s := s
{function startResize}
      + 'function startResize(id,event) {'#13;

   if IsBrowserIE(AContext) and (not IsBrowserIe10(AContext)) then
   begin
     s := s
        + '  var r = document.selection.createRange();'#13
        + '  r.execCommand("Unselect");'#13;
   end
   //Modified - Support other browsers
   else
   begin
     s := s
       + 'if (navigator.appName == "Netscape") {'#13
       + '  window.getSelection().removeAllRanges();'#13
       + '}'#13;
   end;
   //End Modified

         //Modified 10/29/2003 - Optimalization Popup Editing
      if (EditType = edPopup) then
      begin
          s := s
          	+ ' if (inEditing != null) {'#13
						+ '   CancelPopupEdit(inEditing);'#13
						+ '   if (OldSelIndex != null) {'#13
						+ '     if (OldColor != null) {'#13
						+ '       if (El("IT" + (OldSelIndex)) != null) {'#13
						+ '         El("IT" + (OldSelIndex)).bgColor = OldColor;'#13
						+ '         El("IT" + (OldSelIndex)).style.color = OldFontColor;'#13
						+ '       }'#13
						+ '       else {'#13
						+ '		    if (El("ITEM" + (OldSelIndex)) != null) {'#13
						+ '           El("ITEM" + (OldSelIndex)).bgColor = OldColor;'#13
						+ '           El("ITEM" + (OldSelIndex)).style.color = OldFontColor;'#13
						+ '         }'#13
						+ '       }'#13
						+ '     }'#13
						+ '   }'#13
						+ '}'#13;
      end;
      //End Modified

   s := s
      + '  idToResize = id;'#13
      + '  if (selDiv != null) {'#13
      + '    selDiv = null;'#13
      + '  }'#13
      + '  getCoordsR0C0();'#13
      + '  var divItem = id.substring(4,id.indexOf("R"));'#13
      + '  if (El("ITEM" + divItem + "D") != null) {'#13
      + '    var cell = El("ITEM" + divItem);'#13
      + '    var left = 0;'#13
      + '    var el = cell;'#13
      + '    do'#13
      + '      left += el.offsetLeft;'#13
      + '    while ((el = el.offsetParent));'#13
      + '    startResizeLeft = left - '+ IntToStr(Left) + ';'#13
      + '    var top = 0;'#13
      + '    el = cell;'#13
      + '    do'#13
      + '      top += el.offsetTop;'#13
      + '    while ((el = el.offsetParent))'#13
      + '    startResizeTop = top - ' + IntToStr(Top) + ';'#13
      + '    var cell = El("ITEM" + divItem + "RD");'#13
      + '    endResizeLeft = 0;'#13
      + '    el = cell;'#13
      + '    do'#13
      + '      endResizeLeft += el.offsetLeft;'#13
      + '    while ((el = el.offsetParent));'#13
      + '    endResizeTop = 0;'#13
      + '    el = cell;'#13
      + '    do'#13
      + '      endResizeTop += el.offsetTop;'#13
      + '    while ((el = el.offsetParent))'#13
      + '    endResizeTop = endResizeTop - ' + IntToStr(Top) + ';'#13
      + '    endResizeLeft = endResizeLeft - ' + IntToStr(Left) + ';'#13
//      + '    alert(startResizeLeft + ";" + startResizeTop);'#13


      + '    myEl = El("ITEM" + divItem + "D");'#13
      + '    posEl = myEl.parentNode;'#13

      + '    while (posEl = posEl.parentNode)'#13
      + '    {'#13
      + '      if (posEl.offsetLeft){'#13
      + '        left -= posEl.offsetLeft;'#13
      + '        top -= posEl.offsetTop;'#13
      + '      }'#13
      + '    }'#13

      + '    El("ITEM" + divItem + "D").style.visibility = "visible";'#13;

      //Modified - Support use of templates
      if (InTemplate) then
        s := s
          + '    El("ITEM" + divItem + "D").style.left = left + "px";'#13
          + '    El("ITEM" + divItem + "D").style.top = top + "px";'#13
      else
        s := s
          + '    El("ITEM" + divItem + "D").style.left = (left - ' + IntToStr(Left) + ') + "px";'#13
          + '    El("ITEM" + divItem + "D").style.top = (top - ' + IntToStr(Top) + ') + "px";'#13;
      //End Modified

   if FSideBar.Orientation = soVertical then
   begin
     s := s
      + '    var heightDiv = parseInt(endResizeTop - startResizeTop - ' + IntToStr(Top) + ');'#13
      + '    if (heightDiv < 0) {'#13
      + '    heightDiv = - heightDiv;'#13
      + '    }'#13
   end;

   s := s
      + '    if (! isIE) {'#13
      + '      window.document.captureEvents(Event.MOUSEMOVE);'#13
      + '    }'#13
      + '    document.onMouseMove = resizeItem;'#13
      + '    resizeItem(event);'#13
      + '  }'#13
      + '}'#13

{function moveItem}
      + 'function moveItem(e) {'#13;

   if IsBrowserIE(AContext) then
   begin
     s := s
      + '  if (document.compatMode && document.compatMode != "BackCompat"){'#13
        + ' windowX = window.event.x + document.documentElement.scrollLeft;'#13
        + ' windowY = window.event.y + document.documentElement.scrollTop;'#13
      + '  } else {'#13
        + ' windowX = window.event.x + document.body.scrollLeft;'#13
        + ' windowY = window.event.y + document.body.scrollTop;'#13
      + '  }'#13
   end
   else
   begin
     s := s
        //Modified - Support other browsers
        //+ ' windowX = e.pageX + e.pageXOffset;'#13
        //+ ' windowY = e.pageY + e.pageYOffset;'#13
        + ' windowX = e.pageX + 5;'#13
        + ' windowY = e.pageY + 5;'#13;
        //End Modfied
   end;

   if IsBrowserIE(AContext) and (not IsBrowserIe10(AContext)) then
   begin
     s := s
        + '  var r = document.selection.createRange();'#13
        + '  r.execCommand("Unselect");'#13;
   end
   else
   begin
     s := s
        + ' window.getSelection().removeAllRanges();'#13;
   end;

   s := s
      + '  if ((selDiv != null) & (El(selDiv + "D") != null)) {'#13

      //Modified positioning when using inside an IWRegion
      + '  myEl = El(selDiv + "D");'#13
      + '  posEl = myEl.parentNode;'#13
      ;

      //start itemMove if mouse position has moved at least "dm" pixels
      s := s
      + ' dm = 10;'#13
      + ' if ((windowX > xS + dm) || (windowX < xS - dm) || (windowY > yS + dm) || (windowY < yS - dm) || (moveStarted)) {'#13
      + '  moveStarted = true;'#13

      + '  while (posEl = posEl.parentNode)'#13
      + '  {'#13
      + '    if (posEl.offsetLeft){'#13
      + '      windowX -= posEl.offsetLeft;'#13
      + '      moveLeft -= posEl.offsetLeft;'#13
      + '      windowY -= posEl.offsetTop;'#13
      + '    }'#13
      + '  }'#13

      + '    El(selDiv + "D").style.visibility = "visible";'#13
      ;

      //Modified - Support use of templates
      if (InTemplate) then
      begin
//        s := s
//          + '    El(selDiv + "D").style.left = windowX;'#13
//          + '    El(selDiv + "D").style.top = windowY;'#13

        s := s
          + '    dleft = windowX + 5;'#13
          + '    dtop = windowY + 5;'#13;

        if (FSideBar.Orientation = soHorizontal) and (RelativeItemMove) then
        begin
          s := s
            + '    if (moveDelta == null) '#13
            + '      moveDelta = dleft - moveLeft;'#13;
        end
        else
        begin
          s := s
            + '      moveDelta = 0;'#13;
        end;

        s := s
          + '    El(selDiv + "D").style.left = (dleft - moveDelta) + "px";'#13
          + '    El(selDiv + "D").style.top = (dtop) + "px";'#13;
      end
      else
      begin
//        s := s
//          + '    El(selDiv + "D").style.left = windowX - ' + IntToStr(Left) + ';'#13
//          + '    El(selDiv + "D").style.top = windowY - ' + IntToStr(Top) + ';'#13;

        s := s
          + '    dleft = windowX - ' + IntToStr(Left) + ' + 5;'#13
          + '    dtop = windowY - ' + IntToStr(Top) + ' + 5;'#13;

        if (FSideBar.Orientation = soHorizontal) and (RelativeItemMove) then
        begin
          s := s
            + '    if (moveDelta == null) '#13
            + '      moveDelta = dleft - (moveLeft - ' + IntToStr(Left) + ');'#13;
        end
        else
        begin
          s := s
            + '      moveDelta = 0;'#13;
        end;

        s := s
          + '    El(selDiv + "D").style.left = (dleft - moveDelta) + "px";'#13
          + '    El(selDiv + "D").style.top = dtop + "px";'#13;
      end;

      s := s
      //End Modified
      + '    El(selDiv + "D").style.visibility = "visible";'#13

      + ' }'

      + '  }'#13
      + '  else {'#13
      + '    resizeItem(e);'#13
      + '  }'#13
      + '}'#13

{function getCoordsR0C0}
      + 'function getCoordsR0C0() {'#13
      + '  var element;'#13
      + '  if (El("R0C0") != null) {'#13
      + '    element = "R0C0";'#13
      + '  }'#13
      + '  else {'#13
      + '    if (El("PR0C0") != null) {'#13
      + '      element = "PR0C0";'#13
      + '    }'#13
      + '    else {'#13
      + '      if (El("IR0C0") != null) {'#13
      + '        element = "IR0C0";'#13
      + '      }'#13
      + '      else {'#13
      + '        if (El("R0C0NS") != null)'#13
      + '          element = "R0C0NS";'#13
      + '      }'#13
      + '    }'#13
      + '  }'#13
      + '  var cell = El(element);'#13
      + '  leftR0C0 = 0;'#13
      + '  var el = cell;'#13
      + '  do'#13
      + '    leftR0C0 += el.offsetLeft'#13
      + '  while ((el = el.offsetParent));'#13
      + '    topR0C0 = 0;'#13
      + '  el = cell;'#13
      + '  do'#13
      + '    topR0C0 += el.offsetTop;'#13
      + '  while ((el = el.offsetParent))'#13
      + '}'#13;

{Function ItemCall}
			s := s
				+ 'function ItemCall(call,index,selColor,selFontColor,BackColor,FontColor) {'#13
				+ '  if (call == ''Delete'') {'#13;

        //Modified 10/29/2003 - Optimalization Popup Editing
      if (EditType = edPopup) then
      begin
          s := s
          	+ ' if (inEditing != null) {'#13
						+ '   CancelPopupEdit(inEditing);'#13
						+ '   if (OldSelIndex != null) {'#13
						+ '     if (OldColor != null) {'#13
						+ '       if (El("IT" + (OldSelIndex)) != null) {'#13
						+ '         El("IT" + (OldSelIndex)).bgColor = OldColor;'#13
						+ '         El("IT" + (OldSelIndex)).style.color = OldFontColor;'#13
						+ '       }'#13
						+ '       else {'#13
						+ '		    if (El("ITEM" + (OldSelIndex)) != null) {'#13
						+ '           El("ITEM" + (OldSelIndex)).bgColor = OldColor;'#13
						+ '           El("ITEM" + (OldSelIndex)).style.color = OldFontColor;'#13
						+ '         }'#13
						+ '       }'#13
						+ '     }'#13
						+ '   }'#13
						+ '}'#13;
      end;
      //End Modified

        s := s
        +     ClientEvents.DeleteClick.Text;

        if Assigned(FOnAsyncItemDelete) then
        begin
          s := s + 'setToHiddenInput("Delete" + (index));';

          s := s + 'processAjaxEvent(''onItemDelete'', '
            + HTMLControlImplementation.IWCLName
            + ',''' + HTMLName + '.' + 'DoOnAsyncItemDelete' + ''','
            + 'true' + ', null, '
            + 'true' + ');';

          Acontext.WebApplication.RegisterCallBack(HTMLName+'.DoOnAsyncItemDelete', DoOnAsyncItemDelete);
        end
        else
        begin
          s := s
            + '    return SubmitClick('#39 + HTMLName + #39','#39 + 'Delete'' + (index),false)'#13
        end;

        s := s
				+ '  }'#13
				+ '  else {'#13;

      //Modified 10/29/2003 - Optimalization Popup Editing
      if (EditType = edPopup) then
      begin
          s := s
          	+ ' if (inEditing != null) {'#13
						+ '   CancelPopupEdit(inEditing);'#13
						+ '   if (OldSelIndex != null) {'#13
						+ '     if (OldColor != null) {'#13
						+ '       if (El("IT" + (OldSelIndex)) != null) {'#13
						+ '         El("IT" + (OldSelIndex)).bgColor = OldColor;'#13
						+ '         El("IT" + (OldSelIndex)).style.color = OldFontColor;'#13
						+ '       }'#13
						+ '       else {'#13
						+ '		    if (El("ITEM" + (OldSelIndex)) != null) {'#13
						+ '           El("ITEM" + (OldSelIndex)).bgColor = OldColor;'#13
						+ '           El("ITEM" + (OldSelIndex)).style.color = OldFontColor;'#13
						+ '         }'#13
						+ '       }'#13
						+ '     }'#13
						+ '   }'#13
						+ '}'#13;
      end;
      //End Modified
      s :=s
 				+ '    NewSelIndex = index;'#13
        +     ClientEvents.Select.Text
        + '    submval = FindElem('#39+HTMLName+#39').value;'#13
        + '    s_ar = submval.split("|");'#13
        + '    s_ar[0] = NewSelIndex;'#13
        + '    FindElem('#39+HTMLName+#39').value = s_ar.join("|");'#13
				+ '    NewSelColor = selColor;'#13
				+ '    NewSelFontColor = selFontColor;'#13
				+ '    if (call == ''SelectFC'') {'#13
				+ '      if (OldSelIndex != null) {'#13
				+ '        if (OldColor != ''null'') {'#13
				+ '          if (El(''IT'' + (OldSelIndex)) != null) {'#13
				+ '            El(''IT'' + (OldSelIndex)).bgColor = OldColor;'#13
				+ '            El(''IT'' + (OldSelIndex)).style.color = OldFontColor;'#13
				+ '          }'#13
				+ '          else {'#13
				+ '            if (El(''ITEM'' + (OldSelIndex)) != null) {'#13
				+ '               El(''ITEM'' + (OldSelIndex)).bgColor = OldColor;'#13
				+ '               El(''ITEM'' + (OldSelIndex)).style.color = OldFontColor;'#13
				+ '            }'#13
				+ '          }'#13
				+ '        }'#13
				+ '      }'#13
				+ '      OldSelIndex = index;'#13
				+ '      OldSelColor = selColor;'#13
				+ '      OldSelFontColor = selFontColor;'#13
				+ '      if (OldSelColor != ''null'') {'#13
				+ '        if (El(''IT'' + (OldSelIndex)) != null) {'#13
				+ '          El(''IT'' + (OldSelIndex)).bgColor = OldSelColor;'#13
				+ '          El(''IT'' + (OldSelIndex)).style.color = OldSelFontColor;'#13
				+ '        }'#13
				+ '        else {'#13
				+ '          if (El(''ITEM'' + (OldSelIndex)) != null) {'#13
				+ '            El(''ITEM'' + (OldSelIndex)).bgColor = OldSelColor;'#13
				+ '            El(''ITEM'' + (OldSelIndex)).style.color = OldSelFontColor;'#13
				+ '          }'#13
				+ '        }'#13
				+ '      }'#13
				+ '    }'#13
				+ '    if (OldSelIndex == null) {'#13
				+ '      OldSelIndex = index;'#13
				+ '      OldSelColor = selColor;'#13
				+ '      OldSelFontColor = selFontColor;'#13
				+ '      if (OldSelColor != ''null'') {'#13
				+ '        if (El(''IT'' + (OldSelIndex)) != null) {'#13
				+ '          El(''IT'' + (OldSelIndex)).bgColor = OldSelColor;'#13
				+ '          El(''IT'' + (OldSelIndex)).style.color = OldSelFontColor;'#13
				+ '        }'#13
				+ '        else {'#13
				+ '          if (El(''ITEM'' + (OldSelIndex)) != null) {'#13
				+ '            El(''ITEM'' + (OldSelIndex)).bgColor = OldSelColor;'#13
				+ '            El(''ITEM'' + (OldSelIndex)).style.color = OldSelFontColor;'#13
				+ '          }'#13
				+ '        }'#13
				+ '      }'#13
				+ '    }'#13
				+ '    else {'#13
				+ '      if (OldSelIndex == NewSelIndex) {'#13;

			if (EditType = edPopup) then
			begin
				s := s
          +  ClientEvents.EditStart.Text
					+ '              var itObj = El(''ITEM'' + (index));'#13
					+ '              var itemLeft = 0;'#13
					+ '              var el = itObj;'#13
					+ '              do'#13
					+ '                itemLeft += el.offsetLeft;'#13
					+ '              while ((el = el.offsetParent));'#13
					+ '              var itemTop = 0;'#13
					+ '              var el = itObj;'#13
					+ '              do'#13
					+ '                itemTop += el.offsetTop;'#13
					+ '              while ((el = el.offsetParent));'#13;
          //Modified - Support use of templates
          if (InTemplate) then
            s := s
	  				+ '              El(''POPUP_ITEM_'' + (index)).style.left = (itemLeft + 40) + "px";'#13
		  			+ '              El(''POPUP_ITEM_'' + (index)).style.top = (itemTop + 40) + "px";'#13
          else
            s := s
   	  				+ '              El(''POPUP_ITEM_'' + (index)).style.left = (itemLeft + 40 - ' + IntToStr(Left) + ') + "px";'#13
	  	  			+ '              El(''POPUP_ITEM_'' + (index)).style.top = (itemTop + 40 - ' + IntToStr(Top) + ') + "px";'#13;
          //End Modified
          //Modified - Support for < > and & character in notes and caption of popup window
          (*
          + '              var oldCaption = El(''DIVITEM'' + (index));'#13
				  + '              El(''TF_'' + (index)).value = oldCaption.innerHTML;'#13
          //+ '              El(''TF_'' + (index)).value = CaptionArray[(index)];'#13
          *)
        s := s
          + '              var oldCaption = "";'#13
          + '              if (El(''DIVITEM'' + (index)) != null)'#13
          + '                oldCaption = El(''DIVITEM'' + (index));'#13
//          + '              alert(oldCaption.innerHTML);'#13
          + '              if (oldCaption != "") {'#13
          + '                var subjectText = oldCaption.innerHTML;'#13
          + '                subjectText = subjectText.replace(/&amp;/g,"&");'#13
          + '                subjectText = subjectText.replace(/&lt;/g,"<");'#13
          + '                subjectText = subjectText.replace(/&gt;/g,">");'#13
          + '                El(''TF_'' + (index)).value = subjectText;'#13
          + '              }'#13
          + '              else '#13
          + '                El(''TF_'' + (index)).value = "";'#13
					+ '              var text = El(''NOTES_'' + (index)).innerHTML;'#13
          + '              text = text.replace(/&amp;/g,"&");'#13
          + '              text = text.replace(/&lt;/g,"<");'#13
          + '              text = text.replace(/&gt;/g,">");'#13
          //Modified - Support other browsers
      		+ '              if (! isIE) {'#13
      		+ '                text = text.replace(/<BR>/g,"");'#13
     	    + '                text = text.replace(/<br>/g,"");'#13
     	    + '              }'#13
    	    + '              else {'#13
          + '                text = text.replace(/<BR>/g,''\r'');'#13
          + '                text = text.replace(/<br>/g,''\r'');'#13
          + '              }'#13
          //End Modified
					+ '              El(''TA_'' + (index)).value = text;'#13
					+ '              if (El(''TA_'' + (index)).value == "undefined")'#13
					+ '                El(''TA_'' + (index)).value = "";'#13
					+ '              El(''POPUP_ITEM_'' + (index)).style.visibility = "visible";'#13
          + '              El(''TF_'' + (index)).focus();'#13
          //Modified 10/29/2003 Optimalization Popup Editing
          + ' inEditing = index;'#13
          //End Modified
			end
			else
			begin

        if Assigned(FOnAsyncEditStart) then
        begin
          s := s + 'processAjaxEvent(''onEditStart'', '
            + HTMLControlImplementation.IWCLName
            + ',''' + HTMLName + '.' + 'DoOnAsyncEditStart' + ''','
            + 'true' + ', null, '
            + 'true' + ');';

          Acontext.WebApplication.RegisterCallBack(HTMLName+'.DoOnAsyncEditStart', DoOnAsyncEditStart);
        end
        else
        begin
          s := s
            + '                return SubmitClick('#39 + HTMLName + #39','#39 + 'Select'' + (index),false)'#13
        end;
			end;

			s := s
				+ '            }'#13
				+ '            else {'#13
				+ '                if (NewSelColor != ''null'') {'#13
				+ '                    if (El(''IT'' + (NewSelIndex)) != null) {'#13
				+ '                        El(''IT'' + (NewSelIndex)).bgColor = NewSelColor;'#13
				+ '                        El(''IT'' + (NewSelIndex)).style.color = NewSelFontColor;'#13
				+ '                    }'#13
				+ '                    else {'#13
				+ '                        if (El(''ITEM'' + (NewSelIndex)) != null) {'#13
				+ '                            El(''ITEM'' + (NewSelIndex)).bgColor = NewSelColor;'#13
				+ '                            El(''ITEM'' + (NewSelIndex)).style.color = NewSelFontColor;'#13
				+ '                        }'#13
				+ '                    }'#13
				+ '                }'#13
				+ '                if (OldColor != ''null'') {'#13
				+ '                    if (El(''IT'' + (OldSelIndex)) != null) {'#13
				+ '                        El(''IT'' + (OldSelIndex)).bgColor = OldColor;'#13
				+ '                        El(''IT'' + (OldSelIndex)).style.color = OldFontColor;'#13
				+ '                    }'#13
				+ '                    else {'#13
				+ '                        if (El(''ITEM'' + (OldSelIndex)) != null) {'#13
				+ '                            El(''ITEM'' + (OldSelIndex)).bgColor = OldColor;'#13
				+ '                            El(''ITEM'' + (OldSelIndex)).style.color = OldFontColor;'#13
				+ '                        }'#13
				+ '                    }'#13
				+ '                }'#13
				+ '            }'#13
				+ '        }'#13
				+ '        OldColor = BackColor;'#13
				+ '        OldFontColor = FontColor;'#13
				+ '        OldSelIndex = NewSelIndex;'#13
				+ '        OldSelFontColor = NewSelFontColor;'#13
				+ '        OldSelColor = NewSelColor;'#13;

        if Assigned(FOnAsyncItemClick) then
        begin
          s := s + 'processAjaxEvent(''onItemClick'', '
            + HTMLControlImplementation.IWCLName
            + ',''' + HTMLName + '.' + 'DoOnAsyncItemClick' + ''','
            + 'true' + ', null, '
            + 'true' + ');';

          Acontext.WebApplication.RegisterCallBack(HTMLName+'.DoOnAsyncItemClick', DoOnAsyncItemClick);
        end;

        s := s
				+ '    }'#13
				+ '}'#13;

			{Function askCaption}
			s := s
				+ 'function askCaption(id,oldCaption) {'#13
        //Modified - Support '<,>,&' in caption and notes of item
        + '  var subjectText = oldCaption;'#13
        + '  subjectText = subjectText.replace(/&amp;/g,"&");'#13
        + '  subjectText = subjectText.replace(/&lt;/g,"<");'#13
        + '  subjectText = subjectText.replace(/&gt;/g,">");'#13
        + '  var captionText = window.prompt("' + ChangeCaption.TextPromptWindow + '",subjectText);'#13
				//+ '  var captionText = window.prompt("' + ChangeCaption.TextPromptWindow + '",oldCaption);'#13
        //End Modified
				+ '  var send=true;'#13
				+ '  if (captionText == null)'#13
				+ '    send = false;'#13
				+ '  if (captionText == ("undefined"))'#13
				+ '    send = false;'#13
				+ '  if (captionText == "")'#13
				+ '    send = false;'#13
				+ '  if (captionText == " ")'#13
				+ '    send = false;'#13
				+ '  if (send) {'#13
				+ '    var valStr = "ChangeCaption" + "#" + (id);'#13
        + '    submval = FindElem('#39+HTMLName+#39').value;'#13
        + '    s_ar = submval.split("|");'#13
        + '    s_ar[5] = captionText;'#13
        + '    FindElem('#39+HTMLName+#39').value = s_ar.join("|");'#13
        + '    return SubmitClick('#39 + HTMLName + #39',valStr,false)'#13
				+ '  }'#13
				+ '}'#13;

        //Modified 10/07/2003
        if (EditType = edPopup) then
        begin
          s := s
            + 'function CancelPopupEdit(itemIndex) {'#13
    				+ '  var item = El(''POPUP_ITEM_'' + (itemIndex));'#13
		    		+ '  item.style.visibility = "hidden";'#13
            //Modified 10/29/2003 - Optimalization Popup Editing
            + '  inEditing = null;'#13
            //End Modified
    				+ '}'#13

		    		+ 'function ConfirmPopupEdit(itemIndex) {'#13
            //Modified 10/29/2003 - Optimalization Popup Editing
            + '  inEditing = null;'#13
            //End Modified
    				+ '  var itemPopupDiv = El(''POPUP_ITEM_'' + (itemIndex));'#13
		    		+ '  itemPopupDiv.style.visibility = "hidden";'#13
            +    Clientevents.EditDone.Text + #13
    				+ '  temp = El(''TA_'' + itemIndex);'#13
            + '  submval = FindElem('#39+HTMLName+#39').value;'#13
            + '  s_ar = submval.split("|");'#13
            + '  s_ar[4] = temp.value;'#13
            + '  FindElem('#39+HTMLName+#39').value = s_ar.join("|");'#13
				    + '  temp = El(''TF_'' + itemIndex);'#13
            + '  submval = FindElem('#39+HTMLName+#39').value;'#13
            + '  s_ar = submval.split("|");'#13
            + '  s_ar[5] = temp.value;'#13
            + '  FindElem('#39+HTMLName+#39').value = s_ar.join("|");'#13
		    		+ '  var valStr = "PopupEd" + (itemIndex);'#13
            + '  return SubmitClick('#39 + HTMLName + #39',valStr,false)'#13
		    		+ '}'#13;
      end;

      //Modified - Support other browsers
      if not IsBrowserIE(AContext) then
      begin
        s := s
          + '  function context(e){'#13
          + '    var windowX = e.pageX;'#13
       		+ '    var windowY = e.pageY;'#13
//          + '    alert(leftEndRowEndCol + ";" + topEndRowEndCol);'#13
//          + '    alert("Coords : ("  + windowX + ";" + windowY + ")    Begin : (" + PlannerLeft + ";" + PlannerTop + ")  End : (" + leftEndRowEndCol + ";" + topEndRowEndCol + ")");'#13
        	+ '    if ((windowX > leftEndRowEndCol) | (windowX < ' + IntToStr(Left) + ') | (windowY > topEndRowEndCol)  | (windowY < ' + IntToStr(Top) + '))'#13
          + '      return true;'#13
		      + '    else'#13
   		    + '      return false;'#13
		      + '  }'#13

   		    + '  function DisableRightM() {'#13
//          + '    alert("disableRightM");'#13
		      + '    getCoordsEndRowEndColumn();'#13
		      + '    document.oncontextmenu = context;'#13
 		      + ' }'#13;
      end;
      //End Modified


      s := s
        + '</SCRIPT>';


      {$IFDEF FREEWARE}
      s := s + '<script>document.title="TMS IntraWeb Planner demo"</script>';
      {$ENDIF}

    Result := s;
  end;
begin
  FHiddenPopups := '';
  htmlres := MakeScript;
  {$IFDEF TMSDEBUG}
  outputdebugstring(pchar('script length:'+inttostr(length(htmlres))));
  {$ENDIF}
  Items.SetConflicts;
  Display.InitPrecis;

  FIsEditing := False;
  if FEditIndex > -1 then
   FIsEditing := True;

  if Sidebar.Orientation = soVertical then
    htmlres := htmlres + RenderToHTMLCol(AContext)
  else
    htmlres := htmlres + RenderToHTMLRow(AContext);

  // found an item that is in editing mode, put focus on item
  if FIsEditing then
  begin
    TIWComponent40Context(AContext).AddToInitProc('focusel = document.getElementById("EDITITEM'+IntToStr(FEditIndex)+'");if (focusel) { edititemfocus = true; focusel.focus(); }'+#13#10);
  end;

  TIWComponent40Context(AContext).AddToInitProc('FindElem(''' + HTMLName + ''').value="' + IntToStr(FSelectedIndex) + '| | | | |";' + #13#10);
  TIWComponent40Context(AContext).AddToInitProc('SelectCell(0,0);' + #13#10);

  //Modified - Support other browsers
  TIWComponent40Context(AContext).AddToInitProc('getCoordsR0C0();' + #13#10);

  if not IsBrowserIE(AContext) then
  begin
    TIWComponent40Context(AContext).AddToInitProc('document.captureEvents(Event.KEYDOWN);' + #13#10);
    TIWComponent40Context(AContext).AddToInitProc('DisableRightM();' + #13#10);
  end;

  Result := TIWHTMLTag.CreateTag('DIV');
  {$IFDEF TMSDEBUG}
  outputdebugstring(pchar('rendered size:'+inttostr(length(htmlres))));
  {$ENDIF}
  Result.Contents.AddText(htmlres);

  with Result.Contents.AddTag('INPUT') do
  begin
    AddStringParam('TYPE','hidden');
    AddStringParam('NAME',HTMLName);
    AddStringParam('ID',HTMLName+'_INPUT');
  end;
end;

procedure TTIWCustomWebPlanner.SetHeader(const Value: TPlannerHeader);
begin
  FPlannerHeader := Value;
end;

procedure TTIWCustomWebPlanner.SetPositions(const Value: Integer);
begin
  if Value <= 0 then
    ShowMessage('Number of positions needs to be 1 or higher')
  else
    FPositions := Value;
end;

procedure TTIWCustomWebPlanner.SetRelativeItemMove(const Value: Boolean);
begin
  FRelativeItemMove := Value;
end;

procedure TTIWCustomWebPlanner.SetValue(const Value: string);
var
  Temp, strSelection : string;
  SelIndex, RowBegin, RowEnd, Column: integer;
begin
  inherited;

  if (pos('|',Value) <> 0) then
    Temp := Copy(Value,0,pos('|',Value) - 1)
  else
    Temp := Value;

  SelIndex := StrToInt(Temp);

  if (FSelectedIndex <> SelIndex) and (SelIndex > -1) and (SelIndex <= Items.Count) then
  begin
    FSelectedIndex := SelIndex;
    Items.Selected := Items[SelIndex];
  end;

  Temp := Value;

  if (pos('|',Temp) <> 0) then
    Delete(Temp,1,pos('|',Temp));
  FTempValue := Temp;

  strSelection := FTempValue;
  RowBegin := StrToInt(Copy(strSelection,2,pos('|',strSelection) - 2));
  Delete(strSelection,1,pos('|',strSelection));
  RowEnd := StrToInt(Copy(strSelection,2,pos('|',strSelection) - 2));
  Delete(strSelection,1,pos('|',strSelection));
  Column := StrToInt(Copy(strSelection,2,pos('|',strSelection) - 2));

  SelItemBegin := RowBegin;
  SelItemEnd := RowEnd + 1;
  SelPosition := Column;
end;

function TTIWCustomWebPlanner.ColorToStyle(AColor1, AColor2: TIWColor; ADirection: TIWGradientDirection; AContext : TIWBaseComponentContext): string;
var
  GradType : string;
  GradDir : string;
  GradPos : string;
begin
//  if not IsBrowserIE(AContext) or (AColor2 = clNone) or (AColor1 = clNone) then
  if (AColor2 = clNone) or (AColor1 = clNone) then
    Result := ' background-Color:'+HTMLClr(AColor1)+'; '
  else
  begin
    if (ADirection = gdHorizontal) then
    begin
      GradType := '1';
      GradDir := 'left';
      GradPos := 'left top, right top';
    end
    else
    begin
      GradType := '0';
      GradDir := 'top';
      GradPos := 'left top, left bottom';
    end;

    //IE
    Result := ' FILTER: progid:DXImageTransform.Microsoft.Gradient(GradientType=' + GradType +', StartColorStr='#39+HTMLClr(AColor1)+#39',EndColorStr='#39+HTMLClr(AColor2)+#39');';

    //Chrome / Safari
    Result := Result + 'background-image: -webkit-gradient(linear, ' + GradPos + ', from(' + ColorToRGBString(AColor1) + '), to(' + ColorToRGBString(AColor2) + '));';

    //IE10
    if IsBrowserIE(AContext) then
      Result := Result + 'background-image: -ms-linear-gradient(' + GradDir + ', ' + ColorToRGBString(AColor1) + ', ' + ColorToRGBString(AColor2) + ');';

    //FireFox
    Result := Result + 'background-image: -moz-linear-gradient(' + GradDir + ', ' + ColorToRGBString(AColor1) + ', ' + ColorToRGBString(AColor2) + ');';
  end;
end;

function TTIWCustomWebPlanner.HTMLClr(AColor: TIWColor): string;
begin
  Result := ColorToRGBString(AColor);
end;

function TTIWCustomWebPlanner.HTMLAlign(AAlignment: TAlignment): string;
begin
  Result := '';
  case AAlignment of
  taRightJustify: Result := ' text-align:right;';
  taCenter: Result := ' text-align:center;';
  end;
end;

procedure TTIWCustomWebPlanner.IWPaint;
var
  R: TRect;
  DisplayHeight, CaptionHeight, SideBarWidth,i : integer;
  str : string;
  SBWidth : integer;

begin
  R := GetClientRect;

  DisplayHeight := ((Display.DisplayEnd - Display.DisplayStart + 1) * Display.DisplayScale);
  CaptionHeight := 0;

  if FPlannerHeader.Visible then
  begin
    if PositionGroup > 0 then
      CaptionHeight := 2 * FPlannerHeader.CaptionHeight
    else
      CaptionHeight := FPlannerHeader.CaptionHeight;
  end;

  if FSideBar.Position <> spTop then
  begin
    Canvas.Brush.Color := clWhite;
    Canvas.Rectangle(0,0,Width,DisplayHeight + CaptionHeight);
    Canvas.Font.Name := 'Verdana';
    Canvas.Font.size := 10;
    Canvas.TextOut(Width div 2 - (Canvas.TextWidth('TMS software Intraweb ' + ClassName + ' ' + GetVersionString)) div 2 , DisplayHeight + CaptionHeight - 20,'TMS software IntraWeb '+ ClassName + ' ' + GetVersionString);

    if FPlannerHeader.Visible then
    begin
      Canvas.Brush.Color := FPlannerHeader.Background;
      Canvas.Font.Size := FPlannerHeader.Font.Size;
      Canvas.Font.Name := FPlannerHeader.Font.FontName;
      if PositionGroup > 0 then
      begin
        str := 'G R O U P C A P T I O N S';
        Canvas.Brush.Color := FPlannerHeader.Background;
        Canvas.Rectangle(0,0,Width,FPlannerHeader.CaptionHeight);
        Canvas.TextOut(Width div 2 - Canvas.TextWidth(str) div 2,FPlannerHeader.CaptionHeight div 2 - Canvas.TextHeight(str) div 2,str);
        Canvas.Rectangle(0,FPlannerHeader.CaptionHeight,Width,FPlannerHeader.CaptionHeight * 2);
        str := 'C A P T I O N S';
        Canvas.TextOut(Width div 2 - Canvas.TextWidth(str) div 2,FPlannerHeader.CaptionHeight + FPlannerHeader.CaptionHeight div 2 - Canvas.TextHeight(str) div 2 ,str);
      end
      else
      begin
        str := 'C A P T I O N S';
        Canvas.Rectangle(0,0,Width,FPlannerHeader.CaptionHeight);
        Canvas.TextOut(Width div 2 - Canvas.TextWidth(str) div 2, FPlannerHeader.CaptionHeight div 2 - Canvas.TextHeight(str) div 2 ,str);
      end;

      if FSideBar.Visible then
      begin
        Canvas.Brush.Color := FSideBar.FBackground;
        str := 'SIDEBAR';
        if (FSideBar.Position = spLeft) or (FSideBar.Position = spLeftRight) then
        begin
          Canvas.Rectangle(0,CaptionHeight,FSideBar.Width,DisplayHeight + CaptionHeight);
          for i := 0 to length(str) do
            Canvas.TextOut((FSideBar.Width - Canvas.TextWidth(str[i + 1])) div 2,(DisplayHeight) div 2 - (Canvas.TextWidth(str)) + ((i + 1) * 20),Copy(str,i + 1,1));
        end;
        if (FSideBar.Position = spLeftRight) or (FSideBar.Position = spRight) then
        begin
          Canvas.Rectangle(Width - FSideBar.Width, CaptionHeight,Width,DisplayHeight + CaptionHeight);
          for i := 0 to length(str) do
            Canvas.TextOut(Width - FSideBar.Width + (FSideBar.Width - Canvas.TextWidth(str[i + 1])) div 2, (DisplayHeight) div 2 - (Canvas.TextWidth(str)) + ((i + 1) * 20),Copy(str,i + 1,1));
        end;
      end;
    end;
  end
  else
  begin
    Canvas.Brush.Color := clWhite;
    Canvas.Rectangle(0,0,CaptionHeight + ((Display.DisplayEnd - Display.DisplayStart) * Display.DisplayScale) + Header.HeaderItemWidth,Height);
    Canvas.Font.Name := 'Verdana';
    Canvas.TextOut(CaptionHeight + ((Display.DisplayEnd - Display.DisplayStart) * Display.DisplayScale) div 2 - (Canvas.TextWidth('TMS software Intraweb ' + ClassName + ' ' + GetVersionString)) div 2 ,Height - 20,'TMS software IntraWeb '+ ClassName + ' ' + GetVersionString);

    SideBarWidth := 0;
    if FSideBar.Visible then
    begin
      Canvas.Brush.Color := FSideBar.Background;
      Canvas.Rectangle(0,0,CaptionHeight + Header.HeaderItemWidth + ((Display.DisplayEnd - Display.DisplayStart + 1) * Display.DisplayScale),FSideBar.Width);
      Canvas.TextOut(CaptionHeight + ((Display.DisplayEnd - Display.DisplayStart) * Display.DisplayScale) div 2 - Canvas.TextWidth('S I D E B A R'),FSideBar.Width div 2,'S I D E B A R');
      SideBarWidth := FSideBar.Width;
    end;

    if FPlannerHeader.Visible then
    begin
      Canvas.Brush.Color := FPlannerHeader.Background;
      if PositionGroup > 0 then
      begin
        Canvas.Rectangle(0,SideBarWidth,FPlannerHeader.CaptionHeight,Height);
        str := 'GROUPCAPTIONS';
        for i := 0 to length(str) do
          Canvas.TextOut(FPlannerHeader.CaptionHeight div 2,(Height - SideBarWidth) div 2 - (Canvas.TextWidth(str)) + ((i + 1) * 20),Copy(str,i + 1,1));

        Canvas.Rectangle(FPlannerHeader.CaptionHeight,FSideBar.Width,FPlannerHeader.CaptionHeight * 2,Height);
        str := 'CAPTIONS';
        for i := 0 to length(str) do
          Canvas.TextOut(FPlannerHeader.CaptionHeight + FPlannerHeader.CaptionHeight div 2,(Height - SideBarWidth) div 2 - (Canvas.TextWidth(str)) + ((i + 1) * 20),Copy(str,i + 1,1));
      end
      else
      begin
        Canvas.Rectangle(0,SideBarWidth,FPlannerHeader.CaptionHeight,Height);

        str := 'CAPTIONS';
        for i := 0 to length(str) do
          Canvas.TextOut(FPlannerHeader.CaptionHeight div 2,(Height - SideBarWidth) div 2 - (Canvas.TextWidth(str)) + ((i + 1) * 20),Copy(str,i + 1,1));
      end;
    end;

    Canvas.Brush.Color := clWhite;
    if (SideBar.Visible) then
      SBWidth := SideBar.Width
    else
      SBWidth := 0;

    Canvas.Rectangle(CaptionHeight,SBWidth, CaptionHeight + Header.FHeaderItemWidth,Height);
    str := 'HEADERITEMS';
    for i := 0 to length(str) do
      Canvas.TextOut(CaptionHeight + (FPlannerHeader.HeaderItemWidth div 2) - Canvas.TextWidth('H') div 2 ,(Height - SBWidth) div 2 - (Canvas.TextWidth(str)) + ((i + 1) * 20),Copy(str,i + 1,1));

    Canvas.Rectangle(CaptionHeight + Header.HeaderItemWidth,SBWidth,CaptionHeight + Header.HeaderItemWidth + (Display.DisplayEnd - Display.DisplayStart + 1) * Display.DisplayScale,Height);
    Canvas.TextOut(CaptionHeight + Header.HeaderItemWidth
     + ((Display.DisplayEnd - Display.DisplayStart + 1) * Display.DisplayScale) div 2 - Canvas.TextWidth('PLANNERITEMS') div 2, (Height - SBWidth) div 2, 'PLANNERITEMS');
  end;
end;

function TTIWCustomWebPlanner.GetVersionNr: Integer;
begin
  Result := MakeLong(MakeWord(BLD_VER,REL_VER),MakeWord(MIN_VER,MAJ_VER));
end;

function TTIWCustomWebPlanner.GetVersionString: string;
var
  vn: Integer;
begin
  vn := GetVersionNr;
  Result := IntToStr(Hi(Hiword(vn)))+'.'+IntToStr(Lo(Hiword(vn)))+'.'+IntToStr(Hi(Loword(vn)))+'.'+IntToStr(Lo(Loword(vn)))+' '+DATE_VER;
end;

procedure TTIWCustomWebPlanner.SetInActiveDays(const Value: TWeekDays);
begin
  FInActiveDays := Value;
end;

procedure TTIWCustomWebPlanner.InActiveChanged(Sender: TObject);
begin
  FInactive := [];
  if FInactiveDays.FSun then
    FInactive := FInactive + [1];
  if FInactiveDays.FMon then
    FInactive := FInactive + [2];
  if FInactiveDays.FTue then
    FInactive := FInactive + [3];
  if FInactiveDays.FWed then
    FInactive := FInactive + [4];
  if FInactiveDays.FThu then
    FInactive := FInactive + [5];
  if FInactiveDays.FFri then
    FInactive := FInactive + [6];
  if FInactiveDays.FSat then
    FInactive := FInactive + [7];
  InvalidateX;
end;

procedure TTIWCustomWebPlanner.Loaded;
begin
  inherited;
end;

procedure TTIWCustomWebPlanner.ItemEdited(Item: TPlannerItem);
begin
end;

procedure TTIWCustomWebPlanner.ItemMoved(FromBegin, FromEnd, FromPos,
  ToBegin, ToEnd, ToPos: Integer);
begin
end;

procedure TTIWCustomWebPlanner.ItemSelected(Item: TPlannerItem);
begin
end;

procedure TTIWCustomWebPlanner.ItemSized(FromBegin, FromEnd, ToBegin,
  ToEnd: Integer);
begin
end;

procedure TTIWCustomWebPlanner.InvalidateX;
begin
  if (csDesigning in ComponentState) then
    Invalidate;
end;

procedure TTIWCustomWebPlanner.SaveToFile(FileName: string);
var
  FileStream: TFileStream;
begin
  FileStream := TFileStream.Create(FileName, fmCreate);
  try
    SaveToStream(FileStream);
  finally
    FileStream.Free;
  end;
end;

procedure TTIWCustomWebPlanner.LoadFromFile(FileName: string);
var
  FileStream: TFileStream;
begin
  FileStream := TFileStream.Create(FileName, fmOpenRead);
  try
    LoadFromStream(FileStream);
  finally
    FileStream.Free;
  end;
end;

procedure TTIWCustomWebPlanner.InsertFromFile(FileName: string);
var
  FileStream: TFileStream;
begin
  FileStream := TFileStream.Create(FileName, fmOpenRead);
  try
    InsertFromStream(FileStream);
  finally
    FileStream.Free;
  end;
end;

procedure TTIWCustomWebPlanner.SavePositionToStream(Stream: TStream; Position: Integer);
var
  NewItems: TPlannerItems;
  ItemIndex: Integer;
  PlannerIO: TPlannerIO;

begin
  Items.SetTimeTags;

  NewItems := CreateItems;
  try
    for ItemIndex := 0 to Items.Count - 1 do
    begin
      if Items.Items[ItemIndex].ItemPos = Position then
      begin
        NewItems.Add.Assign(Items.Items[ItemIndex]);
      end;
    end;

    PlannerIO := TPlannerIO.Create(Self);
    try
      PlannerIO.Items.Assign(NewItems);
      Stream.WriteComponent(PlannerIO);
    finally
      PlannerIO.Free;
    end;
  finally
    NewItems.Free;
  end;
end;

procedure TTIWCustomWebPlanner.LoadPositionFromStream(Stream: TStream; Position: Integer);
var
  NewItems: TPlannerItems;
  PlannerIO: TPlannerIO;
  ItemIndex: Integer;
  APlannerItem: TPlannerItem;

begin
  PlannerIO := TPlannerIO.Create(Self);
  try
    Stream.ReadComponent(PlannerIO);
    NewItems := CreateItems;
    try
      NewItems.Assign(PlannerIO.Items);
      Items.BeginUpdate;
      for ItemIndex := 0 to NewItems.Count - 1 do
      begin
        NewItems.Items[ItemIndex].ItemPos := Position;
        APlannerItem := Items.Add;
        APlannerItem.Assign(NewItems.Items[ItemIndex]);
        APlannerItem.GetTimeTag;
      end;

    finally
      Items.EndUpdate;
      NewItems.Free;
    end;
  finally
    PlannerIO.Free;
  end;
end;

procedure TTIWCustomWebPlanner.SavePositionToFile(FileName: string; Position: Integer);
var
  FileStream: TFileStream;
begin
  FileStream := TFileStream.Create(FileName, fmCreate);
  try
    SavePositionToStream(FileStream, Position);
  finally
    FileStream.Free;
  end;
end;

procedure TTIWCustomWebPlanner.LoadPositionFromFile(FileName: string; Position: Integer);
var
  FileStream: TFileStream;
begin
  FileStream := TFileStream.Create(FileName, fmOpenRead);
  try
    LoadPositionFromStream(FileStream, Position);
  finally
    FileStream.Free;
  end;
end;

procedure TTIWCustomWebPlanner.SaveLayerToStream(Stream: TStream; Layer: Integer);
var
  NewItems: TPlannerItems;
  ItemIndex: Integer;
  PlannerIO: TPlannerIO;
begin
  Items.SetTimeTags;
  NewItems := CreateItems;
  try
    for ItemIndex := 0 to Items.Count - 1 do
    begin
      if Items.Items[ItemIndex].Layer = Layer then
      begin
        NewItems.Add.Assign(Items.Items[ItemIndex]);
      end;
    end;

    PlannerIO := TPlannerIO.Create(Self);
    try
      PlannerIO.Items.Assign(NewItems);
      Stream.WriteComponent(PlannerIO);
    finally
      PlannerIO.Free;
    end;
  finally
    NewItems.Free;
  end;
end;

procedure TTIWCustomWebPlanner.LoadLayerFromStream(Stream: TStream; Layer: Integer);
var
  NewItems: TPlannerItems;
  PlannerIO: TPlannerIO;
  ItemIndex: Integer;
  APlannerItem: TPlannerItem;
begin
  PlannerIO := TPlannerIO.Create(Self);
  try
    Stream.ReadComponent(PlannerIO);
    NewItems := CreateItems;
    try
      NewItems.Assign(PlannerIO.Items);
      Items.BeginUpdate;
      for ItemIndex := 0 to NewItems.Count - 1 do
      begin
        NewItems.Items[ItemIndex].Layer := Layer;
        APlannerItem := Items.Add;
        APlannerItem.Assign(NewItems.Items[ItemIndex]);
        APlannerItem.GetTimeTag;
      end;
    finally
      Items.EndUpdate;
      NewItems.Free;
    end;
  finally
    PlannerIO.Free;
  end;
end;

procedure TTIWCustomWebPlanner.SaveLayerToFile(FileName: string; Layer: Integer);
var
  FileStream: TFileStream;
begin
  FileStream := TFileStream.Create(FileName, fmCreate);
  try
    SaveLayerToStream(FileStream, Layer);
  finally
    FileStream.Free;
  end;
end;

procedure TTIWCustomWebPlanner.LoadLayerFromFile(FileName: string; Layer: Integer);
var
  FileStream: TFileStream;
begin
  FileStream := TFileStream.Create(FileName, fmOpenRead);
  try
    LoadLayerFromStream(FileStream, Layer);
  finally
    FileStream.Free;
  end;
end;

procedure TTIWCustomWebPlanner.MovePosition(FromPos, ToPos: Integer);
var
  ItemIndex: Integer;
begin
  {Add extra Position}
  Items.BeginUpdate;

  if ToPos > FromPos then
    Inc(ToPos);

  InsertPosition(ToPos);
  if (ToPos < FromPos) then
    Inc(FromPos);
  with Self.Items do
    for ItemIndex := 0 to Count - 1 do
    begin
      if Items[ItemIndex].ItemPos = FromPos then
        Items[ItemIndex].ItemPos := ToPos;
    end;

  Header.Captions.Strings[ToPos + 1] := Header.Captions.Strings[FromPos + 1];

  DeletePosition(FromPos);

  Items.EndUpdate;
end;

procedure TTIWCustomWebPlanner.DeletePosition(Position: Integer);
var
  ItemIndex: Integer;
begin
  Items.BeginUpdate;
  ItemIndex := 0;

  with Self.Items do
    while (ItemIndex < Count) do
    begin
      if Items[ItemIndex].ItemPos > Position then
      begin
        Items[ItemIndex].ItemPos := Items[ItemIndex].ItemPos - 1;
        Inc(ItemIndex);
      end
      else if Items[ItemIndex].ItemPos = Position then
        Items[ItemIndex].Free
      else
        Inc(ItemIndex);

    end;
  Positions := Positions - 1;
  Items.EndUpdate;
  Header.Captions.Delete(Position + 1);
end;

procedure TTIWCustomWebPlanner.InsertPosition(Position: Integer);
var
  ItemIndex: Integer;
begin
  Items.BeginUpdate;
  Positions := Positions + 1;
  with Items do
    for ItemIndex := 0 to Count - 1 do
    begin
      if Items[ItemIndex].ItemPos >= Position then
        Items[ItemIndex].ItemPos := Items[ItemIndex].ItemPos + 1;
    end;
  Items.EndUpdate;
  Header.Captions.Insert(Position + 1, '');
end;

procedure TTIWCustomWebPlanner.SetChangeCaption(const Value : TChangeCaption);
begin
  FChangeCaption.Assign(Value);
end;

procedure TTIWCustomWebPlanner.SaveToStream(Stream: TStream);
var
  PlannerIO: TPlannerIO;
begin
  //save ItemBegin / ItemEnd correct for current display unit
  Items.BeginUpdate;
  Items.SetTimeTags;

  PlannerIO := TPlannerIO.Create(Self);
  try
    PlannerIO.Items.Assign(Self.Items);
    Stream.WriteComponent(PlannerIO);
  finally
    PlannerIO.Free;
  end;
  Items.EndUpdate;
end;

procedure TTIWCustomWebPlanner.LoadFromStream(Stream: TStream);
var
  PlannerIO: TPlannerIO;
begin
  FLoading := True;
  try
    PlannerIO := TPlannerIO.Create(Self);
    try
      Stream.ReadComponent(PlannerIO);
      Items.Assign(PlannerIO.Items);
    finally
      PlannerIO.Free;
    end;
    Items.SetConflicts;
  finally
    FLoading := False;
  end;

  Items.BeginUpdate;
  Items.GetTimeTags;
  Items.OffsetItems(0);
  Items.EndUpdate;
end;

procedure TTIWCustomWebPlanner.InsertFromStream(Stream: TStream);
var
  PlannerIO: TPlannerIO;
  NewItems: TPlannerItems;
  ItemIndex: Integer;
  APlannerItem: TPlannerItem;
begin
  PlannerIO := TPlannerIO.Create(Self);
  try
    Stream.ReadComponent(PlannerIO);

    NewItems := CreateItems;
    try
      NewItems.Assign(PlannerIO.Items);
      Items.BeginUpdate;

      for ItemIndex := 1 to NewItems.Count do
      begin
        APlannerItem := Items.Add;
        APlannerItem.Assign(NewItems.Items[ItemIndex - 1]);
        APlannerItem.GetTimeTag;
      end;
    finally
      NewItems.Free;
      Items.EndUpdate;
    end;
  finally
    PlannerIO.Free;
  end;
end;

(*
function TTIWCustomWebPlanner.GetCacheDir: string;
begin
  {$IFDEF TMSIW51}
  Result := GServerController.CacheURL;
  {$ELSE}
  Result := WebApplication.URLBase + '/cache/';
  {$ENDIF}
end;
*)

procedure TTIWCustomWebPlanner.ItemChanged(Item: TPlannerItem);
begin
  if Assigned(OnItemChange) then
    OnItemChange(Self,Item);
end;

{ TPlannerDisplay }

constructor TPlannerDisplay.Create(AOwner: TTIWCustomWebPlanner);
begin
  inherited Create;
  FOwner := AOwner;
  FDisplayStart := 0;
  FDisplayUnit := 60;
  FDisplayEnd := 23;
  FDisplayScale := 20;
  FOldScale := FDisplayScale;
  FDisplayOffset := 0;
  FActiveStart := 16;
  FActiveEnd := 40;
  InitPrecis;
  FColorActive := clWhite;
  FColorNonActive := clSilver;
  FCurrentPosFrom := -1;
  FCurrentPosTo := -1;
end;

destructor TPlannerDisplay.Destroy;
begin
  inherited;
end;

procedure TPlannerDisplay.SetDisplayUnit(const Value: Integer);
begin
  if csLoading in FOwner.ComponentState then
  begin
    if Value <> 0 then
    begin
      FDisplayUnit := Value;
      UpdatePlanner;
    end;
    Exit;
  end;

  if FOwner.Sidebar.Position = spTop then
  begin
    if Value <> 0 then
      FDisplayUnit := Value
    else
      Exit;

    { Rescale everything }
    FActiveStart := FActiveStartPrecis div FDisplayUnit;
    FActiveEnd := FActiveEndPrecis div FDisplayUnit;
    FDisplayStart := FDisplayStartPrecis div FDisplayUnit;
    FDisplayEnd := ((FDisplayEndPrecis - FDisplayOffset) div FDisplayUnit) - 1;
  end
  else
  begin
    if Value <> 0 then
      FDisplayUnit := Value
    else
      Exit;

    { Rescale everything }
    FActiveStart := FActiveStartPrecis div FDisplayUnit;
    FActiveEnd := FActiveEndPrecis div FDisplayUnit;
    FDisplayStart := FDisplayStartPrecis div FDisplayUnit;
    FDisplayEnd := ((FDisplayEndPrecis - FDisplayOffset) div FDisplayUnit) - 1;
  end;

  FOwner.InvalidateX;
  UpdatePlanner;
end;

procedure TPlannerDisplay.SetDisplayEnd(const Value: Integer);
begin
  FDisplayEnd := Value;
  FDisplayEndPrecis := (FDisplayEnd + 1) * FDisplayUnit + FDisplayOffset;
  UpdatePlanner;
  if ScaleToFit then
    AutoScale;
  FOwner.InvalidateX;
end;

procedure TPlannerDisplay.SetDisplayStart(const Value: Integer);
begin
  FDisplayStart := Value;
  FDisplayStartPrecis := FDisplayStart * FDisplayUnit + FDisplayOffset;
  UpdatePlanner;
  if ScaleToFit then
    AutoScale;
  FOwner.InvalidateX;
end;
procedure TPlannerDisplay.SetDisplayScale(const Value: Integer);
begin
  FDisplayScale := Value;
  UpdatePlanner;
end;

procedure TPlannerDisplay.SetDisplayOffset(const Value: Integer);
begin
  if Value > FDisplayUnit then
    Exit;

  if Value <> FDisplayOffset then
  begin
    FDisplayOffset := Value;
    UpdatePlanner;
  end;
end;

procedure TPlannerDisplay.UpdatePlanner;
var
  ItemIndex: Integer;
begin
  if FUpdateCount > 0 then
    Exit;

  with FOwner do
  begin
    if not (csLoading in ComponentState) then
    begin
      for ItemIndex := 0 to Items.Count - 1 do
        (FOwner.Items.Items[ItemIndex] as TPlannerItem).ReOrganize;
    end;
    Repaint;
  end;
end;

procedure TPlannerDisplay.SetActiveEnd(const Value: Integer);
begin
  FActiveEnd := Value;
  FActiveEndPrecis := (FActiveEnd + FOwner.Display.DisplayStart) *
    FOwner.Display.DisplayUnit + FOwner.Display.DisplayOffset;
  UpdatePlanner;
end;

procedure TPlannerDisplay.SetActiveStart(const Value: Integer);
begin
  FActiveStart := Value;
  FActiveStartPrecis := (FActiveStart + FOwner.Display.DisplayStart) *
    FOwner.Display.DisplayUnit + FOwner.Display.DisplayOffset;
  UpdatePlanner;
end;

procedure TPlannerDisplay.SetColorActive(const Value: TIWColor);
begin
  FColorActive := Value;
//  UpdatePlanner;
end;

procedure TPlannerDisplay.SetColorNonActive(const Value: TIWColor);
begin
  FColorNonActive := Value;
//  UpdatePlanner;
end;

procedure TPlannerDisplay.SetScaleToFit(const Value: Boolean);
begin
  if FScaleToFit <> Value then
  begin
    FScaleToFit := Value;

    if FScaleToFit then
    begin
      FOldScale := DisplayScale;
      AutoScale;
    end
    else
      DisplayScale := FOldScale;
  end;
end;

procedure TPlannerDisplay.AutoScale;
var
  R: TRect;
begin
  R := Rect(0,0,FOwner.Width, FOwner.Height);

  if FOwner.Sidebar.Position = spTop then
    DisplayScale := (R.Right - R.Left - FOwner.SideBar.GetUsedWidth) div (DisplayEnd - DisplayStart + 1)
  else
    DisplayScale := (R.Bottom - R.Top - FOwner.SideBar.GetUsedWidth) div (DisplayEnd - DisplayStart + 1);
end;


procedure TPlannerDisplay.InitPrecis;
begin
  // initialize precise values properly after load
  FDisplayStartPrecis := FDisplayStart * FDisplayUnit + FDisplayOffset;
  FDisplayEndPrecis := (FDisplayEnd + 1) * FDisplayUnit + FDisplayOffset;

  FActiveStartPrecis := (FActiveStart + FDisplayStart) *
    FDisplayUnit + FDisplayOffset;

  FActiveEndPrecis := (FActiveEnd + FDisplayStart) *
    FDisplayUnit + FDisplayOffset;

end;

procedure TPlannerDisplay.BeginUpdate;
begin
  inc(FUpdateCount);
end;

procedure TPlannerDisplay.EndUpdate;
begin
  if FUpdateCount > 0 then
  begin
    dec(FUpdateCount);
    if FUpdateCount = 0 then
      UpdatePlanner;
  end;
end;

procedure TPlannerDisplay.SetDisplayText(const Value: Integer);
begin
  if (FDisplayText <> Value) then
  begin
    FDisplayText := Value;
    UpdatePlanner;
  end;
end;

procedure TPlannerDisplay.Assign(Source: TPersistent);
begin
  if Assigned(Source) then
  begin
    ActiveStart := (Source as TPlannerDisplay).ActiveStart;
    ActiveEnd := (Source as TPlannerDisplay).ActiveEnd;
    CurrentPosFrom := (Source as TPlannerDisplay).CurrentPosFrom;
    CurrentPosTo := (Source as TPlannerDisplay).CurrentPosTo;
    DisplayStart := (Source as TPlannerDisplay).DisplayStart;
    DisplayEnd := (Source as TPlannerDisplay).DisplayEnd;
    DisplayOffset := (Source as TPlannerDisplay).DisplayOffset;
    DisplayScale := (Source as TPlannerDisplay).DisplayScale;
    DisplayUnit := (Source as TPlannerDisplay).DisplayUnit;
    DisplayText := (Source as TPlannerDisplay).DisplayText;
    ColorActive := (Source as TPlannerDisplay).ColorActive;
    ColorNonActive := (Source as TPlannerDisplay).ColorNonActive;
    ScaleToFit := (Source as TPlannerDisplay).ScaleToFit;
  end;
end;

procedure TPlannerDisplay.SetCurrentPosFrom(const Value: Integer);
begin
  if (FCurrentPosFrom <> Value) then
  begin
    FCurrentPosFrom := Value;
    UpdatePlanner;
  end;
end;

procedure TPlannerDisplay.SetCurrentPosTo(const Value: Integer);
begin
  if (FCurrentPosTo <> Value) then
  begin
    FCurrentPosTo := Value;
    UpdatePlanner;
  end;
end;

function TPlannerDisplay.GetDisplay: integer;
begin
  Result := DisplayEnd - DisplayStart + 1;
end;

{ TPlannerItem }

constructor TPlannerItem.Create(Collection: TCollection);
begin
  inherited Create(Collection);
  FAllowDelete := true;
  FPlanner := (Collection as TPlannerItems).FOwner;
  FItemBegin := 5;
  FItemEnd := FItemBegin + 1;
  FItemPos := 0;
//  FShowCaption := true;
  FTrackColor := clBlue;
  FText := TStringList.Create;
  FFixedPos := False;
  FFixedSize := False;
  FReadOnly := False;
  FImageID := -1;
  FVisible := (csDesigning in FPlanner.ComponentState);
  FImageIndexList := TPlannerIntList.Create(Self);
  FImageIndexList.OnChange := ImageChange;
  FName := 'PlannerItem' + IntToStr(Index);
  FObject := nil;
  FPopupMenu := nil;
  FConflicts := 0;
  FConflictPos := 0;
  FIsCurrent := False;

  FColor := clNone;
  FCaptionBkg := clNone;
  FCaptionBkgTo := clNone;
  FCaptionHeight := 20;

  if FPlanner.Sidebar.Orientation = soVertical then
    FAllowOverlap := True
  else
    FAllowOverlap := False;
  FFont := TIWFont.Create;
  FFont.Assign(FPlanner.Font);
  FFont.OnChange := FontChange;
  FText.OnChange := TextChange;
  FCaptionFont := TIWFont.Create;
  FCaptionFont.Assign(FPlanner.Font);
  FCaptionFont.OnChange := FontChange;
  FCaptionAlign := taLeftJustify;
  FSelected := False;
  FSelectColor := clInfoBk;
  FSelectFontColor := clRed;
  FShowSelection := True;
  FTrackColor := clBlue;
  FUniformBkg := false;
  FParentIndex := -1;
  FRelationShip := irParent;
  FPopupEdit := False;
  DefOrganize;

  //Modified - HTMLTemplate
  FTemplate := TStringList.Create;
  //End Modified
end;

destructor TPlannerItem.Destroy;
var
  plItems: TPlannerItems;
  i: Integer;
begin
  //Modified - HTMLTemplate
  FTemplate.Free;
  //End Modified

  plItems := (Collection as TPlannerItems);

  if (RelationShip = irParent) and (ParentIndex <> -1) then
  begin
    i := 0;
    while i < plItems.Count do
      if (plItems.Items[i].ParentIndex = ParentIndex) and (plItems.Items[i].RelationShip = irChild) then
        plItems.Items[i].Free
      else
        inc(i);
  end;

  FImageIndexList.Free;

  if FOwnsItemObject and Assigned(FObject) then
  begin
    FObject.Free;
    FObject := nil;
  end;

  if Self = plItems.Selected then
  begin
    plItems.SelectNext;

    if not Assigned(plItems.Selected) then
    begin
      plItems.Selected := Self;
      plItems.SelectPrev;
    end;
  end;

  FFont.Free;
  FCaptionFont.Free;
  Visible := False;
  FText.Free;
  inherited;

  plItems.SetConflicts;
end;

{$IFNDEF VER90}
function TPlannerItem.GetDisplayName: string;
begin
  Result := FName;
end;
{$ENDIF}

procedure TPlannerItem.SetAlignment(const Value: TAlignment);
begin
  FAlignment := Value;
  Repaint;
end;

{
procedure TPlannerItem.SetShowCaption(const Value: Boolean);
begin
//  FShowCaption := Value;
  if FShowCaption <> Value then
  begin
    FShowCaption := Value;
    Repaint;
    Changed;
  end;
end;
}
procedure TPlannerItem.SetAllowOverlap(const Value: Boolean);
begin
  if FAllowOverlap <> Value then
  begin
    FAllowOverlap := Value;
    Repaint;
    Changed;
  end;
end;

procedure TPlannerItem.SetAllowDelete(const Value : Boolean);
begin
  FAllowDelete := Value;
  Repaint;
  Changed;
end;

//Modified - HTMLTemplate
procedure TPlannerItem.SetTemplate(const Value : TStringList);
begin
  FTemplate.Assign(Value);
  //Changed;
end;
//End Modified

procedure TPlannerItem.SetCaptionType(const Value: TCaptionType);
begin
  if (FCaptionType <> Value) then
  begin
    FCaptionType := Value;
    Repaint;
    Changed;
  end;
end;

procedure TPlannerItem.SetCaptionText(const Value: string);
begin
  FCaptionText := Value;
  Repaint;
  Changed;
end;

procedure TPlannerItem.ImageChange(Sender: TObject);
begin
  Repaint;
  Changed;
end;

procedure TPlannerItem.FontChange(Sender: TObject);
begin
  Repaint;
  Changed;
end;

procedure TPlannerItem.TextChange(Sender: TObject);
begin
  Repaint;
  Changed;
end;

procedure TPlannerItem.SetVisible(const Value: Boolean);
begin
  if FVisible <> Value then
  begin
    FVisible := Value;
    Repaint;
  end;
end;

function TPlannerItem.GetVisible: Boolean;
begin
  Result := FVisible;
end;

procedure TPlannerItem.ScrollInView;
begin
  with (Collection as TPlannerItems).FOwner do
  begin
    if SideBar.Position <> spTop then
    begin
    end
    else
    begin
    end;
  end;
end;

procedure TPlannerItem.PopupEdit;
begin
end;

procedure TPlannerItem.Edit;
begin
end;

procedure TPlannerItem.SetFocus(const Value: Boolean);
begin
//  if (csDestroying in FPlanner.ComponentState) then
//    Exit;
end;

procedure TPlannerItem.SetFont(const Value: TIWFont);
begin
  FFont.Assign(Value);
  Repaint;
  Changed;
end;

procedure TPlannerItem.SetBackground(const Value: Boolean);
begin
  if (FBackGround <> Value) then
  begin
    FBackGround := Value;
    FReadOnly := True;
    FFixedPos := True;
    FFixedSize := True;
    FAllowOverlap := False;
    Repaint;
    Changed;
  end;
end;

procedure TPlannerItem.SetImageID(const Value: Integer);
begin
  FImageID := Value;
  Repaint;
  Changed;
end;

procedure TPlannerItem.SetColor(const Value: TIWColor);
begin
  FColor := Value;
  Repaint;
  Changed;
end;

procedure TPlannerItem.SetTrackColor(const Value: TIWColor);
begin
  FTrackColor := Value;
  Repaint;
  Changed;
end;

procedure TPlannerItem.SetLayer(const Value: Integer);
begin
  FLayer := Value;
  Repaint;
end;

procedure TPlannerItem.SetItemEnd(const Value: Integer);
var
  OldVisible: Boolean;

begin
  OldVisible := Visible;
  Visible := False;
  FItemEnd := Value;

  if not FPlanner.FLoading then
  begin
    FItemEndPrecis := (FItemEnd + EndOffset + FPlanner.Display.DisplayStart) *
      FPlanner.Display.DisplayUnit + FPlanner.Display.DisplayOffset;
  end;

  Visible := OldVisible;
  (Collection as TPlannerItems).SetConflicts;
end;

procedure TPlannerItem.SetItemBegin(const Value: Integer);
begin
  if Assigned(FLinkedItem) and not FPlanner.FLinkUpdate then
  begin
    FPlanner.FLinkUpdate := True;
    FPlanner.Items.ResetUpdate;
  end;

  Visible := False;
  FItemFullBegin := Value;

  if (Value >= 0) then
    FItemBegin := Value
  else
    FItemBegin := 0;

  if not FPlanner.FLoading then
  begin
    FItemBeginPrecis := (FItemBegin - BeginOffset + FPlanner.Display.DisplayStart) *
      FPlanner.Display.DisplayUnit + FPlanner.Display.DisplayOffset; //BeginOffset added
  end;

  Visible := True;
  (Collection as TPlannerItems).SetConflicts;
end;

function TPlannerItem.GetVisibleSpan: Integer;
var
  cb,ce: Integer;
begin
  cb := ItemBegin;
  ce := ItemEnd;
  if FPlanner.SideBar.Position <> spTop then
  begin
    (*
    if cb < FPlanner.FGrid.TopRow then
      cb := FPlanner.FGrid.TopRow;
    if ce > FPlanner.FGrid.VisibleRowCount + FPlanner.FGrid.TopRow then
      ce := FPlanner.FGrid.VisibleRowCount + FPlanner.FGrid.TopRow + 1;
    *)
  end
  else
  begin
    (*
    if cb < FPlanner.FGrid.LeftCol then
      cb := FPlanner.FGrid.LeftCol;
    if ce >= FPlanner.FGrid.VisibleColCount + FPlanner.FGrid.LeftCol then
      ce := FPlanner.FGrid.VisibleColCount + FPlanner.FGrid.LeftCol;
    *)
  end;

  Result := ce - cb;
end;

function TPlannerItem.GetItemRect: TRect;
//var
//  gr1,gr2: TRect;
begin
  if (FPlanner.Sidebar.Orientation = soVertical) then
  begin
    (*
    gr1 := FPlanner.FGrid.CellRectEx(ItemPos + FPlanner.FSideBar.FColOffset, ItemBegin);
    gr2 := FPlanner.FGrid.CellRectEx(ItemPos + FPlanner.FSideBar.FColOffset, ItemEnd - 1);

    CalcConflictRect(gr1, FPlanner.FGrid.ColWidthEx(ItemPos),
      FPlanner.FGrid.RowHeightEx(FPlanner.FSideBar.FRowOffset), True);

    gr1.Bottom := gr2.Bottom;
    Result := gr1;
    *)
  end;
end;

procedure TPlannerItem.DefOrganize;
begin
  FItemBeginPrecis := (FItemBegin + FPlanner.Display.DisplayStart) *
    FPlanner.Display.DisplayUnit + FPlanner.Display.DisplayOffset;

  FItemEndPrecis := (FItemEnd + FPlanner.Display.DisplayStart) *
    FPlanner.Display.DisplayUnit + FPlanner.Display.DisplayOffset;
end;

procedure TPlannerItem.ReOrganize;
begin
  FItemBegin := (FItemBeginPrecis div FPlanner.Display.DisplayUnit) -
    FPlanner.Display.DisplayStart;

  if Frac(FItemEndPrecis / FPlanner.Display.DisplayUnit) > 0 then
    FItemEnd := 1 + (FItemEndPrecis div FPlanner.Display.DisplayUnit) -
      FPlanner.Display.DisplayStart
  else
    FItemEnd := (FItemEndPrecis div FPlanner.Display.DisplayUnit) -
      FPlanner.Display.DisplayStart;

  UpdateWnd;
end;

procedure TPlannerItem.SetText(const Value: TStringList);
begin
  if Assigned(Value) then
    FText.Assign(Value);
  Repaint;
end;

procedure TPlannerItem.UpdateWnd;
begin
  Repaint;
end;

procedure TPlannerItem.Repaint;
begin
  // design time only
end;


procedure TPlannerItem.CalcConflictRect(var Rect: TRect; Width, Height: Integer;
  Position: Boolean);
begin
  if FConflicts > 1 then
  begin
    if Position then
    begin
      Width := Width div FConflicts;
      Rect.Left := Rect.Left + Width * FConflictPos;
      Rect.Right := Rect.Left + Width - FPlanner.ItemGap;
    end
    else
    begin
      Height := Height div FConflicts;
      Rect.Top := Rect.Top + Height * FConflictPos;
      Rect.Bottom := Rect.Top + Height - FPlanner.ItemGap;
    end;
  end
  else
  begin
    if Position then
      Rect.Right := Rect.Right - FPlanner.ItemGap
    else
      Rect.Bottom := Rect.Bottom - FPlanner.ItemGap;
  end;
end;

procedure TPlannerItem.SetItemPos(const Value: Integer);
var
  Difference: Integer;
begin
  if FItemPos <> Value then
  begin
    if Assigned(FLinkedItem) and not FPlanner.FLinkUpdate then
    begin
      FPlanner.FLinkUpdate := True;
      FPlanner.Items.ResetUpdate;
      Difference := Value - FItemPos;
      FLinkedItem.ItemPos := FLinkedItem.ItemPos + Difference;
      FPlanner.FLinkUpdate := False;
    end;

    Visible := False;
    Repaint;

    FItemPos := Value;
    Visible := True;
    FPlanner.Items.SetConflicts;
    Repaint;
  end;
end;

procedure TPlannerItem.Assign(Source: TPersistent);
begin
  if Source is TPlannerItem then
  begin
    ItemPos := TPlannerItem(Source).ItemPos;
    ItemBegin := TPlannerItem(Source).ItemBegin;
    ItemEnd := TPlannerItem(Source).ItemEnd;
    Color := TPlannerItem(Source).Color;
    Text.Text := TPlannerItem(Source).Text.Text;
    ReadOnly := TPlannerItem(Source).ReadOnly;
    FixedSize := TPlannerItem(Source).FixedSize;
    FixedPos := TPlannerItem(Source).FixedPos;
    CaptionType := TPlannerItem(Source).CaptionType;
    ImageID := TPlannerItem(Source).ImageID;
    Alignment := TPlannerItem(Source).Alignment;
    Name := TPlannerItem(Source).Name;
    Visible := TPlannerItem(Source).Visible;
    Tag := TPlannerItem(Source).Tag;
    TrackColor := TPlannerItem(Source).TrackColor;
    CaptionText := TPlannerItem(Source).CaptionText;
    Layer := TPlannerItem(Source).Layer;
    Font.Assign(TPlannerItem(Source).Font);
    Background := TPlannerItem(Source).Background;
    AllowOverlap := TPlannerItem(Source).AllowOverlap;
    AllowDelete := TPlannerItem(Source).AllowDelete;
    Selected := TPlannerItem(Source).Selected;
    SelectColor := TPlannerItem(Source).SelectColor;
    CaptionBkg := TPlannerItem(Source).CaptionBkg;
    CaptionBkgTo := TPlannerItem(Source).CaptionBkgTo;
    CaptionAlign := TPlannerItem(Source).CaptionAlign;
    CaptionFont.Assign(TPlannerItem(Source).CaptionFont);
    CaptionHeight := TPlannerItem(Source).CaptionHeight;
    CaptionType := TPlannerItem(Source).CaptionType;
    InHeader := TPlannerItem(Source).InHeader;
    OwnsItemObject := TPlannerItem(Source).OwnsItemObject;
    Selected := TPlannerItem(Source).Selected;
    SelectColor := TPlannerItem(Source).SelectColor;
    SelectFontColor := TPlannerItem(Source).SelectFontColor;
    PopupMenu := TPlannerItem(Source).PopupMenu;
    ShowSelection := TPlannerItem(Source).ShowSelection;
//    ShowCaption := TPlannerItem(Source).ShowCaption;
    DBTag := TPlannerItem(Source).DBTag;
    Attachement := TPlannerItem(Source).Attachement;
    URL := TPlannerItem(Source).URL;
    UniformBkg := TPlannerItem(Source).UniformBkg;
    Hint := TPlannerItem(Source).Hint;
    CaptionGradientDirection := TPlannerItem(Source).CaptionGradientDirection;
    //Modified - HTMLTemplate
    Template := TPlannerItem(Source).Template;
    //End Modified
  end;
end;

procedure TPlannerItem.AssignEx(Source: TPersistent);
begin
  if Source is TPlannerItem then
  begin
    Color := TPlannerItem(Source).Color;
    Text.Text := TPlannerItem(Source).Text.Text;
    ReadOnly := TPlannerItem(Source).ReadOnly;
    FixedSize := TPlannerItem(Source).FixedSize;
    FixedPos := TPlannerItem(Source).FixedPos;
    CaptionType := TPlannerItem(Source).CaptionType;
    ImageID := TPlannerItem(Source).ImageID;
    Alignment := TPlannerItem(Source).Alignment;
    Name := TPlannerItem(Source).Name;
    Visible := TPlannerItem(Source).Visible;
    Tag := TPlannerItem(Source).Tag;
    TrackColor := TPlannerItem(Source).TrackColor;
    CaptionText := TPlannerItem(Source).CaptionText;
    Layer := TPlannerItem(Source).Layer;
    Font.Assign(TPlannerItem(Source).Font);
    Background := TPlannerItem(Source).Background;
    AllowOverlap := TPlannerItem(Source).AllowOverlap;
    AllowDelete := TPlannerItem(Source).AllowDelete;
    Selected := TPlannerItem(Source).Selected;
    SelectColor := TPlannerItem(Source).SelectColor;
    CaptionBkg := TPlannerItem(Source).CaptionBkg;
    CaptionBkgTo := TPlannerItem(Source).CaptionBkgTo;
    CaptionAlign := TPlannerItem(Source).CaptionAlign;
    CaptionFont.Assign(TPlannerItem(Source).CaptionFont);
    CaptionType := TPlannerItem(Source).CaptionType;
    CaptionGradientDirection := TPlannerItem(Source).CaptionGradientDirection;
    InHeader := TPlannerItem(Source).InHeader;
    OwnsItemObject := TPlannerItem(Source).OwnsItemObject;
    Selected := TPlannerItem(Source).Selected;
    SelectColor := TPlannerItem(Source).SelectColor;
    PopupMenu := TPlannerItem(Source).PopupMenu;
    ShowSelection := TPlannerItem(Source).ShowSelection;
    DBTag := TPlannerItem(Source).DBTag;
    Attachement := TPlannerItem(Source).Attachement;
    URL := TPlannerItem(Source).URL;
    UniformBkg := TPlannerItem(Source).UniformBkg;
  end;
end;

procedure TPlannerItem.SetItemEndTime(const Value: TDateTime);
var
  Hour, Minute, Second, Second100: Word;
  Da, Mo, Ye: Word;
  dte,dts: TDateTime;
  nv, delta, DInc: Integer;

begin
  FItemEndTime := Value;

  with TPlannerItems(Collection).FOwner do
  begin
    Display.InitPrecis; //added

    case Mode.PlannerType of
      plDay:
        begin
          delta := 0;
          BeginOffset := 0;
          EndOffset := 0;

          if Int(Value) > Int(FItemRealStartTime) then
          begin
            InHeader := true;  //added
            delta := 24 * 60;
          end;

          if Int(Value) < Int(FItemRealStartTime) then
          begin
            delta := - 24 * 60;
            InHeader := true; //added
          end;

          DecodeTime(Value, Hour, Minute, Second, Second100);

          Minute := Minute + Hour * 60;
          Minute := Minute + Delta;

          FItemEndPrecis := Minute; // moved after delta

          if FItemEndPrecis < FItemBeginPrecis then
          begin
            FItemEndPrecis := FItemEndPrecis + 60 * 24;
            Minute := Minute + 60 * 24;
          end;

          if (Minute > Display.FDisplayEndPrecis) then
          begin
            EndOffset := (Minute - Display.FDisplayEndPrecis) div Display.DisplayUnit;

            ItemEnd := Display.DisplayEnd - Display.DisplayStart + 1;

            if ((ItemBeginPrecis > Display.FDisplayEndPrecis) or (ItemEndPrecis < Display.FDisplayStartPrecis) or (Minute > MinInDay)) then //added
              InHeader := true;  //added
          end
          else
          begin
            if (ItemBeginPrecis < Display.FDisplayStartPrecis) then
            begin
              if (Minute < Display.FDisplayStartPrecis) then
              begin
                InHeader := true;  //added
                ItemEnd := 0; //added
                //ItemBegin := Display.DisplayStart;  //removed
                //BeginOffset := (Display.FDisplayStartPrecis - Minute) div Display.DisplayUnit;  //changed
                BeginOffset := (Display.FDisplayStartPrecis - ItemBeginPrecis) div Display.DisplayUnit;  //changed
              end
              else
              begin
                BeginOffset := (Display.FDisplayStartPrecis - ItemBeginPrecis) div Display.DisplayUnit;
                ItemEnd := ((Minute - 1) div Display.DisplayUnit) + 1 - Display.DisplayStart;
              end;
            end
            else
              ItemEnd := ((Minute - 1) div Display.DisplayUnit) + 1 - Display.DisplayStart;
          end;

          // removed in 1.6.0.2
          FItemEndPrecis := Minute; // moved after delta

          {$IFDEF TMSDEBUG}
          outputdebugstring(pchar(formatdatetime('hh:nn dd/mm/yyyy',value)));
          {$ENDIF}

          FItemRealEndTime := Value;
        end;

      plTimeLine:
        begin
          DecodeTime(Value, Hour, Minute, Second, Second100);
          Minute := Minute + Hour * 60;
          FItemEndPrecis := Minute;

          DInc := Round(Int(Value) - Int(Mode.TimeLineStart)) * (MININDAY div Display.DisplayUnit);

          Minute := (Minute div Display.DisplayUnit) - Display.DisplayStart;

          if Minute + DInc > Display.FDisplayEnd then
          begin
            EndOffset := Minute + DInc - Display.DisplayEnd - 1;
            ItemEnd := Display.DisplayEnd - Display.DisplayStart + 1;
          end
          else
            ItemEnd := Minute + DInc;

          FItemRealEndTime := Value;
        end;

      plDayPeriod,plMonth,plWeek:
        begin
          dte := FPlanner.Mode.PeriodEndDate;
          dts := FPlanner.Mode.PeriodStartDate;

          FItemRealEndTime := Value;
          EndOffset := 0;

          if Trunc(Value) > dte then
            EndOffset := Trunc(Value - dte);

          nv := Trunc(Value - FPlanner.Mode.PeriodStartDate) + 1;

          if (nv < 0) then // Because of the 0 we can't use the Clip function
            nv := FPlanner.Display.DisplayStart;

          if (nv > FPlanner.Display.DisplayEnd - FPlanner.Display.DisplayStart) then
            nv := FPlanner.Display.DisplayEnd - FPlanner.Display.DisplayStart + 1;

          if Value < dts then
          begin
            Self.Visible := False;
          end
          else
            ItemEnd := nv;

          {$IFDEF TMSCODESITE}
          CodeSite.SendInteger('endtime',ItemEnd);
          CodeSite.SendInteger('set endoffset',EndOffset);
          {$ENDIF}

          {$IFDEF TMSDEBUG}
          outputdebugstring(pchar(formatdatetime('et : hh:nn dd/mm/yyyy',value)));
          {$ENDIF}

          FItemRealEndTime := Value;
        end;
      plHalfDayPeriod:
        begin
          dte := FPlanner.Mode.PeriodEndDate;

          EndOffset := 0;

          {$IFDEF TMSCODESITE}
          codesite.SendFloat('diff:',Value - FPlanner.Mode.PeriodStartDate);
          {$ENDIF}

          nv := Trunc(2 * (Value - FPlanner.Mode.PeriodStartDate));

          if Frac(Value) > 0 then
            inc(nv);

          if nv < 0 then // Because of the 0 we can't use the Clip function
            nv := FPlanner.Display.DisplayStart;

          if nv > FPlanner.Display.DisplayEnd - FPlanner.Display.DisplayStart then
            nv := FPlanner.Display.DisplayEnd - FPlanner.Display.DisplayStart + 1;

          ItemEnd := nv;

          if Value > dte then
            EndOffset := Round(2 * (Value - dte));

          {$IFDEF TMSCODESITE}
          CodeSite.SendInteger('starttime',ItemBegin);
          CodeSite.SendInteger('set beginoffset',BeginOffset);
          {$ENDIF}

          FItemRealEndTime := Value;
        end;
      plMultiMonth:
        begin
          DecodeDate(Value,Ye,Mo,Da);

          while (FPlanner.Mode.Year < Ye) do
          begin
            Mo := Mo + 12;
            dec(Ye);
          end;

          ItemPos := Limit(Mo - FPlanner.Mode.Month, 0, Positions - 1);
          ItemEnd := Da;
        end;
      plCustom, plCustomList:
        begin
          ItemEnd := FPlanner.TimeToIndex(Value);
        end;
    end;
  end;

end;

procedure TPlannerItem.SetItemStartTime(const Value: TDateTime);
var
  Hour, Minute, Second, Second100: Word;
  Da, Mo, Ye: Word;
  dts,dte: TDateTime;
  DInc: integer;

begin
  FItemStartTime := Value;
  with TPlannerItems(Collection).FOwner do
  begin
    case Mode.PlannerType of
      plDay:
        begin
          DecodeTime(Value, Hour, Minute, Second, Second100);
          Minute := Minute + Hour * 60;
          ItemBegin := (Minute div Display.DisplayUnit) - Display.DisplayStart;
          FItemBeginPrecis := Minute;
          FItemRealStartTime := Value;
        end;
      plTimeLine:
        begin
          DecodeTime(Value, Hour, Minute, Second, Second100);
          Minute := Minute + Hour * 60;

          FItemBeginPrecis := Minute;

          DInc := Round(Int(Value) - Int(Mode.TimeLineStart)) * (MININDAY div Display.DisplayUnit);

          {$IFDEF TMSDEBUG}
          outputdebugstring(pchar('day diff: '+inttostr(Dinc)));
          {$ENDIF}

          Minute := (Minute div Display.DisplayUnit) - Display.DisplayStart;

          if Minute + DInc < 0 then
          begin
            ItemBegin := 0;
            BeginOffset := - (Minute + DInc);
          end
          else
            ItemBegin := Minute + DInc;

          FItemRealStartTime := Value;
        end;

      plDayPeriod,plMonth,plWeek:
        begin
          dts := FPlanner.Mode.PeriodStartDate;
          dte := FPlanner.Mode.PeriodEndDate;

          BeginOffset := 0;

          if (Trunc(Value) > dts) and (Trunc(Value) <= dte) then
            ItemBegin := Trunc(Value - dts)
          else
          begin
            BeginOffset := Trunc(dts - Value);

            ItemBegin := 0;
            if Trunc(Value) > dte then
            begin
              Self.Visible := False;
            end;
          end;

          {$IFDEF TMSCODESITE}
          CodeSite.SendInteger('starttime',ItemBegin);
          CodeSite.SendInteger('set beginoffset',BeginOffset);
          {$ENDIF}

          FItemRealStartTime := Value;
        end;
      plHalfDayPeriod:
        begin
          dts := FPlanner.Mode.PeriodStartDate;
          dte := FPlanner.Mode.PeriodEndDate;

          BeginOffset := 0;

          {$IFDEF TMSCODESITE}
          codesite.SendDateTime('dts',dts);
          codesite.SendDateTime('dte',dte);
          {$ENDIF}

          if (Value >= dts) and (Value <= dte) then
            ItemBegin := Trunc(2*(Value - dts))
          else
          begin
            BeginOffset := Trunc(1 + 2*(dts - Value));
            {$IFDEF TMSCODESITE}
            codesite.SendInteger('beginoffset=',beginoffset);
            {$ENDIF}
            ItemBegin := 0;
          end;

          FItemRealStartTime := Value;
        end;
      plMultiMonth:
        begin
          DecodeDate(Value,Ye,Mo,Da);
          while (FPlanner.Mode.Year < Ye) do
          begin
            Mo := Mo + 12;
            dec(Ye);
          end;

          ItemPos := Limit(Mo - FPlanner.Mode.Month, 0, Positions - 1);
          ItemBegin := Da - 1;
        end;
      plCustom, plCustomList:
        begin
          ItemBegin := TimeToIndex(Value);
        end;
    end;
  end;
end;

function TPlannerItem.GetItemEndTime: TDateTime;
var
  res,ID,DInc: Integer;
  da,mo,ye: word;
begin
  with TPlannerItems(Collection).FOwner do
  begin
    Result := 0;
    case Mode.PlannerType of
      plDay:
        begin
          res := FItemEndPrecis;
          ID := 0;
          while (res >= MinInDay) do
          begin
            res := res - MinInDay;
            inc(ID);
          end;
          Result := EncodeTime(res div 60, res mod 60, 0, 0);

          {
          if EndOffset > 0 then
          begin
            res := EndOffset * Display.DisplayUnit;
            Result := Result + EncodeTime(res div 60,res mod 60,0 ,0);
          end;
          }

          Result := Result + PosToDay(ItemPos) + ID;
        end;

      plTimeLine:
        begin
          res := FItemEndPrecis + EndOffset * Display.DisplayUnit;

          while (res >= MININDAY) do
          begin
            res := res - MININDAY;
          end;

          Result := EncodeTime(res div 60, res mod 60, 0, 0);
          DInc := (ItemEnd + EndOffset) div ((MININDAY div Display.DisplayUnit));

          Result := Result + Int(FPlanner.Mode.TimeLineStart + DInc);
        end;

      plMonth,plWeek:
        begin
          Result := Mode.StartOfMonth - 1 + EndOffset + ItemEnd + Frac(FItemRealEndTime);
        end;
      plDayPeriod:
        begin
          Result := FPlanner.Mode.PeriodStartDate -1 + EndOffset + ItemEnd + Frac(FItemRealEndTime);
        end;
      plHalfDayPeriod:
        begin
          Result := FPlanner.Mode.PeriodStartDate + (ItemEnd + EndOffset) / 2;
        end;
      plMultiMonth:
        begin
          ye := FPlanner.Mode.Year;
          mo := FPlanner.Mode.Month + ItemPos;
          da := ItemEnd + 1;

          while (mo > 12) do
          begin
            mo := mo - 12;
            ye := ye + 1;
          end;

          if da > DaysInMonth(mo,ye) then
          begin
            inc(mo);
            if mo > 12 then
            begin
              mo := 1;
              inc(ye);
            end;
            da := 1;
          end;

          Result := EncodeDate(ye,mo,da);
        end;
      plCustom, plCustomList:
        begin
          Result := IndexToTime(ItemEnd);
        end;
    end;
  end;

end;

function TPlannerItem.GetItemStartTime: TDateTime;
var
  res,ID,y,DInc: Integer;
begin
  with TPlannerItems(Collection).FOwner do
  begin
    Result := 0;
    case Mode.PlannerType of
      plDay:
        begin
          res := FItemBeginPrecis;
          ID := 0;

          while (res >= 24 * 60) do
          begin
            res := res - 60 * 24;
            inc(ID);
          end;

          Result := EncodeTime(res div 60, res mod 60, 0, 0);
{
          if BeginOffset > 0 then
          begin
            res := BeginOffset * Display.DisplayUnit;
            Result := Result - EncodeTime(res div 60,res mod 60,0,0);
          end;
}
          Result := Result + PosToDay(ItemPos) + ID;
         end;
      plTimeLine:
        begin
          res := FItemBeginPrecis - BeginOffset * Display.DisplayUnit;

          while (res >= MININDAY) do
            res := res - MININDAY;

          while (res < 0) do
            res := res + MININDAY;

          Result := EncodeTime(res div 60, res mod 60, 0, 0);
          DInc := (ItemBegin) div ((MININDAY div Display.DisplayUnit));
          if BeginOffset > 0 then
            DInc := DInc - 1 - (BeginOffset div (MININDAY div Display.DisplayUnit));

          Result := Result + Int(FPlanner.Mode.TimeLineStart + DInc);
        end;
      plMonth,plWeek:
        begin
          Result := Mode.StartOfMonth - BeginOffset + ItemBegin + Frac(FItemRealStartTime);
        end;
      plDayPeriod:
        begin
          Result := FPlanner.Mode.PeriodStartDate - BeginOffset + ItemBegin + Frac(FItemRealStartTime);
        end;
      plHalfDayPeriod:
        begin
          Result := FPlanner.Mode.PeriodStartDate + (ItemBegin - BeginOffset) / 2;
        end;
      plMultiMonth:
        begin
          res := FPlanner.Mode.Month + ItemPos;
          y := FPlanner.Mode.Year;
          while (res > 12) do
          begin
            res := res - 12;
            inc(y);
          end;
          Result := EncodeDate(y,res,ItemBegin + 1);
        end;
      plCustom, plCustomList:
        begin
          Result := FPlanner.IndexToTime(ItemBegin);
        end;
    end;
    FItemStartTime := Result;
  end;
end;

function TPlannerItem.GetItemSpanTimeStr: string;
begin
  Result := GetCaptionTimeString;
end;

function TPlannerItem.GetItemEndTimeStr: string;
begin
  with TPlannerItems(Collection).FOwner do
  begin
    Result := '';
    case Mode.PlannerType of
      plDay,plTimeLine:
        begin
          Result := PlanTimeToStr(FItemEndPrecis);
        end;
      plMonth,plWeek:
        begin
          Result := DateToStr(Mode.StartOfMonth - 1 + ItemEnd + EndOffset);
        end;
      plDayPeriod:
        begin
          Result := DateToStr(Mode.PeriodStartDate -1 + ItemEnd + EndOffset);
        end;
      plHalfDayPeriod:
        begin
          Result := DateToStr(Mode.PeriodStartDate + (ItemEnd - 1 + EndOffset) / 2);
        end;
      plMultiMonth:
        begin
          if not RealTime then
            Result := DateToStr(GetItemEndTime - 1)
          else
            Result := DateToStr(GetItemRealEndTime);
        end;
      plCustom, plCustomList:
        begin
          Result := FormatDateTime(Mode.DateTimeFormat, ItemEndTime);
        end;
    end;
  end;
end;

function TPlannerItem.GetItemStartTimeStr: string;
begin
  with TPlannerItems(Collection).FOwner do
  begin
    Result := '';
    case Mode.PlannerType of
      plDay,plTimeLine:
        begin
          Result := PlanTimeToStr(FItemBeginPrecis -
            BeginOffset * FPlanner.FDisplay.FDisplayUnit + 60 * 24);
        end;
      plDayPeriod,plMonth,plWeek:
        begin
          Result := DateToStr(Mode.PeriodStartDate + ItemBegin - BeginOffset);
        end;
      plHalfDayPeriod:
        begin
          Result := DateToStr(Mode.PeriodStartDate + (ItemBegin - BeginOffset) / 2);
        end;
      plMultiMonth:
        begin
          if not RealTime then
            Result := DateToStr(GetItemStartTime)
          else
            Result := DateToStr(GetItemRealStartTime);
        end;
      plCustom, plCustomList:
        begin
          Result := FormatDateTime(Mode.DateTimeFormat, ItemStartTime);
        end;
    end;
  end;
end;

procedure TPlannerItem.SetIsCurrent(const Value: Boolean);
begin
  if FIsCurrent <> Value then
  begin
    FIsCurrent := Value;
    Repaint;
  end;
end;

procedure TPlannerItem.SetItemRealEndTime(const Value: TDateTime);
begin
  FItemRealEndTime := Value;
end;

procedure TPlannerItem.SetItemRealStartTime(const Value: TDateTime);
begin
  FItemRealStartTime := Value;
end;

procedure TPlannerItem.SetCaptionAlign(const Value: TAlignment);
begin
  if (FCaptionAlign <> Value) then
  begin
    FCaptionAlign := Value;
    Repaint;
  end;
end;

procedure TPlannerItem.SetCaptionBkg(const Value: TIWColor);
begin
  if (FCaptionBkg <> Value) then
  begin
    FCaptionBkg := Value;
    Repaint;
  end;
end;

procedure TPlannerItem.SetCaptionFont(const Value: TIWFont);
begin
  FCaptionFont.Assign(Value);
  Repaint;
end;

procedure TPlannerItem.SetCaptionGradientDirection(const Value : TIWGradientDirection);
begin
  FCaptionGradientDirection := Value;
  Repaint;
end;

function TPlannerItem.GetCaptionTimeString: string;
var
  s1, s2: string;
  DateTime: TDateTime;
  DOffs1,DOffs2: Integer;
  ho,mi,se,se100: word;
begin
  case FPlanner.FMode.FPlannerType of
    plDay:
      begin
        s1 := FPlanner.PlanTimeToStr(FItemBeginPrecis);
        s2 := FPlanner.PlanTimeToStr(FItemEndPrecis);

        if RealTime or InHeader then
        begin
          DecodeTime(ItemRealStartTime,ho,mi,se,se100);
          mi := mi + ho * 60;
          s1 := FPlanner.PlanTimeToStr(mi);

          DecodeTime(ItemRealEndTime,ho,mi,se,se100);
          mi := mi + ho * 60;
          s2 := FPlanner.PlanTimeToStr(mi);

          if (Int(ItemRealStartTime) <> Int(ItemRealEndTime)) and
             (Int(ItemRealStartTime) <> 0) and (Int(ItemRealEndTime) <> 0) then
            Result := s1 + ' ' + DateToStr(ItemRealStartTime) + ' - ' + s2 + ' ' + DateToStr(ItemRealEndTime)
          else
            Result := s1 + ' - ' + s2;

          {$IFDEF TMSDEBUG}
          outputdebugstring(pchar(datetostr(itemrealendtime)));
          {$ENDIF}
        end
        else
        begin
          if (Int(ItemStartTime) <> Int(ItemEndTime)) and
             (Int(ItemStartTime) <> 0) and (Int(ItemEndTime) <> 0) then
            Result := s1 + ' ' + DateToStr(ItemStartTime) + ' - ' + s2 + ' ' + DateToStr(ItemEndTime)
          else
            Result := s1 + ' - ' + s2;
        end;
      end;
    plTimeLine:
      begin
        s1 := FPlanner.PlanTimeToStr(FItemBeginPrecis -
          BeginOffset * FPlanner.FDisplay.FDisplayUnit + MININDAY);
        s2 := FPlanner.PlanTimeToStr(FItemEndPrecis +
          EndOffset * FPlanner.FDisplay.FDisplayUnit);

        if RealTime or InHeader then
        begin
          s1 := TimeToStr(ItemRealStartTime);
          s2 := TimeToStr(ItemRealEndTime);

          if (Int(ItemRealStartTime) <> Int(ItemRealEndTime)) and
             (Int(ItemRealStartTime) <> 0) and (Int(ItemRealEndTime) <> 0) then
            Result := s1 + ' ' + DateToStr(ItemRealStartTime) + ' - ' + s2 + ' ' + DateToStr(ItemRealEndTime)
          else
            Result := s1 + ' - ' + s2;

          {$IFDEF TMSDEBUG}
          outputdebugstring(pchar(datetostr(itemrealendtime)));
          {$ENDIF}
        end
        else
        begin
          DOffs1 := (((ItemBegin) * FPlanner.Display.DisplayUnit) div MININDAY);
          DOffs2 := (((ItemEnd + EndOffset) * FPlanner.Display.DisplayUnit) div MININDAY);

          if BeginOffset > 0 then
            DOffs1 := DOffs1 - 1 - ((BeginOffset * FPlanner.Display.DisplayUnit) div MININDAY);

          if (Doffs1 <> Doffs2) then
            Result := s1 + ' ' + DateToStr(Planner.Mode.TimeLineStart + DOffs1) + ' - ' + s2 + ' ' + DateToStr(Planner.Mode.TimeLineStart + DOffs2)
          else
            Result := s1 + ' - ' + s2 + ' ' + DateToStr(Planner.Mode.TimeLineStart + DOffs1);
        end;
      end;
    plWeek:
      begin
        s2 := '';
        s1 := FPlanner.GetDayName(ItemBegin - FPlanner.FMode.WeekStart);

        DateTime := FPlanner.Mode.PeriodStartDate + FItemBegin;
        DateTime := DateTime - BeginOffset;

        if FPlanner.Mode.FDateTimeFormat <> '' then
          s1 := s1 + ' ' + FormatDateTime(FPlanner.Mode.FDateTimeFormat,DateTime);

        DateTime := FPlanner.Mode.PeriodStartDate + FItemEnd;
        DateTime := DateTime + EndOffset - 1;

        if FItemEnd > FItemBegin + 1 then
        begin
          s2 := FPlanner.GetDayName(FItemEnd - 1 - FPlanner.FMode.WeekStart);
          if FPlanner.Mode.FDateTimeFormat <> '' then
            s2 := s2 + ' ' + FormatDateTime(FPlanner.Mode.FDateTimeFormat,DateTime);
        end;

        if s2 <> '' then
          s1 := s1 + ' - ' + s2;
        Result := s1;
      end;

    plDayPeriod,plMonth:
      begin
        with FPlanner.FMode do
        begin
          //v1.2
          DateTime := FPlanner.Mode.PeriodStartDate + FItemBegin;
          DateTime := DateTime - BeginOffset;

          s1 := FormatDateTime(FDateTimeFormat, DateTime);

          DateTime := FPlanner.Mode.PeriodStartDate + FItemEnd;
          DateTime := DateTime + EndOffset - 1;

          if FItemEnd > FItemBegin + 1 then
            s2 := '';
          if FItemEnd > FItemBegin + 1 then
            s2 := FormatDateTime(FDateTimeFormat, DateTime);
          if (s2 <> '') then
            s1 := s1 + ' - ' + s2;
          Result := s1;
        end;
      end;
    plHalfDayPeriod:
      begin
        with FPlanner.FMode do
        begin
          DateTime := Trunc(FPlanner.Mode.PeriodStartDate + (FItemBegin - BeginOffset) / 2);

          s1 := FormatDateTime(FDateTimeFormat, DateTime);

          if Odd(FItemBegin - BeginOffset) then
            s1 := s1 + ' PM'
          else
            s1 := s1 + ' AM';

          DateTime := Trunc((FPlanner.Mode.PeriodStartDate + (FItemEnd - 1 + EndOffset) / 2));

          s2 := FormatDateTime(FDateTimeFormat, DateTime);
          if Odd(FItemEnd + EndOffset) then
            s2 := s2 + ' AM'
          else
            s2 := s2 + ' PM';

          Result := s1 + ' - ' + s2;
        end;
      end;
    plMultiMonth:
      begin
        Result := ItemStartTimeStr +  ' - ' + ItemEndTimeStr;
      end;
    plCustom, plCustomList:
      begin
        Result := ItemStartTimeStr +  ' - ' + ItemEndTimeStr;
      end;
  end;
end;

function TPlannerItem.GetCaptionString: string;
begin
  if RelationShip = irChild then
  begin
    Result := ParentItem.GetCaptionString;
    Exit;
  end;

  if CaptionType in [ctTime,ctTimeText] then
  begin
    Result := GetCaptionTimeString;

    if CaptionType = ctTimeText then
      Result := Result + ' ' + CaptionText;
  end
  else
    if not (CaptionType = ctNone) then
      Result := CaptionText
    else
      Result := '';
end;

function TPlannerItem.GetCaptionHeight: Integer;
begin
  Result := FCaptionHeight;
end;

procedure TPlannerItem.SetSelectColor(const Value: TIWColor);
begin
  if FSelectColor <> Value then
  begin
    FSelectColor := Value;
    Repaint;
  end;
end;

procedure TPlannerItem.SetSelectFontColor(const Value: TIWColor);
begin
  if FSelectFontColor <> Value then
  begin
    FSelectFontColor := Value;
    Repaint;
  end;
end;

procedure TPlannerItem.SetSelected(const Value: Boolean);
begin
  if FSelected <> Value then
  begin
    FSelected := Value;
    Repaint;
  end;
end;

procedure TPlannerItem.SetObject(const Value: TObject);
begin
  if FObject = Value then
    Exit;
  if FOwnsItemObject and Assigned(FObject) then
    FObject.Free;
  FObject := Value;
end;

procedure TPlannerItem.SetInHeader(const Value: Boolean);
begin
  if FInHeader <> Value then
  begin
    FInHeader := Value;
    Repaint;
    FPlanner.Items.SetConflicts;
  end;
end;

procedure TPlannerItem.SetItemBeginPrecis(const Value: Integer);
begin
  FItemBeginPrecis := Value;
  ItemBegin := (FItemBeginPrecis div FPlanner.Display.DisplayUnit) -
    FPlanner.Display.DisplayStart;
end;

procedure TPlannerItem.SetItemEndPrecis(const Value: Integer);
begin
  FItemEndPrecis := Value;
  ItemEnd := (FItemEndPrecis div FPlanner.Display.DisplayUnit) -
    FPlanner.Display.DisplayStart;
end;

procedure TPlannerItem.SetPopupMenu(const Value: TPopupMenu);
begin
  FPopupMenu := Value;
end;

procedure TPlannerItem.SetShowSelection(const Value: Boolean);
begin
  FShowSelection := Value;
  Repaint;
end;

procedure TPlannerItem.GetTimeTag;
begin
  ItemBeginPrecis := LoWord(DBTag);
  ItemEndPrecis := HiWord(DBTag);
  FRepainted := False;
end;

procedure TPlannerItem.SetTimeTag;
begin
  ItemBegin := ItemBegin;
  ItemEnd := ItemEnd;
  DBTag := MakeLong(ItemBeginPrecis,ItemEndPrecis);
end;

procedure TPlannerItem.Update;
begin
  FPlanner.UpdateItem(Self);
end;

function TPlannerItem.GetItemText: string;
var
  Index: Integer;
begin
  Result := '';
  for Index := 0 to Text.Count - 1 do
  begin
    if Result <> '' then Result := Result + #13;
    Result := Result + Text.Strings[Index];
  end;
end;

procedure TPlannerItem.SetAttachement(const Value: string);
begin
  if FAttachement <> Value then
  begin
    FAttachement := Value;
    Repaint;
  end;
end;

procedure TPlannerItem.MoveItem(NewBegin, NewEnd, NewPos: Integer; var NewOffset: Integer);
var
  Delta: Integer;
begin
  Delta := NewBegin - ItemBegin;

  RealTime := False;

{$IFDEF CODESITE}
  CodeSite.SendInteger('Moved Begin:',NewBegin);
  CodeSite.SendInteger('Moved End:',NewEnd);
  CodeSite.SendInteger('Delta:',Delta);
{$ENDIF}

  if BeginOffset <> 0 then
  begin
    BeginOffset := BeginOffset - Delta;

    If BeginOffset < 0 then
    begin
      NewBegin := ItemBegin + BeginOffset;
      BeginOffset := 0;
      NewOffset := NewOffset - Delta;
    end
    else
    begin
      NewBegin := NewBegin - Delta;
      NewOffset := NewOffset - Delta;
    end;
  end;

  if EndOffset <> 0 then
  begin
    EndOffset := EndOffset + Delta;

    if EndOffset < 0 then
    begin
      NewEnd := ItemEnd + EndOffset;
      EndOffset := 0;
    end
    else
    begin
      NewEnd := NewEnd - Delta;
      NewOffset := NewOffset;
    end;
  end;

  if NewBegin < 0 then
  begin
    BeginOffset := -NewBegin;
    NewBegin := 0;
    NewOffset := NewOffset - Delta;
  end;

  if FPlanner.Sidebar.Position = spTop then
  begin
//    if NewEnd > FPlanner.FGrid.ColCount then
//    begin
//      EndOffset := NewEnd - FPlanner.FGrid.ColCount;
//      NewEnd := FPlanner.FGrid.ColCount;
//    end;
  end
  else
  begin
//    if NewEnd > FPlanner.FGrid.RowCount then
//    begin
//      EndOffset := NewEnd - FPlanner.FGrid.RowCount;
//      NewEnd := FPlanner.FGrid.RowCount;
//    end;
  end;

{$IFDEF TMSCODESITE}
  CodeSite.SendInteger('NewBegin:',NewBegin);
  CodeSite.SendInteger('NewEnd:',NewEnd);
{$ENDIF}
  ItemBegin := NewBegin;
  ItemEnd := NewEnd;
  ItemPos := NewPos;
end;

procedure TPlannerItem.SizeItem(NewBegin, NewEnd: Integer);
var
  NewPos: Integer;
begin
  RealTime := False;

  NewPos := ItemPos;

//  if Assigned(FPlanner.OnItemPlaceUpdate) then
//    FPlanner.OnItemPlaceUpdate(FPlanner,Self,NewBegin,NewEnd,NewPos);

  if ItemBegin <> NewBegin then
    ItemBegin := NewBegin;
  if ItemEnd <> NewEnd then
    ItemEnd := NewEnd;
  if ItemPos <> NewPos then
    ItemPos := NewPos;
end;

function TPlannerItem.GetItemRealEndTime: TDateTime;
begin
  if RealTime then
    Result := FItemRealEndTime
  else
    Result := ItemEndTime;
end;

function TPlannerItem.GetItemRealStartTime: TDateTime;
begin
  if RealTime then
    Result := FItemRealStartTime
  else
    Result := ItemStartTime;
end;

procedure TPlannerItem.SetURL(const Value: string);
begin
  if FURL <> Value then
  begin
    FURL := Value;
    Repaint;
  end;
end;

function TPlannerItem.GetStrippedItemText: string;
begin
  Result := HTMLStrip(GetItemText);
end;

procedure TPlannerItem.SetFlashing(const Value: Boolean);
begin
  if FFlashing <> Value then
  begin
    FFlashing := Value;
    if not FFlashing then Repaint;
  end;
end;

procedure TPlannerItem.Changed;
begin
  FPlanner.Items.ItemChanged(Self);
end;

procedure TPlannerItem.ChangeCrossing;
var
  delta: Integer;
begin
  delta := ItemEnd + EndOffset - ItemBegin;

  BeginOffset := - (ItemBegin - 1440 div Planner.Display.DisplayUnit);
  ItemBegin := 0;
  EndOffset := 0;
  ItemEnd := delta - BeginOffset;
end;

function TPlannerItem.Planner: TTIWCustomWebPlanner;
begin
  Result := (Collection as TPlannerItems).FOwner;
end;

function TPlannerItem.GetParentItem: TPlannerItem;
var
  i: Integer;
begin
  Result := Self;
  if (ParentIndex = -1) or (RelationShip = irParent) then
    Result := Self
  else
    for i := 1 to (Collection as TPlannerItems).Count do
    begin
      if ((Collection as TPlannerItems).Items[i - 1].ParentIndex = ParentIndex) and
         ((Collection as TPlannerItems).Items[i - 1].RelationShip = irParent) then
      begin
        Result := (Collection as TPlannerItems).Items[i - 1];
        Break;
      end;
    end;
end;


function TPlannerItem.GetCanEdit: Boolean;
begin
  Result := ReadOnly;
end;

procedure TPlannerItem.EnsureFullVisibility;
begin
  if FPlanner.Sidebar.Position = spTop then
  begin
  end
  else
  begin
  end;
end;

procedure TPlannerItem.SetCaptionBkgTo(const Value: TIWColor);
begin
  if (FCaptionBkgTo <> Value) then
  begin
    FCaptionBkgTo := Value;
    Repaint;
  end;
end;

procedure TPlannerItem.SetCaptionHeight(const Value: Integer);
begin
  if (FCaptionHeight <> Value) then
  begin
    FCaptionHeight := Value;
    Repaint;
  end;
end;

{ TPlannerItems }

function TPlannerItems.Add: TPlannerItem;
begin
  Result := TPlannerItem(inherited Add);
//  if Assigned(FOwner.DefaultItem) then
  Result.Assign(FOwner.DefaultItem);
  SetConflicts;
end;

constructor TPlannerItems.Create(AOwner: TTIWCustomWebPlanner);
begin
  inherited Create(GetItemClass);
  FOwner := AOwner;
  FSelected := nil;
  FUpdateCount := 0;
end;

function TPlannerItems.GetItem(Index: Integer): TPlannerItem;
begin
  Result := TPlannerItem(inherited GetItem(Index));
end;

function TPlannerItems.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

function TPlannerItems.Insert(Index: Integer): TPlannerItem;
begin
  Result := TPlannerItem(inherited Insert(Index));
  Result.Assign(FOwner.DefaultItem);
end;

function TPlannerItems.HasItem(ItemBegin, ItemEnd, ItemPos: Integer): Boolean;
var
  ItemIndex: Integer;
  APlannerItem: TPlannerItem;
begin
  Result := False;

  for ItemIndex := 0 to Self.Count - 1 do
  begin
    APlannerItem := Self.Items[ItemIndex];
    if (
      ((APlannerItem.ItemBegin <= ItemBegin) and (APlannerItem.ItemEnd > ItemBegin)) or
      ((APlannerItem.ItemBegin < ItemEnd) and (APlannerItem.ItemEnd >= ItemEnd)) or
      ((ItemBegin < APlannerItem.ItemEnd) and (ItemEnd >= APlannerItem.ItemEnd)) or
      ((ItemBegin <= APlannerItem.ItemBegin) and (ItemEnd > APlannerItem.ItemBegin))
      ) and
      (APlannerItem.ItemPos = ItemPos) and (APlannerItem.Visible) and (APlannerItem <> Selected)
      and (InVisibleLayer(APlannerItem.Layer)) then
    begin
      Result := (not APlannerItem.AllowOverlap) or (not FOwner.FOverlap);
      if Result then
        Break;
    end;
  end;
end;

function TPlannerItems.HasItemEx(AItem: TPlannerItem; ItemBegin, ItemEnd, ItemPos: Integer): Boolean;
var
  ItemIndex: Integer;
  APlannerItem: TPlannerItem;
begin
  Result := False;

  for ItemIndex := 0 to Self.Count - 1 do
  begin
    APlannerItem := Self.Items[ItemIndex];
    if (
      ((APlannerItem.ItemBegin <= ItemBegin) and (APlannerItem.ItemEnd > ItemBegin)) or
      ((APlannerItem.ItemBegin < ItemEnd) and (APlannerItem.ItemEnd >= ItemEnd)) or
      ((ItemBegin < APlannerItem.ItemEnd) and (ItemEnd >= APlannerItem.ItemEnd)) or
      ((ItemBegin <= APlannerItem.ItemBegin) and (ItemEnd > APlannerItem.ItemBegin))
      ) and
      (APlannerItem.ItemPos = ItemPos) and (APlannerItem.Visible)
      and (InVisibleLayer(APlannerItem.Layer))
      and (APlannerItem <> AItem) then
    begin
      Result := (not APlannerItem.AllowOverlap) or (not FOwner.FOverlap) or (APlannerItem.Background);
      if Result then
        Break;
    end;
  end;
end;

function TPlannerItems.FindFirst(ItemBegin, ItemEnd, ItemPos: Integer):
TPlannerItem;
var
  ItemIndex: Integer;
  APlannerItem: TPlannerItem;
begin
  Result := nil;

  FFindIndex := 1;

  for ItemIndex := 1 to Self.Count do
  begin
    APlannerItem := Self.Items[ItemIndex - 1];
    if (
      ((APlannerItem.ItemBegin <= ItemBegin) and (APlannerItem.ItemEnd > ItemBegin)) or
      ((APlannerItem.ItemBegin < ItemEnd) and (APlannerItem.ItemEnd >= ItemEnd)) or
      ((ItemBegin < APlannerItem.ItemEnd) and (ItemEnd >= APlannerItem.ItemEnd)) or
      ((ItemBegin <= APlannerItem.ItemBegin) and (ItemEnd > APlannerItem.ItemBegin))
      )
      and (APlannerItem.ItemPos = ItemPos) then
    begin
      Result := APlannerItem;
      FFindIndex := ItemIndex + 1;
      Break;
    end;
  end;
end;

function TPlannerItems.FindNext(ItemBegin, ItemEnd, ItemPos: Integer):
TPlannerItem;
var
  ItemIndex: Integer;
  APlannerItem: TPlannerItem;
begin
  Result := nil;

  for ItemIndex := FFindIndex to Self.Count do
  begin
    APlannerItem := Self.Items[ItemIndex - 1];
    if (
      ((APlannerItem.ItemBegin <= ItemBegin) and (APlannerItem.ItemEnd > ItemBegin)) or
      ((APlannerItem.ItemBegin < ItemEnd) and (APlannerItem.ItemEnd >= ItemEnd)) or
      ((ItemBegin < APlannerItem.ItemEnd) and (ItemEnd >= APlannerItem.ItemEnd)) or
      ((ItemBegin <= APlannerItem.ItemBegin) and (ItemEnd > APlannerItem.ItemBegin))
      )
      and (APlannerItem.ItemPos = ItemPos) then
    begin
      FFindIndex := ItemIndex + 1;
      Result := APlannerItem;
      Break;
    end;
  end;
end;

procedure TPlannerItems.ClearSelectedRepaints(ItemBegin,ItemPos: Integer);
var
  ItemIndex,ItemEnd: Integer;
  APlannerItem: TPlannerItem;
begin
  ItemEnd := ItemBegin + 1;
  for ItemIndex := 0 to Self.Count - 1 do
  begin
    APlannerItem := Self.Items[ItemIndex];
    if (

      ((ItemBegin >= APlannerItem.ItemBegin) and (ItemBegin < APlannerItem.ItemEnd)) or
      ((ItemEnd > APlannerItem.ItemBegin) and (ItemEnd < APlannerItem.ItemEnd)) or
      ((APlannerItem.ItemBegin >= ItemBegin) and (APlannerItem.ItemBegin < ItemEnd)) or
      ((APlannerItem.ItemEnd > ItemBegin) and (APlannerItem.ItemEnd <= ItemEnd))

      ) and
      (APlannerItem.ItemPos = ItemPos) and (APlannerItem.Visible) and
      (InVisibleLayer(APlannerItem.Layer)) then
    begin
      APlannerItem.Repainted := False;
    end;
  end;
end;

function TPlannerItems.NumItem(ItemBegin, ItemEnd, ItemPos: Integer): TPoint;
var
  ItemIndex: Integer;
  APlannerItem: TPlannerItem;
begin
  Result.X := 0;
  Result.Y := 0;
  for ItemIndex := 0 to Self.Count - 1 do
  begin
    APlannerItem := Self.Items[ItemIndex];
    if (
      ((ItemBegin >= APlannerItem.ItemBegin) and (ItemBegin < APlannerItem.ItemEnd)) or
      ((ItemEnd > APlannerItem.ItemBegin) and (ItemEnd < APlannerItem.ItemEnd)) or
      ((APlannerItem.ItemBegin >= ItemBegin) and (APlannerItem.ItemBegin < ItemEnd)) or
      ((APlannerItem.ItemEnd > ItemBegin) and (APlannerItem.ItemEnd <= ItemEnd))

      ) and
      (APlannerItem.ItemPos = ItemPos) and (APlannerItem.Visible) and
      (InVisibleLayer(APlannerItem.Layer)) and (APlannerItem.ItemEnd > APlannerItem.ItemBegin) then
    begin
      Inc(Result.X);
      if APlannerItem.FConflicts > Result.Y then
        Result.Y := APlannerItem.FConflicts;
      if ItemBegin > APlannerItem.ItemBegin then
        ItemBegin := APlannerItem.ItemBegin;
      if ItemEnd < APlannerItem.ItemEnd then
        ItemEnd := APlannerItem.ItemEnd;
    end;
  end;
end;

function TPlannerItems.NumItemPos(ItemBegin, ItemEnd, ItemPos: Integer): TPoint;
var
  ItemIndex: Integer;
  APlannerItem: TPlannerItem;
begin
  Result.X := 0;
  Result.Y := 0;
  for ItemIndex := 0 to Self.Count-1 do
  begin
    APlannerItem := Self.Items[ItemIndex];
    if (
      ((APlannerItem.ItemBegin <= ItemBegin) and (APlannerItem.ItemEnd > ItemBegin)) or
      ((APlannerItem.ItemBegin < ItemEnd) and (APlannerItem.ItemEnd >= ItemEnd)) or
      ((ItemBegin < APlannerItem.ItemEnd) and (ItemEnd >= APlannerItem.ItemEnd)) or
      ((ItemBegin <= APlannerItem.ItemBegin) and (ItemEnd > APlannerItem.ItemBegin))
      ) and
      (APlannerItem.ItemPos = ItemPos) and (APlannerItem.Visible) and
      (InVisibleLayer(APlannerItem.Layer)) then
    begin
      Inc(Result.X);
      if APlannerItem.FConflictPos > Result.Y then
        Result.Y := APlannerItem.FConflictPos;
    end;
  end;
end;

function TPlannerItems.NumItemPosStart(ItemBegin, ItemPos: Integer): Integer;
var
  ItemIndex: Integer;
  APlannerItem: TPlannerItem;
begin
  Result := 0;
  for ItemIndex := 0 to Self.Count-1 do
  begin
    APlannerItem := Self.Items[ItemIndex];
    if (APlannerItem.ItemBegin = ItemBegin) and
       (APlannerItem.ItemPos = ItemPos) and (APlannerItem.Visible) and
       (InVisibleLayer(APlannerItem.Layer)) then
    begin
      Inc(Result);
    end;
  end;
end;

function TPlannerItems.FindItemInHeader(ItemPos: Integer): TPlannerItem;
var
  ItemIndex: Integer;
  APlannerItem: TPlannerItem;
begin
  Result := nil;
  for ItemIndex := 0 to Self.Count - 1 do
  begin
    APlannerItem := Self.Items[ItemIndex];
    if (APlannerItem.InHeader) and (APlannerItem.ItemPos = ItemPos) and
       (InVisibleLayer(APlannerItem.Layer)) then
    begin
      Result := APlannerItem;
      Break;
    end;
  end;
end;

function TPlannerItems.FindItemInHeaderEx(ItemPos,SubPos: Integer): TPlannerItem;
var
  ItemIndex, SubPosIndex: Integer;
  APlannerItem: TPlannerItem;
begin
  Result := nil;
  SubPosIndex := 0;

  for ItemIndex := 0 to Self.Count - 1 do
  begin
    APlannerItem := Self.Items[ItemIndex];
    if (APlannerItem.InHeader) and (APlannerItem.ItemPos = ItemPos) and
       (InVisibleLayer(APlannerItem.Layer)) then
    begin
      if (SubPosIndex = SubPos) then
      begin
        Result := APlannerItem;
        Break;
      end
      else
        inc(SubPosIndex);
    end;
  end;
end;

function TPlannerItems.FindItem(ItemBegin, ItemPos: Integer): TPlannerItem;
var
  ItemIndex: Integer;
  APlannerItem: TPlannerItem;
begin
  Result := nil;
  for ItemIndex := 0 to Self.Count - 1 do
  begin
    APlannerItem := Self.Items[ItemIndex];
    if (APlannerItem.ItemBegin <= ItemBegin) and
      (APlannerItem.ItemEnd > ItemBegin) and
      not (APlannerItem.InHeader) and//added
      (APlannerItem.ItemPos = ItemPos) and (APlannerItem.Visible) and
      (InVisibleLayer(APlannerItem.Layer)) then
    begin
      Result := APlannerItem;
      Break;
    end;
  end;
end;

function TPlannerItems.FindItemIdx(ItemBegin: Integer): TPlannerItem;
var
  ItemIndex: Integer;
  APlannerItem: TPlannerItem;
begin
  Result := nil;
  for ItemIndex := 0 to Self.Count - 1 do
  begin
    APlannerItem := Self.Items[ItemIndex];
    if (APlannerItem.ItemBegin <= ItemBegin) and
      (APlannerItem.ItemEnd > ItemBegin) and
      (APlannerItem.Visible) and
      (InVisibleLayer(APlannerItem.Layer)) then
    begin
      Result := APlannerItem;
      Break;
    end;
  end;
end;


function TPlannerItems.FindItemPos(ItemBegin, ItemPos, ItemSubPos: Integer): TPlannerItem;
var
  ItemIndex: Integer;
  APlannerItem: TPlannerItem;
begin
  Result := nil;

  for ItemIndex := 0 to Self.Count-1 do
  begin
    APlannerItem := Self.Items[ItemIndex];

    if (APlannerItem.ItemBegin <= ItemBegin) and
      (APlannerItem.ItemEnd > ItemBegin) and
      (APlannerItem.ItemPos = ItemPos) and (APlannerItem.Visible) and
//      not (APlannerItem.InHeader) and //added
      not (APlannerItem.Background and APlannerItem.AllowOverlap) and
      (InVisibleLayer(APlannerItem.Layer)) then
    begin
      if (APlannerItem.FConflicts = 0) or
         (APlannerItem.FConflictPos = ItemSubPos) then
      begin
        Result := APlannerItem;
        Break;
      end;
    end;
  end;
end;

function TPlannerItems.FindBkgPos(ItemBegin, ItemPos, ItemSubPos: Integer): TPlannerItem;
var
  ItemIndex: Integer;
  APlannerItem: TPlannerItem;
begin
  Result := nil;

  for ItemIndex := 0 to Self.Count-1 do
  begin
    APlannerItem := Self.Items[ItemIndex];

    if (APlannerItem.ItemBegin <= ItemBegin) and
      (APlannerItem.ItemEnd > ItemBegin) and
      (APlannerItem.ItemPos = ItemPos) and (APlannerItem.Visible) and
      (APlannerItem.Background and APlannerItem.AllowOverlap) and
      (InVisibleLayer(APlannerItem.Layer)) then
    begin
      if (APlannerItem.FConflicts = 0) or
         (APlannerItem.FConflictPos = ItemSubPos) then
      begin
        Result := APlannerItem;
        Break;
      end;
    end;
  end;
end;

function TPlannerItems.FindItemHeaderPosIdx(ItemPos, ItemSubPos: Integer): TPlannerItem;
var
  ItemIndex: Integer;
  APlannerItem: TPlannerItem;
begin
  Result := nil;
  for ItemIndex := 0 to Self.Count - 1 do
  begin
    APlannerItem := Self.Items[ItemIndex];
    if (APlannerItem.InHeader) and (APlannerItem.ItemPos = ItemPos) and
      (APlannerItem.FConflictPos = ItemSubPos) and
      (InVisibleLayer(APlannerItem.Layer)) then
    begin
      Result := APlannerItem;
      Break;
    end;
  end;
end;

function TPlannerItems.FindItemPosIdx(ItemBegin, ItemPos, ItemSubPos: Integer): TPlannerItem;
var
  ItemIndex: Integer;
  APlannerItem: TPlannerItem;
begin
  Result := nil;
  for ItemIndex := 0 to Self.Count - 1 do
  begin
    APlannerItem := Self.Items[ItemIndex];
    if (APlannerItem.ItemBegin <= ItemBegin) and
      (APlannerItem.ItemEnd > ItemBegin) and
      (APlannerItem.ItemPos = ItemPos) and (APlannerItem.Visible) and
      not (APlannerItem.InHeader) and //added
      not (APlannerItem.Background and APlannerItem.AllowOverlap) and
      (APlannerItem.FConflictPos = ItemSubPos) and
      (InVisibleLayer(APlannerItem.Layer)) then
    begin
      Result := APlannerItem;
      Break;
    end;
  end;
end;

function TPlannerItems.FindItemIndex(ItemBegin, ItemPos, ItemSubIdx: Integer): TPlannerItem;
var
  ItemIndex: Integer;
  APlannerItem: TPlannerItem;
begin
  Result := nil;
  for ItemIndex := 0 to Self.Count - 1 do
  begin
    APlannerItem := Self.Items[ItemIndex];

    if (APlannerItem.ItemBegin <= ItemBegin) and
      (APlannerItem.ItemEnd > ItemBegin) and
      (APlannerItem.ItemPos = ItemPos) and
      (APlannerItem.FConflictPos = ItemSubIdx) and
      (APlannerItem.Visible) and
//      not(APlannerItem.InHeader) and //added
      not (APlannerItem.Background and APlannerItem.AllowOverlap) and
      (InVisibleLayer(APlannerItem.Layer)) then
    begin
      Result := APlannerItem;
      Break;
    end;
  end;
end;

function TPlannerItems.FindBackground(ItemBegin,ItemEnd, ItemPos: Integer): TPlannerItem;
var
  ItemIndex: Integer;
  APlannerItem: TPlannerItem;
begin
  Result := nil;
  for ItemIndex := 0 to Self.Count - 1 do
  begin
    APlannerItem := Self.Items[ItemIndex];

    if (((APlannerItem.ItemBegin <= ItemBegin) and
      (APlannerItem.ItemEnd > ItemBegin)) or ((APlannerItem.ItemBegin >= ItemBegin) and (ItemEnd >= APlannerItem.ItemBegin))) and
      (APlannerItem.ItemPos = ItemPos) and
//      (APlannerItem.AllowOverlap) and
      (APlannerItem.Visible) and (APlannerItem.Background) and
      (InVisibleLayer(APlannerItem.Layer)) then
    begin
      Result := APlannerItem;
      Break;
    end;
  end;
end;


function TPlannerItems.QueryItem(Item: TPlannerItem; ItemBegin, ItemPos:
  Integer): TPlannerItem;
var
  ItemIndex, StartIndex: Integer;
  APlannerItem: TPlannerItem;
begin
  Result := nil;
  if Assigned(Item) then
    StartIndex := Item.Index + 2
  else
    StartIndex := 1;

  for ItemIndex := StartIndex to Self.Count do
  begin
    APlannerItem := Self.Items[ItemIndex - 1];
    if (APlannerItem.ItemBegin <= ItemBegin) and
      (APlannerItem.ItemEnd > ItemBegin) and
      (APlannerItem.ItemPos = ItemPos) and (APlannerItem.Visible) and
      (InVisibleLayer(APlannerItem.Layer)) then
    begin
      Result := APlannerItem;
      Break;
    end;
  end;
end;

procedure TPlannerItems.SetCurrent(ItemCurrent: Integer);
var
  ItemIndex: Integer;
  APlannerItem: TPlannerItem;
begin
  for ItemIndex := 0 to Self.Count - 1 do
  begin
    APlannerItem := Self.Items[ItemIndex];
    APlannerItem.IsCurrent := (APlannerItem.ItemBegin <= ItemCurrent) and
      (APlannerItem.ItemEnd > ItemCurrent) and
      not APlannerItem.InHeader and APlannerItem.Planner.IsCurPos(APlannerItem.ItemPos);
  end;
end;

function TPlannerItems.FocusItem(ItemBegin, ItemPos, ItemSubPos: Integer; Control:
  Boolean): TPlannerItem;
var
  ItemIndex: Integer;
  APlannerItem: TPlannerItem;
begin
  Result := nil;
  FSelected := nil;

  for ItemIndex := 0 to Self.Count - 1 do
  begin
    APlannerItem := Self.Items[ItemIndex];
    if (APlannerItem.ItemBegin <= ItemBegin) and
      (APlannerItem.ItemEnd > ItemBegin) and
      (APlannerItem.ItemPos = ItemPos) and
      (APlannerItem.Visible) and
      not (APlannerItem.Background and APlannerItem.AllowOverlap) and
      (InVisibleLayer(APlannerItem.Layer)) then
    begin
      if (APlannerItem.FConflicts < 0) or
         (APlannerItem.FConflictPos = ItemSubPos) then
      begin
        Result := APlannerItem;
        APlannerItem.Focus := True;
        if Control and APlannerItem.Selected then
          APlannerItem.Selected := False
        else
          APlannerItem.Selected := True;
        FSelected := APlannerItem;
      end
      else
      begin
        APlannerItem.Focus := False;
        if not Control or not FOwner.MultiSelect then
          APlannerItem.Selected := False;
      end;
    end
    else
    begin
      APlannerItem.Focus := False;
      if not Control or not FOwner.MultiSelect then
        APlannerItem.Selected := False;
    end;
  end;
end;

procedure TPlannerItems.UnSelect;
begin
  if Assigned(FSelected) then
  begin
    FSelected.Focus := False;
    FSelected.Selected := False;
    FSelected := nil;
  end;
end;

procedure TPlannerItems.UnSelectAll;
var
  i: Integer;
begin
  for i := 0 to Count-1 do
  with Items[i] do
  begin
    Focus := False;
    Selected := False;
  end;
  Selected := nil;
end;


function TPlannerItems.SelectNext: TPlannerItem;
var
  idx: Integer;
begin
  if FSelected <> nil then
  begin
    FSelected.Focus := False;
    FSelected.Selected := False;
    idx := FSelected.Index;
    while idx + 1 < Self.Count do
    begin
      if not (Self.Items[idx + 1].Background) then
      begin
        FSelected := Self.Items[idx + 1];
        FSelected.Selected := True;
        Break;
      end
      else
        inc(idx);
    end;
    if (idx + 1 = Self.Count) then
      FSelected := nil;
  end
  else
  begin
    if (Self.Count > 0) then
    begin
      FSelected := Self.Items[0];
      FSelected.Focus := True;
      FSelected.Selected := True;
    end;
  end;
  if Assigned(FSelected) then
    FSelected.Focus := True;
  Result := FSelected;
end;

function TPlannerItems.SelectPrev: TPlannerItem;
var
  idx: Integer;
begin
  if FSelected <> nil then
  begin
    FSelected.Focus := False;
    FSelected.Selected := False;
    idx := FSelected.Index;
    while idx > 0 do
    begin
      if not Self.Items[idx - 1].Background then
      begin
        FSelected := Self.Items[idx - 1];
        FSelected.Selected := True;
        Break;
      end
      else
        dec(idx);
    end;
    if (idx < 0) then
      FSelected := nil;
  end
  else
  begin
    if Self.Count > 0 then
    begin
      FSelected := Self.Items[Self.Count - 1];
      FSelected.Selected := True;
    end;
  end;
  if Assigned(FSelected) then
    FSelected.Focus := True;
  Result := FSelected;
end;

function TPlannerItems.InVisibleLayer(Layer: Integer): Boolean;
begin
  Result := (FOwner.Layer = 0) or ((FOwner.Layer > 0) and (Layer and FOwner.Layer > 0));
end;

procedure TPlannerItems.SetItem(Index: Integer; Value: TPlannerItem);
begin
  inherited SetItem(Index, Value);
end;

procedure TPlannerItems.ClearPosition(Position: Integer);
var
  ItemIndex: Integer;
begin
  BeginUpdate;
  for ItemIndex := Count downto 1 do
  begin
    if Items[ItemIndex - 1].ItemPos = Position then
      Items[ItemIndex - 1].Free
  end;
  EndUpdate;
end;

procedure TPlannerItems.ClearLayer(Layer: Integer);
var
  ItemIndex: Integer;
begin
  BeginUpdate;
  for ItemIndex := Count downto 1 do
  begin
    if Items[ItemIndex - 1].Layer = Layer then
      Items[ItemIndex - 1].Free
  end;
  EndUpdate;
end;

procedure TPlannerItems.BeginUpdate;
begin
  inherited;
  Inc(FUpdateCount);
end;

procedure TPlannerItems.EndUpdate;
begin
  if FUpdateCount > 0 then Dec(FUpdateCount);
  if FUpdateCount = 0 then
  begin
    SetConflicts;
  end;
  inherited;
end;

procedure TPlannerItems.ResetUpdate;
var
  i: Integer;
begin
  if FUpdateCount > 0 then Dec(FUpdateCount);
  if FUpdateCount = 0 then
  begin
    SetConflicts;
    ClearRepaints;
    for i := 1 to Count do
      if Items[i - 1].FChanged then
        Items[i - 1].Repaint;
  end;
end;

procedure TPlannerItems.SetConflicts;
type
  tbp = record
    X, Y,Z: Byte;
  end;
  confarray = array of array of tbp;

var
  conf: confarray;

  i, j, k, m, n, mm, NumRows: Integer;
  flg: Boolean;

  function IsOverlapping(APlannerItem: TPlannerItem; ItemBegin: Integer):boolean;
  begin
    Result := False;

    if (APlannerItem.ItemBegin <= ItemBegin) and
       (APlannerItem.ItemEnd > ItemBegin) and
       (APlannerItem.Visible) and not (APlannerItem.InHeader) and
       not (APlannerItem.Background and APlannerItem.AllowOverlap) and
       (InVisibleLayer(APlannerItem.Layer)) then
    begin
      Result := True;
    end;
  end;

begin

  if FUpdateCount > 0 then
    Exit;

  if Count = 0 then
    Exit;

  for i := 1 to Count do
  with Items[i - 1] do
    begin
      FBeginExt := ItemBegin;
      FEndExt := ItemEnd;
      FChanged := False;
    end;

  {Clear counters of overlapping Items}
  if FOwner.SideBar.Orientation = soVertical then
  begin
    NumRows := FOwner.Display.DisplayEnd - FOwner.Display.DisplayStart + 1;
    SetLength(conf,FOwner.Positions + 1,NumRows + 1);
  end
  else
  begin
    NumRows := FOwner.Display.DisplayEnd - FOwner.Display.DisplayStart + 1;
    SetLength(conf,FOwner.Positions + 1,NumRows + 1);
  end;

  {Calculate worst-case item overlap count}

  for i := 1 to Count do
  begin
    m := 0;
    if Items[i - 1].Visible and not(Items[i-1].InHeader) and not (Items[i - 1].Background and Items[i - 1].AllowOverlap) then
      for j := 1 to Count do
      begin
        if (i <> j) and (Items[j - 1].ItemPos = Items[i - 1].ItemPos) then
        for k := Items[i - 1].ItemBegin to Items[i - 1].ItemEnd - 1 do
        begin
          if IsOverlapping(Items[j - 1], k) then
          begin

            if Items[j - 1].FBeginExt < Items[i - 1].FBeginExt then
               Items[i - 1].FBeginExt := Items[j - 1].FBeginExt
            else
               Items[j - 1].FBeginExt := Items[i - 1].FBeginExt;

            if Items[j - 1].FEndExt > Items[i - 1].FEndExt then
               Items[i - 1].FEndExt := Items[j - 1].FEndExt
            else
               Items[j - 1].FEndExt := Items[i - 1].FEndExt;

            Inc(m);
            Break;
          end;
        end;
      end;
    Items[i - 1].FChanged := Items[i - 1].FConflicts <> m + 1;
    Items[i - 1].FConflicts := m + 1;
  end;

  {Count. nr of overlapping Items per cell}
  for i := 1 to Count do
  begin
    with Items[i - 1] do
    begin                                     // +++1.6b+++
      if Visible and InVisibleLayer(Layer) and not InHeader and not (Background and AllowOverlap) then
        for j := FBeginExt to FEndExt - 1 do
        begin
          if (ItemPos < High(Conf)) and (ItemPos >=0) and (j < High(Conf[ItemPos])) then
          begin
            if Items[i - 1].FConflicts > Conf[ItemPos, j].X then
              Conf[ItemPos, j].X := Items[i - 1].FConflicts;
          end;
        end;
    end;
  end;

  {Assign Items conflicts & conflict positions}
  for i := 1 to Count do
  begin
    with Items[i - 1] do                   // +++1.6b+++
      if Visible and InVisibleLayer(Layer) and not InHeader and not (Background and AllowOverlap) then
      begin
        m := 0;
        mm := 0;

        {get last assigned conflict position}
        for j := ItemBegin to ItemEnd - 1 do
        begin
          if (ItemPos < High(Conf)) and (ItemPos >=0) and (j < High(Conf[ItemPos])) then
          begin
            if (Conf[ItemPos, j].Y > m)   then
              m := Conf[ItemPos, j].Y;

            if (Conf[ItemPos, j].Z > mm)   then
              mm := Conf[ItemPos, j].Z;
          end;
        end;

        if mm = m then mm := 0;

        {get nr. of conflicts from extended zone}
        n := 0;
        for j := FBeginExt to FEndExt - 1 do
          if (ItemPos < High(Conf)) and (ItemPos >=0) and (j < High(Conf[ItemPos])) then
            if Conf[ItemPos, j].X > n then
              n := Conf[ItemPos, j].X;

        {scan ... if Assigned Position 0,1,2}
        flg := (FConflicts <> n) or
               (FConflictPos <> m);

        FConflicts := n;

        if m < FConflicts then
        begin
          FChanged := FChanged or (FConflictPos <> m);
          FConflictPos := m
        end
        else
        begin
          FChanged := FChanged or (FConflictPos <> mm);
          FConflictPos := mm;
        end;

        if m >= FConflicts then
        begin
          mm := mm + 1;
          for j := ItemBegin to ItemEnd - 1 do
          begin
            if (ItemPos < High(Conf)) and (ItemPos >=0) and (j < High(Conf[ItemPos])) then
              Conf[ItemPos, j].Z := mm;
          end;
        end;

        if m < FConflicts then
          m := m + 1;

        for j := ItemBegin to ItemEnd - 1 do
        begin
          if (ItemPos < High(Conf)) and (ItemPos >=0) and (j < High(Conf[ItemPos])) then
            Conf[ItemPos, j].Y := m;
        end;

        if Flg then
          Repaint;
      end;
  end;

  {If better than worst case position found, optimize}
  for i := 1 to Count do
  begin
    with Items[i - 1] do
    begin
      m := 0;
      for j := 1 to NumRows do
        if (ItemPos < High(Conf)) and (ItemPos >=0) and (j <= High(Conf[ItemPos])) then
          if Conf[ItemPos, j - 1].Y > m then
            m := Conf[ItemPos, j - 1].Y;

      if FConflicts > m then
      begin
        FChanged := FChanged or (FConflicts <> m);
        FConflicts := m;
        Repaint;
      end;
    end;
  end;
end;

procedure TPlannerItems.ClearConflicts;
var
  ItemIndex: Integer;
begin
  for ItemIndex := 0 to Count-1 do
  begin
    Items[ItemIndex].FConflictPos := 0;
    Items[ItemIndex].FConflicts := 0;
  end;
end;

function TPlannerItems.NumConflicts(var ItemBegin, ItemEnd: Integer; ItemPos: Integer):
  Integer;
var
  ItemIndex: Integer;
  APlannerItem: TPlannerItem;
begin
  Result := 0;
  for ItemIndex := 0 to Self.Count-1 do
  begin
    APlannerItem := Self.Items[ItemIndex];
    if (
      ((APlannerItem.ItemBegin <= ItemBegin) and (APlannerItem.ItemEnd > ItemBegin)) or
      ((APlannerItem.ItemBegin < ItemEnd) and (APlannerItem.ItemEnd >= ItemEnd)) or
      ((ItemBegin < APlannerItem.ItemEnd) and (ItemEnd >= APlannerItem.ItemEnd)) or
      ((ItemBegin <= APlannerItem.ItemBegin) and (ItemEnd > APlannerItem.ItemBegin))
      ) and
      (APlannerItem.ItemPos = ItemPos) and (APlannerItem.Visible) and
      (InVisibleLayer(APlannerItem.Layer)) then
    begin
      Inc(Result);
      if APlannerItem.ItemBegin < ItemBegin then
        ItemBegin := APlannerItem.ItemBegin;
      if APlannerItem.ItemEnd > ItemEnd then
        ItemEnd := APlannerItem.ItemEnd;
    end;
  end;
end;

procedure TPlannerItems.OffsetItems(Offset: Integer);
var
  ItemIndex: Integer;
begin
  BeginUpdate;
  try
    for ItemIndex := 0 to Count - 1 do
    begin
      Items[ItemIndex].ItemBegin := Items[ItemIndex].ItemBegin + Offset;
      Items[ItemIndex].ItemEnd := Items[ItemIndex].ItemEnd + Offset;
    end;
  finally
    EndUpdate;
  end;
end;

procedure TPlannerItems.MoveAll(DeltaPos, DeltaBegin: Integer);
var
  ItemIndex: Integer;
begin
  for ItemIndex := 0 to Count - 1 do
  begin
    Items[ItemIndex].ItemBegin := Clip(Items[ItemIndex].ItemBegin + DeltaBegin,
      FOwner.Display.DisplayStart, FOwner.Display.DisplayEnd);
    Items[ItemIndex].ItemEnd := Clip(Items[ItemIndex].ItemEnd + DeltaBegin,
      FOwner.Display.DisplayStart, FOwner.Display.DisplayEnd);
    Items[ItemIndex].ItemPos := Clip(Items[ItemIndex].ItemPos + DeltaPos,
      FOwner.Display.DisplayStart, FOwner.Display.DisplayEnd);
  end;
end;

procedure TPlannerItems.MoveSelected(DeltaPos, DeltaBegin: Integer);
var
  ItemIndex: Integer;
begin
  for ItemIndex := 0 to Count - 1 do
  begin
    if Items[ItemIndex].Selected and not Items[ItemIndex].Focus then
    begin
      Items[ItemIndex].ItemBegin := Clip(Items[ItemIndex].ItemBegin + DeltaBegin,
        FOwner.Display.DisplayStart, FOwner.Display.DisplayEnd);
      Items[ItemIndex].ItemEnd := Clip(Items[ItemIndex].ItemEnd + DeltaBegin,
        FOwner.Display.DisplayStart, FOwner.Display.DisplayEnd);
      Items[ItemIndex].ItemPos := Clip(Items[ItemIndex].ItemPos + DeltaPos,
        FOwner.Display.DisplayStart, FOwner.Display.DisplayEnd);
    end;
  end;
end;

procedure TPlannerItems.SizeAll(DeltaStart, DeltaEnd: Integer);
var
  ItemIndex: Integer;
begin
  for ItemIndex := 0 to Count - 1 do
  begin
    Items[ItemIndex].ItemBegin := Clip(Items[ItemIndex].ItemBegin + DeltaStart,
      FOwner.Display.DisplayStart, FOwner.Display.DisplayEnd);
    Items[ItemIndex].ItemEnd := Clip(Items[ItemIndex].ItemEnd + DeltaEnd,
      FOwner.Display.DisplayStart, FOwner.Display.DisplayEnd);
  end;
end;

procedure TPlannerItems.SizeSelected(DeltaStart, DeltaEnd: Integer);
var
  ItemIndex: Integer;
begin
  for ItemIndex := 0 to Count - 1 do
  begin
    if Items[ItemIndex].Selected and not Items[ItemIndex].Focus then
    begin
      Items[ItemIndex].ItemBegin := Clip(Items[ItemIndex].ItemBegin + DeltaStart,
        FOwner.Display.DisplayStart, FOwner.Display.DisplayEnd);
      Items[ItemIndex].ItemEnd := Clip(Items[ItemIndex].ItemEnd + DeltaEnd,
        FOwner.Display.DisplayStart, FOwner.Display.DisplayEnd);
    end;
  end;
end;

procedure TPlannerItems.ClearRepaints;
var
  ItemIndex: Integer;
begin
  for ItemIndex := 0 to Count - 1 do
    Items[ItemIndex].Repainted := False;
end;


function TPlannerItems.HeaderFirst(ItemPos: Integer): TPlannerItem;
var
  ItemIndex: Integer;
  APlannerItem: TPlannerItem;
begin
  Result := nil;
  FFindIndex := 1;

  for ItemIndex := 1 to Count do
  begin
    APlannerItem := Items[ItemIndex - 1];
    if (APlannerItem.ItemPos = ItemPos) and (APlannerItem.InHeader) then
    begin
      Result := APlannerItem;
      FFindIndex := ItemIndex + 1;
      Break;
    end;
  end;
end;

function TPlannerItems.HeaderNext(ItemPos: Integer): TPlannerItem;
var
  ItemIndex: Integer;
  APlannerItem: TPlannerItem;
begin
  Result := nil;
  for ItemIndex := FFindIndex to Count do
  begin
    APlannerItem := Items[ItemIndex - 1];
    if (APlannerItem.ItemPos = ItemPos) and (APlannerItem.FInHeader) then
    begin
      Result := APlannerItem;
      FFindIndex := ItemIndex + 1;
      Break;
    end;
  end;
end;


procedure TPlannerItems.ClearAll;
begin
  Clear;
end;

procedure TPlannerItems.ClearDB;
var
  i: Integer;
begin
  i := 0;
  while i < Count do
  begin
    if not Items[i].NonDBItem then
      Items[i].Free
    else
      inc(i);
  end;
  FOwner.FSelectedIndex := -1;
end;

procedure TPlannerItems.GetTimeTags;
var
  i: Integer;
begin
  if not FOwner.FStreamPersistentTime then
    Exit;

  for i := 1 to Count do
    Items[i - 1].GetTimeTag;
end;

procedure TPlannerItems.SetTimeTags;
var
  i: Integer;
begin
  for i := 1 to Count do
    Items[i - 1].SetTimeTag;
end;

function TPlannerItems.FindKey(DBKey: string): TPlannerItem;
var
  i: Integer;
begin
  Result := Nil;

  for i := 1 to Count do
  begin
    if Items[i - 1].DBKey = DBKey then
    begin
      Result := Items[i - 1];
      Break;
    end;
  end;
end;

procedure TPlannerItems.Select(Item: TPlannerItem);
begin
  UnSelect;
  Item.Focus := True;
  Item.Selected := True;
  FSelected := Item;
end;


function TPlannerItems.MatchItem(Item: TPlannerItem; s: string;
  Param: TFindTextParams): Boolean;
var
  SrchText: string;

begin
  Result := False;

  if fnCaptionText in Param then
  begin
    SrchText := Item.CaptionText;

    if fnIgnoreHTMLTags in Param then
      SrchText := HTMLStrip(SrchText);

    if not (fnMatchCase in Param) then
    begin
      SrchText := AnsiUpperCase(SrchText);
      s := AnsiUpperCase(s);
    end;

    if fnMatchFull in Param then
      Result := (s = SrchText)
    else
      if fnMatchStart in Param then
        Result := Pos(s,SrchText) = 1
      else
        if fnMatchRegular in Param then
          Result := MatchStr(s,SrchText,fnMatchCase in Param);
  end;

  if Result then Exit;

  if fnText in Param then
  begin
    SrchText := Item.Text.Text;

    if fnIgnoreHTMLTags in Param then
      SrchText := HTMLStrip(SrchText);

    if not (fnMatchCase in Param) then
    begin
      SrchText := AnsiUpperCase(SrchText);
      s := AnsiUpperCase(s);
    end;

    if fnMatchFull in Param then
      Result := (s = SrchText)
    else
      if fnMatchStart in Param then
        Result := Pos(s,SrchText) = 1
      else
        if fnMatchRegular in Param then
          Result := MatchStr(s,SrchText,fnMatchCase in Param)
            else
              Result := Pos(s,SrchText) > 0;
  end;

end;

function TPlannerItems.FindText(StartItem: TPlannerItem; s: string;
  Param: TFindTextParams): TPlannerItem;
var
  i: Integer;

begin
  Result := nil;
  if Count = 0 then
    Exit;

  if not (fnBackward in Param) then
  begin
    i := 0;

    if Assigned(StartItem) then
    begin
      while i < Count do
      begin
        if Items[i] = StartItem then
          Break
        else
          Inc(i);
      end;

      if i < Count then
        Inc(i)
      else
        i := 0;
    end;

    while i < Count do
    begin
      if MatchItem(Items[i],s,Param) then
      begin
        Result := Items[i];
        if fnAutoGoto in Param then
        begin
          Items[i].ScrollInView;
          Items[i].SetFocus(True);
          Items[i].Selected := True;
        end;
        Break;
      end
      else
        inc(i);
  end;
  end
  else
  begin
    i := Count - 1;

    if Assigned(StartItem) then
    begin
      while i > 0 do
      begin
        if Items[i] = StartItem then
          Break
        else
          Dec(i);
      end;

      if i > 0 then
        Dec(i)
      else
        i := Count - 1;
    end;

    while i > 0 do
    begin
      if MatchItem(Items[i],s,Param) then
      begin
        Result := Items[i];
        if fnAutoGoto in Param then
        begin
          Items[i].ScrollInView;
          Items[i].SetFocus(True);
          Items[i].Selected := True;
        end;
        Break;
      end
      else
        Dec(i);
    end;
  end;
end;

function TPlannerItems.MaxItemsInHeader(Position : integer) : integer;
var
  i, m : integer;
  APlannerItem : TPlannerItem;
begin
  m := 0;
  for i :=1 to Count do
  begin
    APlannerItem := Items[i-1];

    if (APlannerItem.ItemPos = Position) and (APlannerItem.Visible) and
      (InVisibleLayer(APlannerItem.Layer)) and (APlannerItem.InHeader) then
    begin
        m := m + 1;
        APlannerItem.FConflicts := FOwner.Items.MaxItemsInPos(Position);
//        APlannerItem.FConflicts := m;
    end;
  end;

  Result := m;
end;

function TPlannerItems.MaxItemsInPos(Position: Integer): Integer;
var
  i,m : Integer;
  APlannerItem: TPlannerItem;
begin
  m := 1;

  for i := 1 to Count do
  begin
    APlannerItem := Items[i - 1];

    if (APlannerItem.ItemPos = Position) and (APlannerItem.Visible) and
       (InVisibleLayer(APlannerItem.Layer)) then
    begin
      if APlannerItem.Conflicts > m then
      begin
        m := APlannerItem.Conflicts;
      end;
    end;
  end;
  Result := m;
end;

function TPlannerItems.MaxItemsInPosAtRow(Position, Row: Integer): Integer;
var
  i,m : Integer;
  APlannerItem: TPlannerItem;
begin
  m := 1;
  for i :=1 to Count do
  begin
    APlannerItem := Items[i - 1];

    if ((APlannerItem.ItemPos = Position) and (APlannerItem.ItemBegin <= Row)
//    and not (APlannerItem.InHeader) //added
    and (Row < APlannerItem.ItemEnd) and (APlannerItem.Visible) and (InVisibleLayer(APlannerItem.Layer))) then
    begin
      if APlannerItem.Conflicts > m then
        m := APlannerItem.Conflicts;
    end;
  end;
  Result := m;
end;

function TPlannerItems.ItemsAtIndex(Idx: Integer): Integer;
var
  APlannerItem: TPlannerItem;
  i: Integer;
begin
  Result := 0;

  for i := 1 to Count do
  begin
    APlannerItem := Items[i - 1];
    if (APlannerItem.ItemBegin <= Idx) and (APlannerItem.ItemEnd > Idx)
     and APlannerItem.Visible and InVisibleLayer(APlannerItem.Layer) then
      Result := Result + 1;
  end;
end;

function TPlannerItems.ItemsAtPosition(Pos: Integer): Integer;
var
  APlannerItem: TPlannerItem;
  i: Integer;
begin
  Result := 0;

  for i := 1 to Count do
  begin
    APlannerItem := Items[i - 1];
    if (APlannerItem.ItemPos = Pos) and APlannerItem.Visible and
       InVisibleLayer(APlannerItem.Layer) then
      Result := Result + 1;
  end;
end;

function TPlannerItems.GetItemClass: TCollectionItemClass;
begin
  Result := TPlannerItem;
end;

procedure TPlannerItems.ItemChanged(Item:TPlannerItem);
var
  i: Integer;

begin
  if FChanging then
    Exit;

  if Item.ParentIndex = -1 then
    Exit;

  FChanging := True;

  for i := 1 to Count do
  begin
    if (Items[i - 1] <> Item) and (Item.ParentIndex = Items[i - 1].ParentIndex) then
    begin
      Items[i - 1].AssignEx(Item);
    end;
  end;

  FChanging := False;
end;

function TPlannerItems.GetSelCount: Integer;
var
  i: Integer;
begin
  Result := 0;
  for i := 1 to Count do
  begin
    if Items[i - 1].Selected then
      Inc(Result);
  end;
end;

{TPlannerIntList}

constructor TPlannerIntList.Create(Value: TPlannerItem);
begin
  inherited Create;
  FPlannerItem := Value;
end;

procedure TPlannerIntList.SetInteger(Index: Integer; Value: Integer);
begin
  inherited Items[Index] := Pointer(Value);
  if Assigned(OnChange) then
    OnChange(Self);
end;

function TPlannerIntList.GetInteger(Index: Integer): Integer;
begin
  Result := Integer(inherited Items[Index]);
end;

procedure TPlannerIntList.Add(Value: Integer);
begin
  inherited Add(Pointer(Value));
  if Assigned(OnChange) then
    OnChange(Self);
end;

procedure TPlannerIntList.Delete(Index: Integer);
begin
  inherited Delete(Index);
  if Assigned(OnChange) then
    OnChange(Self);
end;

{ TPlannerMode }

constructor TPlannerMode.Create(AOwner: TTIWCustomWebPlanner);
var
  Day, Month, Year: Word;
begin
  inherited Create;
  FOwner := AOwner;
  FPlannerType := plDay;
  FWeekStart := 0;
  FUpdateCount := 0;

  DecoDedate(Now, Year, Month, Day);
  FYear := Year;
  FMonth := Month;

  // default start time in DayPeriod mode
  FPeriodStartYear := Year;
  FPeriodStartMonth := Month;
  FPeriodStartDay := Day;

  // default end time in DayPeriod mode
  DecoDedate(Now + 48, Year, Month, Day);
  FPeriodEndYear := Year;
  FPeriodEndMonth := Month;
  FPeriodEndDay := Day;

  FTimeLineStart := Int(Now);
end;

destructor TPlannerMode.Destroy;
begin
  inherited Destroy;
end;

procedure TPlannerMode.SetMonth(const Value: Integer);
begin
  FMonth := Value;
  //Modified 11/04/2003 - show correct number of days in current month
  if (PlannerType = plMonth) then
    FOwner.Display.DisplayEnd := DaysInMonth(FMonth, FYear) - 1;
  //End modified
end;

procedure TPlannerMode.SetPlannerType(const Value: TPlannerType);
begin
  FPlannerType := Value;
end;

procedure TPlannerMode.SetWeekStart(const Value: Integer);
begin
  FWeekStart := Value;
end;

procedure TPlannerMode.SetYear(const Value: Integer);
begin
  FYear := Value;
  //Modified 11/04/2003 - show correct number of days in current month
  if (PlannerType = plMonth) then
    FOwner.Display.DisplayEnd := DaysInMonth(FMonth, FYear) - 1;
  //End modified
end;

procedure TPlannerMode.SetDateTimeFormat(const Value: string);
begin
  FDateTimeFormat := Value;
end;

procedure TPlannerMode.SetPeriodStartDay(const Value: Integer);
begin
  if (Value > 0) and (Value < 32) then
  begin
    FPeriodStartDay := Value;
    UpdatePeriod;
  end;
end;

procedure TPlannerMode.SetPeriodStartMonth(const Value: Integer);
begin
  if (Value > 0) and (Value < 13) then
  begin
    FPeriodStartMonth := Value;
    UpdatePeriod;
  end;
end;

procedure TPlannerMode.SetPeriodStartYear(const Value: Integer);
begin
  FPeriodStartYear := Value;
  UpdatePeriod;
end;

procedure TPlannerMode.SetPeriodEndDay(const Value: Integer);
begin
  if (Value > 0) and (Value < 32) then
  begin
    FPeriodEndDay := Value;
    UpdatePeriod;
  end;
end;

procedure TPlannerMode.SetPeriodEndMonth(const Value: Integer);
begin
  if (Value > 0) and (Value < 13) then
  begin
    FPeriodEndMonth := Value;
    UpdatePeriod;
  end;
end;

procedure TPlannerMode.SetPeriodEndYear(const Value: Integer);
begin
  FPeriodEndYear := Value;
  UpdatePeriod;
end;

function LimitMonth(Month: word):word;
begin
  Result := Month;
  if Result > 12 then Result := 12;
  if Result < 1 then Result := 1;
end;

function LimitDay(Year, Month, Day: word):word;
const
  NumDays : array[1..12] of word = (31,29,31,30,31,30,31,31,30,31,30,31);
begin

  if Day > NumDays[Month] then
    Result := NumDays[Month]
  else
    Result := Day;

  if (Month = 2) and (Result = 29) then
    if (Year mod 4 <> 0) then
      Result := 28;
end;

function EncodePlannerDate(Year, Month, Day: Word): TDateTime;
begin
   if (Year = 0) or (Month = 0) or (Day = 0) then
    Result := 0
  else
    try
      Month := LimitMonth(Month);
      Day := LimitDay(Year,Month,Day);

      Result := EncodeDate(Year, Month, Day);
    except
      Result := 0;
    end;
end;

function TPlannerMode.GetPeriodEndDate: TDateTime;
begin
  case PlannerType of
  plDay,plHalfDayPeriod: Result := EncodePlannerDate(PeriodEndYear, PeriodEndMonth, PeriodEndDay);
  plMonth,plWeek: Result := GetStartOfMonth +
                     (FOwner.Display.FDisplayEnd - FOwner.Display.FDisplayStart)
  else
    Result := EncodePlannerDate(PeriodEndYear, PeriodEndMonth, PeriodEndDay);
  end;
end;

function TPlannerMode.GetPeriodStartDate: TDateTime;
begin
  case PlannerType of
  plDay,plHalfDayPeriod: Result := EncodePlannerDate(PeriodStartYear, PeriodStartMonth, PeriodStartDay);
  plWeek, plMonth: Result := GetStartOfMonth
  else
    Result := EncodePlannerDate(PeriodStartYear, PeriodStartMonth, PeriodStartDay);
  end;
end;

function TPlannerMode.GetStartOfMonth: TDateTime;
var
  Diff: Integer;
begin
  Result := EncodePlannerDate(Year, Month, 1);
  if PlannerType = plWeek then
  begin
    Diff := (7 - WeekStart) - DayOfWeek(Result);
    if Diff > 0 then
      Diff := Diff - 7;
    Result := Result + Diff;
  end;
end;

procedure TPlannerMode.UpdatePeriod;
var
  DS,DE: TDateTime;
begin
  if FUpdateCount > 0 then
    Exit;

  DS := EncodePlannerDate(FPeriodStartYear,FPeriodStartMonth,FPeriodStartDay);
  DE := EncodePlannerDate(FPeriodEndYear,FPeriodEndMonth,FPeriodEndDay);

  if (DS < DE) and (DS > 0) and (DE > 0) and
     not (csLoading in FOwner.ComponentState) and
     (PlannerType = plDayPeriod) then
  begin
    FOwner.Display.BeginUpdate;
    FOwner.Display.DisplayStart := 0;
    FOwner.Display.DisplayEnd := Trunc(DE - DS);
    FOwner.Display.EndUpdate;
  end;

  if (DS < DE) and (DS > 0) and (DE > 0) and
     not (csLoading in FOwner.ComponentState) and
     (PlannerType = plHalfDayPeriod) then
  begin
    FOwner.Display.BeginUpdate;
    FOwner.Display.DisplayStart := 0;
    FOwner.Display.DisplayEnd := 2 * Trunc(DE - DS) + 1;
    FOwner.Display.EndUpdate;
  end;
end;

procedure TPlannerMode.SetPeriodEndDate(const Value: TDateTime);
var
  Da,Mo,Ye: word;
begin
  DecodeDate(Value,Ye,Mo,Da);
  FPeriodEndYear := Ye;
  FPeriodEndMonth := Mo;
  FPeriodEndDay := Da;
  UpdatePeriod;
end;

procedure TPlannerMode.SetPeriodStartDate(const Value: TDateTime);
var
  Da,Mo,Ye: word;
begin
  DecodeDate(Value,Ye,Mo,Da);
  FPeriodStartYear := Ye;
  FPeriodStartMonth := Mo;
  FPeriodStartDay := Da;
  UpdatePeriod;
end;

procedure TPlannerMode.BeginUpdate;
begin
  Inc(FUpdateCount);
end;

procedure TPlannerMode.EndUpdate;
begin
  if FUpdateCount > 0 then
  begin
    Dec(FUpdateCount);
    if FUpdateCount = 0 then
      UpdatePeriod;
  end;
end;

procedure TPlannerMode.SetTimeLineStart(const Value: TDateTime);
begin
  FTimeLineStart := Value;
end;

procedure TPlannerMode.Assign(Source: TPersistent);
begin
  if (Source is TPlannerMode) then
  begin
    FClip := (Source as TPlannerMode).Clip;
    FDateTimeFormat := (Source as TPlannerMode).DateTimeFormat;
    FMonth := (Source as TPlannerMode).Month;
    PeriodStartDay := (Source as TPlannerMode).PeriodStartDay;
    PeriodStartMonth := (Source as TPlannerMode).PeriodStartMonth;
    PeriodStartYear := (Source as TPlannerMode).PeriodStartYear;
    PeriodEndDay := (Source as TPlannerMode).PeriodEndDay;
    PeriodEndMonth := (Source as TPlannerMode).PeriodEndMonth;
    PeriodEndYear := (Source as TPlannerMode).PeriodEndYear;
    PlannerType := (Source as TPlannerMode).PlannerType;
    TimeLineStart := (Source as TPlannerMode).TimeLineStart;
    WeekStart := (Source as TPlannerMode).WeekStart;
    Year := (Source as TPlannerMode).Year;
  end;
end;

{ TDateTimeList }

procedure TDateTimeList.Add(Value: TDateTime);
var
  tdt: TDateTimeObject;
begin
  tdt := TDateTimeObject.Create;
  tdt.DT := Value;
  inherited Add(tdt);
end;

procedure TDateTimeList.Delete(Index: Integer);
begin
  TDateTimeObject(inherited Items[Index]).Free;
  inherited Delete(Index);
end;

function TDateTimeList.GetDT(index: Integer): TDateTime;
begin
  Result := TDateTimeObject(inherited Items[Index]).DT;
end;

procedure TDateTimeList.Insert(Index: Integer; Value: TDateTime);
var
  tdt: TDateTimeObject;
begin
  tdt := TDateTimeObject.Create;
  tdt.DT := Value;
  inherited Insert(Index,tdt);
end;

procedure TDateTimeList.Clear;
begin
  while Count > 0 do
    Delete(0);
end;

procedure TDateTimeList.SetDT(index: Integer; const Value: TDateTime);
begin
  TDateTimeObject(inherited Items[Index]).DT := Value;
end;

{ TPlannerSideBar }

constructor TPlannerSideBar.Create(AOwner: TTIWCustomWebPlanner);
begin
  inherited Create;
  FOwner := AOwner;
  FOrientation := soVertical;
  FPosition := spLeft;
  FSideBarFont := TIWFont.Create;
  FSideBarFont.FontName := 'Verdana';
  FSideBarFont.Color := clBlack;
  FSideBarFont.Size := 9;
  FOccupiedFont := TIWFont.Create;
  FAlignment := taRightJustify;
  FShowOccupied := true;
  FBackground := clBtnFace;
  FBackgroundTo := clNone;
  FShowOccupied := true;
  FOccupiedColor := clAqua;
  FOccupiedColorTo := clNone;
  FOccupiedFont.Color := clRed;
  FOccupiedFont.FontName := 'Verdana';
  FOccupiedFont.Size := 9;
  FOccupiedFont.Style := [fsItalic];
  FVisible := true;
  FShowHint := true;
end;

destructor TPlannerSideBar.Destroy;
begin
  FreeAndNil(FSideBarFont);
  FreeAndNil(FOccupiedFont);
end;

function TPlannerSideBar.GetOrientation: TSideBarOrientation;
begin
  if Position = spTop then
    Result := soHorizontal
  else
    Result := soVertical;
end;

procedure TPlannerSideBar.SetWidth(const Value: Integer);
begin
  FWidth := Value;
  FOwner.InvalidateX;
end;

procedure TPlannerSidebar.SetBackground(const Value: TIWColor);
begin
  FBackground := Value;
  FOwner.InvalidateX;
end;

procedure TPlannerSideBar.SetAlignment(const Value: TAlignment);
begin
  FAlignment := Value;
end;

procedure TPlannerSideBar.SetOccupiedColor(const Value: TIWColor);
begin
  FOccupiedColor := Value;
end;

procedure TPlannerSideBar.SetOccupiedColorTo(const Value: TIWColor);
begin
  FOccupiedColorTo := Value;
end;

procedure TPlannerSideBar.SetShowHint(const Value: Boolean);
begin
  FShowHint := Value;
end;

procedure TPlannerSideBar.SetSideBarFont(const Value: TIWFont);
begin
  FSideBarFont.Assign(Value);
  FOwner.InvalidateX;
end;

procedure TPlannerSideBar.SetOccupiedFont(const Value: TIWFont);
begin
  FOccupiedFont.Assign(Value);
end;

procedure TPlannerSideBar.SetBackgroundTo(const Value: TIWColor);
begin
  FBackgroundTo := Value;
end;

procedure TPlannerSideBar.SetShowOccupied(const Value: Boolean);
begin
  FShowOccupied := Value;
end;

procedure TPlannerSideBar.SetPosition(const Value: TSideBarPosition);
begin
{
  if (FPosition <> spTop) and (Value = spTop) then
  begin
    FOwner.Height := FOwner.Width;
    FOwner.Width := FOwner.Display.DisplayScale * (FOwner.Display.DisplayEnd - FOwner.Display.DisplayStart + 1) + FOwner.Header.HeaderItemWidth;

    if (FOwner.Header.Visible) then
    begin
      if (FOwner.PositionGroup > 0) then
        FOwner.Width := FOwner.Width + 2 * FOwner.Header.CaptionHeight
      else
        FOwner.Width := FOwner.Width + FOwner.Header.CaptionHeight;
    end;
  end
  else
  begin
    if (FPosition = spTop) and (Value <> spTop) then
    begin
      FOwner.Width := FOwner.Height;
      FOwner.Height := FOwner.Display.DisplayScale * (FOwner.Display.DisplayEnd - FOwner.Display.DisplayStart + 1);
      if (FOwner.Header.Visible) then
      begin
        if (FOwner.PositionGroup > 0) then
          FOwner.Height := FOwner.Height + 2 * FOwner.Header.CaptionHeight
        else
          FOwner.Height := FOwner.Height + FOwner.Header.CaptionHeight;
      end;
    end;
  end;
}
  FPosition := Value;
  if FPosition = spTop then
    Orientation := soHorizontal
  else
    Orientation := soVertical;

  FOwner.InvalidateX;
end;

procedure TPlannerSideBar.SetGradientDirection(const Value: TIWGradientDirection);
begin
  FGradientDirection := Value;
end;

procedure TPlannerSideBar.SetShowDayName(const Value: Boolean);
begin
  FShowDayName := Value;
end;

procedure TPlannerSideBar.SetDateTimeFormat(const Value: String);
begin
  FDateTimeFormat := Value;
end;

procedure TPlannerSideBar.SetVisible(const Value: Boolean);
begin
  FVisible := Value;
  FOwner.InvalidateX;
end;

function TPlannerSideBar.GetUsedWidth: Integer;
begin
  case Position of
  spLeft,spRight,spTop: Result := Width;
  spLeftRight: Result := Width * 2;
  else
    Result := 0;
  end;

  if not Visible then
    Result := 0;
end;

{ TPlannerHeader }

procedure TPlannerHeader.Assign(Source: TPersistent);
begin
  if Assigned(Source) then
  begin
    Alignment := (Source as TPlannerHeader).Alignment;
    Captions.Assign((Source as TPlannerHeader).Captions);
    Background := (Source as TPlannerHeader).Background;
    BackgroundTo := (Source as TPlannerHeader).BackgroundTo;
    CaptionHeight := (Source as TPlannerHeader).CaptionHeight;
    LineColor := (Source as TPlannerHeader).LineColor;
    GroupCaptions.Assign((Source as TPlannerHeader).GroupCaptions);
    HeaderItemWidth := (Source as TPlannerHeader).HeaderItemWidth;
    Visible := (Source as TPlannerHeader).Visible;
    Font := (Source as TPlannerHeader).Font;
  end;
end;

constructor TPlannerHeader.Create(AOwner: TTIWCustomWebPlanner);
begin
  inherited Create;
  FOwner := AOwner;
  FCaptions := TStringList.Create;
  FGroupCaptions := TStringList.Create;
  FCaptionHeight := 20;
  FColor := clBtnFace;
  FHeaderFont := TIWFont.Create;
  FHeaderFont.FontName := 'Verdana';
  FBackgroundTo := clBtnFace;
  FLineColor := clGray;
  FVisible := True;
  FHeaderItemWidth := 80;
end;

destructor TPlannerHeader.Destroy;
begin
  FCaptions.Free;
  FGroupCaptions.Free;
  FreeAndNil(FHeaderFont);
  inherited Destroy;
end;

procedure TPlannerHeader.SetHeaderPosition(const Value : THeaderPosition);
begin
  FPosition := Value;
end;

procedure TPlannerHeader.SetBackgroundTo(const Value: TIWColor);
begin
  FBackgroundTo := Value;
end;

procedure TPlannerHeader.SetAlignment(const Value: TAlignment);
begin
  FAlignment := Value;
end;

procedure TPlannerHeader.SetCaptionHeight(const Value: integer);
begin
  FCaptionHeight := Value;
  FOwner.InvalidateX;
end;

procedure TPlannerHeader.SetVisible(const Value: Boolean);
begin
  FVisible := Value;
  FOwner.InvalidateX;
end;

procedure TPlannerHeader.SetHeaderItemWidth(const Value : integer);
begin
  FHeaderItemWidth := Value;
end;

procedure TPlannerHeader.SetGradientDirection(const Value: TIWGradientDirection);
begin
  FGradientDirection := Value;
end;

procedure TPlannerHeader.SetHeaderFont(const Value: TIWFont);
begin
  FHeaderFont.Assign(Value);
  FOwner.InvalidateX;
end;

procedure TPlannerHeader.SetCaptions(const Value: TStringList);
begin
  if Assigned(Value) then
    FCaptions.Assign(Value);
end;

procedure TPlannerHeader.SetColor(const Value: TIWColor);
begin
  FColor := Value;
  FOwner.InvalidateX;
end;

procedure TPlannerHeader.SetGroupCaptions(const Value: TStringList);
begin
  FGroupCaptions.Assign(Value);
end;

procedure TPlannerHeader.SetLineColor(const Value: TIWColor);
begin
  FLineColor := Value;
  FOwner.FPlannerHeader.FLineColor := Value;
end;

{ TBands }

constructor TBands.Create(AOwner: TTIWCustomWebPlanner);
begin
  inherited Create;
  FOwner := AOwner;
  FActivePrimary := $00FEE7CB;
  FActiveSecondary := $00FAD9AF;
  FNonActivePrimary := clSilver;
  FNonActiveSecondary := $00A8A8A8;
end;

procedure TBands.SetActivePrimary(const Value: TIWColor);
begin
  FActivePrimary := Value;
  FOwner.InvalidateX;
end;

procedure TBands.SetActiveSecondary(const Value: TIWColor);
begin
  FActiveSecondary := Value;
  FOwner.InvalidateX;
end;

procedure TBands.SetNonActivePrimary(const Value: TIWColor);
begin
  FNonActivePrimary := Value;
  FOwner.InvalidateX;
end;

procedure TBands.SetNonActiveSecondary(const Value: TIWColor);
begin
  FNonActiveSecondary := Value;
  FOwner.InvalidateX;
end;

procedure TBands.SetShow(const Value: Boolean);
begin
  FShow := Value;
  FOwner.InvalidateX;
end;

{ TPositionProps }

function TPositionProps.Add: TPositionProp;
begin
  Result := TPositionProp(Inherited Add);
end;

constructor TPositionProps.Create(AOwner: TTIWCustomWebPlanner);
begin
  Inherited Create(TPositionProp);
  FOwner := AOwner;
end;

function TPositionProps.GetItem(Index: Integer): TPositionProp;
begin
  Result := TPositionProp(inherited Items[Index]);
end;

function TPositionProps.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

function TPositionProps.Insert(Index: Integer): TPositionProp;
begin
  Result := TPositionProp(Inherited Insert(Index));
end;

procedure TPositionProps.SetItem(Index: Integer;
  const Value: TPositionProp);
begin
  inherited SetItem(Index, Value);
end;

{ TPositionProp }

procedure TPositionProp.Assign(Source: TPersistent);
begin
  inherited;
  FActiveStart := (Source as TPositionProp).ActiveStart;
  FActiveEnd := (Source as TPositionProp).ActiveEnd;
  FColorActive := (Source as TPositionProp).ColorActive;
  FColorNonActive := (Source as TPositionProp).ColorNonActive;
  FMinSelection := (Source as TPositionProp).MinSelection;
  FMaxSelection := (Source as TPositionProp).MaxSelection;
  FColorNoSelect := (Source as TPositionProp).ColorNoSelect;
  FUse := (Source as TPositionProp).Use;
end;


constructor TPositionProp.Create(Collection: TCollection);
begin
  inherited;
  FActiveStart := TPositionProps(Collection).FOwner.Display.ActiveStart;
  FActiveEnd := TPositionProps(Collection).FOwner.Display.ActiveEnd;
  FColorActive := TPositionProps(Collection).FOwner.Display.ColorActive;
  FColorNonActive := TPositionProps(Collection).FOwner.Display.ColorNonActive;
  FColorNoSelect := clRed;
  FUse := True;
end;

destructor TPositionProp.Destroy;
begin
  TPositionProps(Collection).FOwner.InvalidateX;
//  FBackground.Free;
  inherited;
end;

procedure TPositionProp.SetActiveEnd(const Value: Integer);
begin
  FActiveEnd := Value;
  TPositionProps(Collection).FOwner.InvalidateX;
end;

procedure TPositionProp.SetActiveStart(const Value: Integer);
begin
  FActiveStart := Value;
  TPositionProps(Collection).FOwner.InvalidateX;
end;

procedure TPositionProp.SetColorActive(const Value: TIWColor);
begin
  FColorActive := Value;
  TPositionProps(Collection).FOwner.InvalidateX;
end;

procedure TPositionProp.SetColorNonActive(const Value: TIWColor);
begin
  FColorNonActive := Value;
  TPositionProps(Collection).FOwner.InvalidateX;
end;

procedure TPositionProp.SetColorNoSelect(const Value: TIWColor);
begin
  FColorNoSelect := Value;
  TPositionProps(Collection).FOwner.InvalidateX;
end;

procedure TPositionProp.SetMaxSelection(const Value: Integer);
begin
  FMaxSelection := Value;
  TPositionProps(Collection).FOwner.InvalidateX;
end;

procedure TPositionProp.SetMinSelection(const Value: Integer);
begin
  FMinSelection := Value;
  TPositionProps(Collection).FOwner.InvalidateX;
end;

procedure TPositionProp.SetUse(const Value: Boolean);
begin
  FUse := Value;
  TPositionProps(Collection).FOwner.InvalidateX;
end;

{ TIWWebPlannerClientEvents }

procedure TIWWebPlannerClientEvents.Assign(Source: TPersistent);
begin
  inherited;
end;

constructor TIWWebPlannerClientEvents.Create;
begin
  FEditDone := TStringList.Create;
  FEditStart := TStringList.Create;
  FDeleteClick := TStringList.Create;
  FSelect := TStringList.Create;
end;

destructor TIWWebPlannerClientEvents.Destroy;
begin
  FEditDone.Free;
  FDeleteClick.Free;
  FSelect.Free;
  FEditStart.Free;
  inherited;
end;

procedure TIWWebPlannerClientEvents.SetDeleteClick(
  const Value: TStringList);
begin
  FDeleteClick.Assign(Value);
end;

procedure TIWWebPlannerClientEvents.SetEditDone(const Value: TStringList);
begin
  FEditDone.Assign(Value);
end;

procedure TIWWebPlannerClientEvents.SetEditStart(const Value: TStringList);
begin
  FEditStart.Assign(Value);
end;

procedure TIWWebPlannerClientEvents.SetSelect(const Value: TStringList);
begin
  FSelect.Assign(Value);
end;

{ TGlyphs }

constructor TGlyphs.Create(AOwner: TComponent);
begin
  inherited Create;
  FOwner := AOwner;
  FCancelItem := TBitmap.Create;
  FDeleteItem := TBitmap.Create;
  FEditItem := TBitmap.Create;
  FUpdateItem := TBitmap.Create;
  FCrossingStart := TBitmap.Create;
  FCrossingEnd := TBitmap.Create;
  if (csDesigning in FOwner.ComponentState) then
  begin
    FDeleteItem.LoadFromResourceName(HInstance,'IWTMSDEL');
    FUpdateItem.LoadFromResourceName(HInstance,'IWTMSUPD');
    FCancelItem.LoadFromResourceName(HInstance,'IWTMSCAN');
  end;
end;

destructor TGlyphs.Destroy;
begin
  FCancelItem.Free;
  FDeleteItem.Free;
  FEditItem.Free;
  FUpdateItem.Free;
  FCrossingStart.Free;
  FCrossingEnd.Free;
  inherited;
end;

procedure TGlyphs.SetCancelItem(const Value: TBitmap);
begin
  FCancelItem.Assign(Value);
end;

procedure TGlyphs.SetCancelItemFile(const Value : string);
begin
  FCancelItemFile := Value;
end;

procedure TGlyphs.SetDeleteItem(const Value: TBitmap);
begin
  FDeleteItem.Assign(Value);
end;

procedure TGlyphs.SetDeleteItemFile(const Value: string);
begin
  FDeleteItemFile := Value;
end;

procedure TGlyphs.SetEditItem(const Value: TBitmap);
begin
  FEditItem.Assign(Value);
end;

procedure TGlyphs.SetEditItemFile(const Value: string);
begin
  FEditItemFile := Value;
end;

procedure TGlyphs.SetUpdateItem(const Value: TBitmap);
begin
  FUpdateItem.Assign(Value);
end;

procedure TGlyphs.SetUpdateItemFile(const Value: string);
begin
  FUpdateItemFile := Value;
end;

procedure TGlyphs.SetCrossingStart(const value : TBitmap);
begin
  FCrossingStart.Assign(Value);
end;

procedure TGlyphs.SetCrossingStartFile(const value : string);
begin
  FCrossingStartFile := Value;
end;

procedure TGlyphs.SetCrossingEnd(const Value : TBitmap);
begin
  FCrossingEnd.Assign(Value);
end;

procedure TGlyphs.SetCrossingEndFile(const Value : string);
begin
  FCrossingEndFile := Value;
end;

{ THints }

constructor THints.Create(AOwner: TComponent);
begin
  inherited Create;
  FOwner := AOwner;
end;

destructor THints.Destroy;
begin
  inherited;
end;

//Modified 10/29/2003 - CaptionHint
procedure THints.SetCaptionHint(const Value : TCaptionHint);
begin
  FCaptionHint := Value;
end;
//End modified

procedure THints.SetHintCancelItem(const Value: String);
begin
  FHintCancelItem := Value;
end;

procedure THints.SetHintConfirmDialog(const Value: String);
begin
  FHintConfirmDialog := Value;
end;

procedure THints.SetHintDeleteItem(const Value: String);
begin
  FHintDeleteItem := Value;

end;

procedure THints.SetHintEditItem(const Value: String);
begin
  FHintEditItem := Value;
end;

procedure THints.SetHintUpdateItem(const Value: String);
begin
  FHintUpdateItem := Value;
end;

{ TWeekDays }

procedure TWeekDays.Changed;
begin
  if Assigned(OnChanged) then
    OnChanged(Self);
end;

constructor TWeekDays.Create;
begin
  FSun := True;
  FSat := True;
end;

procedure TWeekDays.SetFri(const Value: Boolean);
begin
  FFri := Value;
  Changed;
end;

procedure TWeekDays.SetMon(const Value: Boolean);
begin
  FMon := Value;
  Changed;
end;

procedure TWeekDays.SetSat(const Value: Boolean);
begin
  FSat := Value;
  Changed;
end;

procedure TWeekDays.SetSun(const Value: Boolean);
begin
  FSun := Value;
  Changed;
end;

procedure TWeekDays.SetThu(const Value: Boolean);
begin
  FThu := Value;
  Changed;
end;

procedure TWeekDays.SetTue(const Value: Boolean);
begin
  FTue := Value;
  Changed;
end;

procedure TWeekDays.SetWed(const Value: Boolean);
begin
  FWed := Value;
  Changed;
end;


{ TPlannerIO }

constructor TPlannerIO.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FItems := (AOwner as TTIWCustomWebPlanner).CreateItems;
end;

destructor TPlannerIO.Destroy;
begin
  FItems.Free;
  inherited Destroy;
end;

{ TPlannerItemIO }

constructor TPlannerItemIO.CreateItem(AOwner: TPlannerItems);
begin
  inherited Create(nil);

  FItem := TPlannerItem(AOwner.GetItemClass.Create(AOwner));
end;

destructor TPlannerItemIO.Destroy;
begin
  FItem.Free;
  inherited;
end;


{ TPopupEdit }

constructor TPopupEdit.Create(AOwner: TTIWCustomWebPlanner);
begin
  FOwner := AOwner;
  FBackgroundImage := TIPicture.Create;
  FSubjectCaption := 'Subject :';
  FNotesCaption := 'Notes :';
  FPopupBorderColor := clBlack;
  FpopupBackColor := clWhite;
  FPopupHeight := 170;
  FPopupWidth := 150;
  FFont := TIWFont.Create;
  FFont.Color := clBlack;
end;

destructor TPopupEdit.Destroy;
begin
  FBackgroundImage.Free;
  FFont.Free;
  inherited;
end;

procedure TPopupEdit.SetBackgroundImage(const Value: TIPicture);
begin
  FBackgroundImage.Assign(Value);
end;

procedure TPopupEdit.SetFont(const Value: TIWFont);
begin
  FFont.Assign(Value);
end;

procedure TPopupEdit.SetNotesCaption(const Value: string);
begin
  FNotesCaption := Value;
end;

procedure TPopupEdit.SetPopupBackColor(const Value: TIWColor);
begin
  FPopupBackColor := Value;
end;

procedure TPopupEdit.SetPopupBorderColor(const Value: TIWColor);
begin
  FPopupBorderColor := Value;
end;

procedure TPopupEdit.SetPopupHeight(const Value : integer);
begin
  FPopupHeight := Value;
end;

procedure TPopupEdit.SetPopupWidth(const Value : integer);
begin
  FPopupWidth := Value;
end;

procedure TPopupEdit.SetSubjectCaption(const Value: string);
begin
  FSubjectCaption := Value;
end;

{ TCaptionChange }

constructor TChangeCaption.Create(AOwner: TTIWCustomWebPlanner);
begin
  FOwner := AOwner;
  FShowDialogWindow := true;
  FShowPromptWindow := true;
  FTextDialogWindow := 'Do you want to change the caption of the item?';
  FTextPromptWindow := 'Enter the captiontext of the item: ';
  FEnterCaptionAfterItemCreate := true;
end;

destructor TChangeCaption.Destroy;
begin
  inherited;
end;

procedure TChangeCaption.SetEnterCaptionAfterItemCreate(
  const Value: Boolean);
begin
  FEnterCaptionAfterItemCreate := Value;
end;

procedure TChangeCaption.SetShowDialogWindow(const Value: Boolean);
begin
  FShowDialogWindow := Value;
end;

procedure TChangeCaption.SetShowPromptWindow(const Value: Boolean);
begin
  FShowPromptWindow := Value;
end;

procedure TChangeCaption.SetTextDialogWindow(const Value: string);
begin
  FTextDialogWindow := Value;
end;

procedure TChangeCaption.SetTextPromptWindow(const Value: string);
begin
  FTextPromptWindow := Value;
end;

initialization

  {$IFDEF FREEWARE}
  {$IFDEF TMSIW51}
  GLicense.ThirdPartyHook(FWNOTICE, True);
  {$ELSE}

  {$IFNDEF DELPHI7_LVL}
  GLicense.ThirdPartyHook(FWNOTICE, True);
  {$ELSE}
  AddThirdPartyLogo(FWNOTICE, True);
  {$ENDIF}

  {$ENDIF}
  {$ENDIF}
end.
